<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Traits classes for arithmetic>

  Traits classes are small classes, which are used to deduce new types in
  specific constructions. By specialization of parametric definitions, they
  combine flexibility and genericity.\ 

  We use the following conventions: The type defined by a trait class is
  denoted by <verbatim|T>. The integer value defined by a trait class is
  denoted by <verbatim|V>. For example, the trait <verbatim|kernelof> (see
  below), defines the arithmetic kernel associated to <verbatim|C> by
  <verbatim|kernelof\<less\>C\<gtr\>::T>.\ 

  Usually the default value is <verbatim|type::null_t> for traits defining a
  type and <verbatim|type::false_t> for boolean traits. Traits on arithmetic
  data types are not directly specialized, each trait class, say
  <verbatim|X>, uses the trait <verbatim|X_> for specialization. The reason
  for this mechanism is that generally the traits do not depend on type
  qualifiers, such as <verbatim|const> or <verbatim|&>. For instance,
  <verbatim|kernelof\<less\>const C&\<gtr\>::T> is associated to
  <verbatim|kernelof_\<less\>C\<gtr\>::T>.\ 

  \;

  See: <verbatim|synaps/arithm/traits.h>

  <subsection| Arithmetic Kernel>

  It is often a difficult task to define all the traits related to a new data
  type. That is why we introduce here the definition of kernels.

  \;

  \ Until now <name|synaps> used <name|gmp> library for extended, rational
  and integer arithmetic, and traits were defined for each <name|gmp>
  particular data type. In this release of <name|synaps> library, the
  <name|gmp> numbers are put together into a kernel named <verbatim|GMP>
  which is simply a data structure containing the following type definition
  (as <verbatim|typedef>):

  <\itemize-dot>
    <item><verbatim|integer>, assumed to be an exact integer implementation.

    <item><verbatim|rational>, assumed to be an exact rational
    implementation.

    <item><verbatim|floating>, assumed to be an extended floating point
    arithmetic.

    <item><verbatim|ieee>, assumed to be a fixed precision ieee compliant
    number type.
  </itemize-dot>

  \;

  All the traits classes for <name|gmp> numbers are now defined by their
  kernel (<verbatim|GMP> kernel), available by the traits class
  <verbatim|kernelof>.\ 

  By default, <verbatim|kernelof_\<less\>X\<gtr\>::T> is
  <verbatim|type::null_t>.

  After an <verbatim|#include \<less\>synaps/arithm/gmp.h\<gtr\>>,
  <verbatim|kernelof\<less\>X\<gtr\>::T> is <verbatim|GMP> for <verbatim|X =
  ><verbatim|ZZ>, <verbatim|QQ>, <verbatim|RR>.

  This new feature will soon allow <name|synaps> library to work with other
  library than <verbatim|GMP>, such as <verbatim|CLN>.

  <subsection| Rings and Fields>

  Here are trait classes which test basic properties of the type:

  <\itemize>
    <item><verbatim|isring\<less\>C\<gtr\>> is a boolean trait, which
    specifies if <verbatim|C> implements a ring.

    <item><verbatim|isfield\<less\>C\<gtr\>> is also provided. Notice that we
    consider <verbatim|double> as field.
  </itemize>

  To get the ring and field associated to a given type, we use the following
  trait classes:

  <\itemize>
    <item><verbatim|ringof\<less\>X,Y\<gtr\>::T> \ specify which data type
    can be used to represent sum and multiplication between type <verbatim|X>
    and type <verbatim|Y>. For example, given a kernel <verbatim|K>,

    <\code>
      \ \ ringof\<less\>K::integer,K::floating\<gtr\>::T
    </code>

    defines <verbatim|K::integer>. It is used for example in the dotproduct
    of two different vector types.\ 

    <item><verbatim|fieldof\<less\>X,Y\<gtr\>::T> is similar for field
    operations. Notice again that we consider <verbatim|double> as a field.
  </itemize>

  Here are our default definitions:

  <\code>
    \ \ ringof\<less\>K::integer,K::floating\<gtr\>::T --\<gtr\> K::floating

    \ \ ringof\<less\>K::integer,K::rational\<gtr\>::T --\<gtr\> K::rational

    \ \ ringof\<less\>K::integer,K::ieee\<gtr\>::T --\<gtr\> K::ieee

    \ \ ringof\<less\>K::floating,K::rational\<gtr\>::T --\<gtr\> K::rational

    \ \ ringof\<less\>K::floating,K::ieee\<gtr\>::T --\<gtr\> K::floating

    \ \ ringof\<less\>K::ieee,K::rational\<gtr\>::T --\<gtr\> K::rational

    \ 

    \ \ fieldof\<less\>K::integer,K::floating\<gtr\>::T --\<gtr\> K::rational

    \ \ fieldof\<less\>K::integer,K::rational\<gtr\>::T --\<gtr\> K::rational

    \ \ fieldof\<less\>K::integer,K::ieee\<gtr\>::T --\<gtr\> K::rational

    \ \ fieldof\<less\>K::floating,K::rational\<gtr\>::T --\<gtr\>
    K::rational

    \ \ fieldof\<less\>K::floating,K::ieee\<gtr\>::T --\<gtr\> K::floating

    \ \ fieldof\<less\>K::ieee,K::rational\<gtr\>::T --\<gtr\> K::rational

    \ \ 
  </code>

  As one can see we neglect the overflow problem that can occurs with
  <verbatim|floating> and <verbatim|ieee>

  data types for ring operations.

  For field operations we take the supset that can represent each inverses,
  when the two data types are inexact we take the best of the two.

  Note that if a type <verbatim|C> is exact for the operations it implement
  then <verbatim|isexact\<less\>C\<gtr\>::T> is <verbatim|true_t>.

  <subsection|Subset, Supset>

  The <verbatim|subsetof\<less\>X,Y\<gtr\>> trait is a boolean trait class,
  specifying if <verbatim|X> is a subset of <verbatim|Y>. For example \ given
  a kernel <verbatim|K>, <verbatim|subsetof\<less\>K::integer,K::rational\<gtr\>><verbatim|::T>
  is <verbatim|true_t.>\ 

  The <verbatim|sup\<less\>X,Y\<gtr\>> is a more general concept used to
  choose a data type enclosing <verbatim|X> and <verbatim|Y> for all these
  operations, <verbatim|>it correspond to default value of <verbatim|ringof>
  and <verbatim|fieldof>, if <verbatim|sup\<less\>X,Y\<gtr\>> is not defined
  but <verbatim|X> <verbatim|issubsetof> of <verbatim|Y>, then
  <verbatim|sup\<less\>X,Y\<gtr\>> is defined by <verbatim|Y>.\ 

  <subsection|Integer, Rational, Floating>

  Any number type is associated to an integer (rational, floating, ieee), by
  the <verbatim|integerof> (<verbatim|rationalof,floatingof,ieeeof>) trait.

  For number type defined outside a kernel, <verbatim|ieeeof> defines
  <verbatim|double>.

  <verbatim|isinteger\<less\>C\<gtr\>::T>
  (<verbatim|isrational,isfloating,isieee>) is <verbatim|true_t> if
  <verbatim|C> is an integer (rational, floating, ieee).

  <subsection|Algebraic closure type>

  This class is a traits class used to specify the type corresponding to the
  algebraic closure of a given type <verbatim|C>. This type is given by
  <verbatim|algebraic_closure\<less\>C\<gtr\>::T>. It can be used as follows:

  <\quote-env>
    <\code>
      algebraic_closure\<less\> complex\<less\>double\<gtr\> \<gtr\>::T
      \ \ \ // is complex\<less\>double\<gtr\>

      algebraic_closure\<less\>double\<gtr\>::T
      \ \ \ \ \ \ \ \ \ \ \ \ \ \ // is complex\<less\>double\<gtr\>
    </code>
  </quote-env>

  <subsection|Real closure type>

  The class <verbatim|realof\<less\>C\<gtr\>> is a traits class used to
  specify the type corresponding to the real closure of <verbatim|C>. This
  type is given by \ <verbatim|realof\<less\>C\<gtr\>::T>. Here is an
  example:

  <\quote-env>
    <\code>
      realof\<less\>complex\<less\>double\<gtr\> \<gtr\>::T \ \ \ // is
      double

      realof\<less\>double\<gtr\>::T \ \ \ \ \ \ \ \ \ \ \ \ \ // is double
    </code>
  </quote-env>

  \;

  \;

  \;
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-10|<tuple|1.7|?>>
    <associate|auto-11|<tuple|1.8|?>>
    <associate|auto-12|<tuple|1.9|?>>
    <associate|auto-13|<tuple|1.10|?>>
    <associate|auto-14|<tuple|1.11|?>>
    <associate|auto-15|<tuple|1.12|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|auto-6|<tuple|1.5|?>>
    <associate|auto-7|<tuple|1.6|?>>
    <associate|auto-8|<tuple|1.6|?>>
    <associate|auto-9|<tuple|1.6|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-10|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.1.1|?>>
    <associate|toc-4|<tuple|1.1.2|?>>
    <associate|toc-5|<tuple|1.2|?>>
    <associate|toc-6|<tuple|1.2.1|?>>
    <associate|toc-7|<tuple|1.2.2|?>>
    <associate|toc-8|<tuple|1.2.2|?>>
    <associate|toc-9|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Traits
      classes for arithmetic> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc> Arithmetic Kernel
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc> Rings and Fields
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Subset, Supset
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc><with|font-family|<quote|tt>|language|<quote|verbatim>|>Integer,
      Rational, Floating <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Algebraic closure type
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.6<space|2spc>Real closure type
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>
    </associate>
  </collection>
</auxiliary>