#ifndef SYNAPS_ARITHM_RG_MPZ_H
#define SYNAPS_ARITHM_RG_MPZ_H


#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

//----------------------------------------------------------------------
std::ostream& Print(std::ostream& os, const MPZ& b)
 { mpz_out_str(stdout, 10, &b);  return os; }
//----------------------------------------------------------------------
void init(MPZ& rep) { mpz_init(&rep); }
void init_set(MPZ& rep, int si)      {mpz_init_set_si(&rep, si);}
void init_set(MPZ& rep, long int sl) {mpz_init_set_si(&rep, sl);}
void init_set(MPZ& rep, const char*& str, unsigned int b) 
{if (mpz_init_set_str(&rep, str, b))
	std::cerr<<"MPZ: The string "<<str<<" is not a valid number in base "<<b<<std::endl;
}
//----------------------------------------------------------------------
void clear(MPZ& rep) { mpz_clear(&rep); }
//----------------------------------------------------------------------
int  cmp(const MPZ& r1, const Rg<MPZ>&  r2){ return mpz_cmp(&r1,&r2.rep()); }
int  cmp(const MPZ& r1, int  r2)           { return mpz_cmp_si(&r1,(long)r2); }
int  cmp(const MPZ& r1, long r2)           { return mpz_cmp_si(&r1,(long)r2); }
int  cmp(const MPZ& r1, unsigned r2)       { return mpz_cmp_ui(&r1, r2); }
int  cmp(const MPZ& r1, unsigned long r2)  { return mpz_cmp_ui(&r1, r2); }
//----------------------------------------------------------------------  
inline void assign(MPZ& p, const MPZ& q)   {  mpz_set(&p,&q); }
//----------------------------------------------------------------------  
inline void add(MPZ& p, const MPZ& a, const MPZ& b) {mpz_add(&p,&a,&b);}
//----------------------------------------------------------------------  
inline void add(MPZ& p, const MPZ& a, int b) {mpz_add_ui(&p, &a,b);}
//----------------------------------------------------------------------
inline void sub(MPZ& p, const MPZ& a, const MPZ& b){mpz_sub(&p,&a,&b);}
//----------------------------------------------------------------------
inline void mul(MPZ& p, const MPZ& a, const MPZ& b) {mpz_mul(&p,&a,&b);}
inline void mul(MPZ& p, const MPZ& a, int b)        {mpz_mul_si(&p,&a,b);}
//----------------------------------------------------------------------
inline void div(MPZ& p, const MPZ& a, const MPZ& b) {mpz_div(&p,&a,&b);}
inline void div(MPZ& p, const MPZ& a, unsigned   b) {mpz_div_ui(&p,&a,b);}
//----------------------------------------------------------------------
inline void mod(MPZ& p, const MPZ& a, const MPZ& b) {mpz_mod(&p,&a,&b);}
inline void mod(MPZ& p, const MPZ& a, unsigned   b) {mpz_mod_ui(&p,&a,b);}
//----------------------------------------------------------------------
// inline void add(MPZ& p, const MPZ q)  { mpz_add(&p,&p,&q); }
// inline void sub(MPZ& p, const MPZ& q) { mpz_sub(&p,&p,&q); }
// inline void mul(MPZ& p, const MPZ& q) { mpz_mul(&p,&p,&q); }
// inline void div(MPZ& p, const MPZ& q) { mpz_div(&p,&p,&q); }
// inline void mod(MPZ& p, const MPZ& q) { mpz_mod(&p,&p,&q); }
// //----------------------------------------------------------------------
// inline void add(MPZ& p, int i) 
//    { if(i>=0)  mpz_add_ui(&p,&p,i); else mpz_sub_ui(&p,&p,-i); }
// inline void add(MPZ& p, unsigned int i) { mpz_add_ui(&p,&p,i); }
// inline void sub(MPZ& p, unsigned int i) { mpz_sub_ui(&p,&p,i); }
// inline void mul(MPZ& p, unsigned int i) { mpz_mul_ui(&p,&p,i); }
// inline void mul(MPZ& p, int i) 
// {
//    if (i >= 0)
//      mpz_mul_ui(&p, &p, (unsigned long) i);
//    else {
//      mpz_mul_ui(&p, &p, (unsigned long) (-i)); mpz_neg(&p, &p);
//    }
// }
// inline void div(MPZ& p, unsigned int ui) { mpz_div_ui(&p, &p, ui); }
// inline void div(MPZ& p, int i) 
// {
//   if (i >= 0)
//     mpz_div_ui(&p, &p, (unsigned long) i);
//   else
//     {
//       p._mp_size = -p._mp_size;
//       mpz_div_ui(&p, &p, ((unsigned long) -i));
//     }
//}
//----------------------------------------------------------------------

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_ARITHM_RG_MPZ_H

