#ifndef SYNAPS_ARITHM_TRAITS_RINGOF_H
#define SYNAPS_ARITHM_TRAITS_RINGOF_H

#include <synaps/arithm/traits/sup.h>  

__BEGIN_NAMESPACE_SYNAPS

namespace arithm {
  
  template<class X, class Y>
  struct ringof_ { typedef null_t T; };
  
  template<>
  struct ringof_<int, unsigned int> { typedef int T; };
  template<>
  struct ringof_<unsigned int, int> { typedef int T; };
  template<>
  struct ringof_<double,int> { typedef double T; };
  template<> 
  struct ringof_<int,double> { typedef double T; };


  template<class K, class X, class Y>
  struct kernel_ringof { typedef null_t T; };
  template<class K>
  struct kernel_ringof< K, typename K::integer, typename K::ieee  > 
  { typedef typename K::ieee T; };
  template<class K>
  struct kernel_ringof< K, typename K::ieee, typename K::integer > 
  { typedef typename K::ieee T; };
  template<class K>
  struct kernel_ringof< K, typename K::integer, typename K::rational  > 
  { typedef typename K::rational T; };
  template<class K>
  struct kernel_ringof< K, typename K::rational, typename K::integer > 
  { typedef typename K::rational T; };
  template<class K>
  struct kernel_ringof< K, typename K::integer, typename K::floating  > 
  { typedef typename K::floating T; };
  template<class K>
  struct kernel_ringof< K, typename K::floating, typename K::integer > 
  { typedef typename K::floating T; };
  template<class K>
  struct kernel_ringof< K, typename K::floating, typename K::rational >
  { typedef typename K::rational T; };
  template<class K>
  struct kernel_ringof< K, typename K::rational, typename K::floating >
  { typedef typename K::rational T; };
  template<class K>
  struct kernel_ringof< K, typename K::floating, typename K::ieee >
  { typedef typename K::floating T; };
  template<class K>
  struct kernel_ringof< K, typename K::ieee, typename K::floating >
  { typedef typename K::floating T; };
  template<class K>
  struct kernel_ringof< K, typename K::rational, typename K::ieee  > 
  { typedef typename K::rational T; };
  template<class K>
  struct kernel_ringof< K, typename K::ieee, typename K::rational > 
  { typedef typename K::rational T; };
  
  /*
  template<class X>
  struct ringof_<double, X >
  {
  typedef typename kernel_ringof< typename kernelof<X>::T, double, X >::T KX;
    };
  */

  /// x (in X) + y (in Y)  is subset of ringof<X,Y>::T
  template<class X, class Y> 
  struct ringof
  {
      /// if ringof is undefined (ringof_<u(X),u(Y)>::T == null_t)
      /// ringof is given as the sup
      typedef typename ucstref<X>::T                _X;
      typedef typename ucstref<Y>::T                _Y;
      typedef typename kernel_ringof< typename kernelof<X>::T,_X,_Y>::T   KXY;
      typedef typename kernel_ringof< typename kernelof<Y>::T,_Y,_X>::T   KYX;
      typedef typename sfirstnn< typename ringof_<_X,_Y>::T, 
				 KXY, KYX, typename sup<X,Y>::T >::T T;
    };
}
__END_NAMESPACE_SYNAPS

#endif
