<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Components of the library>

  The implementation of classes and functions is organised in three levels,
  namely the <em|container>, <em|module> and <em|view> levels<em|>. Let us
  describe them in more details.

  <subsection|The containers and domains>\ 

  <\itemize>
    <item>The <em|containers> specify the internal representation.

    <item>They provide methods or functions for accessing, scanning,
    creating, transforming this representation, often through iterators.\ 
  </itemize>

  Here are some examples:

  <\quote-env>
    <\code>
      \ linalg::rep1d\<less\>C\<gtr\>; linalg::rep2d\<less\>C\<gtr\>;
      lapack::rep2d\<less\>C\<gtr\>; \ \ \ \ 

      \ linalg::sparse2d\<less\>C\<gtr\>; linalg::toeplitz\<less\>C\<gtr\>;
      bezier::rep1d\<less\>C\<gtr\>; \ ...
    </code>
  </quote-env>

  <\itemize>
    <item>The containers are attached to <em|domains> (implemented as
    namespaces):

    <\quote-env>
      <\code>
        \ linalg; lapack; \ upol; bezier; ...
      </code>
    </quote-env>

    <item>The domains allow to define (by an external user) specialized
    algorithms for given containers (through the scope name mechanism) \ or
    by derivation and redefinition of functions.\ 

    <item>They are open and thus can be extended easily.
  </itemize>

  For instance, writting the following code

  <\quote-env>
    <\code>
      \ namespace lapack {

      \ template \<less\>class C\<gtr\>\ 

      \ \ \ void solve(LU,linalg::rep1d\<less\>C\<gtr\> & x, const
      rep2d\<less\>C\<gtr\> & A,\ 

      \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ const linalg::rep1d\<less\>C\<gtr\> &
      b) {...};

      \ }
    </code>
  </quote-env>

  defines a specialized <verbatim|solve> function for the containers of type
  <verbatim|lapack::rep2d\<less\>C\<gtr\>>.

  <subsection|Modules>

  <\itemize>
    <item>They provide collections of generic implementations which apply to
    a ``category'' of objects.

    <item>A minimal set of constraints is imposed on the parameters of these
    generic functions, for containers belonging to a same category.

    <item>They can be combined or extended naturally.
  </itemize>

  They can be combined or extended naturally. The main modules of the library
  are

  <\quote-env>
    <\code>
      \ VECTOR, MATRIX, UPOLDAR, MPOLDST ...
    </code>
  </quote-env>

  These modules are available in files with the suffixe <verbatim|.m>.

  <subsection|Views>

  <\itemize>
    <item>They specify how to manipulate or to see the containers as
    mathematical objects.

    <item>The internal data is available, via the method <verbatim|rep()>.

    <item>They are usualy classes, parameterised by the container type
    <verbatim|R> and sometimes by trait classes which precise the
    implementation:
  </itemize>

  <\quote-env>
    <\code>
      \ VectDse\<less\>double,linalg::rep1d\<less\>C\<gtr\> \<gtr\>;
      //equivalently VectDse\<less\>double\<gtr\>;
    </code>
  </quote-env>

  <\itemize>
    <item> The implementations of these views are based on modules.
  </itemize>

  <\quote-env>
    <\code>
      \ template\<less\>class C, class R\<gtr\>\ 

      \ UPolDse\<less\>C,R\<gtr\>\ 

      \ \ \ \ \ \ operator*(const UPolDse\<less\>C,R\<gtr\>& a, const
      UPolDse\<less\>C,R\<gtr\>& b)

      \ {

      \ \ \ UPolDse\<less\>C,R\<gtr\> r;

      \ \ \ using UPOLDSE::mul; mul(r.rep(),a.rep(),b.rep());

      \ \ \ return r;

      \ }
    </code>
  </quote-env>
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|toc-1|<tuple|<uninit>|?>>
    <associate|toc-2|<tuple|<uninit>|?>>
    <associate|toc-3|<tuple|<uninit>|?>>
    <associate|toc-4|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Components
      of the library> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>The containers and domains
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Modules
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Views
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>