<TeXmacs|1.0.4>

<style|generic>

<\body>
  <section|Template expressions>

  <em|Template expressions> are type manipulations used to guide the compiler
  to produce optimised code. We illustrate their use on the addition of
  vectors. Consider for instance the following instruction:

  <\code>
    \ \ \ v = v1 + v2 + v3;
  </code>

  \ where <verbatim|v, v1,v2, v3> are vectors (say of type
  <verbatim|Vector>). Usual implementations would defined the operator. Usual
  implementations would defined the operator

  <\code>
    \ \ \ Vector & operator+(const Vector & v1, Vector & v2);
  </code>

  \ so that the previous instruction will produce the temporary vector (say
  <verbatim|tmp1>) for the addition of <verbatim|v1> and <verbatim|v2>,
  another one (say <verbatim|vtmp2>) for the addition of <verbatim|vtmp1> and
  <verbatim|v3> and will use the operator

  <\code>
    \ \ \ Vector & Vector::operator=(const Vector & v);
  </code>

  \ to assign <verbatim|v> to the value of <verbatim|vtmp2>, using the loop

  <\code>
    \ \ \ for(index_type i=0; i\<less\> v.size(); ++i) v[i] = vtmp2[i];
  </code>

  In such an implementation, two temporary vectors are allocated and deleted,
  which is time and memory consuming, especially if the vectors are of large
  size. A more optimised implementation would consist in writing the code

  <\code>
    \ \ \ for(index_type i=0; i\<less\>v.size(); ++i) v[i] =
    v1[i]+v2[i]+v3[i];
  </code>

  This code can be generated directly, at compilation time, using
  <em|template expression> techniques. Instead of defining the operator
  <verbatim|+>, as before, we define it with the following signature:

  <\code>
    VAL\<less\>OP\<less\>'+',Vector,Vector\<gtr\> \<gtr\> operator+(const
    Vector & v1, Vector & v2);
  </code>

  \ where <verbatim|VAL\<less\>OP\<less\>'+',Vector,Vector\<gtr\> \<gtr\>> is
  the type of a data-structure which contains references to the two vectors
  <verbatim|v1>, <verbatim|v2>, but which does not compute the sum of this
  two vectors. The instruction <verbatim|v1+v2+v3> we are considering,
  produces a data of type

  <\code>
    \ \ \ VAL\<less\>OP\<less\>'+',Vector,
    VAL\<less\>OP,'+',Vector,Vector\<gtr\> \<gtr\> \<gtr\>;
  </code>

  \ Now, the assignment

  <\code>
    \ \ \ v = v1+v2+v3;<name|>
  </code>

  will involve the operator

  <\code>
    \ \ \ template\<less\>class R\<gtr\> Vector & operator=(const
    VAL\<less\>R\<gtr\> & v);
  </code>

  which is implemented as

  <\code>
    \ \ \ for(index-stype i=0; i\<less\>v.size(); ++i) (*this)[i] =
    _elem(v,i);
  </code>

  The key point is that the compiler will expand the call to the inline
  function <verbatim|_elem(v,i)> into <verbatim|v1[i]+v2[i]+v3[i]>, because
  the ith element of an element <verbatim|v> of type
  <verbatim|VAL\<less\>OP\<less\>'+',...,...\<gtr\> \<gtr\>> is the sum of
  the ith of its two components.\ 

  Here are the arithmetic operations implemented with <em|template
  expressions> and their meaning:

  <\itemize>
    <item><verbatim|OP\<less\>'+',A,B\<gtr\>>, sum of two terms.

    <item><verbatim|OP\<less\>'-',A,B\<gtr\>>, substraction of two terms.

    <item><verbatim|OP1\<less\>'-',A\<gtr\>>, opposite of a term.\ 

    <item><verbatim|OP\<less\>'*',A,B\<gtr\>>, multiplication of two terms.

    <item><verbatim|OP\<less\>'.',A,B\<gtr\>>, multiplication of the term of
    type <verbatim|A> by a scalar of type <verbatim|B>.

    <item><verbatim|OP\<less\>'/',A,B\<gtr\>>, division of two terms.

    <item><verbatim|OP\<less\>'%',A,B\<gtr\>>, division of the term of type
    <verbatim|A> by a scalar of type <verbatim|B>.

    <item><verbatim|OP\<less\>'^',A,B\<gtr\>>, power of the term of type
    <verbatim|A> by a scalar of type <verbatim|B>.
  </itemize>

  Such arithmetic operations will return an unevaluated arithmetic tree and
  the computation will be performed only when the assignment operator will be
  used.
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|toc-1|<tuple|<uninit>|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Template
      expressions> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <pageref|auto-1><vspace|0.5fn>
    </associate>
  </collection>
</auxiliary>