/********************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: Clock.h,v 1.1 2005/07/11 11:23:11 mourrain Exp $
 ********************************************************************/
#ifndef SYNAPS_UTIL_CLOCK_H
#define SYNAPS_UTIL_CLOCK_H
/********************************************************************/


#include <ctime>
#include <synaps/init.h>

__BEGIN_NAMESPACE_SYNAPS

class Clock
{
	public:

		// Construct and start clock immediately.
		//
		Clock() : start_(clock()), running(true) {};

		~Clock() {};

		// Start clock; same effect as constructor.
		//
		void start()
		{
			start_ = clock();
			running = true;
		}

		// Time returned by time() is from start of clock
		// until the call of this function.
		//
		void stop()
		{
			stop_ = clock();
			running = false;
		}

		// Return elapsed CPU time from start of clock until the call
		// of stop(), or the current value if clock is running.
		//
		double time()
		{
			if (running)
				return double(clock() - start_) / CLOCKS_PER_SEC;
			else
				return double(stop_ - start_) / CLOCKS_PER_SEC;
		}

	private:
		clock_t start_;
		clock_t stop_;
		bool running;
};

__END_NAMESPACE_SYNAPS

#endif  // SYNAPS_UTIL_CLOCK_H

