/*********************************************************************
*      This file is part of the source code of AXEL library          *
*   Author(s): G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef synaps_io_axel_ostream_h_
#define synaps_io_axel_ostream_h_
/********************************************************************/
#include <synaps/init.h>
#include <iostream>
#include <fstream>
#include "Color.h"
#include "Width.h"
/*********************************************************************/
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
//! Container for the axel domain
/*!
  Here, we have implemented the structures to store the results 
  we compute in the axel format. 
 */
namespace axel
{
  //------------------------------------------------------------------
  //! The ostream file structure 
  /*!
    Structure which allows us to specify the file in order
    to store the results.
   */
  struct ostream
  {
    /*! The file where we store the result.*/
    char *file;
    Color color;
    Width width;

    /*! The default constructor. */
    ostream(): file("tmp.axl") { init(); }
  
    /*! The default destructor. */
    ~ostream() {}

    /*! 
      \param f a file.
      
      Construct an ostream with the file \c f and initialize
      the options of axel in the file.
    */
    ostream(char *fic) 
    {
      color=Color(255,0,0);
      file=fic;
      init();
    }
    /*!
      Allows us to clean the file and reinitialize.
     */
    void init()
      {
	std::ofstream f(file,std::ios::out);
	if( !f )
	  {
	    std::cerr << "Creation refused" << std::endl;
	    exit(1);
	  }
	f << "<axl>\n";
	f.close();
      }

    /*!
      Allows us to execute axel.
     */
    void view()
      {
	std::ofstream f(file, std::ofstream::out | std::ofstream::app);
	f << "</axl>\n";
	f.close();

	char cmd[1000];
	strcpy(cmd,"axel \0");
	strcat(cmd,file);	\
	strcat(cmd," &");
	system(cmd);
      }
  };

  inline std::ostream& operator<<(std::ostream& os, const ostream & g)
  {
    os <<"Axel("<<g.file<<")"; return os;
  }

  inline ostream& operator<<(ostream & os, const Color & c)
  {
    os.color=c; return os;
  }

  inline ostream& operator<<(ostream & os, const Width & w)
  {
    os.width=w; return os;
  }


  inline ostream& operator<<(ostream & os, char* s)
  {
    std::ofstream f;
    f.open (os.file, std::ofstream::out | std::ofstream::app);
    f<<s;
    f.close();
    return os;
  }

  template<class R>
  inline ostream& operator<<(ostream & os, const R& s)
  {
    std::ofstream f;
    f.open (os.file, std::ofstream::out | std::ofstream::app);
    f<<s;
    f.close();
    return os;
  }
}

/*********************************************************************/
__END_NAMESPACE_SYNAPS 
#endif // synaps_io_axel_mesh_H__
