/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History:
$Id: seq1d.h,v 1.2 2005/08/29 14:30:18 mourrain Exp $
*********************************************************************/
#ifndef SYNAPS_UTIL_SEQ1D_H
#define SYNAPS_UTIL_SEQ1D_H

#include <iterator>
#include <synaps/init.h>
#include <synaps/base/type.h>
#include <synaps/base/shared_object.h>
#include <synaps/base/create_iterator.h>


__BEGIN_NAMESPACE_SYNAPS

/** Unidimensional array of coefficients with a size. **/
template <class C, class I,class N=unsigned int>
struct seq1d {

  typedef C            value_type;
  typedef N            size_type;
  typedef I            iterator;
  typedef iterator     const_iterator;
  typedef std::reverse_iterator<const_iterator>  const_reverse_iterator;
  typedef std::reverse_iterator<iterator>        reverse_iterator;

  size_type size_;
  iterator  begin_, end_;

  seq1d() : size_(0), begin_(), end_() {}
  seq1d(const seq1d<C,I,N> & v):
    size_(v.size_), begin_(v.begin_), end_(v.end_){}
  seq1d(size_type n,  iterator b, iterator e):
    size_(n), begin_(b), end_(e){};

  void reserve(size_type n) {
    if(n >size_){
      C* tmp= new C[n];
      size_=n;
      begin_= create(I(),tmp,0);
      end_  = create(I(),tmp,n);
    }else{
      size_=n;end_=begin_+n;
    }
  }
  void resize(size_type n) {}

  seq1d<C,I,N> & operator=(const seq1d<C,I,N> & v)
    {
      size_=v.size_;begin_=v.begin_;end_ =v.end_; return *this;
    }

  size_type size() const {return size_;}

  iterator        begin()        { return begin_; }
  iterator        begin() const  { return begin_; }

  iterator        end()        { return end_; }
  iterator        end() const  { return end_; }

  reverse_iterator       rbegin()       {return reverse_iterator(end_); }
  const_reverse_iterator rbegin() const {return const_reverse_iterator(end_);}

  reverse_iterator       rend()       {return reverse_iterator(begin_); }
  const_reverse_iterator rend() const {return const_reverse_iterator(begin_);}

  inline       C & operator[] (size_type i)       {return *(begin_+i);}
  inline const C & operator[] (size_type i) const {return *(begin_+i);}
};
//----------------------------------------------------------------------
template<class C, class I,class N>
I create(I s,
	 const seq1d<C,I,N> & a, unsigned int i)
{
  return a.begin()+i;
}
//----------------------------------------------------------------------
namespace let 
{
  template<class C, class I,class N>
  void assign(shared_object<seq1d<C,I,N> > & a,	 
	      const shared_object<seq1d<C,I,N> > & b)
  {

    a->resize( b->size() );
    typename seq1d<C,I,N>::iterator ia=a->begin(),ea=a->end();
    typename seq1d<C,I,N>::const_iterator ib=b->begin();
    for ( ; ia != ea; ia ++, ib++  ) assign(*ia,*ib);
  }
}

namespace type 
{
  template<class X, class I, class N, class Y>
  struct instanceof< seq1d<X,I,N>, Y >
  {
    typedef seq1d< Y, I, N > T;
  };
};


//======================================================================
__END_NAMESPACE_SYNAPS
#endif // SYNAPS_UTIL_SEQ1D_H

