/*********************************************************************
*      This file is part of the source code of SYNAPS                *
*      (C) B. Mourrain, GALAAD, INRIA,                               *
**********************************************************************
History: 
$Id: parser_def.l,v 1.1 2005/07/11 11:25:53 mourrain Exp $
**********************************************************************/
%{
#ifndef min
#define min(a,b) ((a) <= (b) ? (a) : (b))
#endif

char *synaps_input = 0; 
char *synaps_inputptr = 0; /* current position in myinput */
//char *yytext; /* current position in myinput */
char *synaps_inputlim = 0; /* end of data */
char *yylval = 0;

#include <string.h>

#define COEF  10
#define XID   11
#define SUM   12
#define MINUS 13
#define MULT  14
#define TERMINATOR  15
#define SEPARATOR   16
#define BLIST       17
#define ELIST       18

#undef YY_INPUT
#define YY_INPUT(b, r, ms) ( r= synaps__yyinput(b, ms))


%}

DIGIT		[[:digit:]]
INT		{DIGIT}+
FLOAT		{INT}?"."{INT}([eE][-+]?{INT})?
NUM		{INT}|{FLOAT}
VAR		[a-z]({INT})?
SEP             ":"|";"|","

%%

{NUM} {
	int n = strlen(yytext);
	yylval = (char *)malloc(sizeof(char)*(n+1));
	memcpy(yylval,yytext,n+1);
	return COEF;
}

{VAR}("^"{INT})? {
	int n = strlen(yytext);
	yylval = (char *)malloc(sizeof(char)*(n+1));
	memcpy(yylval,yytext,n+1);
	return XID;
}

";" return TERMINATOR;
":" return TERMINATOR;
"," return SEPARATOR;
"+" return SUM;
"-" return MINUS;
"*" return MULT;
"[" return BLIST;
"]" return ELIST;

%%

int yywrap (void) {
	return 1;
}

int synaps__yyinput (char *buf, int max_size) {
	int n = min(max_size, synaps_inputlim - synaps_inputptr);

	if (n>0) {
		memcpy(buf, synaps_inputptr, n);
		synaps_inputptr +=n;
	}
	return n;
}
