<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Linear algebra>

  The main categories of objects for doing linear algebra are\ 

  <\itemize-dot>
    <item><samp|VectDse>: dense vectors,

    <item><samp|MatrDse>: dense matrices,

    <item><samp|MatrStr>: structured matrices,

    <item><samp|MatrSps>: sparse matrices. 0
  </itemize-dot>

  <subsection|Vectors>

  \ If one want to construct vectors with <samp|double> coefficients, one can
  use for instance the definition:

  <\code>
    \ typedef VectDse\<less\>double\<gtr\> vect_t;
  </code>

  This is equivalent to

  <\code>
    \ typedef VectDse\<less\>double,linalg::rep1d\<less\>double\<gtr\>
    \<gtr\> vect_t;
  </code>

  the default value for the second argument of the parameterised type
  <samp|VectDse> being <samp|linalg::rep1d\<less\>C\<gtr\>>. It specifies the
  container type <samp|linalg::rep1d\<less\>C\<gtr\>> which is\ 

  <\itemize-dot>
    <item>a one-dimensional array of coefficients of type <samp|C>,

    <item>with a field corresponding to its size.
  </itemize-dot>

  But one can also use the containor <samp|std::vector\<less\>C\<gtr\>> of
  the standard library or any other convenient container, which provides the
  same functions. See the file <verbatim|synaps/linalg/VectDse.h> for more
  details on the implementation of this <em|view>.\ 

  A vector can be constructed and manipulated as follows:

  <\code>
    \ double u[]={1,-2,1}; vect_t V(3,u);

    \ vect_t W= V*2, U; U -= W+3*V;
  </code>

  File: <verbatim|step2-1.cc>

  <subsection|Matrices>

  For the definition of dense matrices with double coefficients, you can use
  the definition:\ 

  <\code>
    \ typedef MatrDse\<less\>double\<gtr\> matr_t;
  </code>

  Here the default container is <samp|linalg::rep2d\<less\>C\<gtr\>> which
  extends <samp|linalg::rep1d\<less\>C\<gtr\>>, by specifying the number of
  rows and columns If instead, we use the definition

  <\code>
    \ typedef MatrDse\<less\>double,lapack::rep2d\<less\>double\<gtr\>
    \<gtr\> matr_t;
  </code>

  that is the container <samp|lapack::rep2d\<less\>C\<gtr\>>, we are able to
  use the routines of the LAPACK library. The internal representation is also
  a one-dimensional array with a number of rows, a number of columns, a size
  bigger than the product of these two last numbers and a way to access to
  the element of index (i, j). In order to be able to use this container, the
  following include instruction should be given:

  <verbatim| #include \<less\>synaps/linalg/lapack.h\<gtr\>>

  More details on the implementation of the matrix interface can be found in
  the header file <verbatim|synaps/linalg/MatrDse.h>.

  Here are some illustrations of its use:

  <\code>
    \ matr_t A(3,3,"1 2 3 4 5 6 7 8 9 10",ByRow());
  </code>

  The matrix <samp|A> is defined from a string of coefficients. It is a 3x3
  matrix, filed by row. It is printed as:

  <\equation*>
    <left|[><tabular|<tformat|<table|<row|<cell|1>|<cell|2>|<cell|3>>|<row|<cell|4
    >|<cell|5>|<cell|6>>|<row|<cell|7>|<cell|8>|<cell|9>>>>><right|]>
  </equation*>

  The usual operations on matrices and vectors are available from this type:

  <\code>
    \ using std::cout; using std::endl;

    \ vect_t W = A*V; cout \<less\>\<less\> W \<less\>\<less\>endl;
  </code>

  <\equation*>
    <left|[><tabular|<tformat|<table|<row|<cell|0>|<cell|0>|<cell|0>>>>><right|]>
  </equation*>

  <\code>
    \ matr_t B = A*A; cout \<less\>\<less\> B \<less\>\<less\> endl;\ 
  </code>

  <\equation*>
    <left|[><tabular|<tformat|<table|<row|<cell|30>|<cell|36>|<cell|42>>|<row|<cell|66
    >|<cell|81>|<cell|96>>|<row|<cell|102>|<cell|126>|<cell|150>>>>><right|]>
  </equation*>

  <\code>
    \ matr_t C(-A); C += B + A*2; cout \<less\>\<less\>C\<less\>\<less\>
    endl;\ 
  </code>

  <\equation*>
    <left|[><tabular|<tformat|<table|<row|<cell|31>|<cell|38>|<cell|45>>|<row|<cell|70
    >|<cell|86>|<cell|102>>|<row|<cell|109>|<cell|134>|<cell|159>>>>><right|]>
  </equation*>

  <subsection|Numerical linear algebra>

  Standard routines from numerical linear algebra are available on matrices
  with <samp|lapack> containers, one the corresponding header files:

  <verbatim| #include \<less\>synaps/linalg/Svd.h\<gtr\>>

  \ <verbatim|#include \<less\>synaps/linalg/Eigen.h\<gtr\>>

  are included. Some implementation (such as <samp|Svd>) are also available
  for generic matrices.

  <\code>
    \ cout\<less\>\<less\> Svd(A)\<less\>\<less\>endl; cout \<less\>\<less\>
    Eigenval(A) \<less\>\<less\> endl;\ 
  </code>

  <\equation*>
    <left|[><tabular|<tformat|<table|<row|<cell|16.8481>|<cell|1.06837>|<cell|3.06953e-16>>>>><right|]>
  </equation*>

  <\equation*>
    [<tabular|<tformat|<table|<row|<cell|(-1.11684,0)>|<cell|(16.1168,0)>|<cell|(-3.74342e-16,0)>>>>>]
  </equation*>

  Here the type of the eigenvalues is <samp|complex\<less\>double\<gtr\>>.
  They are printed with real and imaginary part between parentheses.

  File: <verbatim|step2-2.cc>

  <\exercise>
    Estimate the mean time for computing the <samp|svd> of a nxn matrix by
    iterating the computation on N random matrices with coefficients between
    -1 and 1 (using the routine <samp|1-(2.0*rand())/RAND MAX> of
    <samp|stdlib.h>). Compare it with the representation using <samp|lapack>
    routine. Compare with different compilation options. To count the time,
    we will use the instructions <samp|Clock c; c.start(); ...; c.stop();
    c.time();> provided by the file <samp|synaps/base/Clock.h>.
    <hlink|<with|mode|math|\<boxdot\>>|step2-exo1.cpp>
  </exercise>

  <subsection|Views on vectors and matrices>

  The linear algebra classes also offers local views on subobjects. A row (or
  a column) of a matrix, for instance, is a view on the corresponding
  elements of the matrix and share the properties of a vector. Indeed, it is
  a <samp|VectDse> with a special container and can be manipulated as such.
  The types of the columns and the rows are accesible as\ 

  <\code>
    \ MatrDse\<less\>double\<gtr\>::row_type

    \ MatrDse\<less\>double\<gtr\>::col_type
  </code>

  The characteristic of these views is that performing operations on the
  vectors will affect the object the view is looking at. For instance,

  <\code>
    \ Col(A,1) = Col(A,0);
  </code>

  transforms the matrix

  <\equation*>
    A\<assign\><left|[><tabular|<tformat|<table|<row|<cell|1>|<cell|2>|<cell|3>>|<row|<cell|4
    >|<cell|5>|<cell|6>>|<row|<cell|7>|<cell|8>|<cell|9>>>>><right|]>
  </equation*>

  \;

  into:\ 

  <\equation*>
    A =<left|[><tabular|<tformat|<table|<row|<cell|1>|<cell|1>|<cell|3>>|<row|<cell|4
    >|<cell|4>|<cell|6>>|<row|<cell|7>|<cell|7>|<cell|9>>>>><right|]>
  </equation*>

  Similarly

  <\code>
    \ Row(A,1) -= Row(A,0)*4;
  </code>

  yields:

  <\equation*>
    A= <left|[><tabular|<tformat|<table|<row|<cell|1>|<cell|1>|<cell|3>>|<row|<cell|0
    >|<cell|0>|<cell|-6>>|<row|<cell|7>|<cell|7>|<cell|9>>>>><right|]>
  </equation*>

  These functions <samp|Col> and <samp|Row> produce vector views on the
  matrix. They can be interprated as references on the internal data.
  Remember that the memory space of the views <samp|Row(A,i)>,
  <samp|Col(A,j)> are not duplicated but shared with the matrix <samp|A>.
  \ Such a vector view can be copied into a standard vector:

  <\code>
    \ vect_t V1 = Col(A,1)*2, V2= Row(A,0)+ Row(A,1);

    \ vect_t W = V1+V2;
  </code>

  yields

  <\equation*>
    W =<left|[><tabular|<tformat|<table|<row|<cell|3>|<cell|1>|<cell|11>>>>><right|]>
  </equation*>

  File: <verbatim|step2-3.cc>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|auto-6|<tuple|1.5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Linear
      algebra> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Vectors
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Matrices
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Numerical linear algebra
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Views on vectors and
      matrices <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>
    </associate>
  </collection>
</auxiliary>