<TeXmacs|1.0.6>

<style|generic>

<\body>
  <section|Polynomial solvers>

  The library provides several type of solvers of polynomial equations. The
  general signature for these functions is\ 

  <\code>
    \ Seq\<less\>S\<gtr\> solve (E, Mth\<less\>S,...\<gtr\>(...))

    \ S \ \ \ \ \ RootOf(E, Mth\<less\>S,...\<gtr\>(...))
  </code>

  \ where

  <\itemize-dot>
    <item><samp|E> is the equation(s) type,

    <item><samp|Mth\<less\>S,...\<gtr\>(...)> is the class which specifies
    the solving method. The first template argument is specifying the type of
    solution we want (eg. a sequence of values of roots of type <samp|S>.
    Some other parameters may be completing the solver definition. Other
    non-static parameters such as eg. the precision could also be given in
    (...).
  </itemize-dot>

  A <samp|solve> function computes a sequence of solutions, where as
  <samp|RootOf> computes one specific root.

  <subsection|Univariate solvers>

  Univariate solvers take as input a polynomial of the \ type <samp|UPolDse>.
  We illustrate their use on several exemples, now.

  Suppose for instance that we are manipulating univariate \ polynomials with
  integer coefficients (<samp|ZZ>) and that we want to compute the real roots
  by the Aberth method. Then after including the file:

  <verbatim| #include \<less\>synaps/usolve/Aberth.h\<gtr\>>

  we can use the instruction:

  <\code>
    \ UpolDse\<less\>ZZ\<gtr\> p("x^7-2324325131254354*x+1");

    \ Seq\<less\>RR\<gtr\> sol= solve(p, Aberth\<less\>RR\<gtr\>());
  </code>

  We obtain

  <\equation*>
    4.30232e<rsup|-16>, -4.99422, 4.99422
  </equation*>

  Instead, if we use <samp|Aberth\<less\>CC\<gtr\>()>, we obtain the sequence
  of all the <with|mode|math|23> complex roots. Notice that these are
  approximate values of the actual roots. The way to decide wether we have a
  real or a complex roots depends on a threshold. The level of approximation
  is controled by the class <samp|Aberth>.

  File: <verbatim|step4-1.cc>

  <\exercise>
    Compute the real (resp. complexe) roots of the polynomial
    <with|mode|math|W<rsub|n>(x)=<big|prod><rsub|1><rsup|n>(x-i) \<in\>
    \<bbb-Q\>[x]> for n=20. Same question for <with|mode|math|W<rsub|20>(x)+
    <frac|1|10<rsup|12>> *x<rsup|12>> . <hlink|<with|mode|math|\<boxdot\>>|step4-exo1.cpp>
  </exercise>

  If one is intereseted in the root of polynomial in an interval, subdivision
  solvers may be used. The following solver is using the Bernstein basis
  representation in order to approximate or isolate the real roots of a
  polynomial. It requires the following files:

  <\code>
    #include \<less\>synaps/usolve/bezier/SlvBzStd.h\<gtr\>

    #include \<less\>synaps/usolve/bezier/SlvBzBdg.h\<gtr\>

    #include \<less\>synaps/usolve/bezier/IslBzStd.h\<gtr\>
  </code>

  and is illustrated now:\ 

  <\code>
    \ UpolDse\<less\>ZZ\<gtr\> p("x^7-2324325131254354*x-1");

    \ cout\<less\>\<less\> solve(p, SlvBzStd\<less\>double\<gtr\>(1e-6))
    \<less\>\<less\> endl;\ 

    \ cout\<less\>\<less\> solve(p, SlvBzBdg\<less\>double\<gtr\>(1e-6),
    -400., 500.) \<less\>\<less\> endl;
  </code>

  The parameter <samp|1e-6> of the class <samp|SlvBzStd> gives the level of
  approximation of the roots. In this computation we look at the roots of
  <samp|p> in the interval <with|mode|math|[-400,500]>. The two solvers
  yields different approximation of the roots:

  <\eqnarray*>
    <tformat|<table|<row|<cell| -363.912>|<cell|-4.76837e<rsup|-07>>|<cell|363.912>>|<row|<cell|-363.956>|<cell|-2.78912e<rsup|-08>>|<cell|
    363.956>>>>
  </eqnarray*>

  <\itemize>
    <item>In the first case, the input coefficients are rounded to
    <samp|double> and the subvision algorithm is applied on this approximate
    polynomial. Its is a fast solver over the <samp|double> but should be
    used with caution, for certified computation when the input coefficients
    are big.

    <item>In the second case <samp|SlvBzBdg\<less\>double\<gtr\>> a lower and
    upper envelloppe with <samp|double> coefficients of the polynomial is
    computed. These enveloppes are used to approximate the roots or to detect
    if the precision is enough and to zoom if necessary. This approximation
    is certified.\ 
  </itemize>

  We can simply be interested by intervals containing one root:

  <\code>
    \ cout\<less\>\<less\> solve(p, IslBzStd\<less\>QQ\<gtr\>(1e-6))
    \<less\>\<less\> endl;
  </code>

  This computes a sequence of intervals with end values in
  <with|mode|math|\<bbb-Q\>> such that <with|mode|math|[a<rsub|i>,b<rsub|i>]>
  contains only one root of <samp|p>:

  <\equation*>
    -2324325131254355, -1, -1, 0, 1, 2324325131254355
  </equation*>

  The naming convention, which we use is the following: <verbatim|Slv...> is
  solver which approximate the real roots, <verbatim|Isl...> is a solver
  which isolate the real roots.\ 

  File: <verbatim|step4-2.cc>

  Roots of polynomials over <with|mode|math|\<bbb-Z\>> can be manipulated
  exactly, as algebraic numbers. Their internal representation will be the
  defining polynomial and an interval of isolation. It requires the following
  file:

  <\code>
    #include \<less\>synaps/usolve/Algebraic.h\<gtr\>
  </code>

  Here is an example, where we compute the real roots with rational isolating
  interval and evaluate the sign of a polynomial at the first root:

  <\code>
    \ UpolDse\<less\>ZZ\<gtr\> p("x^7-2314*x-1"),q("x^3-2*x+1");

    \ typedef algebraic::real\<less\>QQ\<gtr\> algebraic_t;

    \ Seq\<less\>algebraic_t\<gtr\> sol= solve(p,
    Algebraic\<less\>QQ\<gtr\>());

    \ cout\<less\>\<less\>SignAt(q,sol[0])\<less\>\<less\>endl;
  </code>

  It yields

  <\eqnarray*>
    <tformat|<table|<row|<cell|root_of ( *x<rsup|7>-2324325131254354) x-1, 1
    ) \ in>|<cell| [ -<frac|2324325131254355|439804651104>,<frac|
    -2324325131254355 |8796093022208>],>|<cell|approx = -363.956, ,
    \ >>|<row|<cell|root_of( x<rsup|7>-2324325131254354*x-1, 1 ) in>|<cell|[
    0,2324325131254355 ],>|<cell|approx = 363.956, >>|<row|<cell|root_of(
    x<rsup|7>-2324325131254354)x-1, 1 ) \ in >|<cell| [ <frac|
    -2324325131254355 |8796093022208>,0 ],>|<cell|approx = -3.00412
    e<rsup|-11>>>>>
  </eqnarray*>

  File: <verbatim|step4-3.cc>

  <subsection|Multivariate solvers>

  \ Two type of multivariate solvers are available in the library:

  <\itemize>
    <item>a normal form solver, generalising Grobner basis computation and

    <item>a subdivision solver using the representation of multivariate
    polynomials in the Bernstein basis.
  </itemize>

  The normal form solver can be used from the file:

  <\code>
    #include \<less\>synaps/msolve/Newmac.h\<gtr\>
  </code>

  as follows:

  <\code>
    \ typedef MPol\<less\>RR\<gtr\> mpol_t;

    \ std::list\<less\>mpol_t\<gtr\> E;

    \ E.push_back(mpol_t("x^2+y^2+z^2-1"));\ 

    \ E.push_back(mpol_t("x^2+2*x*y-y^2+z^2-1"));\ 

    \ E.push_back(mpol_t("3*x^2-x*y+4*y^2+z^2-2*x-3*y+z-2"));

    \;

    \ Seq\<less\>VectDse\<less\>std:complex\<less\>long double\<gtr\> \<gtr\>
    \<gtr\> sol = solve(E, Newmac\<less\>RR\<gtr\>());

    \ cout\<less\>\<less\> sol \<less\>\<less\>endl;
  </code>

  It yields a sequence of vectors with complex numbers over the <samp|long
  double>:

  <\eqnarray*>
    <tformat|<table|<row|<cell|[>|<cell|(1.40281,0.277651),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (1.40281,0.277651),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (0.450884,-1.72768)>|<cell|],>>|<row|<cell|[>|<cell|(1.40281,-0.277651),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (1.40281,-0.277651),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (0.450884,1.72768)>|<cell|],>>|<row|<cell|[>|<cell|(-0.305627,-1.35784e<rsup|-17>),(-0.30b5627,-2.26906e<rsup|-17>),(-0.901767,-4.39707e<rsup|-17>)>|<cell|],>>|<row|<cell|[>|<cell|(-0.576379,-1.25406e<rsup|-17>),(-1.62598e<rsup|-16>,2.08405e<rsup|-17>),(-0.817183,-2.11964e<rsup|-17>)>|<cell|],>>|<row|<cell|[>|<cell|(1.28819,-0.274842),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (9.78732e<rsup|-17>,3.27854e<rsup|-16>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (0.408591,0.866511)>|<cell|],>>|<row|<cell|[>|<cell|(1.28819,0.274842),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (2.14442e<rsup|-16>,-6.61071e<rsup|-17>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (0.408591,-0.866511)>|<cell|],>>|<row|<cell|[>|<cell|(1.40634e<rsup|-09>,1.43066e<rsup|-08>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (-9.37557e<rsup|-10>,-9.53772e<rsup|-09>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (1,2.382e<rsup|-15>)>|<cell|],>>|<row|<cell|[>|<cell|(-1.40634<rsup|-09>,-1.43066e<rsup|-08>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (9.37557e<rsup|-10>,9.53772e<rsup|-09>),
    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ (1,-7.4936e<rsup|-16>)>|<cell|]>>>>
  </eqnarray*>

  File: <verbatim|step4-4.cc><inactive|>

  The subdivision solver can be used from the file

  <verbatim| #include \<less\>synaps/msolve/sbdslv.h\<gtr\>>\ 

  We define the domain <with|mode|math|[a<rsub|1>,b<rsub|1>]\<times\>\<cdots\>\<times\>[a<rsub|n>,b<rsub|n>]>
  where we search the roots by the vector:

  <\code>
    \ \ VectDse\<less\>double\<gtr\> domain(6,"-1 1.1 -1 1.1 -1 1.1");
  </code>

  and call the subdivision solver with the standard preconditionner:\ 

  <\code>
    \ cout\<less\>\<less\>solve(E, SBDSLV\<less\>double,SBDSLV_RDRDL\<gtr\>(1e-6),
    domain)\<less\>\<less\>endl;
  </code>

  <\itemize>
    <item>The first parameter <samp|><samp|double> of <samp|SBDSLV> is the
    type of coefficients during the computation,

    <item>The second parameter <samp|VectDse\<less\>double\<gtr\>> \ is the
    type of domain,

    <item>The third parameter <samp|SBDSLV_RDRDL> specifies the
    preconditioner,

    <item>The first argument <samp|domain> \ specifies the domain where we
    search the solutions,

    <item>The second argument <samp|eps=1e-6> is precision to which we stop
    the subdivision.
  </itemize>

  It yields a sequence of 3 real vectors:

  <\eqnarray*>
    <tformat|<table|<row|<cell|[>|<cell|5.72114e<rsup|-08>
    \ \ \ \ \ \ \ \ \ \ \ 4.53431e<rsup|-08>
    \ \ \ \ \ \ \ \ \ \ \ \ 1>|<cell|]>>|<row|<cell|[>|<cell| -0.576379
    \ \ \ \ \ \ \ \ -2.21964e<rsup|-10> \ \ \ \ -0.817183
    >|<cell|]>>|<row|<cell|[>|<cell|-0.305627 \ \ \ \ \ \ \ \ \ \ -0.305627
    \ \ \ \ -0.901767 >|<cell|]>>>>
  </eqnarray*>

  \;

  File: <verbatim|step4-4.cc>

  \;
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Polynomial
      solvers> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|Univariate solvers
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|Multivariate solvers
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>
    </associate>
  </collection>
</auxiliary>