AC_DEFUN([RX_LIB_GMP_VERSION], [dnl
  ifelse([$1],,
     [],
     [gmp_min_version="$1"
      AC_MSG_CHECKING([for GNU MP version greater than or equal to $gmp_min_version])
      AC_TRY_RUN([
#ifdef __cplusplus
#include <gmpxx.h>
#else
#include <gmp.h>
#endif
#ifdef __cplusplus
int main (int argc, char **argv)
#else
int main (argc, argv)
  int argc; char **argv;
#endif
{
  int sub, version, i;
  char *vtext;
  vtext = "$1";
  for (sub = version = 0, i = 0; vtext[i] != '\0'; i++) {
    if (vtext[i] >= '0' && vtext[i] <= '9')
      sub = sub * 10 + (vtext[i] - '0');
    else if (vtext[i] == '.') {
      version = version * 100 + sub; sub = 0;
    }
  }
  version = version * 100 + sub;
  if (version == 0) return 0;
  while (version < 10000) version *= 100;
  if (__GNU_MP_VERSION * 10000 + __GNU_MP_VERSION_MINOR * 100
	 + __GNU_MP_VERSION_PATCHLEVEL >= version)
    return 0;
  else
    return 1;
}],
        [AC_MSG_RESULT([yes])
	 $2 ],
        [AC_MSG_RESULT([no])
	 $3 ])   dnl End of AC_TRY_RUN
     ])
])

AC_DEFUN([RX_LIB_SEARCH_GMP], [dnl

AC_ARG_WITH(gmp,
  AC_HELP_STRING([--with-gmp=<DIR>|yes|no],
                 [directory search path of GNU MP installation (defaults to
                  /usr /usr/local /usr/local/gmp). If argument is
		  yes, assume that GNU MP is accessible with the standard
		  compiler and linker paths (usually /usr/include and
		  /usr/lib). If argument is no, nothing is done.]),
  [if test "$withval" = yes ; then
    ac_assume_gmp_ok=yes
   elif test "$withval" != no ; then
    GMP_HOME_PATHS="$withval"
   fi],
  [GMP_HOME_PATHS="${DEFAULT_CHECKING_PATH}"])

if test -z "$GMP_HOME_PATHS" ; then
  GMP_HOME_PATHS="/usr /usr/local /usr/local/gmp"
fi;
AC_MSG_NOTICE([ checking GNU MP library])
if test "$ac_assume_gmp_ok" = yes; then
    AC_DEFINE([SYNAPS_WITH_GMP], 1,
              [Define to 1 if you have the <gmp.h> header file ])
    LIBS="-lgmp -lgmpxx $LIBS"
else
  for GMP_HOME in ${GMP_HOME_PATHS}
  do
    AC_MSG_NOTICE([looking for GNU MP library in $GMP_HOME])
    # disable cached results of previous tests in the loop
    unset ac_cv_header_gmp_h
    unset ac_cv_header_gmpxx_h
    unset ac_cv_lib_gmp___gmpz_init
    unset ac_cv_lib_gmpxx___gmpz_init
    GMP_OLD_LDFLAGS=$LDFLAGS
    GMP_OLD_CPPFLAGS=$CPPFLAGS
    GMP_OLD_LIBS=$LIBS
    LDFLAGS="$LDFLAGS -L${GMP_HOME}/lib"
    CPPFLAGS="$CPPFLAGS -I${GMP_HOME}/include"
    AC_LANG_CASE(C,[dnl
      gmp_header_file="gmp.h"],
    C++,[dnl
      gmp_header_file="gmpxx.h"])
    AC_CHECK_HEADER($gmp_header_file, [gmp_cv_gmp_h=yes], [gmp_cv_gmp_h=no])
    if test "$gmp_cv_gmp_h" = yes; then
      AC_LANG_CASE(C, [dnl
         AC_CHECK_LIB(gmp, __gmpz_init,
		      [gmp_cv_libgmp=yes], [gmp_cv_libgmp=no])],
		   C++, [dnl
         AC_CHECK_LIB(gmpxx, __gmpz_init,
		      [gmp_cv_libgmp=yes], [gmp_cv_libgmp=no], [-lgmp])])
    fi
    if test "$gmp_cv_gmp_h" = yes && test "$gmp_cv_libgmp" = yes
    then
      ifelse([$1],,
	     [gmp_version_ok=yes],
	     [RX_LIB_GMP_VERSION([$1],
				  [gmp_version_ok=yes], [gmp_version_ok=no])])
      if test x"$gmp_version_ok" = xyes; then
         AC_MSG_NOTICE([found GNU MP library in $GMP_HOME])
         gmp_have_libgmp=yes
         AC_DEFINE([SYNAPS_WITH_GMP])
         AC_LANG_CASE(C, [LIBS="-lgmp $LIBS"],
		      C++, [LIBS="-lgmpxx -lgmp $LIBS"])
         break
      fi
    else
      LDFLAGS=$GMP_OLD_LDFLAGS
      CPPFLAGS=$GMP_OLD_CPPFLAGS
      LIBS=$GMP_OLD_LIBS
      AC_MSG_NOTICE([failed to found GNU MP library in $GMP_HOME])
    fi
  done
  if test -z "$gmp_have_libgmp"; then
    AC_MSG_NOTICE([configuring without GNU MP library])
#    AC_MSG_ERROR([No GNU MP library found])
  fi
fi
])dnl
