dnl @synopsis RX_LIB_SEARCH_UDX([ACTION-IF-FOUND[, ACTION-IF-NOT-FOUND]])
dnl
dnl
AC_DEFUN([RX_LIB_SEARCH_UDX], [dnl
AC_REQUIRE([ETR_SOCKET_NSL])
ac_udx_header_found=no
ac_udx_lib_found=no

AC_ARG_WITH(udx,
   AC_HELP_STRING([--with-udx=<DIR>|yes|no],
		  [search for a UDX installation in the DIR directory. If
		   argument is yes, assume that the installation can be found
		   using the default compiler and linker serach paths. If
		   argument is no, no search for UDX is done.]),
	[if test "$withval" = no; then
	  ac_udx_header_found=disable
	  ac_udx_lib_found=disable
	 elif test "$withval" = yes; then
          LIBS="-ludx ${LIBS}"
	  ac_udx_header_found=yes
	  ac_udx_lib_found=yes
         else
	   UDX_PATH="$withval"
	 fi], [UDX_PATH="${DEFAULT_CHECKING_PATH}"])

if test "$ac_udx_header_found" = disable; then
  AC_MSG_NOTICE([configuring without UDX library])
else
  AC_MSG_NOTICE([looking for UDX library in ${UDX_PATH}])
fi

udx_save_CPPFLAGS="${CPPFLAGS}"
udx_save_LDFLAGS="${LDFLAGS}"
udx_save_LIBS="${LIBS}"

# Test for header directly in UDX_PATH
if test "$ac_udx_header_found" = no; then
  UDX_INC_PATH="${UDX_PATH}"
  CPPFLAGS="-I${UDX_INC_PATH} ${CPPFLAGS}"
  unset ac_cv_header_udx_def_h
  AC_CHECK_HEADER([udx_def.h],
		  [ac_udx_header_found=yes],
		  [ac_udx_header_found=no])
  CPPFLAGS="${udx_save_CPPFLAGS}"
fi

# Test for header directly in UDX_PATH/include 
if test "$ac_udx_header_found" = no; then
  UDX_INC_PATH="${UDX_PATH}/include"
  CPPFLAGS="-I${UDX_INC_PATH} ${CPPFLAGS}"
  unset ac_cv_header_udx_def_h
  AC_CHECK_HEADER([udx_def.h],
		  [ac_udx_header_found=yes],
		  [ac_udx_header_found=no])
  CPPFLAGS="${udx_save_CPPFLAGS}"
fi

# Test for header in UDX_PATH/include using include directives written
# as <udx/udx_def.h>
if test "$ac_udx_header_found" = no; then
  UDX_INC_PATH="${UDX_PATH}/include"
  CPPFLAGS="-I${UDX_INC_PATH} ${CPPFLAGS}"
  AC_CHECK_HEADER([udx/udx_def.h],
		  [ac_udx_header_found=yes],
		  [ac_udx_header_found=no])
  CPPFLAGS="${udx_save_CPPFLAGS}"
fi


# UDX with version >= 0.6 puts its lib in ${UDX_PATH}/lib, while the
# older versions put it ${UDX_PATH}/lib/${UDX_ARCH} where UDX_ARCH looks
# like the result of config.guess.

if test "$ac_udx_lib_found" = no; then
  UDX_LIB_PATH="${UDX_PATH}/lib"
  LDFLAGS="-L${UDX_LIB_PATH} ${LDFLAGS}"
  AC_CHECK_LIB(udx, init_udx_ieee,
	       [ac_udx_lib_found=yes], [ac_udx_lib_found=no],
	       ${ETR_SOCKET_LIBS})
  LDFLAGS="${udx_save_LDFLAGS}"
fi

if test "$ac_udx_lib_found" = no; then
# Copied from the configure script of UDX
  UDX_MACH=`uname -m`
  UDX_OS=`uname -s`
  UDX_EXTRAOS=any
  case "${UDX_OS}" in
    Linux) UDX_EXTRAOS="`ldd /bin/sh | grep libc | tr '\t' 'x' `";
          case "${UDX_EXTRAOS}" in
                xlibc.so.5*) UDX_EXTRAOS=libc5;;
                xlibc.so.6*) UDX_EXTRAOS=libc6;;
            esac;;
    SunOS) UDX_EXTRAOS=`uname -r`;
          case "${UDX_EXTRAOS}" in
                5*) UDX_EXTRAOS=solaris5;;
                4*) UDX_EXTRAOS=sunos4;;
          esac;
          case "${UDX_MACH}" in
                sun4*) UDX_MACH=sun4;;
                i386*) UDX_MACH=i686;;
                i486*) UDX_MACH=i686;;
                i586*) UDX_MACH=i686;;
          esac;;
        *)
          case "${UDX_MACH}" in
                sun4*) UDX_MACH=sun4;;
                i386*) UDX_MACH=i686;;
                i486*) UDX_MACH=i686;;
                i586*) UDX_MACH=i686;;
          esac;
        ;;
  esac
  UDX_ARCH="${UDX_MACH}_${UDX_OS}_${UDX_EXTRAOS}"
  UDX_LIB_PATH="${UDX_PATH}/lib/${UDX_ARCH}"
  LDFLAGS="-L${UDX_LIB_PATH} ${LDFLAGS}"
  AC_CHECK_LIB(udx, init_udx_ieee,
	       [ac_udx_lib_found=yes], [ac_udx_lib_found=no],
	       ${ETR_SOCKET_LIBS})
  LDFLAGS="${udx_save_LDFLAGS}"
fi

LIBS=${udx_save_LIBS}
LDFLAGS=${udx_save_LDFLAGS}
CPPFLAGS=${udx_save_CPPFLAGS}

if test "$ac_udx_header_found" = yes && test "$ac_udx_lib_found" = yes
then
  AC_DEFINE([SYNAPS_WITH_UDX], 1, [Define to 1 if you have UDX])
  CPPFLAGS="-I${UDX_INC_PATH} $CPPFLAGS"
  LIBS="-ludx $LIBS ${ETR_SOCKET_LIBS}"
  LDFLAGS="${LDFLAGS} -L${UDX_LIB_PATH}"
  AC_MSG_NOTICE([found UDX from ${UDX_PATH}])
  $1
else
  if test "$withval" != no; then
    AC_MSG_NOTICE([failed to found UDX library in ${UDX_PATH}])
    $2
    :
  fi
fi
])dnl
