<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Linear algebra package>

  <subsection|Matrix decomposition>

  The general interface for matrix decomposition is given by the function:

  <\code>
    template\<less\>class Mthd, class Mat\<gtr\> Mat decomp(const Mat & m);
  </code>

  The first parameter specifies the type of decomposition. It can be:

  <\itemize>
    <item><verbatim|LU> for Lower-Upper triangular decomposition.

    <item><verbatim|Bareiss> for Lower-Upper triangular decomposition based
    on Bareiss method. It applies the following pivoting scheme:

    <\equation*>
      a <rsup|(k)><rsub|i,j >= <frac|a <rsup|(k-1) ><rsub|i,j >a
      <rsup|(k-1)><rsub|k>- a <rsup|(k-1)><rsub|i,k >a <rsup|(k-1)><rsub|k,j
      >|a <rsup|(k-2)><rsub|k-1,k-1>>,
    </equation*>

    where <with|mode|math|A<rsup|(k)>=(a<rsub|i,j><rsup|(k)>) > is the
    transformed matrix at step <with|mode|math|k> and
    <with|mode|math|A<rsup|(0)>\<assign\>A> is the initial matrix. At the end
    of this process, after some permutation of the rows, the matrix is upper
    triangular. If A is a square matrix, the last entry
    <with|mode|math|A<rsup|(n)><rsub|n,n>> on the diagonal at the end of the
    process is the determinant of the matrix <with|mode|math|A>. Moreover, if
    <with|mode|math|A> has its coefficients in a ring, it is the same for the
    matrices <with|mode|math|A<rsup|(k)>>, since the previous division is
    exact

    <item><verbatim|QR> for Orthogonal-Upper triangular decomposition.
  </itemize>

  Here are examples of its use:

  <\code>
    \ \ matrix_t T = Decomp\<less\>LU\<gtr\>(M);\ 

    \ \ matrix_t R = Decomp\<less\>QR\<gtr\>(M);
  </code>

  See: <verbatim|synaps/linalg/decomp.h>

  <subsection|Solving a linear system>

  Several methods for solving linear systems

  <\equation*>
    A X=B
  </equation*>

  where <with|mode|math|A> and <with|mode|math|B> are
  <with|mode|math|n\<times\>m> and <with|mode|math|n\<times\> p> matrices.
  The result <with|mode|math|> is a <with|mode|math|m\<times\>p> matrix. \ On
  can use the following methods

  <\itemize-dot>
    <item><verbatim|LU> for solving using LU decomposition with column
    pivoting.

    <item>LS for least square solving.
  </itemize-dot>

  Here are examplesof its use.

  <\code>
    \ \ matrix_t B = ...;

    \ \ matrix_t X = solve\<less\>LU\<gtr\>(A,B);

    \ \ matrix_t Y = solve\<less\>LS\<gtr\>(A,B);
  </code>

  See: <verbatim|synaps/linalg/solve.h>

  <subsection|Determinant>

  Different methods for computing the determinant of a matrix are discribed
  here: Gauss, Bareiss or Berkowitz methods.\ 

  <\itemize>
    <item><verbatim|LU> for Gauss method uses classical pivoting techniques
    (ie. LU decomposition) to compute the determinant.

    <item><verbatim|Bareiss> for Bareiss method .

    <item>Berkowitz method is based on the following recursion formula, based
    on Cayley-Hamilton identity:

    <\equation*>
      d<rsub|0>:= Trace(A); A<rsub|0> := A; A<rsub|k+1>:= A<rsub|k> *A -
      <frac|1|k+1> \ Trace(A<rsub|k>) A.
    </equation*>

    <item>A mixed strategy, which expands the determinant with respect to the
    first row and Bareiss method is applied to the subdeterminants.
  </itemize>

  See: <verbatim|synaps/linalg/Det.h>

  <subsection|SVD>

  <\theorem>
    <cite|GLVL96> For a real matrix <with|mode|math|A> of size
    <with|mode|math|m\<times\>n>, there exists two orthogonal matrices

    <\equation*>
      U=[u<rsub|1>\<cdots\>,u<rsub|m>]\<in\><with|math-font|Bbb|R><rsup|m\<times\>m>,
      and V=[v<rsub|1>,\<cdots\>,<space|1fn>v<rsub|n>]\<in\><with|math-font|Bbb|R><rsup|n\<times\>n>
    </equation*>

    such that

    <\equation*>
      U<rsup|t>A*V=diag (\<sigma\><rsub|1>,\<cdots\>,\<sigma\><rsub|p>)\<in\><with|math-font|Bbb|R><rsup|m\<times\>n><space|1fn>p=min{m,n}<with|font-family|rm|,>
    </equation*>

    <\equation*>
      \<sigma\><rsub|1>\<geq\>\<sigma\><rsub|2>\<geq\>\<cdots\>\<geq\>\<sigma\><rsub|p>\<geq\>0<space|2fn><space|2fn><space|2fn><space|2fn><space|2fn><space|2fn>
    </equation*>

    <with|mode|math|\<sigma\><rsub|i>> are called the <with|mode|math|i>th
    singular value of A, <with|mode|math|u<rsub|i>> and
    <with|mode|math|v<rsub|i>> are respectively the <with|mode|math|i>th left
    and right singular vectors.\ 
  </theorem>

  <\definition>
    \ For a matrix <with|mode|math|A>, the numerical rank of A is defined by:

    <\equation*>
      rank(A,\<epsilon\>)=min{rank(B):<left|\|><frac|A|\<\|\|\>A\<\|\|\>>-B<right|\|>\<leq\>\<epsilon\>}.
    </equation*>
  </definition>

  <\theorem>
    <label|t><cite|GLVL96> If <with|mode|math|A\<in\><with|math-font|Bbb|R><rsup|m\<times\>n>>
    is of rank <with|mode|math|r> and <with|mode|math|k\<less\>r> then

    <\equation*>
      \<sigma\><rsub|k+1>=min<rsub|rank(B)=k>\<\|\|\>A-B\<\|\|\>.
    </equation*>
  </theorem>

  The singular value decomposition of a matrix <with|mode|math|M> is the
  decomposition of the form\ 

  <\equation*>
    M = U \<Sigma\> V<rsup|t>
  </equation*>

  where <with|mode|math|U> and <with|mode|math|V> are orthognal matrices
  <with|mode|math|U U<rsup|t> = Id>, <with|mode|math|V \ V<rsup|t>=Id> and
  <with|mode|math|\<Sigma\>=diag( \<sigma\><rsub|1>,\<ldots\>,
  \<sigma\><rsub|n>)> is a diagonal matrix such that
  <with|mode|math|\<sigma\><rsub|1> \<geqslant\> \<cdots\> \<geqslant\>
  \<sigma\><rsub|n> \<geqslant\> 0>. The values
  <with|mode|math|\<sigma\><rsub|i>> are called the singular values of
  <with|mode|math|M>.

  <\code>
    \ \ vector_t s = \ Svd(m);
  </code>

  See: <verbatim|synaps/linalg/svd.h>

  <subsection|Rank>

  <no-indent>Theorem <reference|t> shows that the smallest singular value is
  the distance between A and all the matrices of rank
  <with|mode|math|\<less\>p=min{m,n}>. Thus if
  <with|mode|math|r<rsub|\<epsilon\>>=rank(A,\<epsilon\>)> then

  <\equation*>
    \<sigma\><rsub|1>\<geq\>\<cdots\>\<geq\>\<sigma\><rsub|r<rsub|\<epsilon\>>>\<gtr\>\<epsilon\>\<sigma\><rsub|1>\<geq\>\<sigma\><rsub|r<rsub|\<epsilon\>+1>>\<geq\>\<cdots\>\<geq\>\<sigma\><rsub|p>,<space|1fn>p=min{m,n}.
  </equation*>

  As a result, in order to determine the rank of a matrix <with|mode|math|A>,
  we find the singular values <with|mode|math|(\<sigma\><rsub|i>)<rsub|i>> so
  that

  <\equation*>
    rank (A,\<epsilon\>)=max{i;<frac|\<sigma\><rsub|i>|\<sigma\><rsub|1>>\<gtr\>\<epsilon\>}.
  </equation*>

  The function\ 

  <\code>
    \ \ Rank(A);
  </code>

  computes the rank of a matrix, as described above with a default value for
  <with|mode|math|\<varepsilon\>>.\ 

  See: <verbatim|synaps/linalg/rank.h>

  <subsection|Eigenvalues, eigenvectors>

  The computation of the eigenvalues and eigenvectors of a matrix are
  performed by the functions <verbatim|eigen> and <verbatim|eigen_vect>. They
  also provide generalised eigenvalues and eigenvectors, for pairs of
  matrices <verbatim|(a,B)>. If the internal representation of type
  <verbatim|lapack::rep2d\<less\>double\<gtr\>>, the routine <verbatim|dgeeg>
  or <verbatim|dgegv> are called.

  <\code>
    \ \ VectDse\<less\>std::complex\<less\>double\<gtr\> \<gtr\> v =
    eigen(A);

    \ \ VectDse\<less\>double\<gtr\> \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ w =
    eigen(A, Real());

    \ \ VectDse\<less\>double\<gtr\> \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ w =
    eigen(A,B); \ 
  </code>

  See: <verbatim|synaps/linalg/eigen.h>

  <subsection|Fast Fourier Transform>

  A set of functions are available, for performing FFT on generic vectors.

  \;

  See: <verbatim|synaps/linalg/FFT.h>

  \;
</body>

<\initial>
  <\collection>
    <associate|language|english>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-10|<tuple|1.7.2|?>>
    <associate|auto-11|<tuple|1.7.2|?>>
    <associate|auto-12|<tuple|1.7.1|?>>
    <associate|auto-13|<tuple|1.7.2|?>>
    <associate|auto-14|<tuple|1.7.1|?>>
    <associate|auto-15|<tuple|1.7.2|?>>
    <associate|auto-16|<tuple|1.7.2|?>>
    <associate|auto-17|<tuple|1.7.1|?>>
    <associate|auto-18|<tuple|1.7.2|?>>
    <associate|auto-19|<tuple|1.7.1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-20|<tuple|1.7.2|?>>
    <associate|auto-21|<tuple|1.7.2|?>>
    <associate|auto-22|<tuple|1.7.1|?>>
    <associate|auto-23|<tuple|1.7.2|?>>
    <associate|auto-24|<tuple|1.7.2|?>>
    <associate|auto-25|<tuple|1.7.1|?>>
    <associate|auto-26|<tuple|1.7.2|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|auto-6|<tuple|1.5|?>>
    <associate|auto-7|<tuple|1.6|?>>
    <associate|auto-8|<tuple|1.7|?>>
    <associate|auto-9|<tuple|1.7.1|?>>
    <associate|ra|<tuple|4|?>>
    <associate|t|<tuple|3|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-10|<tuple|1.7|?>>
    <associate|toc-11|<tuple|1.8|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.1.1|?>>
    <associate|toc-4|<tuple|1.1.2|?>>
    <associate|toc-5|<tuple|1.2|?>>
    <associate|toc-6|<tuple|1.3|?>>
    <associate|toc-7|<tuple|1.4|?>>
    <associate|toc-8|<tuple|1.5|?>>
    <associate|toc-9|<tuple|1.6|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|bib>
      GLVL96

      GLVL96
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Linear
      algebra package> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Matrix decomposition
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Solving a linear system
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Determinant
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>SVD
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Rank
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.6<space|2spc>Eigenvalues, eigenvectors
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>

      <with|par-left|<quote|1.5fn>|1.7<space|2spc>Fast Fourier Transform
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-8>>
    </associate>
  </collection>
</auxiliary>