/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
#ifndef SYNAPS_LINALG_SOLVE_H
#define SYNAPS_LINALG_SOLVE_H
//-------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/linalg/MethodName.h>
__BEGIN_NAMESPACE_SYNAPS
//====================================================================
/** \file Solve.h
    \brief Resolution of linear systems.
 **/
//--------------------------------------------------------------------
/**
 Resolution of @A X = B@ by the method @MTH()@.
 Calling the function @solve(MTH(),X.rep(),A.rep(),B.rep())@, 
 where @X@ is a copy of @B@. Example:
 \code
   B1 := Solve<LU>(A,B0);
 \endcode
**/
template <class MTH, class M, class R> inline 
R Solve(M & A, R & B)
{
  R x(B);
  solve(MTH(),x.rep(),A.rep(),B.rep());
  return x;
}
//====================================================================
__END_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
#endif // SYNAPS_LINALG_SOLVE_H
