#ifndef SYNAPS_LINALG_USCHUR_H
#define SYNAPS_LINALG_USCHUR_H
//--------------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/linalg/MATRIX.m>
#include <synaps/base/Range.h>
#include <synaps/linalg/MethodName.h>

__BEGIN_NAMESPACE_SYNAPS


//--------------------------------------------------------------------------
/**
 Compute the upper schur complement of size @n@ of the matrix @mt@, given
by the formula $A -B\, D^{-1}\, C$ where $A$ is the first diagonal submatrix 
block of size @n@. 
\help{USchur}
 **/
template <class M> inline 
M USchur(M & m, int n)
{
	M  A = m(Range2d(0,n-1,0,n-1));
	M  B=m(Range2d(0,n-1,n,m.nbcol()-1)),
	C=m(Range2d(n,m.nbrow()-1,0,n-1)),
	D=m(Range2d(n,m.nbrow()-1,n,m.nbcol()-1));
	using namespace MATRIX;
	solve(LU(),C.rep(),D.rep());
	B *= C;
	A -= B;
	return A;
}
//--------------------------------------------------------------------------

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_LINALG_USCHUR_H

