
#include <synaps/init.h>
//#include <synaps/linalg/clapack/clap.h>
#include "lapack.h"
#include "lapack_extern.h"


__BEGIN_NAMESPACE_SYNAPS

//======================================================================
namespace lapack 
{
//--------------------------------------------------------------------
#define LAPACK_TOL 1e-6

//--------------------------------------------------------------------
inline void eigbuild(rep2d<std::complex<double> > & A, 
	      int n, double* t, double* vr, double* vi) 
{
  for(int j=0;j< n; j++){
    if(vi[j]==0){
      for(int i=0; i< n; i++) A(i,j)= std::complex<double>(t[i+n*j],0);
    }else{
      for(int i=0; i< n; i++) {
	A(i,j)  = std::complex<double>(t[i+j*n], t[i+(j+1)*n]);
	A(i,j+1)= std::complex<double>(t[i+j*n],-t[i+(j+1)*n]);
      }
      j++;
    }
  }
}
//----------------------------------------------------------------------
inline void eigbuild(rep2d<double> & A, 
		       int n, double* t, double* vr, double* vi, Real) 
{
  int c=0;
  for(int j = 0; j < n; j++){
    if(vi[j]==0){
      for(int i = 0; i < n; i++) A(i,c)= t[i+n*j];
      c++;
    }
  }
}
//----------------------------------------------------------------------
inline void eigbuild(
      rep2d<double > & A, 
      int m, int mr, double* t, double* beta, double* vr, double* vi, Real) 
{
  int c=0;
  for(int j = 0; j < m; j++){
    if(vi[j]==0 && beta[j] !=0){
      for(int i = 0; i < mr; i++) A(i,c)= t[i+m*j];
      c++;
    }
  }
}

//----------------------------------------------------------------------
inline void eigbuild(linalg::rep1d<double> & V, rep2d<double > & A, 
		       int n, double* t, double * beta, 
		       double* vr, double* vi, Real) 
{
  int c=0;
  for(int j = 0; j < n; j++){
    if(vi[j]==0){
      for(int i = 0; i < n; i++) A(i,c)= t[i+n*j];
      V[c] = vr[j]/beta[j];
      c++;
    }
  }
}
//----------------------------------------------------------------------
inline int count(double* t,int n){
  int c=0;
  for(int i=0; i<n;i++) if(t[i]==0) c++;
  return(c);
}
//----------------------------------------------------------------------
inline int count(double* t,double* d, int n){
  int c=0;
  for(int i=0; i<n;i++) if(t[i]==0 && d[i]!=0) c++;
  return(c);
}
//----------------------------------------------------------------------
// /// Multiplication using blas. 
// void mul(rep2d<double> & R, const rep2d<double> & A, const rep2d<double> & B)
// {
//   MATRIX::init(R,0.);
//   char TRANSA = 'N', TRANSB = 'N';  
//   int M=A.nbrow(), N=A.nbcol(), K=B.nbcol();
//   int LDR= R.lda_, LDA=A.lda_, LDB=B.lda_;
//   double ALPHA = 1.0,  BETA = 1.0; 
//   F77NAME(dgemm)(&TRANSA, &TRANSB, &M, &N, &K, &ALPHA, 
// 		 A.tab_, &LDB, B.tab_, &LDB, &BETA, R.tab_, &LDR);

// }
//----------------------------------------------------------------------
/*
 Transposition of the matrix. If it is a square matrix, the operation is
 performed inplace. Otherwise, a new memory space is allocated and 
 restored.
*/
// // void MATRIX::transpose(rep2d<double> & A)
// {
//   typedef rep2d<double>::size_type index_t;
//   if(A.nbcol() == A.nbrow())
//     for(index_t i = 0; i < A.nbrow(); i++)
//       for(index_t j = i+1; j < A.nbcol(); j++) 
//         swap(A(i, j),A(j, i));
//   else {
//     rep2d<double> res(A.nbcol(), A.nbrow());
//     for(index_t i = 0; i <A.nbrow(); i++)
//       for(index_t j = 0; j < A.nbcol(); j++) 
//         res(j, i) = A(i, j);
//     assign(A,res);
//   }
// }
#ifdef SYNAPS_WITH_LAPACK
//----------------------------------------------------------------------
/** 
  Compute the QR decomposition of the matrix A, without pivoting.
  The operation is done inplace. The output matrix is upper-triangular.
 **/
inline integer decomp(QR qr, rep2d<double> & A){
  //if(IsMatrixEmpty(A))  return;
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info;
  double *tau   = new double[m];
  integer lwork = 4*n;
  double *work  = new double[lwork];
//   if(!work)
//     Error("%s %s dcmp_QR(int *, COEFF *)\n",ERR_SPACE, ERR_FUNCTION);
  F77NAME(dgeqrf)(&m, &n, A.tab_, &l, tau, work, &lwork, &info);
  
  using MATRIX::setupper; setupper(A);
  delete [] tau;
  delete [] work;
  return (info);
   
};
//----------------------------------------------------------------------
/** 
  Compute the QR decomposition of the matrix @A@. The operation is
  done inplace, without pivoting. The matrix Q is 
  represented by the lower part of @A@ and the coefficients in @tau@.
  Use the {\sc lapack} routine @dgeqrf@.
**/
inline integer decomp(QR qr,rep2d<double> & A, double * tau){
  //if(IsMatrixEmpty(A))  return;
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info, lwork = n;
  double  *work = new double[3*n+1];
//   if(!work)
//     Error("%s %s dcmp_QR(int *, COEFF *)\n",ERR_SPACE, ERR_FUNCTION);

  F77NAME(dgeqrf)(&m, &n, A.tab_, &l, tau, work, &lwork, &info);
  
  using MATRIX::setupper; setupper(A);
  return (info);
   
};

//----------------------------------------------------------------------
/** 
  Compute the QR decomposition of the matrix @A@. The operation is
  done inplace. The pivots are stored in @pivot@ and the matrix Q is 
  represented by the lower part of @A@ and the coefficients in @tau@.
  Use the {\sc lapack} routine @dgeqpf@.
**/
inline integer decomp(QR qr, rep2d<double> & A, 
		       unsigned int * pivot, double * tau) 
{
  //if(IsMatrixEmpty(A))  return;
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info;
  integer *pvt = new integer[n];
  double *work = new double[3*n+1];
// if(!work)
//     Error("%s %s dcmp_QR(int *, COEFF *)\n",ERR_SPACE, ERR_FUNCTION);

  F77NAME(dgeqpf)(&m, &n, A.tab_, &l, pvt, tau, work,&info);

  for(int i=0; i< n;i++)  pivot[i]= pvt[i]-1;
  return (info);
  
};
//----------------------------------------------------------------------
/** 
  Compute the LU decomposition of the matrix @A@. The pivots on the
  columns are stored in @pivot@. Use the {\sc lapack} routine @dgetrf@.
**/
inline integer decomp(LU lu, rep2d<double> & A) 
{
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info;
  integer *pivot= new integer[n];
  F77NAME(dgetrf)(&m, &n, A.tab_, &l, pivot, & info);
  using MATRIX::setupper; setupper(A);
  delete [] pivot;
  return(info);
}
//----------------------------------------------------------------------
/** 
  Compute the LU decomposition of the matrix @A@. The pivots on the
  columns are stored in @pivot@. Use the {\sc lapack} routine @dgetrf@.
**/
inline integer decomp(LU lu, rep2d<double> & A, rep2d<double> & L) 
{
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info;
  integer *pivot= new integer[n];
  F77NAME(dgetrf)(&m, &n, A.tab_, &l, pivot, &info);
  L.resize(m,m);
  for(unsigned j=0 ; j< A.nbcol(); j++)
    for(unsigned i=j+1; i < A.nbrow(); i++)
      L(i,j) =A(i,j);
  for(unsigned i=0 ; i< A.nbrow(); i++)   L(i,i)=1;  
  using MATRIX::setupper; setupper(A);
  delete [] pivot;
  return(info);
}
//----------------------------------------------------------------------
/** 
  Compute the LU decomposition of the matrix @A@. The pivots on the
  columns are stored in @pivot@. Use the {\sc lapack} routine @dgetrf@.
**/
inline integer decomp(LU lu,rep2d<double> & A, integer *pivot) 
{
  integer m = A.nbrow(), n = A.nbcol(), l = A.lda_;
  integer info;
  F77NAME(dgetrf)(&m, &n, A.tab_, &l, pivot, &info);
  for(int i=0; i< n;i++) pivot[i]= pivot[i]-1;
  using MATRIX::setupper; setupper(A);
  return(info);
}
//----------------------------------------------------------------------
/** 
  Compute a triangulation of A, based on the LU decomposition of the matrix 
  @A@. Return the determinant, which is the product of terms on the diagonal.
  \help{Triang,Gauss Lapack}
**/
template <class C>
C triang(Gauss, rep2d<C> & A) 
{
  decomp(LU(),A);
  using MATRIX::setupper; setupper(A);
  C d=1;
  for(typename rep2d<C>::size_type i=0; 
      i< std::min(A.nbrow(),A.nbcol());++i) d*=A(i,i);
  return d;
}
//----------------------------------------------------------------------//
/**
  Solve the system $A\,x = b$ by LU decomposition of the matrix @A@.
  Use the {\sc lapack} routine @dgesv@.
**/
inline void solve(LU m, linalg::rep1d<double> & x, 
		   const rep2d<double> & A, const linalg::rep1d<double> & b)
{
      integer N  = A.nbrow(), NRHS=1, lb = A.nbrow();
      integer info;
      integer *ipiv = new integer[A.nbcol()];
      integer s = N*N;
      double * tmp  = new double[s];
      for(int i=0 ; i < s; i++) tmp[i]=A.tab_[i];
      x=b;
      F77NAME(dgesv)(&N, &NRHS, tmp, &N, ipiv, x.tab_, &lb, &info);
}
//----------------------------------------------------------------------//
/**
  Solve the system $A\,x = B$ by LU decomposition of the matrix @A@.
  @B@ is a matrix. Use the {\sc lapack} routine @dgesv@.
**/
inline void solve(LU lu, rep2d<double> & x, 
		   const rep2d<double> & A, const rep2d<double> & B)
{
      integer N  = A.nbrow(), NRHS=B.nbcol(), lb = A.nbrow();
      integer info;
      integer *ipiv = new integer[A.nbcol()];
      integer s = N*N;
      double * tmp  = new double[s];
      for(int i=0 ; i < s; i++) tmp[i]=A.tab_[i];
      x=B;
      F77NAME(dgesv)(&N, &NRHS, tmp, &N, ipiv, x.tab_, &lb, &info);
}
//----------------------------------------------------------------------//
/**
  Solve the system $A\,X = B$ where @B@ is a matrix by LU decomposition 
  of the matrix @A@.  Use the {\sc lapack} routine @dgesv@.
**/
inline void solve(LU lu, rep2d<double> & X, 
		   const rep2d<double> & A)
{
  integer M  = A.nbrow(), NRHS=X.nbcol(), lb = X.lda_;
  integer *info = new integer;
  integer *ipiv = new integer[A.nbcol()];
  integer s = M*M;
  double * tmp  = new double[s];
  for(int i=0 ; i < s; i++) tmp[i]=A.tab_[i];
  F77NAME(dgesv)(&M, &NRHS, tmp, &M, ipiv, X.tab_, &lb, info);
}
//----------------------------------------------------------------------//
/**
  Solve the system $A\,x = b$ by a Least Square method.
  Put the result in @b@.
  Use the {\sc lapack} routine @dgelsx@.
**/
inline void solve(LS ls, const rep2d<double> & A, linalg::rep1d<double> & b)
{      
      integer M = A.nbrow(), N=A.nbcol(), NRHS=1;
      integer la = A.lda_, lb = M;
      integer *info = new integer;
      integer *JPVT = new integer[N], *RANK= new integer;
      doublereal RCOND = 0.0001;
      int n = std::max(std::min(M,N)+3*N,2*std::min(M,N)+NRHS);
      doublereal *WORK = new doublereal[n];
      F77NAME(dgelsx)(&M,&N,&NRHS, A.tab_, &la, b.tab_, &lb, 
		     JPVT, &RCOND, RANK, WORK, info);
      // cout << "Rank: " <<*RANK <<" with RCOND= " << RCOND << endl;
      // cout << "Info: " <<*info << endl;
}
//----------------------------------------------------------------------//
/**
  Solve the system $A\,x = b$ by a Least Square method.
  A copy of @A@ is made and a copy of @b@ is put in @x@.
  Then the previous function 
  @solve_ls(rep2d<double> & A, linalg::rep1d<double> & b)@ is used. 
**/
inline void solve(LS ls,linalg::rep1d<double> & x, 
		   const rep2d<double> & A, const linalg::rep1d<double> & b)
{      
      rep2d<double> tmp=A;
      x = b;
      solve(ls,tmp,x);

}
//----------------------------------------------------------------------//
/**
 Compute the Singular values of  the matrix @A@, calling the {\sc lapack} 
 subroutine @dgesvd@. The result, put in @x@, is the vector of singular values 
 in decreasing order.  The matrix @A@ is copied.
**/   
inline void svd(linalg::rep1d<double> & x,const rep2d<double> & A)
{
//if(IsMatrixEmpty(A)) return;
  integer m = A.nbrow(), n = A.nbcol(), lda = A.lda_;
  rep2d<double> U, V, W=A;
  integer ldu = m, ldvt = n, info;
  integer lwork = std::max((3*std::min(m, n) + std::max(m, n)), (5*std::min(m, n)-4)) + 10;
  double  *work = new double[lwork+1];

  //if(!work)
  //     Error("%s %s SVD_Lapack_Decomposition(Matrix &, Type *, Matrix &, Matrix &, intege
  // r *) \n",ERR_SPACE, ERR_FUNCTION);

  F77NAME(dgesvd)("N", "N", &m, &n, W.tab_, &lda, x.tab_, U.tab_, &ldu, V.tab_, &ldvt, work, &lwork, &info);

  delete [] work; 
  //  return info;

}
//----------------------------------------------------------------------
/**
 Compute the Singular value of  the matrix, calling {\sc lapack} subroutine
 @dgesvd@. The result, put in @x@, is the vector of singular values 
 in decreasing order.  The matrix @A@ is copied.
 The singular left vectors are stored in @U@.
**/   
inline void svd(linalg::rep1d<double> & x,
		 const rep2d<double> & A, rep2d<double> & U)
{
//if(IsMatrixEmpty(A)) return;
  integer m = A.nbrow(), n = A.nbcol(), lda = A.lda_;
  U.resize(m,m);
  integer ldu = U.lda_, ldvt = m, *info = new integer;
  integer lwork = std::max((3*std::min(m, n) + std::max(m, n)), (5*std::min(m, n)-4)) + 10;
  double  *work = new double[lwork+1];
  rep2d<double> V;

  //if(!work)
  //     Error("%s %s SVD_Lapack_Decomposition(Matrix &, Type *, Matrix &, Matrix &, intege
  // r *) \n",ERR_SPACE, ERR_FUNCTION);
  
  F77NAME(dgesvd)("A", "N", &m, &n, A.tab_, &lda, x.tab_, U.tab_, &ldu, 
		  V.tab_, &ldvt, work, &lwork, info);
  //  return info;

}
//----------------------------------------------------------------------
/** Compute the Singular values of the matrix, calling the {\sc lapack}
   subroutine @dgesvd@. The result, put in @x@, is the vector of singular
   values in decreasing order. The matrix @A@ is copied in a temporary
   matrix.
   
   The  matrices @U@ and @V@ are orthogonal matrices, which satisfy
   @A=U S V@$^t$.
   The correct space for the matrix @U@ and @V@ should be allocated
   before using this procedure.
**/
inline void svd(linalg::rep1d<double> & S,
	 const rep2d<double> & A, 
	 rep2d<double> & U, rep2d<double> & V)
{
  #ifdef VERBATIM
     cout <<"Using Lapack for the svd" <<endl;
  #endif
//if(IsMatrixEmpty(A)) return;
  typedef rep2d<double>::size_type index_t;
  integer m = A.nbrow(), n = A.nbcol(), lda = A.lda_;
  integer ldu = m, ldvt = n, *info = new integer;
  integer lwork = std::max((3*std::min(m, n) + std::max(m, n)), (5*std::min(m, n)-4)) + 10;
  double  *work = new double[lwork+1];
  rep2d<double> W=A;
  U.resize(m,m);
  V.resize(n,n);
  //if(!work)
  //     Error("%s %s SVD_Lapack_Decomposition(Matrix &, Type *, Matrix &, Matrix &, intege
  // r *) \n",ERR_SPACE, ERR_FUNCTION);
  
  F77NAME(dgesvd)("A", "A", &m, &n, W.tab_, &lda, S.tab_, U.tab_, &ldu, V.tab_, &ldvt, work, &lwork, info);
}
//----------------------------------------------------------------------
/// The same but we have @A=U\, V@$^t$.
inline void svdx(linalg::rep1d<double> & S,
		  const rep2d<double> & A, 
		  rep2d<double> & U, rep2d<double> & V)
{
  typedef rep2d<double>::size_type index_t;
  svd(S,A,U,V);
  for(index_t j=0;j<U.nbcol();j++)
    for(index_t i=0;i< U.nbrow();i++) 
      U(i,j)*=S[j];
}
//----------------------------------------------------------------------
/** 
  Compute the eigenvalues of @A@, put them in the complex
  vector @V@. Use the {\sc lapack} routine @dgeev@.
  The matrix @A@ is copied. 
**/
inline void eignval(linalg::rep1d<std::complex<double> >& V, const rep2d<double> & A)
{
//   if(A.nbrow() != A.nbcol())
//     Error("%s %s rep2d<double>::leigvct MatrTbXX not square !!! \n",ERR_DIM, ERR_FUNCTION);
  typedef rep2d<double>::size_type index_t;
  // Copy of A
  rep2d<double> cA(A);
  integer n = cA.nbrow(), l = cA.lda_;
  integer  ldvl = n, ldvr = n, info;
  integer lwork = 4*n + 10;
  double 
    *valR = new double[cA.nbrow()],
    *valI = new double[cA.nbrow()],
    *tl= new double,   //= new double[A.nbcol()*A.nbrow()];
    *tr= new double,   // = new double[A.nbcol()*A.nbrow()];
    *work  = new double[lwork+1];

  F77NAME(dgeev)("N", "N", &n, cA.tab_, &l, valR, valI, 
		 tl, &ldvl, tr, &ldvr, work, &lwork, &info);
  for(index_t i=0; i<A.nbrow();i++) V[i] = std::complex<double>(valR[i],valI[i]);

  delete [] valR;  delete [] valI; delete [] work; delete tr; delete tl;

}

//----------------------------------------------------------------------
/** 
  Compute the  real eigenvalues of @A@ and put them in the real
  vector @V@. Use the {\sc lapack} routine @dgeev@.
  The matrix @A@ is copied. 
**/
inline void eignval(
      linalg::rep1d<double>& V, const rep2d<double> & A, Real)
{
//   if(A.nbrow() != A.nbcol())
//     Error("%s %s rep2d<double>::leigvct MatrTbXX not square !!! \n",ERR_DIM, ERR_FUNCTION);
  typedef rep2d<double>::size_type index_t;
  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n;
  integer * info = new integer;
  integer lwork = 4*n + 10;
  double * valR = new double[A.nbrow()];
  double * valI = new double[A.nbrow()];
  double * tl= new double;//= new double[A.nbcol()*A.nbrow()];
  double * tr= new double;// = new double[A.nbcol()*A.nbrow()];
  double *work  = new double[lwork+1];
  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for(index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];
  
//   if(!work)
//     Error("%s %s rep2d<double>::leigvct \n",ERR_SPACE, ERR_FUNCTION);

  F77NAME(dgeev)("N", "N", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr, &ldvr, work, &lwork, info);

  // count the number of real eigenvalues.
  int c=count(valI,A.nbrow());

  V.resize(c);
  c=0;
  for(index_t i=0; i<A.nbrow();i++)
    if(valI[i]==0){
      V[c]=valR[i];
      c++;
    }
  //  delete [] work; delete [] valR; delete [] valI;
}

//----------------------------------------------------------------------//
/** 
  Compute the eigenvalues of @A@, put them in the complex
  vector @V@ and store in @error[i]@ a bound on the error on the 
  eigenvalue @V[i]@. Use the {\sc lapack} routine @dgeevx@.
  The matrix @A@ is copied.
**/
inline void eignval(linalg::rep1d<std::complex<double> >& V, const rep2d<double> & A, 
		     linalg::rep1d<double>& error)
{
//   if(A.nbrow() != A.nbcol())
//     Error("%s %s rep2d<double>::leigvct MatrTbXX not square !!! \n",ERR_DIM, ERR_FUNCTION);
  typedef rep2d<double>::size_type index_t;

  integer n = A.nbrow(), lda = A.lda_;
  integer ldvl = n, ldvr = n;
  integer* info = new integer, *ilo = new integer, *ihi= new integer;
  double*  scale= new double[n], *abnrm= new double;
  double*  valR = new double[n];
  double*  valI = new double[n];
  double*  tl= new double[A.nbcol()*n];
  double*  tr= new double[A.nbcol()*n];
  double*  rcondv= new double[A.nbcol()];
  integer  lwork = n*n + 6*n;
  integer* iwork= new integer[2*n-2];
  
  double* work  = new double[lwork];
  // Copy of A
  double* tab = new double[A.nbcol()*n];
  for(index_t i=0;i<A.nbcol()*n;i++) tab[i] = A.tab_[i];
  
  F77NAME(dgeevx)("P", "V", "V", "B", &n, tab, &lda, valR, valI, 
		  tl, &ldvl, tr,&ldvr, ilo, ihi, scale, abnrm,
		  error.tab_, rcondv, work, &lwork, iwork, info);

  double epsmch= F77NAME(dlamch)("E");
  
  epsmch*=(*abnrm);  
  for(int i=0; i<n;i++){
    V[i] = std::complex<double>(valR[i],valI[i]);
    error[i] = epsmch/error[i];
  }
  
}
//----------------------------------------------------------------------
/** 
  Compute the eigenvalues of the pencil $A - z\,B$ and put it in the complex
  vector @V@. Use the {\sc lapack} routine @dgegv@.
  The matrices @A@ and @B@ are copied.
**/
inline void eignval(
      linalg::rep1d<std::complex<double> >& V, 
      const rep2d<double> & A, const rep2d<double> & B)
{
  assert(A.nbrow() == A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double;//= new double[A.nbcol()*n];
  double*  VR= new double;//= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n +1;
  double* work  = new double[lwork];
  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "N", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
		  VL, &n, VR, &n, work, &lwork, info);
  for(int i=0; i<n;i++){
    V[i] = std::complex<double>(WR[i]/BETA[i],WI[i]/BETA[i]);
  }
}
//----------------------------------------------------------------------
/** 
  Compute the real eigenvalues of the pencil $A - z\,B$ and put it in 
  the complex  vector @V@. Use the {\sc lapack} routine @dgegv@.
  The matrices @A@  and @B@ are copied.
**/
inline void eignval(
      linalg::rep1d<double>& V, 
      const rep2d<double> & A, const rep2d<double> & B, Real real)
{
  assert(A.nbrow()==A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double;//= new double[A.nbcol()*n];
  double*  VR= new double;//= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n+1;
  double* work  = new double[lwork];
  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "N", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
		  VL, &n, VR, &n, work, &lwork, info);
  V.resize(count(WI,BETA,n));
  int c=0;
  for(int i=0; i<n;i++){
    if(WI[i]==0 && BETA[i] !=0) {V[c]=WR[i]/BETA[i];c++;}
  }
  
}
//----------------------------------------------------------------------
/** 
  Compute the real eigenvalues of the pencil $A - z\,B$ and put it in 
  the complex  vector @V@. Use the {\sc lapack} routine @dgegv@.
  The matrices @A@  and @B@ are copied.
**/
inline void eignval(
      linalg::rep1d<double>& V, 
      const rep2d<double> & A, const rep2d<double> & B, 
      linalg::rep1d<double>& Er, 
      Real real)
{
  assert(A.nbrow()==A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double;//= new double[A.nbcol()*n];
  double*  VR= new double;//= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n+1;
  double* work  = new double[lwork];
  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "N", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
		    VL, &n, VR, &n, work, &lwork, info);
  V.resize(count(WI,BETA,n));
  int c=0;
  for(int i=0; i<n;i++){
    if(WI[i]==0 && BETA[i] !=0) {V[c]=WR[i]/BETA[i];c++;}
  }
  
}
//----------------------------------------------------------------------
/** 
  Compute the left eigenvectors of @A@ and put it in the complex
  matrix @E@. Use the {\sc lapack} routine @dgeev@. The matrix @A@ 
  is copied.
**/
inline void leigvct(rep2d<std::complex<double> > & E, 
		     const rep2d<double> & A)
{
  assert(A.nbrow() == A.nbcol());

  typedef rep2d<double>::size_type index_t;

  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n;
  integer * info = new integer;
  double * valR = new double[A.nbrow()];
  double * valI = new double[A.nbrow()];
  double * tl = new double[A.nbcol()*A.nbrow()];
  double * tr = new double[A.nbcol()*A.nbrow()];
  integer lwork = 4*n + 10;
  double *work  = new double[lwork+1];

//   if(!work)
//     Error("%s %s lapack::rep2d<double>::leigvct \n",ERR_SPACE, ERR_FUNCTION);

  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for(index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];

  F77NAME(dgeev)("V", "N", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr,&ldvr, work, &lwork, info);
  eigbuild(E,A.nbrow(),tl,valR,valI); // XXX
}

//-------------------------------------------------------------------------//
/** 
  Compute the left eigenvectors and eigenvalues of @A@.
  The matrix of eigenvector is put in the complex matrix @E@ and 
  the eigenvalues are stored in @V@. The matrix @A@ 
  is copied.
**/
inline void leigvct(rep2d<std::complex<double> > & E, 
	     linalg::rep1d<std::complex<double> >& V, const rep2d<double> & A)
{
//   if(A.nbrow() != A.nbcol())
//     Error("%s %s rep2d<double>::leigvct MatrTbXX not square !!! \n",ERR_DIM, ERR_FUNCTION);
  typedef rep2d<double>::size_type index_t;

  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n;
  integer * info = new integer;
  double * valR = new double[A.nbrow()];
  double * valI = new double[A.nbrow()];
  double * tl = new double[A.nbcol()*A.nbrow()];
  double * tr = new double[A.nbcol()*A.nbrow()];
  integer lwork = 4*n + 10;
  double *work  = new double[lwork+1];

//   if(!work)
//     Error("%s %s rep2d<double>::leigvct \n",ERR_SPACE, ERR_FUNCTION);

  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for(index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];

  F77NAME(dgeev)("V", "N", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr,&ldvr, work, &lwork, info);

  for(index_t i=0; i<A.nbrow();i++) V[i] = std::complex<double>(valR[i],valI[i]);
  eigbuild(E,A.nbrow(),tr,valR, valI); //XXX
}

//  ---------------------------------------------------------------------  //
/** 
  Compute the right eigenvectors of the matrix @A@.
  The complex matrix of eigenvector is put in @E@.
  Use the {\sc lapack} routine @dgeev@. The matrix @A@ 
  is copied.
**/
inline void reigvct(rep2d<std::complex<double> > & E, 
		     const rep2d<double> & A)
{

  assert(A.nbrow() == A.nbcol());

  typedef rep2d<double>::size_type index_t;
  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n, info;
  double* valR = new double[A.nbcol()];
  double* valI = new double[A.nbcol()];
  double* tl = new double[A.nbcol()*A.nbrow()];
  double* tr = new double[A.nbcol()*A.nbrow()];
  integer lwork = 4*n + 10;
  double* work = new double[lwork+1];

//   if(!work)
//     Error("%s %s MatrixLa<double>::reigvct \n",ERR_SPACE, ERR_FUNCTION);

  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for(index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];

  F77NAME(dgeev)("N", "V", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr, &ldvr, work, &lwork, &info);

  eigbuild(E,n,tr,valR,valI);

  delete [] valR; delete [] valI; delete [] tl; delete [] tr;
  delete [] work; delete [] tab;
}
//----------------------------------------------------------------------
/** 
  Compute the real right eigenvectors of the matrix @A@.
  The complex matrix of eigenvector is put in @E@.
  Use the {\sc lapack} routine @dgeev@. The matrix @A@ 
  is copied.
**/
inline void reigvct(rep2d<double > & E, 
		     const rep2d<double> & A, const Real &)
{

  assert(A.nbrow() == A.nbcol());

  typedef rep2d<double>::size_type index_t;
  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n, info;
  double* valR = new double[A.nbcol()];
  double* valI = new double[A.nbcol()];
  double* tl = new double[A.nbcol()*A.nbrow()];
  double* tr = new double[A.nbcol()*A.nbrow()];
  integer lwork = 4*n + 10;
  double* work = new double[lwork+1];

//   if(!work)
//     Error("%s %s MatrixLa<double>::reigvct \n",ERR_SPACE, ERR_FUNCTION);

  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for(index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];

  F77NAME(dgeev)("N", "V", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr, &ldvr, work, &lwork, &info);

  E.resize(n,count(valI,n));
  eigbuild(E,n,tr,valR, valI, Real());
  
  delete [] valR; delete [] valI; delete [] tl; delete [] tr;
  delete [] work; delete [] tab;
}
//----------------------------------------------------------------------
/** 
  Compute the left eigenvectors and eigenvalues of @A@.
  The matrix of eigenvector is put in the std::complex matrix @E@ and 
  the eigenvalues are stored in @V@. The {\sc lapack} routine is
  @dgeev@. The matrix @A@ is copied.
**/
inline void reigvct(rep2d<std::complex<double> > & E, 
		     linalg::rep1d<std::complex<double> > & V, const rep2d<double> & A)
{
  assert(A.nbrow() == A.nbcol());
  typedef rep2d<double>::size_type index_t;

  integer n = A.nbrow(), l = A.lda_;
  integer ldvl = n, ldvr = n;
  integer * info = new integer;
  double * valR = new double[A.nbrow()];
  double * valI = new double[A.nbrow()];
  double * tl = new double[A.nbcol()*A.nbrow()];
  double * tr = new double[A.nbcol()*A.nbrow()];
  integer lwork = 4*n + 10;
  double *work  = new double[lwork+1];

//   if(!work)
//     Error("%s %s lapack::rep2d<double>::leigvct \n",ERR_SPACE, ERR_FUNCTION);

  // Copy of A
  double* tab = new double[A.nbcol()*A.nbrow()];
  for (index_t i=0;i<A.nbcol()*A.nbrow();i++) tab[i] = A.tab_[i];
  
  F77NAME(dgeev)("V", "N", &n, tab, &l, valR, valI, 
		 tl, &ldvl, tr,&ldvr, work, &lwork, info);

  for (index_t i=0; i<A.nbrow();i++) V[i] = std::complex<double>(valR[i],valI[i]);

  eigbuild(E,A.nbcol(),tr,valR,valI);
}
//----------------------------------------------------------------------//
/**
   Compute the complex eigenvectors of the pencil $A - z\, B$.
   The matrices @A@ and @B@ are copied.
**/ 
inline void reigvct(rep2d<std::complex<double> > & E, 
		     const rep2d<double> & A, const rep2d<double> & B)
{
  assert(A.nbrow() == A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double[A.nbcol()*n];
  double*  VR= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n +1;
  double*  work  = new double[lwork];

  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "V", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
                  VL, &n, VR, &n, work, &lwork, info);
  E.resize(n,n);
  eigbuild(E,n,VR,WR,WI);
}
//----------------------------------------------------------------------//
/**
   Compute the real eigenvectors of the pencil $A - z\, B$ and keep only 
   their m first coordinates.  The matrices @A@ and @B@ are copied.
**/ 
inline void reigvct(rep2d<double> & E, 
		     const rep2d<double> & A, const rep2d<double> & B, 
		     const Real & real,
		     unsigned int M)
{
  assert(A.nbrow() == A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double[A.nbcol()*n];
  double*  VR= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n+1;
  double*  work  = new double[lwork];

  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "V", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
                  VL, &n, VR, &n, work, &lwork, info);
  int c = count(WI,BETA,n);
  E.resize(M,c);
  eigbuild(E,n,M,VR,BETA,WR,WI,Real());
}
//----------------------------------------------------------------------
/**
   Compute the real eigenvectors of the pencil $A - z\, B$.
   The matrices @A@ and @B@ are copied.
**/ 
inline void reigvct(rep2d<double> & E, 
		     const rep2d<double> & A, const rep2d<double> & B, 
		     const Real & real)
{
  reigvct(E,A,B,real,A.nbrow());
}
//----------------------------------------------------------------------//
/** Compute the real eigenvectors of the pencil $A - z\,B$. Put the 
    eigenvalues in @V@.  The matrices @A@ and @B@ are copied.
 **/
inline void reigvct(rep2d<double> & E, 
		     const rep2d<double> & A, const rep2d<double> & B, 
		     const Real &, linalg::rep1d<double> & V)
{
  assert(A.nbrow() == A.nbcol());

  integer n = A.nbrow(), lda = A.lda_, ldb = B.lda_;
  integer* info = new integer;
  double*  WR = new double[n];
  double*  WI = new double[n];
  double*  BETA = new double[n];

  double*  VL= new double[A.nbcol()*n];
  double*  VR= new double[A.nbcol()*n];
  integer  lwork = n*n + 6*n +1;
  double* work  = new double[lwork];
  // Copy of A and B
  double* At = new double[n*n];
  double* Bt = new double[n*n];
  
  for(int i=0;i<n*n;i++) {
    At[i] = A.tab_[i]; Bt[i] = B.tab_[i];
  }
 
  F77NAME(dgegv) ("N", "V", &n, At,  &lda, Bt, &ldb, WR, WI, BETA, 
                  VL, &n, VR, &n, work, &lwork, info);

//   for(int i=0;i<n;i++) cout << WR[i]<<" "; cout <<endl;
//   for(int i=0;i<n;i++) cout << WI[i]<<" "; cout <<endl;
//   for(int i=0;i<n;i++) cout << BETA[i]<<" "; cout <<endl;  

  int c = count(WI,BETA,n);
  E.resize(n,c);
  V.resize(c);
  eigbuild(V,E,n,VR,BETA,WR,WI,Real());
}
//----------------------------------------------------------------------//
/** Compute the rank of @M@, using a QR-decomposition (use the routine
@dgeqpf@. The matrix @M@ is copied.
**/
template<class C>
typename rep2d<C>::size_type rank(const rep2d<C> & M)
{
  // Copy of M
  rep2d<double> Mc(M);
  integer n = M.nbrow(), m = M.nbcol(), lda = M.lda_;

  integer info, *jpvt = new integer[m];
  double  *tau = new double[std::min(n,m)], *work = new double[3*m];
  
  F77NAME(dgeqpf)(&n,&m,Mc.tab_,&lda,jpvt,tau,work,&info);

  integer rk=0;
  while ( (rk<std::min(m,n)) && (fabs(Mc(0,0)*LAPACK_TOL/Mc(rk,rk)) < 1)) {
    rk++; 
  }

  delete [] jpvt;  delete [] tau; delete [] work;
  
  return rk;
}
//---------------------------------------------------------------------------
/** Compute the rank of @M@, using a QR-decomposition (use the routine
@dgeqpf@. The matrix @M@ is copied. The permutation of rows and 
columns needed to get a leading square matrix of std::maximal rank
are stored in @permut_row@  and @permut_col@.
**/
template<class C>
typename rep2d<C>::size_type rank(const rep2d<C> & M,
		  typename rep2d<C>::size_type * permut_row,
		  typename rep2d<C>::size_type * permut_col)
{
  integer n = M.nbrow(), m = M.nbcol(), lda = M.lda_;
  integer *info= new integer;
  integer *jpvt = new integer[std::max(m,n)];
  double *tau = new double[std::max(n,m)];
  double *work = new double[std::max(m,n)];
  rep2d<double> M_val(M);
  double eps=LAPACK_TOL;
  
  F77NAME(dgeqpf)(&n,&m,M_val.tab_,&lda,jpvt,tau,work,info);

  integer rk=0;
  while ( (rk<std::min(m,n)) && (fabs(M_val(0,0)*eps/M_val(rk,rk)) < 1)) {
    rk++; 
  }
  for (unsigned int i=0; i< M.nbcol(); i++) permut_col[i]=jpvt[i]-1;

  M_val=M; MATRIX::transpose(M_val);
  lda = M_val.lda_;

  for(int i=0;i<std::max(m,n); i++) jpvt[i]=0;

  F77NAME(dgeqpf)(&m,&n,M_val.tab_,&lda,jpvt,tau,work,info);

  for (unsigned int i=0; i< M.nbrow(); i++) permut_row[i]=jpvt[i]-1;
  return rk;

}
//-----------------------------------------------------------------------
/** Inplace inversion of triangular matrices.  The value of @s@ should be
  @'L'@ for lower or @'U'@ for upper triangular matrix. The value of @u@
  should be @'U'@ for unitary (implicitly 1 on the diagonal) or @'N'@ for
  the default. Return an integer which is $0$ if the computation
  runs correctly. 
**/
inline integer inv_tr(rep2d<double> & A, char s, char u){
  integer n = A.nbcol(), l = A.lda_;
  integer *info = new integer;
  F77NAME(dtrtri)(&s,&u, &n, A.tab_, &l, info);
  return (*info);
}; 
//----------------------------------------------------------------------
#endif //SYNAPS_WITH_LAPACK


} //CLOSE lapack

__END_NAMESPACE_SYNAPS


