#ifndef synaps_linalg_lapack_H
#define synaps_linalg_lapack_H
//----------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/arithm/traits.h>
#include <synaps/linalg/MethodName.h>
#include <synaps/linalg/rep1d.h>
#include <synaps/linalg/rep2d.h>
#include <synaps/linalg/MATRIX.m>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//------------------------------------------------------------------------
/// Domain gathering containers and specialisations  for lapack connection.
namespace lapack { 
//----------------------------------------------------------------------
//  extern double TOL;
//----------------------------------------------------------------------
/**
  Container for the connection with the fortran library LAPACK.
  Class derived from @linalg::rep2d<C>@, with a new @lda_@ field.
  It indicates the leading dimension array, that is the number 
  of rows in the internal allocated array. It can differ from the number
  of rows returned by @nbrow()@ which is smaller than @lda_@. This field
  is used only in an internal way.
 */
//----------------------------------------------------------------------
  template<class C>
  struct rep2d;
  template <class C>
  struct rep2d: public linalg::rep2d<C> 
  {
    unsigned int lda_;
    rep2d(): linalg::rep2d<C>() {}
    rep2d(const rep2d<C> & v): 
      linalg::rep2d<C>(v), lda_(v.lda_){}
    rep2d(unsigned int i, unsigned int j=1): 
      linalg::rep2d<C>(i,j), lda_(i){}
    rep2d(unsigned int i, unsigned int j, C* t): 
      linalg::rep2d<C>(i,j,t), lda_(i){}
    rep2d(unsigned int i, unsigned int j, C* t, char c): 
       linalg::rep2d<C>(i,j,t,c), lda_(i) {}
    rep2d(unsigned int i, unsigned int j, unsigned int k, C* t): 
       linalg::rep2d<C>(i,j,t), lda_(i) {this->nbrow_=k;}
    rep2d(const char * nm): linalg::rep2d<C>(nm) {this->lda_ = this->nbrow_;}
    
    rep2d<C> & operator=(const rep2d<C> & v);
    
    C & operator()(unsigned int i, unsigned int j)       
      {return this->tab_[i+this->lda_*j];}
    const C & operator()(unsigned int i, unsigned int j) const 
      {return this->tab_[i+this->lda_*j];}

    void resize(unsigned int i, unsigned int j)
      {
	this->linalg::rep2d<C>::resize(i,j);
	this->lda_=i;           //XXX
      } 
};
  //----------------------------------------------------------------------
  template<class C>
  void assign(rep2d<C> & r, rep2d<C>* v) 
  {
    assign((typename linalg::rep2d<C> &)r, (typename linalg::rep2d<C>*) v);
    r.lda_=v->lda_;
  }
  template<class C>
  rep2d<C> & rep2d<C>::operator=(const rep2d<C> & v)
  {
    this->linalg::rep2d<C>::operator=(v);
    lda_ = v.lda_;
    return *this;
  } 
  //----------------------------------------------------------------------
  //  template<class C> void transpose(rep2d<C> & A);
  // void mul(rep2d<double> & R, const rep2d<double> & A, const rep2d<double> & B);
  
  template<class C> void solve(LS,typename linalg::rep1d<C> & x, 
			       const rep2d<C> & A, const typename linalg::rep1d<C> & b);
  //  template<class C> void solve(LS,rep2d<C> & A, linalg::rep1d<C> & b);
  
  template<class C> void solve(LU,typename linalg::rep1d<C> & x, 
			       const rep2d<C> & A, const typename linalg::rep1d<C> & b);
  template<class C> void solve(LU,rep2d<C> & X, const rep2d<C> & A);

  template<class C> C triang(Gauss, rep2d<C> & A);
  template<class C> int decomp(LU,rep2d<C> & A);
  template<class C> int decomp(LU,rep2d<C> & A, rep2d<C> & U);
  template<class C> int decomp(LU,rep2d<C> & A, int *pivot);
  template<class C> int decomp(QR,rep2d<C> & A);
  template<class C> int decomp(QR,rep2d<C> & A, C * tau); 
  template<class C> int decomp(QR,rep2d<C> & A, unsigned int * pivot,C * tau); 
  template<class C> void svd(typename linalg::rep1d<C> & x, const rep2d<C> & A);
  template<class C> void svd(typename linalg::rep1d<C> & x, const rep2d<C> & A, rep2d<C> & U);
  template<class C> void svd(typename linalg::rep1d<C> & x, const rep2d<C> & A, 
		  rep2d<C> & U, rep2d<C> & V);

//   template<class C> void svdx(linalg::rep1d<C> & x, const rep2d<C> & A, 
// 		   rep2d<C> & U, rep2d<C> & V);
  
  template<class C> void eignval(typename linalg::rep1d<typename algebraic_closure<C>::T>& V, 
				 const rep2d<C> & A);
  template<class C> void eignval(typename linalg::rep1d<typename realof<C>::T>& V, 
				 const rep2d<C> & A, Real);
  template<class C> void eignval(typename linalg::rep1d<typename algebraic_closure<C>::T>& V, 
				 const rep2d<C>& , typename linalg::rep1d<C> &);
  template<class C> void eignval(typename 
				 linalg::rep1d<typename algebraic_closure<C>::T>& V, 
				 const rep2d<C>& , const rep2d<C> &);
  template<class C> void eignval(typename 
				 linalg::rep1d<typename realof<C>::T>& V, 
				 const rep2d<C>& , const rep2d<C> &, Real);
  template<class C> void leigvct(rep2d<typename algebraic_closure<C>::T> & E, 
				 const rep2d<C> & A);
  template<class C> void leigvct(
				 rep2d<typename algebraic_closure<C>::T> & E, 
				 linalg::rep1d<typename algebraic_closure<C>::T> & V, 
				 const rep2d<C> & A);
  template<class C> void reigvct(rep2d<typename algebraic_closure<C>::T> & E, 
				 const rep2d<C> & A);
  template<class C> void reigvct(rep2d<C> &E, 
				 const rep2d<C> &A, const Real &);
  template<class C> void reigvct(rep2d<typename algebraic_closure<C>::T> & E, 
				 linalg::rep1d<typename algebraic_closure<C>::T> & V, 
				 const rep2d<C> & A);
  template<class C> void reigvct(rep2d<C> &E, 
				 const rep2d<C> &A, const rep2d<C> &B, 
				 const Real &);
  template<class C> void reigvct(rep2d<C> &E, 
				 const rep2d<C> &A, const rep2d<C> &B, 
				 const Real &,
				 typename rep2d<C>::size_type);
  template<class C> void reigvct(rep2d<C> &E, 
				 const rep2d<C> &A, const rep2d<C> &B, 
				 const Real &, linalg::rep1d<C> & Val);

  template<class C> typename rep2d<C>::size_type rank(const rep2d<C> & M);
  template<class C> typename rep2d<C>::size_type rank(const rep2d<C> & M, 
				     typename rep2d<C>::size_type* permut_col,
				     typename rep2d<C>::size_type* permut_row);

  template<class C> int inv_tr(rep2d<C> & A, char s='L', char u='N');
  //----------------------------------------------------------------------
  void eigbuild(rep2d<std::complex<double> > & A, 
		int n, double* t, double* vr, double* vi);
  void eigbuild(rep2d<double > & A, 
		int n, double* t, double* vr, double* vi, Real);
  void eigbuild(rep2d<double > & A, 
		int m, int n, double* t, double * beta, 
		double* vr, double* vi, Real);
  void eigbuild(linalg::rep1d<double> & V, rep2d<double > & A, 
		int n, double* t, double * beta, double* vr, double* vi, Real);
  void eigbuild(rep2d<std::complex<double> > & A, 
		int n, double* t, double* vr, double* vi);
  int  count(double* t,int n);
  int  count(double* t,double* d, int n);
  //----------------------------------------------------------------------


} //CLOSE lapack
//====================================================================
__END_NAMESPACE_SYNAPS
//======================================================================
#ifndef SEPARATE_COMPILATION
#ifdef SYNAPS_WITH_LAPACK
#include "f2c.h"
#include "lapack.C"
#endif //SYNAPS_WITH_LAPACK
#endif
//====================================================================
#endif // SYNAPS_LINALG_LAPACK_H

