#ifndef LAPACK_CPX_H
#define LAPACK_CPX_H
//----------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/arithm/AlgClos.h>
#include <synaps/arithm/RealOf.h>
#include <synaps/linalg/matrix.h>
#include <synaps/linalg/MethodName.h>
//----------------------------------------------------------------------
#include <synaps/linalg/lapack_extern.h>



__BEGIN_NAMESPACE_SYNAPS

//------------------------------------------------------------------------
namespace lapack {
//----------------------------------------------------------------------
/**
  Container for the connection with the fortran library MAT_REP.
  Class derived from @linalg::rep2d<C>@, with a new @lda_@ field.
  It indicates the leading dimension array, that is the number 
  of rows in the internal allocated array. It can differ from the number
  of rows returned by @nbrow()@ which is smaller than @lda_@. This field
  is used only in an internal way.
**/
//----------------------------------------------------------------------
  template <class C>
  struct rep2dcplx: public linalg::rep1d<C> 
  {
    typedef complex<C>                      value_type;
    typedef C*                              col_iterator;
    typedef stride_iterator<C,C*,size_type> row_iterator;
        
    size_type nbrow_, nbcol_, lda_;
    rep2dcplx(): linalg::rep1d<C>() {}
    rep2dcplx(const rep<C> & v): 
      linalg::rep1d<C>(v), nbrow_(v.nbrow_),nbcol_(v.nbcol_),lda_(v.lda_){}
    rep2dcplx(unsigned int i, unsigned int j=1): 
      linalg::rep1d<C>(2*i*j),nbrow_(i),nbcol_(j),lda_(2*i){}
    rep2dcplx(unsigned int i, unsigned int j, C* t): 
      linalg::rep1d<C>(2*i*j,t),nbrow_(i),nbcol_(j),lda_(2*i){}
    rep2dcplx(unsigned int i, unsigned int j, C* t, char c): 
      linalg::rep1d<C>(2*i*j,t,c),nbrow_(i),nbcol_(j),lda_(2*i) {}
    rep2dcplx(unsigned int i, unsigned int j, unsigned int k, C* t): 
      linalg::rep1d<C>(2*i*j,t),nbrow_(i),nbcol_(j),lda_(2*i) {nbrow_=k;}
    rep2dcplx(const char * nm):linalg::rep1d<C>(nm) {lda_ = nbrow_;}

    rep2dcplx<C> & operator=(const rep2dcplx<C> & v);

    size_type nbrow() const {return nbrow_;}
    size_type nbcol() const {return nbcol_;}

    complex<C> & operator()(unsigned int i, unsigned int j)       
      {return complex<C>(tab_[2*i+lda_*j],tab_[2*i+lda_*j+1]);}
    complex<C>  operator()(unsigned int i, unsigned int j) const 
      {return complex<C>(tab_[2*i+lda_*j],tab_[2*i+lda_*j+1]);}

    void resize(rep2dcplx<C> & m, unsigned int i,unsigned int j)
      {
	this->linalg::rep1d<C>::resize(2*i*j);
	rbrow_=i;nbcol_=j;
	lda_=i;
      }


};
  //----------------------------------------------------------------------
  template<class C>
  void assign(rep2dcplx<C> & r, rep2dcplx<C>* v) 
  {
    assign((linalg::rep1d<C> &)r, (linalg::rep1d<C>*) v);
    r.lda_=v->lda_;
  }
  //----------------------------------------------------------------------
  //  template<class C> void transpose(rep2dcplx<C> & A);
  //  template<class C> void mult(rep2dcplx<C> & R, 
  //    		      const rep2dcplx<C> & A, const rep2dcplx<C> & B);
  
  template<class C> void solve(LS,linalg::rep1d<C> & x, 
			       const rep2dcplx<C> & A, const linalg::rep1d<C> & b);
  template<class C> void solve(LS,rep2dcplx<C> & A, linalg::rep1d<C> & b);
  
  template<class C> void solve(LU,linalg::rep1d<C> & x, 
			       const rep2dcplx<C> & A, const linalg::rep1d<C> & b);
  template<class C> void solve(LU,rep2dcplx<C> & X, const rep2dcplx<C> & A);

  template<class C> C triang(Gauss, rep2dcplx<C> & A);
  template<class C> integer decomp(LU,rep2dcplx<C> & A);
  template<class C> integer decomp(LU,rep2dcplx<C> & A, rep2dcplx<C> & U);
  template<class C> integer decomp(LU,rep2dcplx<C> & A, integer *pivot);
  template<class C> integer decomp(QR,rep2dcplx<C> & A);
  template<class C> integer decomp(QR,rep2dcplx<C> & A, C * tau); 
  template<class C> integer decomp(QR,rep2dcplx<C> & A, unsigned int * pivot,C * tau); 
  template<class C> void svd(linalg::rep1d<C> & x, const rep2dcplx<C> & A);
  template<class C> void svd(linalg::rep1d<C> & x, const rep2dcplx<C> & A, rep2dcplx<C> & U);
  template<class C> void svd(linalg::rep1d<C> & x, const rep2dcplx<C> & A, 
		  rep2dcplx<C> & U, rep2dcplx<C> & V);

//   template<class C> void svdx(linalg::rep1d<C> & x, const rep2dcplx<C> & A, 
// 		   rep2dcplx<C> & U, rep2dcplx<C> & V);
  
  template<class C> void eignval(linalg::rep1d<typename AlgClos<C>::TYPE>& V, 
				 const rep2dcplx<C> & A);
  template<class C> void eignval(linalg::rep1d<typename RealOf<C>::TYPE>& V, 
				 const rep2dcplx<C> & A, Real);
  template<class C> void eignval(linalg::rep1d<AlgClos<C>::TYPE>& V, 
				 const rep2dcplx<C>& , linalg::rep1d<C> &);
  template<class C> void eignval(linalg::rep1d<AlgClos<C>::TYPE>& V, 
				 const rep2dcplx<C>& , const rep2dcplx<C> &);
  template<class C> void eignval(linalg::rep1d<typename RealOf<C>::TYPE>& V, 
				 const rep2dcplx<C>& , const rep2dcplx<C> &, Real);
  template<class C> void leigvct(rep2dcplx<AlgClos<C>::TYPE> & E, 
				 const rep2dcplx<C> & A);
  template<class C> void leigvct(rep2dcplx<AlgClos<C>::TYPE> & E, 
		      linalg::rep1d<AlgClos<C>::TYPE> & V, const rep2dcplx<C> & A);
  template<class C> void reigvct(rep2dcplx<AlgClos<C>::TYPE> & E, 
				 const rep2dcplx<C> & A);
  template<class C> void reigvct(rep2dcplx<C> &E, 
				 const rep2dcplx<C> &A, const Real &);
  template<class C> void reigvct(rep2dcplx<AlgClos<C>::TYPE> & E, 
				 linalg::rep1d<AlgClos<C>::TYPE> & V, 
				 const rep2dcplx<C> & A);
  template<class C> void reigvct(rep2dcplx<C> &E, 
				 const rep2dcplx<C> &A, const rep2dcplx<C> &B, 
				 const Real &);
  template<class C> void reigvct(rep2dcplx<C> &E, 
				 const rep2dcplx<C> &A, const rep2dcplx<C> &B, 
				 const Real &,
				 rep2dcplx<C>::size_type);
  template<class C> void reigvct(rep2dcplx<C> &E, 
				 const rep2dcplx<C> &A, const rep2dcplx<C> &B, 
				 const Real &, linalg::rep1d<C> & Val);

  template<class C> typename rep2dcplx<C>::size_type rank(const rep2dcplx<C> & M);
  template<class C> typename rep2dcplx<C>::size_type rank(const rep2dcplx<C> & M, 
				     typename rep2dcplx<C>::size_type* permut_col,
				     typename rep2dcplx<C>::size_type* permut_row);

  template<class C> integer inv_tr(rep2dcplx<C> & A, char s='L', char u='N');
  //----------------------------------------------------------------------
  void eigbuild(rep2dcplx<complex<double> > & A, 
		int n, double* t, double* vr, double* vi);
  void eigbuild(rep2dcplx<double > & A, 
		int n, double* t, double* vr, double* vi, Real);
  void eigbuild(rep2dcplx<double > & A, 
		int m, int n, double* t, double * beta, 
		double* vr, double* vi, Real);
  void eigbuild(linalg::rep1d<double> & V, rep2dcplx<double > & A, 
		int n, double* t, double * beta, double* vr, double* vi, Real);
  void eigbuild(rep2dcplx<complex<double> > & A, 
		int n, double* t, double* vr, double* vi);
  int  count(double* t,int n);
  int  count(double* t,double* d, int n);
  //----------------------------------------------------------------------

} //======================================================================

__END_NAMESPACE_SYNAPS

#ifndef SEPARATE_COMPILATION
#include "synaps/lapack_cpx.C"
#endif

#endif // lapack_cpx_H


