********************************************************************
*      This file is part of the source code of SYNAPS library.       *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
**********************************************************************/
#ifndef synaps_linalg_lisp_H
#define synaps_linalg_lisp_H
//--------------------------------------------------------------------
#include <synaps/init.h>
#include <synaps/base/lisp.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
template<class C, class R> struct VectDse;
template<class C, class R> struct MatrDse;
template<class R> struct MatrStr;
template<class C, class R> struct MatrSps;
//--------------------------------------------------------------------
namespace lisp 
{
  //--------------------------------------------------------------------
  template<class OSTREAM, class X>
    OSTREAM& print_lisp(OSTREAM& os, const X& x) 
  { 
    os<<x; return os; 
  }
  
  //--------------------------------------------------------------------
  namespace VECTOR 
  {
    template<class OSTREAM, class R>
      OSTREAM& lisp(OSTREAM& os, const R &V) 
    {
      os<<"([]";
      for(typename R::size_type i = 0; i < V.size(); i++) {
	os<<" ";
	print_lisp(os,V[i]); 
      }
      os<<")";
      return os;
    }
  } 
  //----------------------------------------------------------------------
  /// Output operator for the vector @V@ in lisp format.
  template<class OSTREAM, class C,class R>
      OSTREAM& print_lisp (OSTREAM& os, const VectDse<C,R> &V) 
    { 
      return VECTOR::lisp(os, V);
    }
  //----------------------------------------------------------------------
  namespace MATRIX 
  {  
    template <class OSTREAM, class R>
      inline OSTREAM & lisp(OSTREAM & os,const R & M) 
    {
      typedef typename R::size_type  size_type;
      if(M.nbrow()) {
	os<<"([] ";
	for(size_type i = 0; i < M.nbrow(); i++) {
	  os << " ([]";
	  for(size_type j = 0; j < M.nbcol(); j++)
	    {
	      os<<" ";
	      print_lisp(os,M(i, j)); 
	    }
	  os<<")";
	}
	os<<")";
      }
      return os;
    }
  }
  //----------------------------------------------------------------------
  /// Lisp output operator for dense matrices.
  template<class OSTREAM, class C,class R>
  OSTREAM& print_lisp(OSTREAM& os, const MatrDse<C,R> &M) 
  {
    return MATRIX::lisp(os,M);
  }
  
  /// Lisp output operator for structured matrices.
  template<class OSTREAM, class R>
  OSTREAM& print_lisp(OSTREAM& os, const MatrStr<R> &M) 
  {
    return MATRIX::lisp(os,M);
  }
  /// Lisp output operator for sparse matrices.
  template<class OSTREAM, class C, class R>
  OSTREAM& print_lisp(OSTREAM& os, const MatrSps<C,R> &M) 
  {
    return MATRIX::lisp(os,M);
  }
}
//----------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//----------------------------------------------------------------------
#endif //synaps_base_lisp_H
