#ifndef SYNAPS_LINALG_MATRDSE_ROW_AND_COL_ACCESSORS_H
#define SYNAPS_LINALG_MATRDSE_ROW_AND_COL_ACCESSORS_H

#include <synaps/base/type.h>

__BEGIN_NAMESPACE_SYNAPS

template<class matrix>
struct row_access
{
  typedef typename matrix::rep_type             rep_type;
  typedef typename rep_type::row_iterator       iterator;
  typedef const iterator               const_iterator;
  typedef std::reverse_iterator<const_iterator>  const_reverse_iterator;
  typedef std::reverse_iterator<iterator>        reverse_iterator;

  typedef typename matrix::coeff_t              coeff_t;
  typedef typename matrix::size_type            size_type;
  iterator begin_, end_;
  row_access( const matrix &  m, unsigned int i )
  { 
    begin_ = begin_row( m.rep(), i ); 
    end_  = end_row(m.rep(),i);
  };
  iterator begin() { return begin_; };
  const_iterator begin() const { return begin_; };
  iterator end() { return end_; };
  const_iterator end() const { return end_; };
    reverse_iterator       rbegin()       {return reverse_iterator(end_); }
  const_reverse_iterator rbegin() const {return const_reverse_iterator(end_);}

  reverse_iterator       rend()       {return reverse_iterator(begin_); }
  const_reverse_iterator rend() const {return const_reverse_iterator(begin_);}

  coeff_t & operator[]( int i ) { return *(begin_+i); };
  const coeff_t & operator[]( int i ) const { return *(begin_+i); };
  size_type size() const { return end_ - begin_; };
  void resize( size_type newsz ) { assert(newsz==size()); };
};

template<class matrix>
struct col_access
{
  typedef typename matrix::rep_type             rep_type;
  typedef typename rep_type::col_iterator       iterator;
  typedef const iterator               const_iterator;
    typedef std::reverse_iterator<const_iterator>  const_reverse_iterator;
  typedef std::reverse_iterator<iterator>        reverse_iterator;

  typedef typename matrix::coeff_t              coeff_t;
  typedef typename matrix::size_type            size_type;
  iterator begin_, end_;
  col_access( const matrix &  m, unsigned int i )
  { 
    begin_ = begin_col( m.rep(), i ); 
    end_  = end_col(m.rep(),i);
  };
  iterator begin() { return begin_; };
  const_iterator begin() const { return begin_; };
  iterator end() { return end_; };
  const_iterator end() const { return end_; };
  reverse_iterator       rbegin()       {return reverse_iterator(end_); }
  const_reverse_iterator rbegin() const {return const_reverse_iterator(end_);}

  reverse_iterator       rend()       {return reverse_iterator(begin_); }
  const_reverse_iterator rend() const {return const_reverse_iterator(begin_);}

  coeff_t & operator[]( int i ) { return *(begin_+i); };
  const coeff_t & operator[]( int i ) const { return *(begin_+i); };
  size_type size() const { return end_ - begin_; };
  void resize( size_type newsz ) { assert(newsz==size()); };
};


// resulting_instanceof< X, C > -> instanceof<X,C> 
namespace type {
  
  template<class matrix, typename coeff>
  struct instanceof< row_access< matrix >, coeff > 
  { typedef typename instanceof< typename matrix::vector_type, coeff >::T T; };
  
  template<class matrix, typename coeff>
  struct instanceof< col_access< matrix >, coeff > 
  { typedef typename instanceof< typename matrix::vector_type, coeff >::T T; };
}

__END_NAMESPACE_SYNAPS

#endif
