/*********************************************************************
*      This file is part of the source code of SYNAPS kernel.        *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
History: 
$Id: toeplitz.C,v 1.1 2005/07/11 08:33:37 mourrain Exp $
**********************************************************************/

#include <synaps/init.h>
#include <synaps/arithm/traits.h>

__BEGIN_NAMESPACE_SYNAPS


namespace linalg  {
  //--------------------------------------------------------------------
template<class C>
void toeplitz<C>::resize(unsigned int i, unsigned int j)
{
  this->linalg::rep1d<C>::resize(i+j-1);
  nbrow_ = i; nbcol_= j;
}
  //--------------------------------------------------------------------
template<class C>
void toeplitz<C>::transpose() 
{ 
  size_type i=0,j=this->size()-1;
  while(i <j){
    std::swap((*this)[i],(*this)[j]);
    i++;j--;
  }
}

//----------------------------------------------------------------------
/**
  Multiplication Toeplitz  Matrix - VectDse.
  Result,  given by the first parameter r,  is a VectDse representation type
**/
template<class C,class V> 
void mul_vect(V & r, const toeplitz<C> & a, const V & b)
{
  //#ifdef VERBATIM 
	std::cout <<"Using fft for the multiplication in toeplitz<C>"<<std::endl;
	//#endif

  unsigned long col = a.nbcol(), row = a.nbrow();
  unsigned long N = (int) pow(2, 1 + (int) (log(row+col-1)/log(2)));
  
  typedef typename realof<C>::T real_t;
  typedef typename std::complex<real_t> complex_t;
  complex_t w;
  Root_Unity(w, N);
  complex_t * XA = new complex_t[N];
  complex_t * XB = new complex_t[N];
  for ( unsigned i = 0; i < row; i ++ )
    XA[i] = a(i,0);
  for ( unsigned i = N-col+1, k = 1; i < N ; i++, k++)
    XA[i] = a(0,col-k);
  
  // Bit reversal permutation, to allow Butterfly algorithm to calculate FFT
  // on Matrix and VectDse values
  Permut(XA, N);
  FFT(N, w, XA,0);
  
  for (unsigned long i = 0; i < col ; i++) XB[i] = b[i];
  Permut(XB, N);
  FFT(N, w, XB, 0);
      
  // multiplication of FFT(Matrix) by FFT(VectDse)

  for (unsigned long k = 0; k < N; k++) XA[k] *= XB[k] ; 
      
  // result of this multiplication is also permuted for the same reason
  Permut(XA, N);
  // Backward FFT , to retrieve the result of multiplication 
  // of Matrix-VectDse
  FFT(N, pow(w,-1), XA, 1);
      
  for ( unsigned long k = 0; k < row; k++)  
    {
      XA[k]/=N;
      //using namespace let;
     r[k] = XA[k].real();
    };

  delete [] XA;
  delete [] XB;
}
/********************************************************************/
} //! /namespace toepliz

__END_NAMESPACE_SYNAPS


#include <synaps/init.h>


