#define USING_NAMESPACE_SYNAPS 1

#include <cassert>
#include <iostream>
#include <synaps/mpol.h>
#include <synaps/linalg.h>     // General header file for linear algebra.
#include <synaps/base/maple.h>      // For output in maple format.
#include <synaps/arithm/gmp.h>
using std::cout;
using std::cin;
using std::endl;

typedef VectDse<double, linalg::rep1d<double> > VECT;
  // Vector definition based on array1d container, with double coefficients.
typedef MatrDse<double,lapack::rep2d<double> > MAT;
  // Marix definition based on lapack container, with double coefficients.
  // This will allow us to use the routines of the lapack library.

int main(int argc, char** argv)
{
#ifdef SYNAPS_WITH_LAPACK

//  int status;

  double v[]={1,2,1,1,3,4,2,2,5,5,5,5,-2, 1,2,3,1, -2.8,-2.4,1,.2,5.8};
  MAT A(4,4,v,ByRow()),
      B(4,4,"1 2 1 1 3 4 2 2 5 5 5 5 -2  1 2 3 1 .2 5.8", ByCol());

  cout<<"A="<<A<<", B="<<B<<endl;

  // Arithmetic operations.
  MAT C, R;
  C= A+B; cout<<C<<endl;
  R = MAT(4,4,"2 5 6 -1 5 8 7 3 6 7 10 7 -1 3 7 6", ByRow());
  assert(C == R);
  C= A-B; cout<<C<<endl;
  assert(C == MAT(4, 4, "0 -1 -4 3 1 0 -3 1 4 3 0 3 -3 -1 -3 0", ByRow()));
  C= A*B; cout<<C<<endl;
  assert(C == MAT(4,4,"7 15 25 5 15 33 55 8 25 55 100 20 5 8 20 18",ByRow()));
  C= A*B*A; cout<<C<<endl;
  assert(C == MAT(4,4,"167 204 172 177 373 445 372 380 650 790 675 695 93 160 157 175",ByRow()));
  C+=A*2; cout<<C<<endl;
  
  C= 2*A; cout<<C<<endl;
  assert(C == MAT(4,4,"2 4 2 2 6 8 4 4 10 10 10 10 -4 2 4 6",ByRow()));
  // Construction of a Vector from to iterators.
  VECT V(v+1,v+5);  cout<<V<<endl;
//   assert(V == VECT(4, "2 1 1 3"));
  VECT W(A*V);      cout<<W<<endl;
//   assert(W == VECT(4, "8 18 35 8"));

//   //Extractions of sub matrices of vectors:
   W = col(A,1);                  cout<<"col(A,1) = "<<W<<endl;
   //   assert(W == VECT(4, "2 4 5 1"));
   W = row(A,2);                  cout<<"row(A,2) = "<<W<<endl;
   //   assert(W == VECT(4, "5 5 5 5"));
   B = A(Range(1,3),Range(1,3));  cout<<"A(1..3,1..3) = "<<B <<endl;
//   assert(B.nbcol() == 3);
//   assert(B.nbrow() == 3);
//   assert(B == MAT(3,3,"4 2 2 5 5 5 1 2 3",ByRow()));
   B = row(A,Range(1,2));         cout<<B<<endl;
   assert(B.nbcol() == 4);
   assert(B.nbrow() == 2);
   assert(B == MAT(2,4,"3 4 2 2 5 5 5 5",ByRow()));
   B = col(A,Range(1,2));         cout<<B<<endl;
   assert(B.nbcol() == 2);
   assert(B.nbrow() == 4);
   assert(B == MAT(4,2,"2 1 4 2 5 5 1 2",ByRow()));

//   cout <<"-----------"<<endl;
   A(Range(0,2),Range(0,2)) = Eval(A(Range(1,3),Range(1,3)));  cout<<A<<endl;
//   //Row(A,1) =  Row(A,1)-Row(A,0);  cout <<A<<endl;
//   assert(A == MAT(4,4, "1 2 1 1 5 6 3 3 5 5 5 5 -2 1 2 3", ByRow()));
   // Output to the maple format.
   cmaple <<"A := "<<A<<endl;
   cout<<" "<<endl;
#endif
  return 0;
};
