/*********************************************************************
*   This file is part of the source code of the SYNAPS.              *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
$Id: MONOMIAL.h,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#ifndef SYNAPS_MPOL_MONOMIAL_H
#define SYNAPS_MPOL_MONOMIAL_H

#include <synaps/init.h>
#include <synaps/mpol/Variables.h>

__BEGIN_NAMESPACE_SYNAPS

/// Module for monomials
namespace MONOMIAL
{
  //--------------------------------------------------------------------   
  template <class R, class VARIABLES> inline
  std::ostream & print_pp (std::ostream & os, const R & m, const VARIABLES& V)
  { 
    bool first=true;
    for (typename R::size_type i = 0; i < m.rep().size(); ++i)
      {
	if (m[i] !=0)
	  {
	    if(first)
	      first =false;
	    else
	      os<<'*';
	    os << V[i];
	    if (m[i] != 1)
	      os << '^' <<m[i];
	  }
      }
    return os;
  }
  //--------------------------------------------------------------------
  template <class OSTREAM, class R, class VARIABLES> inline
  OSTREAM & print (OSTREAM & os, const R & m, const VARIABLES& V)
  {
    if (m.coeff() ==0 )
      os<<"0";
    else {
      int i=m.nvars();
      while(i> -1 && m[i]==0) i--;
      if(i<0)
	os <<m.coeff();
      else{
      if(m.coeff() != 1) 
	if(m.coeff()==-1) os << "-";
	else 
	  {  os << m.coeff(); os <<"*"; }
      print_pp(os,m,V);
      }
    }
    return os;
  }
  //--------------------------------------------------------------------
  template <class OSTREAM, class R> inline
  OSTREAM& print (OSTREAM& os, const R& m)
  {
    return print(os,m,Variables::default_);
  }
  //--------------------------------------------------------------------
  template <class X> inline
  bool has_no_sign(const X & x)
  {
    return true;
  }

template <class X> inline
bool has_no_sign(const int & x) {return (x>0);}

inline bool has_no_sign(const unsigned & x)    {return true;}
inline bool has_no_sign(const float & x)       {return (x>0);}
inline bool has_no_sign(const double & x)      {return (x>0);}
inline bool has_no_sign(const long double & x) {return (x>0);}

//======================================================================
} //CLOSE MONOMIAL

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_MPOL_MONOMIAL_H

