/*********************************************************************
*      This file is part of the source code of SYNAPS library.       *
*   Author(s): B. Mourrain, GALAAD, INRIA                            *
**********************************************************************
$Id: Monom.C,v 1.1 2005/07/11 11:17:56 mourrain Exp $
**********************************************************************/
#include <cstdlib>

__BEGIN_NAMESPACE_SYNAPS

/// Construction of the monomial $x_{i}^{d}$.
template <class C, class R>
Monom<C,R>::Monom(int i, int d):_coef(1)
{
  if(d == 0)
    rep().resize(0);
  else
    {
      rep().reserve(i+1);
      for(int j=0;j<i;j++) rep().setExponent(j,0);
      rep().setExponent(i,d);
    }
}
//----------------------------------------------------------------------

/// Construction of the monomial $x_{i}^{d}$.
template <class C, class R>
Monom<C,R>::Monom(const std::string & s, int d, 
 		  Variables& vars 		  ):_coef(1)
{
  if(d == 0)
    rep().resize(0);
 else
     {
       int i=vars[s];
       //  std::cout <<":: "<<s<<" "<<i<<std::endl;
       rep().reserve(i+1);
       for(int j=0;j<i;j++) rep().setExponent(j,0);
       rep().setExponent(i,d);
     }
 }

//----------------------------------------------------------------------
/// Construction of the monomial $c\,x_{i}^{d}$.
template <class C, class R>
Monom<C,R>::Monom(const C & c, int i, int d ):_coef(c)
{
  if(d == 0)
    rep().resize(0);
  else
    {
      rep().reserve(i+1);
      for(int j=0;j<i;j++) rep().setExponent(j,0);
      rep().setExponent(i,d);
    }
}
//----------------------------------------------------------------------
// arithemtic operators
//----------------------------------------------------------------------
template <class C, class R> inline
bool Monom<C,R>::operator== (int n) const
{
  return (coeff()==n);
}
//----------------------------------------------------------------------
// template <class C, class R> inline
// int Monom<C,R>::index_of_var(const std::string & s)
// {
//   //cout <<">> "<<s<<endl;
//   if(s[0]=='x' && s.size()>1 )
//     {
//       const char* var=s.data()+1;
//       return atoi(var);
//     }
//   else
//     {
//       int i=_index_of_var[s]-1;
//       if(i<0)
// 	{
// 	  while(_var_of_index[_number_var]!="")
// 	    {
// 	      _number_var++;
// 	    }
// 	  i=_number_var-1;
// 	  _index_of_var[s]=_number_var;
// 	  _var_of_index[_number_var]=s;
// 	}
//       return i;
//     }
// }
// //----------------------------------------------------------------------
// template <class C, class R> inline
// std::string Monom<C,R>::var_of_index(int i)
// {
//   if(_var_of_index[i+1]!="")
//     return _var_of_index[i+1];
//   else
//     {
//       int sz=1,m=1;
//       while(m<i) {m*=10; sz++;}
//       char str[sz];
//       sprintf(str,"%d",i);
//       return std::string("x")+str;
//     }
// }
//----------------------------------------------------------------------
// arithemtic operators
//----------------------------------------------------------------------
template <class C, class R> inline
Monom<C,R> & Monom<C,R>::operator += (const Monom<C,R> & m)
{
  coeff() += m.coeff();
  if (coeff()==0) erase(rep());
  return *this;
}

template <class C, class R> inline
Monom<C,R> & Monom<C,R>::operator -= (const Monom<C,R> & m)
{
  coeff() -= m.coeff();
  if (coeff()==0) erase(rep());
  return *this;
}

template <class C, class R> inline
Monom<C,R>  operator +  (const Monom<C,R> & m1, const Monom<C,R> & m2)
{
  Monom<C,R> res(m1);
  res.coeff() += m2.coeff();
  return (*res);
}

template <class C, class R> inline
Monom<C,R> operator - (const Monom<C,R> & m1, const Monom<C,R> & m2)
{
        Monom<C,R> res(m1);
	res.coeff() -= m2.coeff();
	return (res);
}

template <class C, class R> inline
Monom<C,R> & Monom<C,R>::operator *= (const Monom<C,R> & m)
{
  using namespace VECTOR;
  coeff() *= m.coeff();
  if (coeff()!=0)
    add(rep(),m.rep());
  else
    erase(rep());
  return *this;
}

template <class C, class R> inline
Monom<C,R> & Monom<C,R>::operator *= (const C & c)
{
  coeff() *= c;
  if (coeff()==0) erase(rep());
  return *this;
}

template <class C, class R> inline
Monom<C,R> & Monom<C,R>::operator /= (const C & c)
{
  assert(c !=0);
  coeff() /= c;
  if (coeff()==0) erase(rep());
  return *this;
}

template <class C, class R> inline
Monom<C,R> operator * (const Monom<C,R> & m1, const C & c2)
{
  Monom<C,R> res(m1);
  res.coeff() *= c2;
  if (res.coeff() ==0) erase(res.rep());
  return (res);
}
template <class C, class R> inline
Monom<C,R> operator * (const C & c2, const Monom<C,R> & m1)
{
  return(m1*c2);
}

template <class C, class R> inline
Monom<C,R> operator / (const Monom<C,R> & m1, const C & c2)
{
  Monom<C,R> res(m1);
  res.coeff() /= c2;
  if (res.coeff() ==0) erase(res.rep());
  return (res);
}


//----------------------------------------------------------------------
// operateurs d'entrees/sorties
//----------------------------------------------------------------------
/// Output operator.
template <class C, class R> inline
std::ostream & operator << (std::ostream & os, const Monom <C,R> & m)
{
  if (m.coeff() ==0 )    os<<"(0)";
  else {
    MONOMIAL::print(os,m);
  }
  return os;
}
//----------------------------------------------------------------------

__END_NAMESPACE_SYNAPS

