
/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef synaps_mpol_mpol_dse_BEZiER_m
#define synaps_mpol_mpol_dse_BEZiER_m
//===================================================================
#include <synaps/init.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================

namespace BEZIER {

//====================================================================
template<class MATR, class MPOL, class X>
void monomial2bezier2d(MATR& M, const MPOL& p, 
		       const X& x0, const X& x1, const X& y0, const X& y1)
{
  using let::assign;
  typedef typename MATR::coeff_t coeff_t;
  M.resize(degree(p,1)+1, degree(p,0)+1);
  for(typename  MPOL::const_iterator it=p.begin(); it != p.end(); it++)
    {
      assign(M((*it)[1],(*it)[0]),it->coeff());
    }
  for(unsigned i=0;i<M.nbrow();i++)
    {
      VectDse<coeff_t> V = row(M,i), BZ(M.nbcol(),AsSize());
      monomials2bezier(BZ,V,V.size(),x0,x1);
      //      row(M,i)=BZ;
      for(unsigned j=0;j<M.nbcol();j++) M(i,j)=BZ[j];
    }

  for(unsigned j=0;j<M.nbcol();j++)
    {
      VectDse<coeff_t> V= col(M,j), BZ(M.nbrow(),AsSize());
      monomials2bezier(BZ,V,V.size(),y0,y1);
      //      col(M,j)=BZ;
      for(unsigned i=0;i<M.nbrow();i++) M(i,j)=BZ[i];
    }
}
//--------------------------------------------------------------------
template<class matr_t>
void deCasteljau( matr_t& left, matr_t& right, ByRow M ) 
{
    typedef typename matr_t::value_type coeff_t;

    int col=left.nbcol(),row=left.nbrow();
    assert( col > 1 );
    coeff_t middle = coeff_t(0.5);
    for ( int i = 0 ; i < row ; ++i )
      {
	for ( int j = 1 ; j < col ; ++j )
	  {
	    left( i, j-1) = right( i, 0); 
	    for ( int k = 0 ; k < col - j ; ++k ) 
	      {
		right( i, k ) = ( right( i, k ) + right( i, k+1) ) * middle;
	      }
	  }
	left( i, col-1 ) = right( i, 0);
      }
}  
//--------------------------------------------------------------------
template<class matr_t>
void deCasteljau( matr_t & left, matr_t & right, ByCol M ) 
{

  typedef typename matr_t::value_type coeff_t;

  int col=right.nbcol(),row=right.nbrow();
  assert( row > 1 );
  coeff_t middle = coeff_t(0.5);
  for ( int i = 0 ; i < col ; ++i )
    {
      for ( int j = 1 ; j < row ; ++j )
	{
	  left( j-1, i) = right( 0, i); 
	  for ( int k = 0 ; k < row - j ; ++k ) 
	    {
	      right( k, i ) = ( right( k, i ) + right( k+1, i ) ) * middle;
	    }
	}
      left( row-1, i) = right( 0, i);
    }
}  
//--------------------------------------------------------------------
template<class LIST, class CELL>
void split(LIST& L, const CELL& eqs)
{
  typedef CELL                  cell_t;
  typedef typename CELL::syst_t syst_t;
  
  syst_t M1(eqs.syst), M2(eqs.syst);
  for(unsigned i=0;i<eqs.syst.size();i++)  deCasteljau(M1[i],M2[i],ByRow());
  syst_t M11(M1),M21(M1);

  for(unsigned i=0;i<eqs.syst.size();i++)  deCasteljau(M11[i],M21[i],ByCol());
  L.push_back(cell_t(eqs.x0,(eqs.x0+eqs.x1)/2,eqs.y0,(eqs.y0+eqs.y1)/2,M11));
  L.push_back(cell_t(eqs.x0,(eqs.x0+eqs.x1)/2,(eqs.y0+eqs.y1)/2,eqs.y1,M21));

  syst_t M12(M2),M22(M2);
  for(unsigned i=0;i<eqs.syst.size();i++)  deCasteljau(M12[i],M22[i],ByCol());
  L.push_back(cell_t((eqs.x0+eqs.x1)/2,eqs.x1,eqs.y0,(eqs.y0+eqs.y1)/2,M12));
  L.push_back(cell_t((eqs.x0+eqs.x1)/2,eqs.x1,(eqs.y0+eqs.y1)/2,eqs.y1,M22));
}

}
//===================================================================
__END_NAMESPACE_SYNAPS
#endif //
