#ifndef SYNAPS_MPOLDSE_MODULE_H
#define SYNAPS_MPOLDSE_MODULE_H

#include <synaps/mpol/mpoldse/eenv.h> 
#include <synaps/mpol/mpoldse/vctops.h>
#include <synaps/mpol/mpoldse/binomials.h>
//#include <synaps/mpol/mpoldse/eenv.h>

__BEGIN_NAMESPACE_SYNAPS

namespace MPOLDSE
{

  typedef mpoldse::eenv eenv;
  
  template<class MPLBASE, class C >
  void scadd (MPLBASE & mpl, const C & c)
  {
    vct::scadd (mpl.begin (), c, mpl.esz (), 1);
  };
  
  template<class MPLBASE, class MPLBASE1, class C >
  void scadd ( MPLBASE& r, const MPLBASE1 &a, const C & c)
  {
    if (r.env != a.env) 
      scadd (r = a, c);
    else 
      vct::scadd (r.begin (), a.begin (), c, r.esz ());
  };
  
  template<class MPLBASE0,class MPLBASE1>
    void conv( MPLBASE0 &r, const MPLBASE1 &a );

  template<class MPLBASE0, class MPLBASE1, class MPLBASE2 >
  void conv( MPLBASE0& r, const MPLBASE1& a, const MPLBASE2& b )
  {
    if (&a == &r) { conv (r, b); return; };
    if (&b == &r) { conv (r, a); return; };
    clear (r, eenv::mul (a.env, b.env));

    unsigned *oa = new unsigned[a.esz () + b.esz ()];
    unsigned *ob = oa + a.esz ();

    eenv::oaddress (r.env, oa, a.env);
    eenv::oaddress (r.env, ob, b.env);
    
    typedef typename MPLBASE0::value_type rreal;
    typedef typename MPLBASE1::value_type areal;
    typedef typename MPLBASE2::value_type breal;
    
    unsigned *ea;
    unsigned *eb;
    rreal * pr = r.begin();
    const areal * pa;
    const breal * pb;
    
    for (ea = oa, pa = a.begin (); ea != oa + a.esz (); ea++, pa++)
      for (eb = ob, pb = b.begin (); eb != ob + b.esz ();
	   pr[*ea + *eb++] += (*pa) * (*pb++));
    delete[]oa;
  };
  
  template<class MPLBASE0,class MPLBASE1>
  void conv( MPLBASE0 &r, const MPLBASE1 &a )
  {
    MPLBASE0 tmp;
    r.swap (tmp);
    conv (r, tmp, a);
  };
  
  template < class MPLBASE  > 
  inline void convertm2b ( MPLBASE  & mpl ) { 
    typedef typename MPLBASE::value_type C;
    convertm2b (mpl.begin (), mpl.env, binomials < C >::default_); 
  };
  template < class MPLBASE > 
  inline void convertb2m ( MPLBASE  & mpl ) { 
    typedef typename MPLBASE::value_type C;
    convertb2m (mpl.begin (), mpl.env, binomials < C >::default_); };
  
  template< class MPLBASE >  inline 
  void binoms( MPLBASE& mpl )
  {
    typedef typename MPLBASE::value_type C;
    binoms( mpl.begin(), mpl.env, binomials<C>::default_ ) ;
  };

};
__END_NAMESPACE_SYNAPS

#endif
