#ifndef SYNAPS_SOLVE_SBDSLV_BINOMIALS_H
#define SYNAPS_SOLVE_SBDSLV_BINOMIALS_H

#include <synaps/init.h>
#include <vector>
#include "vctops.h"

__BEGIN_NAMESPACE_SYNAPS

template<typename C>
struct binomials
{
  static binomials default_;
  std::vector<int> offs;
  std::vector<C>   data;
  binomials(const binomials& b):offs(b.offs),data(b.data) {}
  binomials();
  // return the list [ binomial(d,i) ], for i = 0..d
  const C* operator[]( int d );
  const C& operator()( int i, int j ) { return (*this)[i][j]; };
};

// IMPLEMENTATION

template<class C>
binomials<C>::binomials() : offs(2), data(3) 
{
  offs[0] = 0;       data[0] = 1;
  offs[1] = 1; data[offs[1]] = 1; data[offs[1]+1] = 1;
};

template<class C> binomials<C> binomials<C>::default_;

template<class C> 
const C* binomials<C>::operator[]( int d )
{
  int s,j,x;
  if ( (s=offs.size()) > d ) return &*(data.begin())+offs[d];
  for ( x = 0, j = s; j <= d; x += j+1, j ++ );
  offs.resize(d+1);
  data.resize(data.size()+x);
  for ( j = s; j <= d; j ++ )
    {
      offs[j] = offs[j-1] + j;
      for ( int i = 0; i < j-1; i ++ )
	data[offs[j]+i+1] = data[offs[j-1]+i] + data[offs[j-1]+i+1];
      data[offs[j]] = data[offs[j]+j] = 1;
    };
  return &*(data.begin())+offs[d];
};


template<class C> inline
void convertb2m( C * bzc, unsigned sz, int st, binomials<C>& binom  )
{
  int i,k;
  for ( i = st; i != st*int(sz); i += st )
    for ( k = (sz-1)*st; k != i-st; k -= st )
      bzc[k] -= bzc[k-st];
  vct::pmul(bzc,binom[sz-1],sz,st,1);
};
  

template<class C> inline
void convertm2b( C * bzc, unsigned sz, int st, binomials<C>& binoms )
{
  C tmp[sz];
  const C * bin;
  int i,k,p,l;
  
  for ( p = 0, i = 0; i < sz; i++, p += st ) 
    { tmp[i] = bzc[p]; bzc[p] = 0; };
  for ( p = 0, i = 0; i < sz; i++, p += st )
    for ( bin = binoms[sz-i-1], l = p, k = 0; k < sz-i; k++, l += st )
      bzc[l] += tmp[i]*bin[k];
  bin = binoms[sz-1];
  for ( i = 0, l = 0; i < sz; i ++, l += st  )
    bzc[l] /= bin[i];
};

__END_NAMESPACE_SYNAPS

#endif
