/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef synaps_mpol_triangular_h
#define synaps_mpol_triangular_h
//===================================================================
#include <synaps/init.h>
#include <synaps/linalg/rep1d.h>
#include <synaps/linalg/VECTOR.m>
#include <synaps/mpol/MPol.h>
#include <synaps/mpol/mpoldse/binomials.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
namespace triangular
{
  template<class C>
  struct monomial : public linalg::rep1d<C> 
  {
    typedef monomial<C>      self_t;
    typedef linalg::rep1d<C> base_t;
    
    unsigned nbvar_;

    monomial():base_t(), nbvar_(0) {}
    monomial(const self_t& p):base_t((const base_t&)p), nbvar_(p.nbvar_) {}

    monomial(const MPol<C>& p);
    unsigned index_of(const typename MPol<C>::monom_t& m, binomials<int>& nbmon);
  };


  template<class C> 
  unsigned monomial<C>::index_of(const typename MPol<C>::monom_t& m, binomials<int>& binom)
  {
    int dg = degree(m)-1;
    unsigned idx = binom(dg+nbvar_,nbvar_)-1;
    for(unsigned i=0;i<m.nbvar();i++)
      {
	dg -= m[i];
	if(dg<0) idx++; else idx+=binom(dg+nbvar_-i-1, nbvar_-i-1);  
      }
    return idx;
  }

  template<class C> 
  monomial<C>::monomial(const  MPol<C>& p)
  {
    nbvar_=p.nbvar();
    binomials<int> binom;
    this->resize(binom(degree(p)+nbvar_,nbvar_));
    for(typename MPol<C>::const_iterator it=p.begin();it!=p.end();it++)
      (*this)[index_of(*it, binom)] =it->coeff();
  }

  template<class C> inline
  void add(monomial<C>& r, const monomial<C>& a, const monomial<C>& b)
  {
    VECTOR::add(r,a,b);
  }

  template<class C> inline
  void add(monomial<C>& r, const monomial<C>& a)
  {
    VECTOR::add(r,a);
  }

  template<class OSTREAM, class C> inline
  void print(OSTREAM& os, const monomial<C>& a)
  {
    VECTOR::print(os,a);
  }
  
}

//===================================================================
__END_NAMESPACE_SYNAPS
#endif //synaps_mpol_triangular_h
