#include <synaps/init.h>
#include <list>
#include "synaps/mpol/MPol.h"
#include "synaps/mpol/bezoutian.h"
#include "synaps/mpol/macaulay.h"
#include "synaps/linalg/MatrDse.h"
#include "synaps/linalg/lapack.h"
#include "synaps/linalg/Svd.h"
#include "synaps/linalg/Eigen.h"
#include "synaps/linalg/Uschur.h"

using std::cout;
using std::cin;
using std::endl;

typedef MPol<double> Pol;
typedef MatrDse<double,lapack::rep2d<double> > Matr;
typedef VectDse<double> Vect;

int main (int argc, char **argv)
{

  unsigned n=3;
  linalg::rep1d<Pol> l(n);
  
  l[0] = Pol("x0");
  l[1] = Pol("2*x0^2+2*x0*x1+2");
  l[2] = Pol("3*x1^2+3*x1+3");

  int d=1;
  for(int i=1; i< (int)n;i++) d*=MPOLDST::degree(l[i]);
  cout <<"Bezout bound: "<<d<<endl;

  Matr S=macaulay<Matr>(l);
  cout <<"Macaulay matrix of size "<<S.nbrow()<<"x"<<S.nbcol()<<endl;
  cout <<S<<endl;
  
  for(unsigned int i=d;  i<S.nbrow(); i++) 
    if(S(i,i)==0) 
      cout<<"The system is not generic. Zero coefficient at "
	  <<i<<" "<<i<<endl; 
   Matr D = S(Range(d,S.nbrow()-1),Range(d,S.nbcol()-1));
   cout <<"Lower diag. block of size "<< D.nbrow()<<endl;

#ifdef SYNAPS_WITH_LAPACK
   cout <<"Singular values of the block D: "<<endl;
   Vect Sv= Svd(D);  cout <<Sv<<endl;

   Matr U= USchur(S,d);
   cout <<"Eigenvalues of the Schur complement: "<<endl << Eigenval(U)<<endl;
#else
  std::cout <<"  Lapack is not connected"<<std::endl;
#endif
}
