<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Resultant-based methods for solving bivariate polynomial
  equations>

  <\with|par-mode|right>
    <name|H. Khalil>
  </with>

  This section describes an algorithm, which uses generalized eigenvalues and
  eigenvectors, for solving a system of two bivariate polynomials
  <with|mode|math|p(x,y)=q(x,y)=0>, with <with|mode|math|p,q\<in\><with|math-font|Bbb|R>[x,y]>.
  It treats multiple roots, which are usually considered as obstacles for
  numerical linear algebra techniques. The numerical difficulties are handled
  with the help of singular value decomposition (SVD) of the matrix of
  eigenvectors associated to each eigenvalue. The extension to
  <with|mode|math|n> equations could be treated similarly.

  <subsection|Generalized eigenvalues and eigenvectors>

  <label|eigen>Let <with|mode|math|A,B> be two square matrices of size
  <with|mode|math|n\<times\>n>. A <with|font-shape|italic|generalized
  eigenvalue> of <with|mode|math|A> and <with|mode|math|B> is a value in the
  set

  <\equation*>
    \<lambda\>(A,B):={\<lambda\>\<in\><with|math-font|Bbb|C>;det
    (A-\<lambda\>B)=0}.
  </equation*>

  A vector <with|mode|math|x\<neq\>0> is called a
  <with|font-shape|italic|generalized eigenvector> associated to the
  eigenvalue <with|mode|math|\<lambda\>\<in\>\<lambda\>(A,B)> if
  <with|mode|math|A*x=\<lambda\>B*x>. The matrices <with|mode|math|A> and
  <with|mode|math|B> have <with|mode|math|n> generalized eigenvalues if and
  only if <with|mode|math|rank(B)=n>. If <with|mode|math|rank(B)\<less\>n>,
  then <with|mode|math|\<lambda\>(A,B)> can be finite, empty, or infinite.
  Note that if <with|mode|math|0\<neq\>\<mu\>\<in\>\<lambda\>(A,B)> then
  <with|mode|math|1/\<mu\>\<in\>\<lambda\>(B,A)>. Moreover, if
  <with|mode|math|B> is invertible then <with|mode|math|\<lambda\>(A,B)=\<lambda\>(B<rsup|-1>A,I)=\<lambda\>(B<rsup|-1>A)>,
  which is the ordinary spectrum of <with|mode|math|B<rsup|-1>A>.<next-line>

  Recall that an <with|mode|math|n\<times\>n> matrix <with|mode|math|T(x)>
  with polynomial coefficients can be equivalently written as a polynomial
  with <with|mode|math|n\<times\>n> matrix coefficients. If
  <with|mode|math|d=max<rsub|i,j>{deg (T<rsub|i*j>(x))}>, we obtain
  <with|mode|math|T(x)=T<rsub|d>x<rsup|d>+T<rsub|d-1>x<rsup|d-1>+\<cdots\>+T<rsub|0>>,
  where <with|mode|math|T<rsub|i>> are <with|mode|math|n\<times\>n> matrices.

  <\definition>
    The companion matrices of <with|mode|math|T(x)> are the two matrices
    <with|mode|math|A> and <with|mode|math|B> given by:

    <\equation*>
      <tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|c>|<table|<row|<cell|A=<left|(><tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|c>|<cwith|1|-1|2|2|cell-halign|c>|<cwith|1|-1|3|3|cell-halign|c>|<cwith|1|-1|4|4|cell-halign|c>|<table|<row|<cell|0>|<cell|I>|<cell|\<cdots\>>|<cell|0>>|<row|<cell|\<vdots\>>|<cell|\<ddots\>>|<cell|\<ddots\>>|<cell|\<vdots\>>>|<row|<cell|0>|<cell|\<cdots\>>|<cell|0>|<cell|I>>|<row|<cell|T<rsub|0><rsup|t>>|<cell|T<rsub|1><rsup|t>>|<cell|\<cdots\>>|<cell|T<rsub|d-1><rsup|t>>>>>><right|)>,<space|0.25em>B=<left|(><tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|c>|<cwith|1|-1|2|2|cell-halign|c>|<cwith|1|-1|3|3|cell-halign|c>|<cwith|1|-1|4|4|cell-halign|c>|<table|<row|<cell|I>|<cell|0>|<cell|\<cdots\>>|<cell|0>>|<row|<cell|0>|<cell|\<ddots\>>|<cell|>|<cell|\<vdots\>>>|<row|<cell|\<vdots\>>|<cell|>|<cell|I>|<cell|0>>|<row|<cell|0>|<cell|\<cdots\>>|<cell|0>|<cell|-T<rsub|d><rsup|t>>>>>><right|)>>>>>>.
    </equation*>
  </definition>

  And we have the following interesting property :

  <\proposition>
    <label|AB>With the above notation, the following equivalence holds:

    <\equation*>
      <space|1em>T<rsup|t>(x)v=0\<Leftrightarrow\>(A-x*B)<left|(><tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|c>|<table|<row|<cell|v>>|<row|<cell|x*v>>|<row|<cell|\<vdots\>>>|<row|<cell|x<rsup|d-1>v>>>>><right|)>=0.
    </equation*>
  </proposition>

  <subsection|Intersection of planar curves>

  From now on we will assume that <with|mode|math|math-font|Bbb*|K> is an
  algebraically closed field and suppose given two bivariate polynomials
  <with|mode|math|p(x,y)> and <with|mode|math|q(x,y)> in
  <with|mode|math|<with|math-font|Bbb*|K>[x,y]>. Our aim will be to study and
  compute their common roots. This problem can be interpreted more
  geometrically. Polynomials <with|mode|math|p> and <with|mode|math|q>
  actually define two algebraic curves in the affine plane
  <with|mode|math|<with|math-font|Bbb|A><rsup|2>> (having coordinates
  <with|mode|math|(x,y)>) and we would like to know their intersection
  points.

  Hereafter we will consider systems <with|mode|math|p(x,y)=q(x,y)=0> having
  only a <with|font-shape|italic|finite> number of roots. This condition is
  not quite restrictive since it is sufficient (and necessary) to require
  that polynomials <with|mode|math|p> and <with|mode|math|q> are coprime in
  <with|mode|math|<with|math-font|Bbb*|K>[x,y]> (otherwise one can divide
  them by their <with|mode|math|gcd>).

  In the case where one of the curve is a line, i.e. one of the polynomial
  has degree 1, the problem is reduced to solving a univariate polynomial.
  Indeed, one may assume e.g. that <with|mode|math|q(x,y)=y> and thus we are
  looking for the roots of <with|mode|math|p(x,0)=0>. Let us denote them by
  <with|mode|math|z<rsub|1>,\<ldots\>,z<rsub|s>>. Then we know that

  <\equation*>
    p(x,0)=c(x-z<rsub|1>)<rsup|\<mu\><rsub|1>>(x-z<rsub|2>)<rsup|\<mu\><rsub|2>>\<cdots\>(x-z<rsub|s>)<rsup|\<mu\><rsub|s>>,
  </equation*>

  where the <with|mode|math|z<rsub|i>>'s are assumed to be distinct and
  <with|mode|math|c> is a non-zero constant in
  <with|mode|math|math-font|Bbb*|K>. The integer
  <with|mode|math|\<mu\><rsub|i>>, for all <with|mode|math|i=1,\<ldots\>,s>,
  is called the <with|font-shape|italic|multiplicity> of the root
  <with|mode|math|z<rsub|i>>, and it turns out that
  <with|mode|math|<big|sum><rsub|i=1><rsup|s>\<mu\><rsub|i>=deg (p)> if
  <with|mode|math|p(x,y)> do not vanish at infinity in the direction of the
  <with|mode|math|y>-axis (in other words, if the homogeneous part of
  <with|mode|math|p> of highest degree do not vanish when
  <with|mode|math|y=0>). This later condition can be avoid in the projective
  setting: let <with|mode|math|p<rsup|h>(x,y,t)> be the homogeneous
  polynomial obtained by homogenizing <with|mode|math|p(x,y)> with the new
  variable <with|mode|math|t>, then we have

  <\equation*>
    p(x,0,t)=c(x-z<rsub|1>t)<rsup|\<mu\><rsub|1>>(x-z<rsub|2>t)<rsup|\<mu\><rsub|2>>\<cdots\>(x-z<rsub|s>t)<rsup|\<mu\><rsub|s>>t<rsup|\<mu\><rsub|\<infty\>>>,
  </equation*>

  where <with|mode|math|\<mu\><rsub|\<infty\>>> is an integer corresponding
  to the multiplicity of the root at infinity, and
  <with|mode|math|\<mu\><rsub|\<infty\>>+<big|sum><rsub|i=1><rsup|s>\<mu\><rsub|i>=deg
  (p)>. Moreover, it turns out that the roots
  <with|mode|math|z<rsub|1>,\<ldots\>,z<rsub|s>> and their corresponding
  multiplicities can be computed by eigenvalues and eigenvectors computation.

  In the sequel we will generalize this approach to the case where
  <with|mode|math|p> and <with|mode|math|q> are bivariate polynomials of
  arbitrary degree. For this we first need to recall the notion of
  multiplicity in this context. Then we will show how to recover the roots
  from multiplication maps.

  <subsubsection|Intersection multiplicity>

  Let <with|mode|math|p(x,y),q(x,y)> be two coprime polynomials in
  <with|mode|math|<with|math-font|Bbb*|K>[x,y]>,
  <with|mode|math|<with|math-font|cal|C><rsub|p>> and
  <with|mode|math|<with|math-font|cal|C><rsub|g>> the corresponding algebraic
  plane curves, <with|mode|math|I:=(p,q)> the ideal they generate in
  <with|mode|math|<with|math-font|Bbb*|K>[x,y]> and
  <with|mode|math|<with|math-font|cal|A>:=<with|math-font|Bbb*|K>[x,y]/I> the
  associated quotient ring. We denote by <with|mode|math|z<rsub|1>=(x<rsub|1>,y<rsub|1>),\<ldots\>,z<rsub|s>=(x<rsub|s>,y<rsub|s>)>
  the distinct intersection points in <with|mode|math|<with|math-font|Bbb|A><rsup|2>>
  of <with|mode|math|<with|math-font|cal|C><rsub|p>> and
  <with|mode|math|<with|math-font|cal|C><rsub|q>> (i.e. the distinct roots of
  the system <with|mode|math|p(x,y)=q(x,y)=0>).

  A modern definition of the intersection multiplicity of
  <with|mode|math|<with|math-font|cal|C><rsub|p>> and
  <with|mode|math|<with|math-font|cal|C><rsub|q>> at a point
  <with|mode|math|z<rsub|i>> is (see <cite|Ful84>)

  <\equation*>
    i(z<rsub|i>,<with|math-font|cal|C><rsub|p>\<cap\><with|math-font|cal|C><rsub|q>):=dim<rsub|<with|math-font|Bbb*|K>><with|math-font|cal|A><rsub|z<rsub|i>>\<less\>\<infty\>,
  </equation*>

  where the point <with|mode|math|z<rsub|i>\<in\><with|math-font|Bbb|A><rsup|2>>
  is here abusively (but usually) identified with its corresponding prime
  ideal in <with|mode|math|<with|math-font|Bbb*|K>[x,y]>,
  <with|mode|math|<with|math-font|cal|A><rsub|z<rsub|i>>> denoting the
  ordinary localization of the ring <with|mode|math|math-font|cal|A> by this
  prime ideal. As a result, the finite <with|mode|math|math-font|Bbb*|K>-algebra
  <with|mode|math|math-font|cal|A> (which is actually finite if and only if
  <with|mode|math|p(x,y)> and <with|mode|math|q(x,y)> are coprime in
  <with|mode|math|<with|math-font|Bbb*|K>[x,y]>) can be decomposed as the
  direct sum

  <\equation*>
    <with|math-font|cal|A>=<with|math-font|cal|A><rsub|z<rsub|1>>\<oplus\><with|math-font|cal|A><rsub|z<rsub|2>>\<oplus\>\<cdots\>\<oplus\><with|math-font|cal|A><rsub|z<rsub|s>>
  </equation*>

  and consequently <with|mode|math|dim<rsub|<with|math-font|Bbb*|K>><with|math-font|cal|A>=<big|sum><rsub|i=1><rsup|s>i(z<rsub|i>,<with|math-font|cal|C><rsub|p>\<cap\><with|math-font|cal|C><rsub|q>).>

  The intersection multiplicities can be computed using a resultant. The main
  idea is to project ``algebraically'' the intersection points on the
  <with|mode|math|x>-axis. To do this let us see both polynomials
  <with|mode|math|p> and <with|mode|math|q> in <with|mode|math|A[y]> where
  <with|mode|math|A:=<with|math-font|Bbb*|K>[x]>, that is to say as
  univariate polynomials in <with|mode|math|y> with coefficients in the ring
  <with|mode|math|A> which is a domain; we can rewrite

  <\equation>
    <label|A[x]>p(x,y)=<big|sum><rsub|i=0><rsup|d<rsub|1>>a<rsub|i>(x)y<rsup|i>,<space|.2cm>q(x,y)=<big|sum><rsub|i=0><rsup|d<rsub|2>>b<rsub|i>(x)y<rsup|i>.
  </equation>

  Their resultant (with respect to <with|mode|math|y>) is an element in
  <with|mode|math|A> which is non-zero, since <with|mode|math|p> and
  <with|mode|math|q> are assumed to be coprime in <with|mode|math|A[y]>, and
  which can be factorized, assuming that <with|mode|math|a<rsub|d<rsub|1>>(x)>
  and <with|mode|math|b<rsub|d<rsub|2>>(x)> do not vanish simultaneously, as\ 

  <\equation>
    <label|resform>Res(p,q)=c<big|prod><rsub|i=1><rsup|r>(x-\<alpha\><rsub|i>)<rsup|\<beta\><rsub|i>>
  </equation>

  where the <with|mode|math|\<alpha\><rsub|i>>'s are distinct elements in
  <with|mode|math|math-font|Bbb*|K> and <with|mode|math|{\<alpha\><rsub|1>,\<ldots\>,\<alpha\><rsub|r>}={x<rsub|1>,\<ldots\>,x<rsub|s>}>
  (as sets). For instance, if all the <with|mode|math|x<rsub|i><rprime|'>s>
  are distinct then we have <with|mode|math|r=s> and
  <with|mode|math|\<alpha\><rsub|i>=x<rsub|i>> for all
  <with|mode|math|i=1,\<ldots\>,s>. Moreover, we have (see e.g. <cite|Ful84>
  (S)):

  <\proposition>
    <label|mul>For all <with|mode|math|i\<in\>{1,\<ldots\>,r}>, the integer
    <with|mode|math|\<beta\><rsub|i>> equals the sum of all the intersection
    multiplicity of <with|mode|math|<with|math-font|cal|C><rsub|p>> and
    <with|mode|math|<with|math-font|cal|C><rsub|q>> at the points
    <with|mode|math|z<rsub|j>=(x<rsub|j>,y<rsub|j>)> such that
    <with|mode|math|x<rsub|j>=\<alpha\><rsub|i>>.
  </proposition>

  As a corollary, if all the <with|mode|math|x<rsub|i>>'s are distinct (this
  can be easily obtained by a linear change of coordinates
  <with|mode|math|(x,y)>) then <with|mode|math|i(z<rsub|i>,<with|math-font|cal|C><rsub|p>\<cap\><with|math-font|cal|C><rsub|q>)>
  is nothing but the valuation of <with|mode|math|Res(p,q)> at
  <with|mode|math|x=x<rsub|i>>. Another corollary of this result is the
  well-know Bezout theorem for algebraic plane curves, which is better stated
  in the projective context. Let us denote by
  <with|mode|math|p<rsup|h>(x,y,t)> and <with|mode|math|q<rsup|h>(x,y,t)> the
  homogeneous polynomials in <with|mode|math|<with|math-font|Bbb*|K>[x,y,t]>
  obtained from <with|mode|math|p> and <with|mode|math|q> by homogenization
  with the new variable <with|mode|math|t>.

  <\proposition>
    [Bezout theorem] If <with|mode|math|p> and <with|mode|math|q> are coprime
    then the algebraic projective plane curves associated to
    <with|mode|math|p<rsup|h>(x,y,t)> and <with|mode|math|q<rsup|h>(x,y,t)>
    intersect in <with|mode|math|deg (p)deg (q)> points in
    <with|mode|math|<with|math-font|Bbb|P><rsup|2>>, counted with
    multiplicities.
  </proposition>

  <\proof>
    It follows straightforwardly from the previous proposition and the
    well-known fact that <with|mode|math|Res(p<rsup|h>,q<rsup|h>)> is a
    homogeneous polynomial in <with|mode|math|<with|math-font|Bbb*|K>[x,t]>
    of degree <with|mode|math|deg (p)deg (q)> (see e.g. <cite|Lang02>,
    <cite|VWe48II>).
  </proof>

  <subsubsection|Intersection points>

  We take again the notation of <reference|A[x]> and <reference|resform>. We
  denote by <with|mode|math|S(x)> the Sylvester matrix of
  <with|mode|math|p(x,y)> and <with|mode|math|q(x,y)> seen in
  <with|mode|math|A[y]> (assuming that both as degree at least one in
  <with|mode|math|y>); thus <with|mode|math|det (S(x))=Res(p,q)>. From
  <reference|resform>, we deduce immediately that <with|mode|math|det (S(x))>
  vanishes at a point <with|mode|math|x<rsub|0>\<in\><with|math-font|Bbb*|K>>
  if and only if either there exists <with|mode|math|y<rsub|0>> such that
  <with|mode|math|p(x<rsub|0>,y<rsub|0>)=q(x<rsub|0>,y<rsub|0>)=0>, or either
  <with|mode|math|a<rsub|d<rsub|1>>(x<rsub|0>)=b<rsub|d<rsub|2>>(x<rsub|0>)=0>
  (in which case the intersection point is at infinity). Therefore one may
  ask the following question : being given a point <with|mode|math|x<rsub|0>>
  such that <with|mode|math|det (S(x<rsub|0>))=0>, how to recover all the
  possible point <with|mode|math|y<rsub|0>> such that
  <with|mode|math|p(x<rsub|0>,y<rsub|0>)=q(x<rsub|0>,y<rsub|0>)=0>?

  Suppose given such a point <with|mode|math|x<rsub|0>> and assume that
  <with|mode|math|a<rsub|d<rsub|1>>(x<rsub|0>)> and
  <with|mode|math|b<rsub|d<rsub|2>>(x<rsub|0>)> are not both zero. If
  <with|mode|math|kerS<rsup|t>(x<rsub|0>)> is of dimension one, then it is
  easy to check that there is only one point <with|mode|math|y<rsub|0>> such
  that <with|mode|math|p(x<rsub|0>,y<rsub|0>)=q(x<rsub|0>,y<rsub|0>)=0> and
  moreover any element in <with|mode|math|ker (S(x<rsub|0>)<rsup|t>)> is a
  multiple of the vector <with|mode|math|[1,y<rsub|0>,\<cdots\>,y<rsub|0><rsup|d<rsub|1>+d<rsub|2>-1>]>.
  Therefore to compute <with|mode|math|y<rsub|0>>, we only need to compute a
  basis of <with|mode|math|ker (S(x<rsub|0>)<rsup|t>)> and compute the
  quotient of its second coordinate by its first one. In case of multiple
  roots, this construction can be generalized as follows:

  <\proposition>
    <label|x0>Let <with|mode|math|\<Lambda\><rsub|1>,\<cdots\>,\<Lambda\><rsub|d>>
    be any basis of the kernel <with|mode|math|ker S(x<rsub|0>)<rsup|t>>,
    <with|mode|math|math-font-family|bf|\<Lambda\>> be the matrix whose
    <with|mode|math|i<rsup|<with|mode|text|font-size|0.71|th>>> row is the
    vector <with|mode|math|\<Lambda\><rsub|i>>,
    <with|mode|math|\<Delta\><rsub|0>> be the
    <with|mode|math|d\<times\>d>-submatrix of <with|mode|math|\<Lambda\>>
    corresponding to the first <with|mode|math|d> columns and
    <with|mode|math|\<Delta\><rsub|1>> be the
    <with|mode|math|d\<times\>d>-submatrix of <with|mode|math|\<Lambda\>>
    corresponding to the columns number <with|mode|math|2,3,\<ldots\>,d+1>.
    Then <with|mode|math|\<lambda\>(\<Delta\><rsub|1>,\<Delta\><rsub|0>)> is
    the set of roots of the equations <with|mode|math|p(x<rsub|0>,y)=q(x<rsub|0>,y)=0>
    (i.e. the set of <with|mode|math|y>-coordinates of the intersection
    points of <with|mode|math|<with|math-font|cal|C><rsub|p>> and
    <with|mode|math|<with|math-font|cal|C><rsub|q>> above
    <with|mode|math|x=x<rsub|0>>.
  </proposition>

  In this theorem, observe that the construction of
  <with|mode|math|\<Delta\><rsub|0>> and <with|mode|math|\<Delta\><rsub|1>>
  is always possible because we assumed that both polynomials
  <with|mode|math|p> and <with|mode|math|q> depend on the variable
  <with|mode|math|y>, which implies that the number of rows in the matrix
  <with|mode|math|S(x<rsub|0>)> is always strictly greater than the
  <with|mode|math|deg (gcd (p(x<rsub|0>,y),q(x<rsub|0>,y)))>.

  The previous theorem shows how to recover the
  <with|mode|math|y>-coordinates of the roots of the system
  <with|mode|math|p=q=0> from the Sylvester matrix. However, instead of the
  Sylvester matrix we can use the Bezout matrix to compute
  <with|mode|math|Res(p,q)\<in\>A[x]> (see section <reference|sylvres>). This
  matrix have all the required properties to replace the Sylvester matrix in
  all the previous results, except in proposition <reference|x0>. Indeed, the
  Bezout matrix being smaller than the Sylvester matrix, the condition given
  in remark <reference|rem> is not always fulfilled; using Bezout matrix in
  proposition <reference|x0> requires that for all roots
  <with|mode|math|x<rsub|0>> of <with|mode|math|Res(p,q)>,

  <\equation*>
    max (deg (p(x<rsub|0>,y),deg (q(x<rsub|0>,y))\<gtr\>deg (gcd
    (p(x<rsub|0>,y),q(x<rsub|0>,y)).
  </equation*>

  This condition may failed: take for instance <with|mode|math|x<rsub|0>=-1>
  in the system

  <\equation*>
    <left|{><tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|l>|<table|<row|<cell|p(x,y)=x<rsup|2>y<rsup|2>-2y<rsup|2>+x*y-y+x+1>>|<row|<cell|q(x,y)=y+x*y>>>>><right|.>
  </equation*>

  Note that the use of the Bezout matrix gives, in practice, a faster method
  because it is a smaller matrix than the Sylvester matrix, even if its
  computation takes more time (the save in the eigen-computations is
  greater).

  <subsection|The algorithm>

  We are now ready to describe our resultant-based solver. According to
  proposition <reference|AB>, we replace the computation of the resultant,
  its zeroes, and the kernel of <with|mode|math|S<rsup|t>(x<rsub|0>)> (or the
  Bezout matrix) for each root <with|mode|math|x<rsub|0>> by the computation
  of generalized eigenvalues and eigenvectors of the associated companion
  matrices <with|mode|math|A> and <with|mode|math|B>. This computation can be
  achieved with the QZ algorithm <cite|GLVL96>. Here are some numerical
  problems which might occur:

  <\enumerate-alpha>
    <item>In order to compute the <with|mode|math|x>-coordinates of
    intersection points, it is necessary to compute the real generalized
    eigenvalues of <with|mode|math|(A,B)>. Numerically, some of these values
    can contain a nonzero imaginary part. Generally such a problem is
    encountered for multiple eigenvalues.

    <item>What does it mean by a numerical multiple
    <with|mode|math|x>-coordinate?

    <item>How do we choose the linearly independent vectors among the
    computed eigenvectors?
  </enumerate-alpha>

  For difficulty ``a'' we choose <with|mode|math|\<epsilon\>>, and any
  eigenvalue whose imaginary part is of absolute value less than
  <with|mode|math|\<epsilon\>> will be considered real.

  For difficulty ``b'', we choose possibly a different
  <with|mode|math|\<epsilon\>> and gather the points which lay in an interval
  of size <with|mode|math|\<epsilon\>>. According to the following
  proposition, the algorithm remains effective.

  <\proposition>
    <label|ra>Assume that <with|mode|math|{\<xi\><rsub|1>\<cdots\>\<xi\><rsub|n>}\<subseteq\>\<lambda\>(A,B)>
    with corresponding generalized eigenvectors
    <with|mode|math|v<rsub|1><rsup|(1)>,\<cdots\>,v<rsub|k<rsub|1>><rsup|(1)>,\<cdots\>,v<rsub|1><rsup|(n)>,>
    <with|mode|math|\<cdots\>,v<rsub|k<rsub|n>><rsup|(n)>> with
    <with|mode|math|<big|sum>k<rsub|i>\<less\>d<rsub|1>+d<rsub|2>>. We define
    <with|mode|math|math-font-family|bf|\<Lambda\>> the matrix whose its rows
    are <with|mode|math|v<rsub|1><rsup|(1)>,\<cdots\>,v<rsub|k<rsub|1>><rsup|(1)>,\<cdots\>,v<rsub|1><rsup|(n)>,\<cdots\>,v<rsub|k<rsub|n>><rsup|(n)>.>
    Then, if <with|mode|math|\<Delta\><rsub|0>> is the first left
    <with|mode|math|(k<rsub|1>+\<cdots\>+k<rsub|n>)\<times\>(k<rsub|1>+\<cdots\>+k<rsub|n>)>
    block of <with|mode|math|math-font-family|bf|\<Lambda\>> and
    <with|mode|math|\<Delta\><rsub|1>> the second
    <with|mode|math|(k<rsub|1>+\<cdots\>+k<rsub|n>)\<times\>(k<rsub|1>+\<cdots\>+k<rsub|n>)>
    block, then the generalized eigenvalues of
    <with|mode|math|\<Delta\><rsub|1>> and <with|mode|math|\<Delta\><rsub|0>>
    give the set of ordinates of the intersections above
    <with|mode|math|\<xi\><rsub|1>\<cdots\>\<xi\><rsub|n>>; i.e. this
    algorithm can be used to gather clusters of
    <with|mode|math|x>-coordinates and regarding them as the projection of
    only one point.
  </proposition>

  For difficulties ``c'', it is necessary to use the singular value
  decomposition: If <with|mode|math|x> a generalized eigenvalue of
  <with|mode|math|A> and <with|mode|math|B>, and <with|mode|math|E> the
  matrix of the associated eigenvectors, then the singular value
  decomposition of <with|mode|math|E> is

  <\equation*>
    E=U\<Sigma\>V<rsup|t>, with \<Sigma\>=<left|(><tabular*|<tformat|<cwith|1|-1|1|1|cell-halign|c>|<cwith|1|-1|2|2|cell-halign|c>|<cwith|1|-1|3|3|cell-halign|c>|<cwith|1|-1|4|4|cell-halign|c>|<cwith|1|-1|5|5|cell-halign|c>|<cwith|1|-1|6|6|cell-halign|c>|<table|<row|<cell|\<sigma\><rsub|1>>|<cell|>|<cell|>|<cell|>|<cell|>|<cell|>>|<row|<cell|>|<cell|\<ddots\>>|<cell|>|<cell|>|<cell|>|<cell|>>|<row|<cell|>|<cell|>|<cell|\<sigma\><rsub|r>>|<cell|>|<cell|>|<cell|>>|<row|<cell|>|<cell|>|<cell|>|<cell|0>|<cell|>|<cell|>>|<row|<cell|>|<cell|>|<cell|>|<cell|>|<cell|\<ddots\>>|<cell|>>|<row|<cell|>|<cell|>|<cell|>|<cell|>|<cell|>|<cell|0>>>>><right|)><space|1em>
  </equation*>

  and <with|mode|math|r> is the rank of <with|mode|math|E>. Let

  <\equation*>
    E<rprime|'>=E\<cdot\>V=U\<cdot\>\<Sigma\>=[\<sigma\><rsub|1>u<rsub|1>\<cdots\>\<sigma\><rsub|r>u<rsub|r><space|0.25em>0\<cdots\>0]=[e<rprime|'><rsub|1>\<cdots\>e<rprime|'><rsub|r><space|0.25em>0\<cdots\>0].
  </equation*>

  The matrix <with|mode|math|E<rprime|'>> is also a matrix of eigenvectors,
  because the changes are done on the columns, and V is invertible. Hence
  <with|mode|math|E<rprime|'>> and <with|mode|math|E> have the same rank
  <with|mode|math|r>, and <with|mode|math|E<rprime|''>=[e<rprime|'><rsub|1>\<cdots\>e<rprime|'><rsub|r>]>
  is a matrix of linearly independent eigenvectors. Consequently
  <with|mode|math|E<rprime|''>> can be used to build
  <with|mode|math|\<Delta\><rsub|0>> and <with|mode|math|\<Delta\><rsub|1>>.

  This leads to the following complete algorithm:

  <\algorithm>
    \ <with|font-shape|small-caps|Numerical approximation of the root of a
    square bivariate system> <vspace|-0.3cm>

    <with|font-shape|small-caps|Input:> Two bivariate polynomials
    <with|mode|math|p(x,y),q(x,y)\<in\><with|math-font|Bbb*|R>[x,y].>

    <\enumerate>
      <item><samp|Compute the Bezout matrix <with|mode|math|B(x)> of
      <with|mode|math|p> and <with|mode|math|q>, and compute the matrices
      <with|mode|math|A> and <with|mode|math|B>;>

      <item><samp|Compute the generalized eigenvalues and eigenvectors of
      <with|mode|math|(A,B)> to get the <with|mode|math|x> coordinates of the
      roots;>

      <item><samp|Eliminate the imaginary eigenvalues and the values at
      infinity, and gather the close points (with the input
      <with|mode|math|\<epsilon\>>);>

      <item><samp|For each of <samp|>all the close points represented by
      <with|mode|math|\<xi\>>, take the matrix <with|mode|math|E> of the
      associated eigenvectors. If their number
      <with|mode|math|\<geq\>d<rsub|1>+d<rsub|2>> then <with|mode|math|E>
      gives a basis of <with|mode|math|ker S<rsup|t>(\<xi\>)>;>

      <item><samp|Compute a singular value decomposition of
      <with|mode|math|E>;>

      <item><samp|Compute the rank of <with|mode|math|E> by testing the
      <with|mode|math|\<sigma\><rsub|i>> until
      <with|mode|math|\<sigma\><rsub|r+1>\<less\>\<sigma\><rsub|r>\<epsilon\>>
      is found (the rank is thus determined to be <with|mode|math|r>);>

      <item><samp|Let <with|mode|math|E<rprime|''>> be the <with|mode|math|r>
      first columns of <with|mode|math|E<rprime|'>=E\<cdot\>V>. Define
      <with|mode|math|\<Delta\><rsub|0>> as the first
      <with|mode|math|r\<times\>r> block in <with|mode|math|E<rprime|''>>,
      and <with|mode|math|\<Delta\><rsub|1>> as the second
      <with|mode|math|r\<times\>r> block in <with|mode|math|E<rprime|''>>;>

      <item><samp|Compute the generalized eigenvalues of
      <with|mode|math|(\<Delta\><rsub|1>,\<Delta\><rsub|0>)> to get the
      <with|mode|math|y> coordinates;>
    </enumerate>

    <with|font-shape|small-caps|Output:> an approximation of the real roots
    of <with|mode|math|p(x,y)=0>, <with|mode|math|q(x,y)=0>.\ 
  </algorithm>

  <subsection|Implementation>

  \;

  The function which implement this algorithm is\ 

  <\verbatim>
    template\<less\>class C\<gtr\> Seq\<less\>VectDse\<less\>C\<gtr\> \<gtr\>
    solve(p,q,EigenRes\<less\>C\<gtr\>());
  </verbatim>

  \;

  See: <verbatim|synaps/resultant/bivariate_res.h>.

  <subsection|Example>

  File: <verbatim|BivariateRes.cc><verbatim|>
</body>

<\references>
  <\collection>
    <associate|AB|<tuple|2|?>>
    <associate|A[x]|<tuple|1|?>>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.2.1|?>>
    <associate|auto-5|<tuple|1.2.2|?>>
    <associate|auto-6|<tuple|1.3|?>>
    <associate|auto-7|<tuple|1.4|?>>
    <associate|auto-8|<tuple|1.5|?>>
    <associate|eigen|<tuple|1.1|?>>
    <associate|mul|<tuple|3|?>>
    <associate|ra|<tuple|6|?>>
    <associate|resform|<tuple|2|?>>
    <associate|x0|<tuple|5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|bib>
      Ful84

      Ful84

      Lang02

      VWe48II

      GLVL96
    </associate>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Resultant-based
      methods for solving bivariate polynomial equations>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>Generalized eigenvalues and
      eigenvectors <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Intersection of planar
      curves <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|3fn>|1.2.1<space|2spc>Intersection multiplicity
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|3fn>|1.2.2<space|2spc>Intersection points
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>The algorithm
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|1.5fn>|1.4<space|2spc>Implementation
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>

      <with|par-left|<quote|1.5fn>|1.5<space|2spc>Example
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-8>>
    </associate>
  </collection>
</auxiliary>