/********************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): Ph. Tre'buchet, GALAAD, INRIA
 *   $Id: Newmac.h,v 1.2 2005/08/29 14:30:17 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_Newmac_H
#define synaps_solve_Newmac_H
//--------------------------------------------------------------------
/** \file                                                          **/
//--------------------------------------------------------------------
#include <sstream>
#include <synaps/init.h>
#include <complex>
#include <synaps/linalg/MatrDse.h>
#include <synaps/linalg/sparse.h>
#include <synaps/msolve/newmac/mynumexp.h>
#include <synaps/msolve/newmac/solve.h>

//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
/** This class specifies the multivariate solver, based on generalised 
    normal form computation. The type @T@ is the type of coefficients used 
    to compute this normal form.
*/
template<typename T, typename OUT=Seq<VectDse<std::complex<long double> > > >
struct Newmac 
{
  typedef OUT sol_t;
};

//--------------------------------------------------------------------
/**
   Solve a zero-dimensional system of polynomial equations. 
   The result is a dense matrix, where its number of rows ir the number
   of solutions, each row representing the coordinnates of a complex 
   solution. 
*/
template<typename T, typename CF_T, typename OUT>
OUT solve(const T &t, const Newmac<CF_T,OUT> & N)
{
  //typedef Monom<coeff , numexp<'x',4,long long int > > mon;
  //typedef mon Mon;
  typedef typename T::value_type Poly;
  typedef typename Poly::monom_t mon;
  typedef mon Mon;
  serveur<mon> serv;
  std::list<Poly> dummy;
  std::list<mon> stockmon;
  MatrDse<std::complex< long double> > tmp(0,0);
  std::list<sparse::rep2d<CF_T> > lmat;
  std::list<dumpstruct<pol<mon,CF_T> > > dump;
  sparse::rep2d<CF_T> m;
  Base<predicat<mon> > b;
  //cout<<"avant algo"<<endl;
  algo<T,std::list<pol<mon,CF_T> >,std::list<dumpstruct<pol<mon,CF_T> > >
    ,Base<predicat<mon> >, sparse::rep2d >(t,dump,b,serv);
#ifdef TRACE_PROG
  std::cout<<"apres algo "<<dump.size()<<std::endl;
#endif
  if (dump.size())
    {
      computestockmon(stockmon,b,dump);
      for(int i=0;i<b.nbvar();i++)
	{
	  matmul(m,i,dump,b,stockmon,serv);
	  lmat.push_back(m);
	}
      realsolve2(lmat,b,tmp);
  }
  forget(b,dump,0,dummy);
  OUT res;
  using let::assign;  assign(res,tmp);
  return res;
}
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
#endif // synaps_solve_Newmac_H

