
#include <synaps/init.h>


__BEGIN_NAMESPACE_SYNAPS

//
//classe declaree comme struct pour conserver le 
//parallele avec superlu 
//
template<typename T>
struct harewell {
  typedef T coeff_t;
  int nrow;
  int ncol;
  int nnz;
  int *rowind;
  int *colptr;
  void *nzval;
  
  harewell(){
    //Pour eviter le undefined behavior on met tous lespointeurs non initialises a NULL
    nrow=0;
    ncol=0;
    nnz=0;
    rowind=NULL;
    colptr=NULL;
    nzval=NULL;
    //parce que sans ca ca n'est pas ds le std
  };
  harewell(const int nrow, const int ncol);
  harewell(const int nrow, const int ncol, const int nz);
  harewell(const int nrow, const int ncol, const int nz,
	  T* coeffs, const int *idx);
  
  ~harewell();
  
  T & operator() (const int & i,const int & j,const T &d) ;
  T & operator() (const int &i,const int &j);
  T  operator() (const int &i,const int &j)const ;
  inline void destroystore();
};
  
template<typename T> 
harewell<T>::harewell(const int nrow,const int ncol)
{
  //Pour eviter le undefined behavior on met tous lespointeurs non initialises a NULL
  this->nrow=nrow;
  this->ncol=ncol;
  this->rowind=(int*)malloc(sizeof(int)*(this->ncol));
  this->colptr=(int*)malloc(sizeof(int)*(1+this->ncol));
  this->nzval=malloc(sizeof(T)*(this->ncol));
  this->nnz=this->ncol;
  for(int i=0;i<this->ncol;i++)
    {
      this->rowind[i]=i;
      this->colptr[i]=i;
      ((T*)this->nzval)[i]=(T)0;
    }
  this->colptr[this->ncol]=this->nnz;
}

template<>
harewell<Scl<MPQ> >::harewell(const int nrow,const int ncol)
{
  //Pour eviter le undefined behavior on met tous lespointeurs non initialises a NULL
  this->nrow=nrow;
  this->ncol=ncol;
  this->rowind=(int*)malloc(sizeof(int)*(this->ncol));
  this->colptr=(int*)malloc(sizeof(int)*(1+this->ncol));
  this->nzval=MAC_REV_MALLOC<Scl<MPQ> >(sizeof(Scl<MPQ>)*(this->ncol));
  this->nnz=this->ncol;
  for(int i=0;i<this->ncol;i++)
    {
      this->rowind[i]=i;
      this->colptr[i]=i;
      //    ((T*)this->nzval)[i]=0;
    }
  this->colptr[this->ncol]=this->nnz;
}

template<>
harewell<Scl<MPF> >::harewell(const int nrow,const int ncol)
{
  //Pour eviter le undefined behavior on met tous lespointeurs non initialises a NULL
  this->nrow=nrow;
  this->ncol=ncol;
  this->rowind=(int*)malloc(sizeof(int)*(this->ncol));
  this->colptr=(int*)malloc(sizeof(int)*(1+this->ncol));
  this->nzval=MAC_REV_MALLOC<Scl<MPF> >(sizeof(Scl<MPF>)*(this->ncol));
  this->nnz=this->ncol;
  for(int i=0;i<this->ncol;i++)
    {
      this->rowind[i]=i;
      this->colptr[i]=i;
      //    ((T*)this->nzval)[i]=0;
    }
  this->colptr[this->ncol]=this->nnz;
}

template<typename T> 
inline harewell<T>::~harewell()
{
  //destroystore();
}

template<typename T> 
inline void harewell<T>::destroystore()
{
  free(rowind);
  free(colptr);
  free(nzval);
}


inline void harewell<Scl<MPF> >::destroystore()
{
  free(rowind);
  free(colptr);
  for(int i=0;i<nnz;i++) mpf_clear(&((Scl<MPF>*)nzval)[i].rep());
  free(nzval);
}
template<> 
inline void harewell<Scl<MPQ> >::destroystore()
{
  free(rowind);
  free(colptr);
  for(int i=0;i<nnz;i++) mpq_clear(&((Scl<MPQ>*)nzval)[i].rep());
  free(nzval);
}
//  template<typename poly,typename mon,typename T> 
//  harewell<T> & operator|(list<poly> P , list<mon> M);

template<typename T>
T & harewell<T>::operator() (const int  &i,const int &j) 
{
  static T d=T(0);
  int colbegin=colptr[j]
    ,colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t;
  //for(t=colbegin;(t<colend)&&(i>rowind[t]);t++) ;
  //le 17/07/2001 idee pouvoir permutter les ligne plus facilement 
  for(t=colbegin;(t<colend)&&(i!=rowind[t]);t++);
  if (t>=colend) return d;
  return ((i==(rowind[t]))?(((T*)(nzval))[t]):d);
}

template<typename T>
T  harewell<T>::operator() (const int  &i,const int &j) const
{
  static T d=(T)0;
  int colbegin=colptr[j]
    ,colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t;
  //for(t=colbegin;(t<colend)&&(i>rowind[t]);t++) ;
  //le 17/07/2001 idee pouvoir permutter les ligne plus facilement
  for(t=colbegin;(t<colend)&&(i!=rowind[t]);t++);
  if (t>=colend) return d;
  return ((i==(rowind[t]))?(((T*)(nzval))[t]):d);
}
// operateur () d'ecriture il suppose que si on ecrit un nul ca fait rien
template<typename T>
T  &harewell<T>::operator() (const int &i,const int &j,const T &d)
{
  static T zero=(T)0;
  int colbegin=colptr[j],colend=(j+1<this->ncol)?colptr[j+1]:nnz;
  int t,carry=0;;
  //for(t=colbegin;(t<colend)&&(i>(rowind[t]));t++);
  //le 17/07/2001 idees tjrs la meme
  for(t=colbegin;(t<colend)&&(i!=(rowind[t]));t++);
  if (t>=colend) {t--;carry=1;};
  //cout<<"je suis dedans t "<<t<<" i "<<i<<" rowind[t] "<<rowind[t]<<endl;
  if (rowind[t]==i) // je suis tombe sur un coeff qui etait deja pas nul
    {
      if (d!=(T)0)//super on ne vire rien on change un valeur!!!!
	{
	  return ((T *)(nzval))[t]=d;
	}
      else//aie on vire un truc
	{
	  if ((t==colbegin)&&(colend==colbegin+1))
	    return ((T *)(nzval))[t]=d;//on ne cree pas de colonnes de zeros!!!
	  else
	    {
	      //ici avant   S->nnz--;
	      //if (t==colbegin);
	      //   S->colptr[j]=t+1;//S->rowind[t+1];
	      //on ne cree jamais de colonnes de 0
	      //le formatNC ne les gerent pas!!!
	      CLEAR_ELEM<T>((T*)nzval+t,sizeof(T));
	      memmove(((T*)(nzval)+t),((T*)(nzval))+t+1,(nnz-t-1)*sizeof(T));
	      memmove(((int*)(rowind)+t),((int*)(rowind))+t+1
		      ,(nnz-t-1)*sizeof(int));
	      //for(int k=t;k<nnz-1;k++)
	      //	{
	      //	  ((T*)(nzval))[k]= ((T*)(nzval))[k+1];
	      //	  rowind[k]=rowind[k+1];
	      //	}
	      //cout<<"je passe ici"<<endl;
	      //cout<<"valeur de d"<<d<<endl;
	      nnz--;
	      //reglage des colptr
	      for(int k=j+1;k<this->ncol;k++)
		colptr[k]--;
	      //reglage des nnz
	      colptr[this->ncol]=nnz;
	      //  cout<<"sortie par la"<<endl;
	      return zero;
	    }
	}
    }
    //on fait un petit memory "leak" mais ca evite de reallouer
  
  else//il faut creer la place pour le nouveau coeff
    // il n'y a pas l'equivalent realloc en C++ !!!!!
    if (d==(T)0) {return zero;}
    else
      {
	nnz++;
	//	int k;
	t+=carry;
	//cout<<"je rajoute un truc nnz"<<nnz<<endl;
	//
	//les champs dynamiques sont supposes alloues par malloc
	//et pas par new!!!!!! c pour connexion avec super lu 
	// qui bricole sa memoire avec malloc
	//
	T* tmpnzval=(T*)malloc(sizeof(T)*(nnz));
	//cout<<"le nzval "<<endl;
	int *tmprowind=(int*)malloc(sizeof(int)*nnz);
	//cout<<" t "<<t<<" nnz "<<nnz<<endl;
	memcpy(((T*)(tmpnzval)),((T*)(nzval)),(t)*sizeof(T));
	memcpy(tmprowind,(rowind),(t)*sizeof(int));
	//for (int kk=0;kk<t;kk++)
	//  {
	//   tmpnzval[kk]=((T*)(nzval))[kk];
	//  tmprowind[kk]=rowind[kk];
	// }
	INIT_ELEM<T>(&tmpnzval[t],sizeof(T));
	tmpnzval[t]=d;
	tmprowind[t]=i;
	if (t==colbegin) colptr[j]=t;//on met a jour pour la nvlle valeur
	for (int i=j+1;i<this->ncol;i++)
	  colptr[i]++;//il faut decaler les indices de 1
	memcpy(((T*)(tmpnzval))+t+1,((T*)(nzval))+t,(nnz-t-1)*sizeof(T));
	memcpy(tmprowind+t+1,rowind+t,(nnz-t-1)*sizeof(int));
	//for (k=t;k<nnz-1;k++)
	//{
	//  tmpnzval[k+1]=((T*)(nzval))[k];
	//  tmprowind[k+1]=rowind[k];
	//};
	free((T*)(nzval));
	nzval=(void*)tmpnzval;
	free(rowind);
	rowind=tmprowind;
	colptr[this->ncol]=nnz;//le colptr[ncol] c nnz recopie pour 
	// usage interne a la lib apres.
	return (((T *)(nzval))[t]);
      }
}

template<typename typmat,typename coeff>
void allocstore(typmat &m,int nrow,int ncol,coeff val)
{
  
  m.ncol=ncol;
  m.nrow=nrow;
  m.rowind=(int*)malloc(sizeof(int)*(m.ncol));
  m.colptr=(int*)malloc(sizeof(int)*(1+m.ncol));
  m.nzval=malloc(sizeof(coeff)*(m.ncol));
  m.nnz=m.ncol;
  for (int i=0;i<m.ncol;i++)
     {
      m.rowind[i]=i;
      m.colptr[i]=i;
      ((coeff*)m.nzval)[i]=val;
    }
  m.colptr[m.ncol]=m.nnz;
  return;
}

namespace matrixof {
	template<typename T>
		void build(harewell<T> & A, int nl, 
				int nc, int nz)
		{
			A.nzval=(void *)malloc(sizeof(typename harewell<T>::coeff_t)*nz);
			A.colind=(int*)malloc(sizeof(int)*nz);
			A.rowptr=(int*)malloc(sizeof(int)*nl);
		}
}






__END_NAMESPACE_SYNAPS
