/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): F.Carreras, B.Mourrain, O.Ruatta, C.Tanasescu
 *   $Id: bivariate_sth.H,v 1.2 2005/09/28 06:30:39 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_bivariate_sth_H_
#define synaps_solve_bivariate_sth_H_
/********************************************************************/
#include <cstdio>
#include <synaps/init.H>
#include <synaps/linalg/array1d.H>
#include <synaps/upol/UPolDse.H>
//#include <synaps/linalg.H>
#include <synaps/mpol.H>
#include <synaps/arithm/let.H>
#include <synaps/arithm/RR.H>
#include <synaps/resultant/UniReslt.H>
#include <synaps/resultant/SturmSeq.H>
#include <synaps/upol/SquareFree.H>
#include <synaps/solve/Aberth.H>
#include <synaps/solve/bezier/Std.H>
#include <synaps/solve/method_name.H>

__BEGIN_NAMESPACE_SYNAPS

namespace let {

	template<class C>
		UPolDse<double> convert(const UPolDse<C> & P, AsType<double> mth)
		{
			UPolDse<double> tmp(Degree(P)+1, AsSize());
			for(int k=0; k< tmp.size();++k)
				let::assign(tmp[k],P[k]);
			return tmp;
		}

}
template<class C>
UPolDse<double> eval_to_double(const UPolDse< UPolDse<C> > & P, double x)
{
	UPolDse<double> tmp(Degree(P)+1, AsSize());
	for(int k=0; k< tmp.size();++k)
		let::assign(tmp[k],P[k](x));
	return tmp;
}

template<class C>
Seq<Seq<C> > convertXToXMinusY(int n)
{
	Seq<C> lista, lista1;
	Seq<Seq<C> > listaFin; 

	if (n==0) 
	{
		lista.push_back(1); 
		//    cout << "Lista " << lista << endl;
		listaFin.push_back(lista);
	}
	else
	{
		if (n==1)
		{
			lista.push_back(1); 
			listaFin.push_back(lista);
			lista.push_back(-1);
			//      cout << "Lista " << lista << endl;
			listaFin.push_back(lista);
		}
		else
		{
			listaFin=convertXToXMinusY<C>(n-1);
			lista1=listaFin[listaFin.size()-1];
			lista.push_back(lista1[0]);
			for (int i=0; i<n-1; ++i)
				lista.push_back(-lista1[i] + lista1[i+1]);
			lista.push_back(-lista1[n-1]);
			//    cout << "Lista " << lista << endl;
			listaFin.push_back(lista);
		}
	}
	//  cout << "LFin " << listaFin << endl;
	return listaFin;
}

template<class C>
UPolDse<UPolDse<C> > changeOfVariable(const UPolDse<UPolDse<C> >& p)
{
	int maxDegX=0; int newDegY=0;
	for (int i=0; i<=Degree(p); ++i)
	{
		int d=UPOLDAR::degree(p[i]);
		if (maxDegX<d) {maxDegX=d;}
		if (newDegY<i+d) {newDegY=i+d;}
	}

	//  cout << "deg " << p<<" "<<maxDegX << "  " << newDegY << endl;

	Seq<Seq<C> > lista=convertXToXMinusY<C>(maxDegX);

	//  cout << "Seq " << lista << endl;

	UPolDse<UPolDse<C> > newP (newDegY+1,AsSize());
	for (int i=0; i<=newDegY; ++i)
	{
		UPolDse<C> inter (maxDegX+1,AsSize());
		newP[i]=inter;
	}
	for (int i=0; i<=UPOLDAR::degree(p); ++i)
	{
		for (int j=0; j<=UPOLDAR::degree(p[i]); ++j)
		{
			for (int k=0; k<=j; ++k)
				newP[i+k][j-k]+=p[i][j]*lista[j][k];
		}
	}
	//  cout <<"newP "<<newP<<" "<<UPOLDAR::degree(newP)<<endl;
	for (int i=0; i<=UPOLDAR::degree(newP); ++i)
	{
		UPOLDAR::checkdegree(newP[i]);
		int j=UPOLDAR::degree(newP[i]);
		bool ok=true;
		while (j>=0 && ok)
		{
			if (newP[i][j]!=0) ok=false; else j--;
		}
		//    UPolDse<C> temp;
		if (ok) // in this case newP[i] (which is a polynomial in x) is identically zero
			//temp.resize(0);
			newP[i].resize(0);
		else // in this case newP[i] is not of degree newP[i].size()-1 but of degree j-1
		{
			newP[i].resize(j+1);
			//temp.resize(j+1);
			//      for (int k=0; k<j+1; ++k)	temp[k]=newP[i][k];
		}
		//    cout <<"temp "<<temp<<endl;
		//    newP[i]=temp;
	}

	UPOLDAR::checkdegree(newP);
	//   UPolDse<UPolDse<C> > newPP;
	//   int i=UPOLDAR::degree(newP);
	//   bool ok=true;
	//   while (i>=0 and ok)
	//   {
	//     if (Degree(newP[i])>-1) ok=false; else i--;
	//   }
	//   if (ok) // in this case newP (which is a polynomial in y, whose coefficients are polynomials in x) is identically zero
	//     newPP.resize(0);
	//   else // in this case newP is not of degree newP.size()-1 but of degree i-1
	//   {
	//     newPP.resize(i);
	//     for (int k=0; k<i; ++k)
	//       newPP[k]=newP[k];
	//   }

	return newP;
}

template<class C>
UPolDse<UPolDse<C> > convert_to_upol_upol(const MPol<C> & p)
{
	int d= Degree(p,1);
	UPolDse<UPolDse<C> > u(d+1,AsSize());
	for(typename MPol<C>::const_iterator it=p.begin(); it !=p.end(); ++it)
		u[(*it)[1]] += UPolDse<C>(it->coeff(), (*it)[0]);
	return u;
}

#ifdef SYNAPS_WITH_MPSOLVE
struct Aberth_RR
{
	typedef Seq<RR> sol_t;
	typedef HABICHT sturmseq_t;

	template<class POL>  static 
		sol_t solve(const POL& p)
		{
			return solve(p,Aberth<RR>(10));
		}
};
#else
struct Aberth_RR
{
	typedef Seq<double> sol_t;
	typedef HABICHT sturmseq_t;

	template<class POL>  static 
		sol_t solve(const POL& p)
		{
			return solve(p,Bezier<double>());
		}
};
#endif //SYNAPS_WITH_GMP

template<class C, class SLV>
Seq<VectDse<double> > solve(const MPol<C> & f, const  MPol<C> & g, 
		const Habicht<SLV> & mth)
{

  typedef UPolDse<C> upol_t;
  typedef std::vector< UPolDse<C> > vct_upol_t;
  typedef typename SLV::sturmseq_t sturmseq_t;
  
  VectDse<double> pt(2,AsSize());
  Seq<VectDse<double> > sol;
  
  UPolDse<UPolDse<C> > p,q;
  if(Degree(f,1) < Degree(g,1))
    {
		p=convert_to_upol_upol(g);
		q=convert_to_upol_upol(f);
    }
  else
    {
      p=convert_to_upol_upol(f);
		q=convert_to_upol_upol(g);
    }
  Variable< UPolDse<UPolDse<C> > >()="y";
  
  UPolDse<C> qq=p[Degree(p)];
  //  cout <<p<<"\n"<<qq<<" "<<qq.size()<<endl;
  while (qq.size()>1)
    {
      //std::cout << "Change of variable occured" << std::endl;
      p=changeOfVariable(p);
      q=changeOfVariable(q);
      qq=p[Degree(p)];
    }
	
  SturmSeq< UPolDse<C> > s(p,q,sturmseq_t());
  //  cout <<"\nSturm "<< s<<endl;
  
  vct_upol_t r=CoeffStHa<vct_upol_t>(s);

  //std::cout << "psc" << std::endl;
  //for (int j=0; j< r.size(); ++j)  cout <<"psc" << j << ": " << r[j] <<endl;
  
  upol_t disc=SquareFree(r[r.size()-1]);
  //cout << "disc: "<< disc <<endl;
  
  vct_upol_t Desc;
  upol_t  Fi = disc;
  upol_t  Fin;
  for (int j=0; j < r.size()-1; ++j)
    {
      Fin=PGcd(Fi,r[r.size()-j-2]);
      Desc.push_back(Fi/Fin);
      Fi=Fin;
    }
  //std::cout << "---  Desc[i]" << std::endl;
  //for (int j=0; j<Desc.size(); ++j)  cout <<"Desc " << j << " " << Desc[j] << endl;
  //cout << "------" << endl;
  
  using let::assign;
  using let::convert;
  
  for (int j=0; j<Desc.size(); ++j)
    {
      if (Degree(Desc[j])>0)
	{
	  // std::cout << "start---------------------------------------\n";
	  //     std::cout << "Decomposition " << j <<" "<<Desc[j]<<endl;
	  //      UPolDse<double> tmp;
	  //      let::assign(tmp,Desc[j]);
	  
	  typename SLV::sol_t SolX = SLV::solve(Desc[j]);
	  //      cout <<"X: "<<SolX<<endl;
	  //Seq<RR> SolX = solve(tmp, AberthReal());
	  //Seq<RR> SolX = solve(tmp, Bezier<double,Opt>(1.e-10));
	  
	  UPolDse< UPolDse<C> > tp = s[s.size()-2-j];
	  
	  //cout << "tp " << tp << ", dg: " << Degree(tp) << endl;
	  //cout << "StHa: " << s[ s.size() - j - 2] << endl;
	  
	  if ((Degree(tp) >= j) && (j < s.size()-3) )
	    for(int k=0; k < SolX.size(); ++k)
	      {
		assign(pt[0],SolX[k]);
		pt[1] = -convert(tp[j],AsType<double>())(pt[0])/
		  convert(tp[j+1],AsType<double>())(pt[0])/(j+1);
		sol.push_back(pt);
	      }
	  else
	    {
	      for(int k=0; k< SolX.size();++k)
		{
		  std::cout <<" Non generic situation"<<std::endl;
		  assign(pt[0],SolX[k]);
		  unsigned u= s.size()-2-j;
		  while(u>=0 && s[u]==0) u--;
		  UPolDse<double> tmp1 = eval_to_double(s[u],pt[0]);
		  //    cout << "NG case " << j <<": "
		  //		  << "x-val: " << pt[0] << ", "
		  //		  <<s[u]<<" --- "<<tmp1<<endl;
		  Seq<double> sy = solve(tmp1,Bezier<double>());
		  //    cout<<"Y:"<<sy<<endl;
		  for(unsigned l=0;l<sy.size();l++)
		    {
		      pt[1]=sy[l];
		      sol.push_back(pt);
		    }
		}
	    }
	  //cout << "end---------------------------------------\n";
	}
    }
  //      cout << " X coordinates of the roots " << solx << endl;
  //      cout << " Y Coordinates of the roots " << soly << endl;
  return sol;
}

//--------------------------------------------------------------------
template<class C>
Seq<VectDse<double> > solve(const MPol<C> & f, const  MPol<C> & g,
		const HABICHT & mth)
{
	return solve(f,g,Habicht<Aberth_RR>());
}
/********************************************************************/
__END_NAMESPACE_SYNAPS
#endif //synaps_solve_bivariate_sth_H_
