/********************************************************************
 *   This file is part of the source code of the SYNAPS library.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: binomials.hpp,v 1.1 2005/07/11 10:03:55 jppavone Exp $
 ********************************************************************/
#ifndef SYNAPS_SOLVE_SBDSLV_BERNSTEIN_BINOMIALS_H
#define SYNAPS_SOLVE_SBDSLV_BERNSTEIN_BINOMIALS_H
//--------------------------------------------------------------------
#include <vector>
#include <assert.h>
#include <synaps/init.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
namespace bernstein
{
  template<typename T>
  struct binomials
  {
    static binomials _default_;
    std::vector< T * > m_data;
    binomials( const binomials& );
    static void pwsum( T * dst, T * src, int sz ) { for(int i=0;i<sz-1;dst[i]=src[i]+src[i+1],i++); };
    binomials( int dmax = 100 ) 
    {
      m_data.push_back( new T[1] );
      m_data[0][0] = 1;
    };
    ~binomials() { 
      for ( unsigned i = 0; i < m_data.size(); i ++ ) 
	delete[] m_data[i]; 

    };
    inline T * get( unsigned i ) 
    {
      if ( m_data.size() > i ) return m_data[i];
      for ( unsigned j = m_data.size(); j <= i; j ++ )
	{
	  m_data.push_back( new T[j+1] );
	  pwsum(m_data[j]+1,m_data[j-1],j);
	  m_data[j][0] = 1;
	  m_data[j][j] = 1;
	};
      return m_data[i];
    };
    const T& binomial( unsigned n, unsigned i )
    { return get(n)[i]; };
  };
  template<typename T>
  binomials<T> binomials<T>::_default_;
  template<typename T>
  T binomial( int n, int i ) { return binomials<T>::_default_.binomial(n,i); };
};
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
/********************************************************************/
#endif //
