#ifndef SYNAPS_SHAPE_MPOLDSE_RATIONAL_CURVE_H
#define SYNAPS_SHAPE_MPOLDSE_RATIONAL_CURVE_H

#include <synaps/mpol/MPolDse.h>
#include <synaps/shape/wrappers.h>
#include <synaps/shape/interfaces/IRationalCurve.h>

__BEGIN_NAMESPACE_SYNAPS

template<class C, int N, class R > struct RationalCurve;
template<class X> struct interfaceof;
template<class C, int N, class R> struct interfaceof< RationalCurve<C,N,R> > 
{ 
  typedef IRationalCurve T; 
};


template<class K, class C, int N, class R>
void range( K & a, K & b, const RationalCurve<C,N,R> & x )
{
  a = x.m_dmn[0];
  b = x.m_dmn[1];
};
template< class C, int N, class R, class S>
void setRange( RationalCurve<C,N,R> & c, const S & a, const S & b )
{
  using let::assign;
  assign(c.m_dmn[0],a);
  assign(c.m_dmn[1],b);
};
template<class C, int N, class R, class Point >
void eval( Point& value, const RationalCurve<C,N,R> & crv, const typename Point::value_type & t )
{
  typename Point::value_type w;
  eval(w,crv[0],&t);
  for ( int i = 1; i <= N; eval( value[i-1], crv[i], &t ), value[i-1] /= w, i ++  );
};

template<class C, int N, class R, class Point, class Prm >
void eval( Point& value, Point& dvalue, const RationalCurve<C,N,R>& crv, const Prm& t )
{
  typename Point::value_type w, wt;
  MPolDse<C> dp;
  eval(w,crv[0],&t);
  diff(dp,crv[0],0);
  eval(wt,dp,&t);
  for ( int i = 1; i <= N; i ++ )
    {
      typename Point::value_type ft;
      diff( dp, crv[i], 0 );
      eval( value[i-1], crv[i], &t ); 
      eval( ft, dp, &t );
      dvalue[i-1] = (ft*w-wt*value[i-1])/(w*w);
    };
};

template<class C, class R>
void setEquations( RationalCurve<C,3,R>  & c, const char * w, const char * x, const char * y, const char * z )
{
  c.m_coords[0] = w;
  c.m_coords[1] = x;
  c.m_coords[2] = y;
  c.m_coords[3] = z;
};

template<class C, int N = 3, class R= MPolDse<C> >
struct RationalCurve : wrapper<RationalCurve<C,N,R> ,IRationalCurve>
{
  typedef IRationalCurve base_t;
  typedef RationalCurve<C,N,R> self_t;
  typedef self_t rep_t;
  typedef C          coeff_t;
  typedef MPolDse<C> polynomial_t;
  C m_dmn[2];
  polynomial_t  m_coords[N+1];
  polynomial_t& operator[]( int i ) { return m_coords[i]; };
  const polynomial_t& operator[]( int i ) const { return m_coords[i]; };
  RationalCurve() { m_dmn[0] = 0; m_dmn[1] = 1; };

  RationalCurve( const char * w, const char * x, const char * y, const char * z )
  {
    m_dmn[0] = 0; m_dmn[1] = 1;
    this->setEquations(w,x,y,z);
  }
  void setEquations( const char * w, const char * x, const char * y, const char * z )
  {
    m_coords[0] = w;
    m_coords[1] = x;
    m_coords[2] = y;
    m_coords[3] = z;
  };

};

/**
   The \c operator \c << .\n
   This operator allows us to display a rational curve.
**/
template<class C, int N, class R>
std::ostream & operator<< (std::ostream & os, 
			   const RationalCurve<C,N,R> & s)
{
  os<<"RationalCurve("<<s[0];
  for (unsigned i=1; i<N+1;++i) os<<","<<s[i];
  os <<", ["<<s.m_dmn[0]<<","<<s.m_dmn[1]<<"]";
  os <<")";
  return os;
}


__END_NAMESPACE_SYNAPS

#include <synaps/shape/wrappers/WRationalCurve.h> 
#endif
