#ifndef SYNAPS_SHAPE_FPARAMETRICCURVE_H
#define SYNAPS_SHAPE_FPARAMETRICCURVE_H

#include <synaps/shape/interfaces/IParametricCurve.h>
#include <synaps/shape/functions/FParametric.h>
#include <synaps/shape/functions/SubParametricCurve.h>

__BEGIN_NAMESPACE_SYNAPS


template<class Scalar, class X, class R> inline
void domain( Scalar & tmin, Scalar & tmax, const X& x, const R & r ) 
{ 
  domain(tmin,tmax,(const typename X::base_t&) r,r); 
};

template<class Scalar, class R> inline
void domain( Scalar & tmin, Scalar & tmax, const R & r ) 
{ 
  domain(tmin,tmax,(const typename R::base_t&) r,r); 
};

template<class Scalar, class R> inline
void domain( Scalar & tmin, Scalar & tmax, const IParametricCurve&, const R & c ) 
{ 
  tmin = Scalar(0); 
  tmax = Scalar(1); 
};

template<class Point, class R> inline
void eval( Point & p, const IParametricCurve &, const R & c, const typename Point::value_type & t )
{ eval(&p,c,&t,1); };

template<class Point, class X, class Curve> inline
void eval( Point & p, const X & x, const Curve & c, const typename Point::value_type & t )
{
  eval( p, (const typename X::base_t&)x, c, t);
};

template<class Point, class Curve> inline
void eval( Point & p, const Curve & c, const typename Point::value_type & t )
{
  eval(p,(const typename Curve::base_t&)c,c,t);
};

template<class Point, class R> inline
void evald( Point& dp, const IParametricCurve& , const R & c, const typename Point::value_type & t )
{
  typedef typename Point::value_type C;
  Point p;
  C a,b;
  domain(a,b,c);
  C h = (b-a)/C(1000);
  a = t - h;
  b = t + h;
  eval(p,c,a);
  eval(dp,c,b);
  for ( int i = 0; i < Point::_dimension_ ; i ++ )
    dp[i] = (dp[i]- p[i])/(C(2)*h);
};

template<class Point, class X, class R> inline
void evald( Point & dp, const X & x, const R & c, const typename Point::value_type &t )
{
  evald(dp,(const typename X::base_t&)x,c,t);
};

template<class Point, class R> inline
void evald( Point & dp, const R & c, const typename Point::value_type & t )
{
  evald(dp,c,c,t);
};

template<class Point, class R> inline
void eval( Point & p, Point & dp, const IParametricCurve&, const R & c, const typename Point::value_type & t )
{
  typedef typename Point::value_type C;
  C a,b;
  domain(a,b,c);
  C h = (b-a)/C(1000);
  a = t - h;
  b = t + h;
  eval(p,c,a);
  eval(dp,c,b);
  for ( int i = 0; i < Point::_dimension_ ; i ++ )
    dp[i] = (dp[i]- p[i])/(C(2)*h);
  eval(p,c,t);
};

template<class Point, class R> inline
void eval( Point & p, Point & dp, const R & c, const typename Point::value_type & t )
{ eval(p,dp,c,c,t); };

template<class R, class Point> inline
void eval( Point * smp, const IParametricCurve &, const R & c, const typename Point::value_type * t, int n )
{ for ( Point * o = smp; o != smp + n; eval(*o++,c,*t++) ); };

//template<class Point, class Curve> inline
//void eval( Point * smp, const Curve& c, const typename Point::value_type * t, int n )
//{ eval( smp, c, c, t, n ); };

template<class R, class Scalar> inline
void tsample( Scalar * t, const IParametricCurve&, const R & c, int n )
{
  Scalar a,b;
  domain(a,b,c);
  tsample(t,a,b,n);
};

template<class Curve, class Scalar> inline  
void tsample( Scalar * t, const Curve & c, int n ) { tsample( t, c, c, n ); };
template<class Point, class Scalar, class R> inline
void sample( Point * p, Scalar * s, const IParametricCurve & , const R & c, int n )
{
  tsample(s,c,n);
  eval(p,c,s,n);
};

template<class Point, class Scalar, class Curve> inline
void sample( Point *  p, Scalar * s, const Curve & c, int n )
{
  sample(p,s,c,c,n);
  //for ( int i = 0; i< n; i ++ ) std::cout << p[i] << std::endl;
};

template<class R, class C> inline
void split( IParametricCurve *& left,
	    IParametricCurve *& right,
	    const IParametricCurve &,
	    const R & c,
	    const C & t )
{
  C tmin,tmax;
  domain(tmin,tmax,c);
  assert(t>=tmin && t<=tmax);
  left  = subcurve(c,tmin,t);
  right = subcurve(c,t,tmax);
};

template<class R, class C> inline
void split( IParametricCurve *& left, 
	    IParametricCurve *& right, 
	    const R & c, 
	    const C & t )
{
  split(left,right,c,c,t);
};

template<class F, class C>
C simpson_rule(const F & f, const C&  a, const C&  b, int n )
{
  C h = (b - a)/(2 * n);
  C sum_even, sum_odd;
  sum_even = 0;
  sum_odd = 0;
  for(int i = 1; i <= n; i++ ) sum_odd = sum_odd  + f(a + (2 * i - 1) * h);
  for(int j = 1; j <  n; j++ ) sum_even = sum_even + f(a + 2 * j * h);
  return ((f(a) + f(b) + 4 * sum_odd + 2 * sum_even) * h / 3);
};


template<class F, class C>
C simpson_int( const F & f, const C & a, const C & b, const C & eps  )
{
  using std::abs;
  int n = 16;
  C prev =  simpson_rule(f,a, b, 8);
  C curr =  simpson_rule(f,a, b, n);
  while( abs(curr - prev) > eps)
    {
      n *= 2;
      prev = curr;
      curr = simpson_rule(f,a, b, n);
    };
  return curr;
};

template<class R, class C>
struct parametric_curve_length_function
{
  const R & rep;
  C operator()( const C & x ) const 
  { 
    fxv<C,3> p;
    evald(p,rep,x);
    C s = p*p;
    return sqrt(s);
  };
  parametric_curve_length_function( const R & r ) : rep(r){};
};

template<class R, class C> inline
C length( const IParametricCurve & , const R & c, const C & eps )
{
  C a,b;
  domain(a,b,c);
  parametric_curve_length_function<R,C> f(c);
  return simpson_int( f, a, b, eps );
};


template<class X, class R, class C> inline
C length( const X & x, const R& r, const C& eps )
{
  return length((const typename X::base_t&)x,r,eps);
};

template<class R, class C> inline
C length( const R & c, const C & eps )
{
  return length((const typename R::base_t&)c,c,eps);
};

template<class R> inline
IGLGood * mesh( const IParametricCurve& , const R & c )
{
  return compute_mesh(c);  
};

__END_NAMESPACE_SYNAPS

#endif
