#ifndef SYNAPS_SHAPE_GAIA_LSEGMENT_
#define LSEGMENT_H
#include "fsvector.h"
#include "igraph.h"
#include "vcode.h"

  template<class FunctionType>
  struct lsegment_t
  {
    typedef typename FunctionType::point_t Point;
    typedef typename FunctionType::number_t number;
    typedef short              rid_t;  
    typedef unsigned short     coord_t;
    typedef igraph   graph_t; 
    typedef fsvector<number,3> point3;
    FunctionType * Fct;
    Point *  src;   coord_t  w,h;
    number m_du,m_dv;
    
    number uvvalue( coord_t i  ) { return i*m_du + Fct->umin(); };
    number vvalue ( coord_t i  ) { return i*m_dv + Fct->vmin(); };
    
    void DiffUEval( point3& p, const number& u, const number& v ) 
    {
      point3 a,b;
      Fct->Eval(a,u+1.0/104856.0,v);
      Fct->Eval(b,u-1.0/104856.0,v);
      for ( unsigned d = 0; d < 3; d++ )
	p[d] = 52428*(a[d]-b[d]);
    };
    
    void DiffVEval( point3& p, const number& u, const number& v ) 
    {
      point3 a,b;
      Fct->Eval(a,u,v+1.0/104856.0);
      Fct->Eval(b,u,v-1.0/104856.0);
      for ( unsigned d = 0; d < 3; d++ )
	p[d] = 52428*(a[d]-b[d]);
    };
    
    struct mark_t
    {
      rid_t     head;
      coord_t   j;
      vcode_t code;
      mark_t  * next;
      coord_t a() { return j;             };
      coord_t b() { return (this+1)->a(); };
    };
      
    mark_t   *  marks;  
    unsigned _size;  
    unsigned *  lines;
    
    unsigned nmarks  () const { return _size; };
    void pushm( unsigned code, unsigned j )
    { 
      marks[_size].code = code;
      marks[_size].j    = j;
      marks[_size].head =-1;
      marks[_size].next = 0;
      _size++;
    };
  
    void lines_changes()
    {
      int i,j;
      lines = (unsigned*)(malloc(sizeof(unsigned)*nrows()));
      unsigned max_marks = w*h;
      while ( !(marks = (mark_t*)(malloc(sizeof(mark_t)*max_marks))) ) max_marks >>= 1;
      Point * ptr  = source();
      for ( i = 0; i < h-1; i++ )
	{
	  lines[i] = nmarks();
	  vcode_t code = vcode(*ptr,*(ptr+w),*(ptr+1));
	  pushm(code,0);
	  for (  j = 0; j < w-1; j++, ptr++ )
	    {
	      vcode_t tmp = vcode(*ptr,*(ptr+w),*(ptr+1));
	      if ( tmp != code )
		{ code = tmp; pushm(code,j); };
	    };
	  pushm(code,w-1);
	  ptr++;
	};
      lines[h-1] = nmarks();
      marks = (mark_t*)realloc(marks,sizeof(mark_t)*nmarks());
    };
  
  public:
    typedef coord_t bounds_t[2];
  
    struct region_t
    {
      coord_t _umin, _umax;
      vcode_t _code; 
      void  * data;
      bool operator<( const region_t& b ) const { return _umin < b._umin;         };
      coord_t min   ( coord_t u         ) const { return ((bounds_t*)data)[u][0]; };
      coord_t max   ( coord_t u         ) const { return ((bounds_t*)data)[u][1]; };
      coord_t umin  ()                    const { return _umin; };
      coord_t umax  ()                    const { return _umax; };

      bool inside( coord_t u, coord_t v )
      {
	if  ( u >= umin() && u < umax() )
	  if ( v >= min(u) && v < max(u) ) return true;
	return false;
      };
	
      vcode_t code() const { return _code; };
      friend class lsegment_t;
    };
    
    void rfree( region_t& r )
    {
      free((void*)(((bounds_t*)(r.data)) + r._umin));
    };
    
    
    mark_t * begin( unsigned i ) { return &(marks[lines[i]]); };
    mark_t * end  ( unsigned i ) { return &(marks[lines[i+1]-1]);};
    
    std::vector< region_t > regions;
    graph_t * grp;
    
    void pushr( coord_t i, mark_t * p )
    {
      p->head =  regions.size();
      regions.push_back( region_t() );
      region_t * r =&(regions.back());
      r->_umin = i;
      r->_umax = 1;
      r->data = (void*)p;
      r->_code = p->code;
      grp->add_node();
    };
     
    void addneighbors( mark_t * m0, mark_t * m1 )
    {
      if ( coherent_code( m0->code, m1->code )  )
	{
	  grp->add_link(m0->head,m1->head);
	  grp->add_link(m1->head,m0->head);
	};
    };
    
    void find_regions()
    {
      mark_t * p;
      for ( p = begin(0); p != end(0); p++ )
	pushr(0,p);
    
      for ( p = begin(0)+1; p != end(0); p++ )
	addneighbors(p,p-1);
      
#ifdef WIN32
      mark_t ** neighbors = new mark_t*[ncols()];
#else
      mark_t * neighbors[ncols()];
#endif
      unsigned n_neigh;
      coord_t i;
      //      cout << "collect\n";
      #define INTERVALTEST ( std::abs(std::min(down->b(),up->b())-std::max(down->a(),up->a())) > 1 )
      for ( i = 0; i < nrows()-2; i++ )
	{
	  mark_t * startup = begin(i);
	  for ( mark_t * down = begin(i+1); down != end(i+1); down++ )
	    {
	      while ( startup->b() <= down->a() ) startup++;
	      mark_t * up = startup;
	      n_neigh = 0;
	      while( down->b() > up->a() )
		if ( (up->next == 0) && ( up->code == down->code ) && INTERVALTEST)
		  
		  { 
		    if ( INTERVALTEST )
		      {
			regions[up->head]._umax++, up->next = down, 
			  down->head = up->head, up++; break; 
		      }
		  }
		else 
		  neighbors[n_neigh++] = up, up++;
	      
	      if ( down->head == -1 ) pushr( i+1, down );
	    
	      for ( unsigned k = 0; k < n_neigh; k ++ ) addneighbors(down,neighbors[k]);
	      
	      while( down->b() > up->a() )
		addneighbors(down,up), up++;
	    };
	};
#ifdef WIN32
      delete[] neighbors;
#endif
    };

    void promote( region_t& r )
    {
      mark_t     * mpointer;
      bounds_t * bpointer; 
      mpointer = (mark_t*)(r.data);
      r.data   = (void*)(malloc(sizeof(bounds_t)*(r._umax)));
      bpointer = (bounds_t*)(r.data);

      for ( unsigned i = 0; i < r._umax; i++ )
	{
	  bpointer[i][0] = mpointer->a();
	  bpointer[i][1] = mpointer->b();
	  mpointer = mpointer->next;
	};
    
      r._umax += r._umin;
      r.data  = (void*)(((bounds_t*)(r.data))-r._umin);
    };
    
    void convert_regions()
    {
      for ( unsigned i = 0; i < regions.size(); i++ )
	promote( regions[i] );
    };

  public:

    Point * source() const { return src;         };
    coord_t ncols()  const { return w;           };
    coord_t nrows()  const { return h;           };
  

    void sample( )
    {
      m_du = (Fct->umax()-Fct->umin())/(h-1);
      m_dv = (Fct->vmax()-Fct->vmin())/(w-1);
      number   u ;
      number   v;
      unsigned i,j;
      for ( i = 0, u = Fct->umin(); i < h; i++, u += m_du )
	for ( j = 0, v = Fct->vmin(); j < w; j++, v += m_dv )
	  Fct->Eval(src[i*w+j],u,v);
    };

    void   free_src()
    {
      delete[] src;
      src = 0;
    };
    
    double _lt;

    lsegment_t( FunctionType * f , unsigned _w, unsigned _h ) : Fct(f)
    {
      _lt = time();
      src  = new Point[_w*_h];
      w    = _w;
      h    = _h;
      sample();
      _size = 0;
      
      lines_changes  ();
      grp = new graph_t();
      find_regions   ();
      convert_regions();
      //      std::cout <<  regions.size() << std::endl;
      delete[] marks;
      delete[] lines;
      //      cout << "lsegment: ok\n";
    };
  
    unsigned size() const { return regions.size(); };
    region_t& operator[]( unsigned i ) { return regions[i]; };
    
//     rid_t region( coord_t u, coord_t v )
//     {
//       for ( unsigned i = 0; i < regions.size(); i++ )
// 	{
// 	  if ( regions[i].inside(u,v) ) return i;
// 	};
//       return -1;
//     };
    
    
    ~lsegment_t()
    {
      if ( src ) delete[] src;
      delete grp;
      for ( unsigned i = 0; i < regions.size(); i++ )
	rfree(regions[i]);
    };
  };
#endif




