#ifndef SYNAPS_SHAPE_GAIA_QNODE_H
#define SYNAPS_SHAPE_GAIA_QNODE_H

#include <synaps/shape/gaia/gaia.h>
#include <synaps/shape/gaia/sample.h>

__BEGIN_NAMESPACE_SYNAPS

namespace gaia
{
  struct qnode
  {
    aabb3 box;
    coord_t umin,umax,vmin,vmax;
    qnode * l, *r;
    qnode * father;
    bool leaf() const { return (l == 0); };
    qnode(): l(0), r(0) {};
    void split_u();
    void split_v();
    void fill( vector3 ** qp, sample * s  ) const;
    void fill( vector3 * qp, sample * s  ) const;
    void convert( vector2 *s, sample * s, int n ) const ;
    void mbox( sample * s );
    void split( sample * s );
    ~qnode();
  };
  std::ostream& operator<<( std::ostream& o, const qnode& );
  inline coord_t du  ( const qnode *  q ) { return q->umax-q->umin; };
  inline coord_t dv  ( const qnode *  q ) { return q->vmax - q->vmin; };
  inline coord_t umin( const qnode *  q ) { return q->umin; };
  inline coord_t umax( const qnode *  q ) { return q->umax; };
  inline coord_t vmin( const qnode *  q ) { return q->vmin; };
  inline coord_t vmax( const qnode *  q ) { return q->vmax; };
  inline bool leaf( qnode const * const q ) { return (q->l == 0); };
  inline bool unit( const qnode * q ) { return du(q)==1 && dv(q) == 1; };
  inline bool inside( coord_t u, coord_t v, qnode * q )
  {
    if ( q->umin  > u ) return false;
    if ( q->umax <= u ) return false;
    if ( q->vmin  > v ) return false;
    if ( q->vmax <= v ) return false;
    return true;
  };
  qnode * search( coord_t u, coord_t v, qnode * start );
  inline qnode * up  ( qnode * q )  { return search( q->umin + 1, q->vmin, q ); };
  inline qnode * down( qnode * q )  { return search( q->umin - 1, q->vmin, q ); };
  inline qnode * left( qnode * q )  { return search( q->umin, q->vmin - 1, q ); };
  inline qnode * right( qnode * q ) { return search( q->umin, q->vmin + 1, q );  };
};

__END_NAMESPACE_SYNAPS
#endif
