#ifndef SYNAPS_PARAMETRIC_CURVE_H
#define SYNAPS_PARAMETRIC_CURVE_H 

#include <synaps/shape/Curve.h>
#include <synaps/shape/base/fxv.h>
#include <synaps/shape/gmv/stream.h>
__BEGIN_NAMESPACE_SYNAPS

VIEW_DCL(Curve,ParametricCurve);

/// expected interface on parametric curves 

{
  eval( p, dp, view(c), t );
};

template<class Point, class R>
void eval( Point & p, Point & dp, const ParametricCurve<R> & c, const typename Point::value_type & t )
{
  typedef typename Point::value_type C;
  C a,b;
  domain(a,b,repr(c));
  C h = (b-a)/C(1000);
  a = t - h;
  b = t + h;
  eval(p,repr(c),a);
  eval(dp,repr(c),b);
  for ( int i = 0; i < Point::_dimension_ ; i ++ )
    dp[i] = (dp[i]- p[i])/(C(2)*h);
  eval(p,repr(c),t);
};



template<class Point, class Curve, class Scalar> inline
void eval( Point * smp, const Curve& c, const Scalar * t, int n )
{
  eval( smp, view(c), t, n );
};

template<class R, class Point> inline
void eval( Point * smp, const ParametricCurve<R> & c, const typename Point::value_type * t, int n )
{
  for ( Point * o = smp; o != smp + n; eval(*o++,repr(c),*t++) );
};



template<class Scalar> 
void tsample( Scalar * t, const Scalar&  a_, const Scalar & b, int n )
{
  Scalar a = a_;
  Scalar h = (b-a)/Scalar(n-1);
  for ( Scalar * p = t; p != t + n; *p++ = a, a += h );
};

template<class Curve, class Scalar> inline  
void tsample( Scalar * t, const Curve & c, int n )
{
  tsample( t, view(c), n );
};

template<class R, class Scalar> inline
void tsample( Scalar * t, const ParametricCurve<R> & c, int n )
{
  Scalar a,b;
  domain(a,b,repr(c));
  tsample(t,a,b,n);
};

template<class Point, class Scalar, class Curve> inline
void sample( Point *  p, Scalar * s, const Curve & c, int n )
{
  sample(p,s,view(c),n);
};

template<class Point, class Scalar, class R> inline
void sample( Point * p, Scalar * s, const ParametricCurve<R> & c, int n )
{
  tsample(s,repr(c),n);
  eval(p,repr(c),s,n);
};


template<class C>
struct ParametricCurveSample
{
  //  typedef ParametricCurveSample<C>  self_t;
  //  typedef ParametricCurve< self_t > view_t;
  typedef fxv<C,3> point_t;
  std::vector< point_t > m_cvals;
  std::vector< C >       m_tvals;
  const C & tval( int i ) const { return m_tvals[i]; };
  const point_t & operator[]( int i ) const { return m_cvals[i]; };
  int size() const { return m_cvals.size(); };
  ParametricCurveSample() {};
  ParametricCurveSample( int n ) : m_cvals(n), m_tvals(n) {};
};

template<class C> 
gmv::stream & operator<<( gmv::stream & o, const ParametricCurveSample<C>& csmp )
{
  o.polyline( (C*)&(csmp.m_cvals[0]), csmp.size() );
  return o;
};

template<class C> 
struct ParametricCurveUniformSample : ParametricCurveSample<C>
{
  template<class Curve>
  ParametricCurveUniformSample( const Curve & c, int n ) : ParametricCurveSample<C>(n)
  {
    sample(&(this->m_cvals[0]),&(this->m_tvals[0]), repr(c), n );
  };
};

template<class PointArray, class CoefficientArray, class Curve>
void regular_sample( PointArray& psmp, CoefficientArray& tsmp, const Curve & c, 
		     const typename CoefficientArray::value_type & eps )
{
  regular_sample(psmp,tsmp,view(c),eps);
};

template<class PointArray, class CoefficientArray, class R>
void regular_sample( PointArray& psmp, CoefficientArray& tsmp, const ParametricCurve<R>& c, const typename CoefficientArray::value_type & eps )
{
  typedef typename CoefficientArray::value_type inputc;
  typedef typename PointArray::value_type       point;
  typedef typename ringof<inputc,typename point::value_type>::T C;
  point p,dp,dfp;
  C f;
  inputc t, tmax, tprev;
  domain(t,tmax,repr(c));
  while ( t < tmax )
    {
      eval( p, dfp, repr(c), t );
      psmp.push_back(p);
      tsmp.push_back(t);
      t = t + eps/sqrt(dotprod(dfp,dfp));
      do
	{
	  eval(p,dfp,repr(c),t);
	  dp = p-psmp.back();
	  f = dotprod(dp,dp)-eps*eps;
	  tprev = t;
	  t = tprev - f/(2*dotprod(dp,dfp));
	}
      while( !(std::abs(f) < 1e-12 || std::abs(tprev-t) < 1e-8) );
    };
  eval(p,repr(c),tmax);
  psmp.push_back(p);
  tsmp.push_back(tmax);
};

template<class C> 
struct ParametricCurveRegularSample : ParametricCurveSample<C>
{
  template<class Curve>
  ParametricCurveRegularSample( const Curve & c, const C & eps  ) : ParametricCurveSample<C>()
  {
    regular_sample( this->m_cvals, this->m_tvals, repr(c), eps );
  };
};

template<class Point, class Curve, class Scalar> inline
void project( Scalar& t, const Curve & c, const Point & p, const Scalar & e )
{
  project( t, view(c), p, e );
};


template<class CurveRep, class C> 
C length( const ParametricCurve<CurveRep> & c, const C &  eps )
{
  C tmin,tmax;
  domain(tmin,tmax,c);
  return length(c,tmin,tmax,eps);
};


template<class Curve, class C> inline
C length( const Curve & c, const C & tmin, const C & tmax, const C & eps ) 
{ length( view(c), tmin, tmax,eps ); };
template<class X, class C>
C length( const Thing<X> & c, const C & tmin, const C & tmax, const C & eps ) 
{ 
  std::cerr << "no length function on " << TYPE<X>() << " concept path\n"; 
  return C(0);
};



__END_NAMESPACE_SYNAPS


#endif
