#ifndef SYNAPS_SHAPE_SISL_BSPLINE_SURFACE_H
#define SYNAPS_SHAPE_SISL_BSPLINE_SURFACE_H

#include <synaps/shape/wrappers.h>
#include <synaps/shape/surfaces/SISLSurfaceBase.h>

__BEGIN_NAMESPACE_SYNAPS
namespace SISL{ struct BSPS; };
template<> struct interfaceof<SISL::BSPS>{ typedef IBSplineSurface T; };
namespace SISL {
  struct BSPS : SurfaceBase, wrapper<SISL::BSPS,IBSplineSurface>
  {
    typedef IBSplineSurface base_t;
    typedef fxv<double,3>   cp;
    typedef double          cp_value_type;
    typedef cp *            cp_iterator;
    typedef const cp *      const_cp_iterator;
    typedef double          kn_value_type;
    typedef kn_value_type * kn_iterator;
    typedef const kn_value_type * const_kn_iterator;
    
    // iterator access to knots values
    inline       kn_iterator kn_begin_u()       { return       (kn_iterator)(sisl_data()->et1);         };
    inline const_kn_iterator kn_begin_u() const { return (const_kn_iterator)(sisl_data()->et1);         };  
    inline       kn_iterator kn_end_u  ()       { return       (kn_iterator)(sisl_data()->et1) + nbku(); };
    inline const_kn_iterator kn_end_u  () const { return (const_kn_iterator)(sisl_data()->et1) + nbku(); };
    
    inline       kn_iterator kn_begin_v()       { return       (kn_iterator)(sisl_data()->et2);         };
    inline const_kn_iterator kn_begin_v() const { return (const_kn_iterator)(sisl_data()->et2);         };  
    inline       kn_iterator kn_end_v  ()       { return       (kn_iterator)(sisl_data()->et2) + nbkv(); };
    inline const_kn_iterator kn_end_v  () const { return (const_kn_iterator)(sisl_data()->et2) + nbkv(); };
    
    // iterator access to control points
    inline       cp_iterator cp_begin()       { return       (cp_iterator)(sisl_data()->ecoef);      };
    inline const_cp_iterator cp_begin() const { return (const_cp_iterator)(sisl_data()->ecoef);      };
    inline       cp_iterator cp_end  ()       { return       (cp_iterator)(sisl_data()->ecoef)+nbp();};
    inline const_cp_iterator cp_end  () const { return (const_cp_iterator)(sisl_data()->ecoef)+nbp();};
    
    inline       cp_iterator cp_begin_u( int u )       { return cp_begin() + u*nbpv();  };
    inline const_cp_iterator cp_begin_u( int u ) const { return cp_begin() + u*nbpv();  };
    inline       cp_iterator cp_end_u  ( int u )       { return cp_begin_u(u) + nbpv(); };
    inline const_cp_iterator cp_end_u  ( int u ) const { return cp_begin_u(u) + nbpv(); };


    // random access to control points
    inline const cp & operator[](int i) const { return cp_begin()[i]; };
    inline       cp & operator[](int i)       { return cp_begin()[i]; };
    
    inline const cp & operator()(int i, int j ) const { return cp_begin()[i*nbpv()+j]; };
    inline       cp & operator()(int i, int j )       { return cp_begin()[i*nbpv()+j]; };
    
    inline const kn_value_type & uknot( int i ) const { return kn_begin_u()[i]; };
    inline kn_value_type & uknot( int i ) { return kn_begin_u()[i]; };
    inline const kn_value_type & vknot( int i ) const { return kn_begin_v()[i]; };
    inline kn_value_type & vknot( int i ) { return kn_begin_v()[i]; };
  };
  
  
    
  inline void setDefinition( BSPS & c, 
			     int nbpu, int nbpv,
			     int ordu, int ordv,
			     const double * uknots, 
			     const double * vknots,
			     const fxv<double,3> * controls )
    {
      if ( c.sisl_data_ ) freeSurf(c.sisl_data_);
      c.sisl_data_ = newSurf(nbpu,nbpv,ordu,ordv,(double*)uknots,(double*)vknots,(double*)controls,1,3,1);
    };
  
};

__END_NAMESPACE_SYNAPS
#endif
