/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef synaps_implicit_qsic_h
#define synaps_implicit_qsic_h
//===================================================================
#include <synaps/init.h>
#include <synaps/linalg/VECTOR.m>
#include <synaps/shape/surfaces/quadric/quadric.h>
#include <synaps/usolve/Algebraic.h>
#include <synaps/linalg/Det.h>
#include <synaps/arithm/sign.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
template<class U>
Seq<VectDse<int> > qsic_type(const implicit::quadric<U>& Q1, const implicit::quadric<U>&Q2)
{
  using std::cout;
  using std::endl;
  typedef U                coeff_t;
  typedef MPol<coeff_t>    mpol_t;
  typedef UPolDse<coeff_t> upol_t;

  MatrDse<U> A,B;

  assign(A,Q1);
  assign(B,Q2);

  mpol_t t("x0"), u("x1");

  unsigned N=4;
  MatrDse<mpol_t> C(N,N);
  C=B; C*=t;  C+=A; C-=diag(4,u);
  mpol_t dt(Det(C));
  std::cout<<dt<<std::endl;
  VectDse<upol_t> r(5,AsSize());
  
  for(typename mpol_t::const_iterator it=dt.begin();it!=dt.end();it++)
    {
      if(it->nbvar()<2)
	r[0]+= upol_t(it->coeff(),(*it)[0]);
      else
	r[(*it)[1]] += upol_t(it->coeff(),(*it)[0]);
    }
  std::cout<<r<<std::endl;


  Seq<Algebraic<ZZ>::root_t> sol =solve(r[0],Algebraic<ZZ>());
  cout<<sol<<endl;

  int c0,c1,c2,nbr,mu, n1;
  QQ m;


  VectDse<int> V(2,AsSize());
  Seq<VectDse<int> > res;


  VectDse<int> sm(sol.size()+1);
  int d1 = N+1;

  //  std::vector<int> S(d1);

  std::vector<int> v1(d1);
  for(unsigned k=0;k<d1; k++)
    if(r[k] !=0)
      v1[k]=sign(r[k][degree(r[k])]);
    else
      v1[k]=0;
  for(unsigned k=0;k<d1; k+=2)
    v1[k]*=-1;
  sm[0]= sign_variation(v1);
  for(unsigned j=1;j<sol.size(); j++)
    {
      m= (sol[j].interval().inf()+sol[j-1].interval().sup())/2;
      for(unsigned k=0;k<d1; k++)
        v1[k]=SignAt(r[k],m);
      sm[j]= sign_variation(v1);
    }
  sm[sol.size()]=N-sm[0];
  res.push_back(sm);
  for(unsigned j=0;j<sol.size(); j++)
    {
      for(unsigned k=1;k<d1; k++)
        v1[k-1]=sign_at(r[k],sol[j]);
      v1[d1-1]=0;
      c1= sign_variation(v1);
      for(unsigned k=0;k<d1; k+=2)
        v1[k]*=-1;
      c2= sign_variation(v1);
      nbr=N-c1-c2;
      n1 = sm[j]-c1;
      mu = sol[j].multiplicity();

      if(mu==1)
        {
          V[1]=sm[j]-c1;
          V[0]=1-V[1];
        }
      else if(mu==2)
        {
          if(nbr==1)
            {
              V[0]=sm[j]-c1;
              V[0]*=2;
              V[1]=2-V[0];
            }
          else if(nbr==2)
            {
              V[1]=sm[j]-c1;
              if(V[1]==2)
                {
                  V[0]=0;
                  V[1]=11;
                }
              else if(V[1]==1)
                V[0]=1;
              else
                V[0]=11;
            }
        }
      else if(mu==3)
        {
          if(nbr==1)
            {
              V[1]=sm[j]-c1;
              V[1]*=3;
              V[0]=3-V[1];
            }
          else if(nbr==2)
            {
              V[1]=sm[j]-sm[j+1]+1; V[1]/=2;
              V[0]=1-V[1];
              c0= sm[j]+sm[j+1]-1-2*c1; c0*=2;
              if(V[0]) V[0]+=10*c0; else V[0]+=c0;
              if(V[1]) V[1]+=10*(2-c0); else V[1]+=2-c0;
            }
          else if(nbr==3)
            {
              V[1]=sm[j]-c1;
              if(V[1]==2)
                {
                  V[0]=1;
                  V[1]=11;
                }
              else if(V[1]==3)
                {
                  V[0]=0;
                  V[1]=111;
                }
              else
                {
                  V[0]=111;
                }
            }
        }
      else if(mu==4)
        {
          if(nbr==1)
            {
              V[0]=sm[j]-c1;
              V[0]*=4;
              V[1]=4-V[1];
            }
          else if(nbr==2)
            {
              if(sm[j]==sm[j+1])
                {
                  n1 = sm[j]-c1;
                  if(n1==0)
                    {
                      V[0]=0;
                      V[1]=22;
                    }
                  else if(n1==1)
                    {
                      int s = sign_at(diff(r[1]),sol[j]);
                      if(s==1)
                        {
                          V[0]=1;
                          V[1]=3;
                        }
                      else if(s==-1)
                        {
                          V[0]=1;
                          V[1]=3;
                        }
                      else
                        {
                          V[0]=2;
                          V[1]=2;
                        }
                    }
                  else
                    {
                      V[0]=22;
                      V[1]=0;
                    }
                }
              else if(sm[j]<sm[j+1])
                {
                  V[0]=31;
                  V[1]=0;
                }
              else
                {
                  V[0]=0;
                  V[1]=31;
                }
            }
          else if(nbr==3)
            {
              V[1]=sm[j]-sm[j+1]+2; V[1]/=2;
              if(V[1]==0)
                V[0]=11;
              else if(V[1]==1)
                V[0]=1;
              else
                {
                  V[0]=0;
                  V[1]=11;
                }
              c0= sm[j]+sm[j+1]-2-2*c1; c0*=2;
              if(V[0]>10)
                V[0]+=100*c0;
              else if(V[0])
                V[0]+=10*c0;
              else V[0]+=c0;
              if(V[1]>10)
                V[1]+=100*(2-c0);
              else if(V[1])
                V[1]+=10*(2-c0);
              else
                V[1]+=2-c0;
            }
          else if(nbr==4)
            {
              V[1]=sm[j]-c1;
              if(V[1]==0)
                V[0]=1111;
              else if(V[1]==1)
                V[0]=111;
              else if(V[1]==2)
                {
                  V[0]=11;
                  V[1]=11;
                }
              else if(V[1]==3)
                {
                  V[0]=1;
                  V[1]=111;
                }
              else
                V[0]=1111;
            }
        }
      res.push_back(V);
    }
  return res;
}
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//===================================================================
#endif // synaps_implicit_qsic_h
