#include <synaps/shape/interfaces/IParametricSurface.h>
#include <synaps/shape/ssi/SSIQTSL.h>
#include <synaps/shape/gmv/stream.h>
#include <synaps/shape/wrappers.h>

__BEGIN_NAMESPACE_SYNAPS


static void compute_normals ( fxv<double,3> * normals, const fxv<double,3> * smp, int m, int n )
{
  int _N[m]; std::fill(_N,_N+m,-n);  _N[0]   = 0;
  int _S[m]; std::fill(_S,_S+m,n);   _S[m-1] = 0;
  int _E[n]; std::fill(_E,_E+n,1);   _E[n-1] = 0;
  int _W[n]; std::fill(_W,_W+n,-1);  _W[0] = 0;

  int *N,*S,*E,*W;

  for ( N = _N, S = _S; N != _N + m; N++, S++ )
    for (  E = _E, W = _W; E != _E + n; E++, W++,  normals++, smp ++  )
      {
	fxv<double,3> du,dv;
	sub(du,*(smp+*S),*(smp+*N));
	//	scmul(du,9.0/2.0);
	sub(dv,*(smp+*E),*(smp+*W));
	//	scmul(dv,9.0/2.0);
	crossprod(*normals,du,dv);
	div(*normals,sqrt(dotprod(*normals,*normals)));
	//	std::cout << *normals<<  std::endl;
      };

};

IGLGood * mesh( const IParametricSurface * ps, int m, int n )
{
  IGLGood * result = new IGLGood(IGLGood::E_QUAD,m*n,4*(m-1)*(n-1),true);
  double * uvals = new double[m+n];
  double * vvals = uvals + m;
  ps->sample(result->vertices,uvals,vvals,m,n);

  compute_normals((fxv<double,3>*)(result->normals),(const fxv<double,3>*)(result->vertices),m,n);
  
  int c = 0;
  for ( int i = 0; i < m-1; i ++ )
    for ( int j = 0; j < n-1; j ++, c += 4 )
      {
	int a;
	result->indexes[c]   = (a = i*n+j);
	result->indexes[c+1] = a + n;
	result->indexes[c+2] = a + n + 1;
	result->indexes[c+3] = a + 1;
      };
  
  delete[] uvals;
  return result;
};



struct IntersectionResult;
template<> struct interfaceof< IntersectionResult > { typedef IShape T; };

struct IntersectionResult : wrapper<IntersectionResult,IShape>
{
  IGLGood * glg;
  IntersectionResult(){};
  IntersectionResult( int type, int nbv, int nbi ) 
  { glg = new IGLGood(type,nbv,nbi,false); };
  ~IntersectionResult() { delete glg;};  
};

IGLGood * mesh( const IntersectionResult & r ) { return r.glg; };

IShape * intersect( IParametricSurface * srfa, IParametricSurface * srfb )
{
  SSIQTSL ssi( srfa, srfb, 100 );
  
  IntersectionResult * result = new IntersectionResult(IGLGood::E_LINE,ssi.spcs.size(),ssi.spcs.size());
  std::copy(ssi.spcs.begin(),ssi.spcs.end(),(fxv<double,3>*)(result->glg->vertices));
  for ( int i = 0; i < result->glg->nbv; i ++ )
    result->glg->indexes[i] = i;
  return result;
  
};

__END_NAMESPACE_SYNAPS
