#include <synaps/shape/gaia/qnode.h>
__BEGIN_NAMESPACE_SYNAPS
namespace gaia {    

  std::ostream& operator<<( std::ostream& o, const qnode & q )
  {
    o << "[ " << q.umin << ", " << q.umax << " ] x ";
    o << "[ " << q.vmin << ", " << q.vmax << " ]";
    return o;
  };
  
  void qnode::split_u() 
  {
    l = new qnode();
    r = new qnode();
    l->umin = umin;  
    l->umax = (umin+umax)/2;
    r->umin = (umin+umax)/2;
    r->umax = umax;
    l->vmin = r->vmin = vmin;
    l->vmax = r->vmax = vmax;
    l->father = r->father = this;
  };

void qnode::split_v()
{
  l = new qnode();
  r = new qnode();
  l->vmin = vmin;
  l->vmax = (vmin+vmax)/2;
  r->vmin = l->vmax;
  r->vmax = vmax;
  l->umin = r->umin = umin;
  l->umax = r->umax = umax;
  l->father = this;
  r->father = this;
};


qnode::~qnode()
{
  if  (l ) delete l;
  if ( r ) delete r;
};

void qnode::fill( vector3 ** qp, sample *  s  ) const
{
  qp[0] = s->base() + this->umin*s->m_ncols + this->vmin;
  qp[1] = s->base() + this->umax*s->m_ncols + this->vmin;
  qp[2] = qp[1]  + this->vmax - this->vmin;
  qp[3] = qp[0]  + this->vmax - this->vmin;
};

void qnode::split( sample * s)
{

  if ( l ) return;
  if ( du(this) > dv(this) ) 
    {
      this->split_u();
      l->mbox(s);
      r->mbox(s);
    }
  else if ( dv(this) > 1 ) 
    {
      this->split_v();
      l->mbox(s);
      r->mbox(s);
    };

};

void qnode::mbox( sample * s )
{
  vector3 * qp[4];
  fill( qp, s );
  for ( int i = 0; i < 3; i ++ ) box[i].m = box[i].M = qp[0]->data[i];
  for ( int i = 1; i < 4; i ++ )
    for ( int d = 0; d < 3; d ++ )
      {
	if ( qp[i]->data[d] < box[d].m ) box[d].m = qp[i]->data[d];
	else {  if ( qp[i]->data[d] > box[d].M ) box[d].M = qp[i]->data[d]; };
      };
};
 
void qnode::fill( vector3   *qp, sample * s  ) const
{
  vector3  *r = s->base() + this->umin*s->m_ncols + this->vmin;
  qp[0] = *r;
  qp[3] = *(r + this->vmax - this->vmin);
  r = s->base() + this->umax*s->m_ncols + this->vmin;
  qp[1] = *r;
  qp[2] = *(r + this->vmax - this->vmin);
};

qnode * search( coord_t u, coord_t v, qnode * start )
{
  qnode * tmp = start;
  do { tmp=tmp->father; } while(tmp && !inside(u,v,tmp) );
  if ( !tmp ) return 0;
  while(1){ 
    if ( leaf(tmp) ) return tmp;
    if ( inside(u,v,tmp->l) ) { tmp = tmp->l; continue; };
    if ( inside(u,v,tmp->r) ) { tmp = tmp->r; continue; };
    return 0;
  };
};


void qnode::convert( vector2 * p, sample * s, int n  ) const 
{
  assert(du(this)==1 && dv(this)==1);
  double u = s->uvalue(umin);
  double v = s->vvalue(vmin);
  double vu = s->uvalue(umax)-u;
  double vv = s->vvalue(vmax)-v;
  for ( int i = 0; i < n; i ++ )
    {
      p[i][0] = u + p[i][0]*vu; 
      p[i][1] = v + p[i][1]*vv;
    };
};

}
__END_NAMESPACE_SYNAPS
       
    
