#include <synaps/shape/gmv/stream.h>
#include <vector>
#include <math.h>
#include <iostream>

__BEGIN_NAMESPACE_SYNAPS

namespace gmv
{
  
  struct ellipse_drawer
  {
    //    typedef float point3[3];
    struct point3
    {
      float data[3];
      float& operator[]( unsigned i ) { return data[i]; };
    };
    struct edge { int m,a,b;float scr; edge(int _a, int _b ) { a = _a; b = _b; m = -1;scr = 1.0;}};
    float m_d;
    float m_iell2[3];
    std::vector< point3 >   m_pts;
    std::vector< unsigned > m_idx;
    void edge_point( point3& p, edge * edge, int n  )
    {

      for ( int d = 0; d < 3; d ++ )
	p[d] = (m_pts[edge->a][d] + m_pts[edge->b][d])/2.0;
      
      float sc = 0.0;
      for ( int d = 0; d < 3; d ++ )
	sc += m_iell2[d] * (p[d] * p[d]);
      float lmbd = sqrt(1.0/sc);
      for ( int d = 0; d < 3; d ++ ) p[d] *= lmbd;
    };
    
    void edge_point( edge * e, int n )
    {
      if ( e->m == -1 )
	{
	  point3 tmp;
	  edge_point( tmp, e, n );
	  m_pts.push_back(tmp);
	  e->m = m_pts.size()-1;
	};
    };
    
    void sbd( edge * a, edge * b, edge * c, int n  )
    {
      if ( n == 0 )
	{
	  m_idx.push_back( a->a );
	  m_idx.push_back( b->a );
	  m_idx.push_back( c->a );
	  return;
	};
      
      edge_point(a,n);
      edge_point(b,n);
      edge_point(c,n);

      //      edge_point(a);
      //      edge_point(b);
      //      edge_point(c);
      
      edge edge0(a->a,a->m);
      edge edge1(a->m,c->m);
      edge edge2(c->m,a->a);
      sbd( &edge0, &edge1, &edge2, n-1 );
      edge edge3(c->m,b->m);
      edge edge4(b->m,a->m);
      edge edge5(a->m,a->b);
      edge edge6(a->b,b->m);
      edge edge7(b->m,b->b);
      sbd( &edge5, &edge6, &edge4, n-1 );
      edge edge8(b->b,c->m);
      sbd( &edge7, &edge8, &edge3, n-1 );
      std::swap(edge1.a,edge1.b);
      std::swap(edge3.a,edge3.b);
      std::swap(edge4.a,edge4.b);
      sbd( &edge1, &edge4, &edge3, n-1 );
      std::swap(edge1.a,edge1.b);
      std::swap(edge3.a,edge3.b);
      std::swap(edge4.a,edge4.b);
    };
    
    
    ellipse_drawer( float a, float b, float c, float d, int n = 4 )
    {
      assert(d>0);
      assert(a>0&&b>0&&c>0);
      float sd = sqrt(d);
      a *= sd;
      b *= sd;
      c *= sd;
      m_iell2[0] = 1.0/(a*a);
      m_iell2[1] = 1.0/(b*b);
      m_iell2[2] = 1.0/(c*c);
      float oct[8][3] = { { -a, 0, 0 }, { 0, -b, 0 }, { 0, 0, -c } };
      for ( unsigned i = 0; i < 3; i ++ )
	{
	  point3 tmp;
	  for ( unsigned d = 0; d < 3; d ++ )
	    tmp[d] = oct[i][d];
	  m_pts.push_back( tmp );
	};
      //unsigned itrigs[] ={ { 0, 1, 4 }, { 1, 2, 4 }, { 2, 3, 4 }, { 3, 0, 4 },
      //			   { 5, 1, 0 }, { 5, 2, 1 }, { 5, 3, 2 }, { 5, 0, 3 } };
      edge e0(0,1);
      edge e1(1,2);
      edge e2(2,0);
      sbd(&e0,&e1,&e2,n);
      
      unsigned np = m_pts.size();
      unsigned ni = m_idx.size();
      std::vector< point3 >   pts( 8*np );
      std::vector< unsigned > idx( 8*ni );
      for ( unsigned i = 0; i < np; i++ ) pts[i] = m_pts[i];
      for ( unsigned i = 0; i < ni; i++ ) idx[i] = m_idx[i];
      
      int dim[7] = { 0, 1, 2, 0, 1, 0, 2 };
      int prv[7]  = { 0, 1, 2, 3, 4, 5, 4 };
      for ( unsigned k = 0; k < 7; k ++ )
	{
	  for ( unsigned i = 0; i < np; i ++ )
	    {
	      pts[(k+1)*np+i] = pts[prv[k]*np+i];
	      pts[(k+1)*np+i][dim[k]] = -pts[(k+1)*np+i][dim[k]];
	    };
	  for ( unsigned i = 0; i < ni; i ++ )
	    {
	      idx[(k+1)*ni+i] = idx[(prv[k]+1)*ni-i-1] + (k+1-prv[k])*np;
	      //	      std::cout << idx[(k+1)*ni+i] << std::endl;
	    };
	};
      m_idx.swap(idx);
      m_pts.swap(pts);
    };
  };

  void stream::sphere( float * p, float r )
  {
    fprintf(m_fd,"{ = SPHERE\n");
    fprintf(m_fd,"%f\n%f %f %f\n",r,p[0],p[1],p[2]);
    fprintf(m_fd,"}\n");
  };
  


  void stream::spheres( float * _centers, float r, unsigned n, float * color, int dice  )
  {
    ellipse_drawer tmp( 1.0, 1.0, 1.0,r*r, dice );
    unsigned ne  = tmp.m_pts.size();
    unsigned ni  = tmp.m_idx.size();
    unsigned np  = n*ne;
    unsigned nfe = ni / 3;
    unsigned nf  = nfe * n;
    unsigned fsz = (color?9:5);
    unsigned sz  = 3 + np*3 + fsz*nf;
    typedef float point3[3];
    point3 * centers = (point3*)_centers;
    unsigned * data = new unsigned[ sz ];
    data[0] = np;
    data[1] = nf;
    data[2] = nf*3;
    point3 * pts = (point3*)(data+3);

    //    points((float*)(&(tmp.m_pts[0])), ne );
    //    return;

    for ( unsigned i = 0; i < n; i ++ )
      for ( unsigned k = 0; k < ne; k ++ )
	for ( unsigned d = 0; d < 3; d ++ )
	  pts[i*ne+k][d] = centers[i][d]+tmp.m_pts[k][d];
    // points((float*)pts,np);


    unsigned * idx = (unsigned*)(pts+np);
    //    std::cout << (idx-data) << " " << np << std::endl;
    //    std::cout << nfe << std::endl;
    for ( unsigned i = 0; i < n; i ++, idx += nfe*fsz )
      for ( unsigned k = 0; k < nfe; k ++ )
	{
	  idx[ k*fsz ]     = 3;
	  idx[ k*fsz + 1]  = tmp.m_idx[k*3]   + i*ne;
	  idx[ k*fsz + 2]  = tmp.m_idx[k*3+1] + i*ne;
	  idx[ k*fsz + 3]  = tmp.m_idx[k*3+2] + i*ne;
	  if ( fsz == 5 )  
	    idx[ k*fsz + 4]  = 0;
	  else
	    {
	      idx[ k*fsz + 4 ] = 4;
	      idx[ k*fsz + 5 ] = *((unsigned*)(color+0));
	      idx[ k*fsz + 6 ] = *((unsigned*)(color+1));
	      idx[ k*fsz + 7 ] = *((unsigned*)(color+2));
	      idx[ k*fsz + 8 ] = *((unsigned*)(color+3));
	    };
	};
    swab4(data,sz );
    fprintf(m_fd,"{ = OFF BINARY\n");
    fwrite(data,4,sz,m_fd);
    delete[] data;
    fprintf(m_fd,"}\n");
  };
  
  void stream::ellipse( float * prms )
  {
    ellipse_drawer tmp( prms[0], prms[1], prms[2], prms[3], 3 );
    fprintf(m_fd,"{ = OFF BINARY #gmv::stream::ellipse\n");
    unsigned np = tmp.m_pts.size();
    unsigned nf = tmp.m_idx.size()/3;
    unsigned * data = new unsigned [3+3*np+5*nf];
    data[0] = tmp.m_pts.size();
    data[1] = tmp.m_idx.size()/3;
    data[2] = 3*data[1];
    float * pts = (float*)(data + 3);
    std::copy((float*)(&(tmp.m_pts[0])),(float*)(&(tmp.m_pts[0])) + np * 3, pts ); 
    unsigned * idx = (unsigned*)(pts + np*3);
    unsigned c = 0;
    for ( unsigned i = 0; i < tmp.m_idx.size(); i += 3, c += 5 )
      {
	idx[c] = 3;
	idx[c+1] = tmp.m_idx[i];
	idx[c+2] = tmp.m_idx[i+1];
	idx[c+3] = tmp.m_idx[i+2];
	idx[c+4] = 0;
      };
    swab4(data,3+3*np+5*nf);
    fwrite(data,4,3+3*np+5*nf,m_fd);
    fprintf(m_fd,"}\n");
    delete[] data;
  };
};
__END_NAMESPACE_SYNAPS
