#include <synaps/shape/gmv/stream.h>
#include <algorithm>
#include <math.h>
#include <iostream>

__BEGIN_NAMESPACE_SYNAPS
namespace gmv
{
  struct point3{
    float data[3];
    float& operator[]( unsigned i ) { return data[i]; };
  };

  float angle( point3& a, point3& b, point3 c )
  {
    point3 t0,t1;
    float  s0,s1;
    s0 = s1 = 0;
    for (unsigned i = 0; i < 3; i ++ )
      {
	t0[i] = b[i]-a[i];
	s0 += t0[i]*t0[i];
	t1[i] = c[i]-b[i];
	s1 += t1[i]*t1[i];
      };
    float sc = t0[0]*t1[1]-t0[1]*t1[0];
    sc *= sc;
    float tmp = t0[1]*t1[2]-t0[2]*t1[1];
    sc += tmp*tmp;
    tmp =t0[0]*t1[2]-t0[2]*t1[0];
    sc += tmp*tmp;
    return sc/(s0*s1);
  };
  
  void angles( float * angles, point3 * pts, unsigned n )
  {
    angles[0]   = 0;
    angles[n-1] = 0;
    for ( unsigned i = 1; i < n-1; i += 2 )
      angles[i] = angle( pts[i-1], pts[i], pts[i+1] );
  };
  
  unsigned simplify( point3 * pts, unsigned n )
  {
    float * _angles = new float[n];
    angles( _angles, pts, n );
    float acc = 0.0;
    unsigned c = 1;
    for ( unsigned i = 1; i < n; i ++ )
      {
	acc += _angles[i-1];
	if ( acc > 1e-5 )
	  {
	    acc = 0.0;
	    pts[c++] = pts[i];
	  };
      };
    pts[c] = pts[n-1];
    return c;
  };
  
  void orth( point3& u, point3& a, point3& b )
  {
    float s,s0,s1;
    s = s0 = s1 = 0;
    float mx = 0;
    for ( int i = 0; i < 3; i ++ )
      if ( fabs(u[i]) > mx ) mx = fabs(u[i]);
    for ( int i = 0; i < 3; i ++ ) u[i] /= mx;
    for ( int i = 0; i < 3; i ++ )
      {
	s  += u[i] * u[i];
	s0 += a[i] * u[i];
	s1 += b[i] * u[i];
      };

    s0 /= s;
    s1 /= s;

    for ( int i = 0; i < 3; i ++ )
      {
	a[i] -= s0 * u[i];
	b[i] -= s1 * u[i];
      };
    mx = fabs(a[0]);
    for ( int i = 0; i < 3; i ++ )
      if ( fabs(a[i]) > mx ) mx = fabs(a[i]);
    for ( int i = 0; i < 3; i ++ ) a[i] /= mx;
    mx = fabs(b[0]);
    for ( int i = 0; i < 3; i ++ )
      if ( fabs(b[i]) > mx ) mx = fabs(b[i]);
    for ( int i = 0; i < 3; i ++ ) b[i] /= mx;
    s0 = 0;
    s1 = 0;
    for ( int i = 0; i < 3; i ++ )
      {
	s0 += a[i]*a[i];
	s1 += b[i]*a[i];
      };
    s1 /= s0;
    s = 0;
    for ( int i = 0; i < 3; i ++ )
      {
	b[i] -= s1*a[i];
	s += b[i]*b[i];
      };
    s0 = sqrt(s0);
    s1 = sqrt(s);
    for ( int i = 0; i < 3; i ++ )
      {
	a[i] /= s0;
	b[i] /= s1;
      };
  };

  void pipe( point3 * dst, point3 * pts, unsigned m, float thick )
  {
    point3 dp,a,b;
    unsigned c = 0;
    for ( int i = 0; i < 3; i ++ )
      {
	a[i] = ((double)rand()/RAND_MAX) - 0.5;
	b[i] = ((double)rand()/RAND_MAX) - 0.5;
      };
    for ( unsigned i = 0; i < m-1; i ++ )
      {
	point3 p;
	for  (int d = 0; d < 3; d ++ )
	  dp[d] = pts[i+1][d]-pts[i][d];
	orth(dp,a,b);
	/*
	std::cout << dp[0] << ", " ;
	std::cout << dp[1] << ", " ; 
	std::cout << dp[2] << ", " ;
	std::cout << a[0] << ", " ;
	std::cout << a[1] << ", " ;
	std::cout << a[2] << ", " ;
	std::cout << b[0] << ", " ;
	std::cout << b[1] << ", " ;
	std::cout << b[2] ;
	std::cout << std::endl;
	*/
	for ( int d = 0; d < 3; d ++ )
	  p[d] = pts[i][d] + thick*a[d] + thick*b[d];
	dst[c++] = p;
	for ( int d = 0; d < 3; d ++ )
	  p[d] = pts[i][d] + thick*a[d] - thick*b[d];
	dst[c++] = p;
	for ( int d = 0; d < 3; d ++ )
	  p[d] = pts[i][d] -thick*a[d] - thick*b[d];
	dst[c++] = p;
	for ( int d = 0; d < 3; d ++ )
	  p[d] = pts[i][d] -thick*a[d] + thick*b[d];
	dst[c++] = p;
	for ( int d = 0; d < 3; d ++ )
	  p[d] = pts[i][d] + thick*a[d] + thick*b[d];
	dst[c++] = p;
      };
  };
  
  void stream::stick( float * pts, unsigned m, float thick )
  {

    //    unsigned k = simplify( (point3*)pts, m );
    unsigned k = m;
    point3  tmp[5*k];
    pipe(tmp,(point3*)pts,k,thick);
    float * colors = new float[ 4*m * 5 ];
    for ( unsigned i = 0; i < m * 5; i ++ )
      for ( int d = 0; d < 4; d ++ )
	colors[4*i+d] = m_lcolor[d];
    uvsmp((float*)tmp,k-1,5,colors);
    delete[] colors;
    //    delete[] tmp;
  };
  
}
__END_NAMESPACE_SYNAPS

