#include <fstream>
#include <synaps/shape/curves/SISLBSplineCurve.h>
#include <synaps/shape/curves/RationalCurve.h>
#include <synaps/shape/surfaces/RationalSurface.h>
#include <synaps/shape/wrappers/WShapeFactory.h>
#include <synaps/shape/surfaces/SISLBSplineSurface.h>
#include <synaps/shape/surfaces/MPolImplicitSurface.h>
#include <synaps/arithm/gmp.h>
#include <synaps/base/Debug_util.h>
#include <synaps/shape/gmv/stream.h>

void test_rational_curve  ( IShapeFactory * factory )
{
  IShape * shape = factory->interface( "SYNAPS::IRationalCurve", 0 );
  IRationalCurve * c = dynamic_cast<IRationalCurve*>(shape);
  c->setEquations( "1", "t", "t", "t" );
  std::cout << "length = " << c->length(1e-3)  << std::endl;
};

void test_rational_surface( IShapeFactory * factory )
{
  IShape * shape1 = factory->interface( "SYNAPS::IRationalSurface", 0 );
  IRationalSurface * r1 = dynamic_cast<IRationalSurface*>(shape1);
  r1->setEquations( "1", "u^2-v^2*u+v^3", "u^3-u*v", "v^3-v", "u v");
  r1->setRange(-1.0,1.0,-1.0,1.0);
  double a,b,c,d;
  r1->range(a,b,c,d);
  std::cout << "a = " << a << std::endl;
  std::cout << " mesh of rational surface\n";
  shape1->mesh();
  std::cout << " passed\n";
  IParametricSurface * psf = dynamic_cast<IParametricSurface*>(shape1);
  IShape * s = selfintersection( psf );
  IGLGood * gl = s->mesh();
  //  gmv::stream out("self.gmv");
  //  fxv<double,3> * p = (fxv<double,3>*)(gl->vertices);
  //  out.points(gl->vertices,gl->nbv);
};


void test_bspline_curve( IShapeFactory * factory )
{
  IShape        * shape = factory->interface("SYNAPS::IBSplineCurve",0);
  IBSplineCurve * bspl  = dynamic_cast<IBSplineCurve*>(shape);
  double controls [] = { 0,0,0, 1,1,1 };
  double knots    [] = { 0, 0, 1, 1  };
  bspl->setDefinition( 2, 2, knots, controls );
  IGLGood * glg = shape->mesh();
  for ( int i = 0; i < glg->nbv; i += 10 )
    std::cout << ((fxv<double,3>*)(glg->vertices))[i] << std::endl;
  IParametricCurve * pcrv = dynamic_cast<IParametricCurve*>(shape);
  IParametricCurve * a;
  IParametricCurve * b;
  pcrv->split(a,b,0.5);
  
};

void test_algebraic_surface( IShapeFactory * factory )
{
  IShape * shape = factory->interface("SYNAPS::IAlgebraicSurface",0);
  IAlgebraicSurface * s = dynamic_cast<IAlgebraicSurface*>(shape);
  assert(s);
  s->setEquation("x^2+y^2+z^2-1");
  std::cout << " MESH ? ";
  IGLGood* tmp = s->mesh();
};

//SYNAPS::IBSplineCurve
typedef type::gentlist< 
  //    MPolImplicitSurface<double>,
  RationalCurve<double,3>, 
  SISLBSplineCurve,
  RationalSurface<double>,
  SISL::BSPS >::T List;


int main( int argc, char ** argv )
{
  WShapeFactory< List > Factory;
  test_rational_surface( &Factory );
  //  test_rational_curve( &Factory );
  //test_algebraic_surface( & Factory );
  return 0;
}
;

