/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define EV_CV_OFF

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
   ev_cv_off(SISLCurve *pc1,int ider,double ax,int *ileft,
	       double offset,double eder[],int *jstat)
#else
void ev_cv_off(pc1,ider,ax,ileft,offset,eder,jstat)
     SISLCurve *pc1;
     int ider;
     double ax;
     int *ileft;
     double offset;
     double eder[];
     int *jstat;
#endif
                                     
{
  int kstat=0;        
  int kpos=0;         
  int kn;             
  int kk;             
  int kdim;           
  int kleft;          
  int kder;           
  int kind;           
  int ki,kj,kih,kjh;  
  int kl,kl1,kl2;     
  double *st;         
  double *scoef;      
  double tt;          
  double *ebder=SISL_NULL; 
  double *sder=SISL_NULL;  
  double hlpnorm;     
  
  
  
  kn = pc1 -> in;
  kk = pc1 -> ik;
  st = pc1 -> et;
  kdim = pc1 -> idim;
  kind = pc1 ->ikind;
  
  if (kind == 2 || kind == 4)
    {
      scoef = pc1 -> rcoef;
      kdim +=1;
    }
  else
      scoef = pc1 -> ecoef;

  sder = newarray(kdim*(ider+2),DOUBLE); 
  if (sder==SISL_NULL) goto err101;
  
  
  
  if (pc1->idim != 2) goto err102;
  
  if (kk < 1) goto err110;
  
  if (kn < kk) goto err111;
  
  if (st[kk-1] == st[kk] || st[kn-1] == st[kn]) goto err112;
  
  if (ider < 0 || ider > 2) goto err178;
  
  kder = min(kk-1,ider+1);
  
  
  
  ebder = newarray(kk*(kder+1),double);
  if (ebder == SISL_NULL) goto err101;
  
  
  
  for (ki=0; ki<(ider+2)*kdim; ki++) sder[ki] = DZERO;
  
  
  
  s1220(st,kk,kn,ileft,ax,kder,ebder,&kstat);
  
  if (kstat < 0) goto error;
  
  kleft = *ileft;
  
  
  
  
  
  kih = 0;
  for (ki=kleft-kk+1; ki<=kleft; ki++)
    {
      
      
      
      kjh = 0; kl1 = kdim*ki;
      for (kj=0; kj<=kder; kj++)
	{
	  
	  
	  
	  tt = ebder[kih++]; kl2 = kl1;
	  for (kl=0; kl<kdim; kl++,kjh++,kl2++)
	    {
	      sder[kjh] += scoef[kl2]*tt;
	    }
	}
    }
  
  
  
  if (kind == 2 || kind == 4)
    {
      s6ratder(sder,pc1->idim,ider,sder,&kstat);
      if (kstat<0) goto error;
    }
  
  
  
  hlpnorm = sqrt(sder[2]*sder[2] + sder[3]*sder[3]);
  if (hlpnorm < REL_COMP_RES) goto err113;
  
  eder[0] = sder[0] + offset*(-sder[3]/hlpnorm);
  eder[1] = sder[1] + offset*(sder[2]/hlpnorm);
  if (ider > 0)
    {
       eder[2] = sder[2] + offset*(sder[2]*sder[3]*sder[4] - sder[2]
				   *sder[2]*sder[5])/pow(hlpnorm,3); 
       eder[3] = sder[3] + offset*(sder[3]*sder[3]*sder[4] - sder[2]
				   *sder[3]*sder[5])/pow(hlpnorm,3);
    }
  if (ider > 1)
    {
       eder[4] = sder[4] + offset*((pow(sder[2],3)*sder[3] 
		 + sder[2]*pow(sder[3],3))*sder[6] - (pow(sder[2],4) 
		 + (sder[2]*sder[2])*(sder[3]*sder[3]))*sder[7] 
		 + (pow(sder[3],3) - 2*sder[3]*(sder[2]*sder[2]))*
		 (sder[4]* sder[4]) + 2*(pow(sder[2],3) 
		 - 2*sder[2]*(sder[3]*sder[3]))*(sder[4]*
                 sder[5]) + 3*(sder[2]*sder[2])*sder[3]*sder[5]*sder[5])/
	         pow(hlpnorm,5);
       eder[5] = sder[5] + offset*(((sder[2]*sder[2])*(sder[3]*sder[3]) 
		 + pow(sder[3],4))*sder[6] - (pow(sder[2],3)*sder[3] 
		 + sder[2]*pow(sder[3],3))*sder[7] -3*sder[2]
		 *(sder[3]*sder[3])*(sder[4]*sder[4]) + 2*(2*(sder[2]
		 *sder[2]*sder[3]) - pow(sder[3],3))*(sder[4]*sder[5]) + 
		 (2*sder[2]*(sder[3]*sder[3]) - pow(sder[2],3))
		  *(sder[5]*sder[5]))/pow(hlpnorm,5);
    }
  
  
  
  freearray(ebder);
  freearray(sder);
  
  
  
  *jstat = 0;
  goto out;

 err101: *jstat = -101;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err102: *jstat = -102;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err110: *jstat = -110;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err111: *jstat = -111;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err112: *jstat = -112;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err113: *jstat = -113;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

 err178: *jstat = -178;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;


 error:  *jstat = kstat;
         s6err("ev_cv_off",*jstat,kpos);
         goto out;

out: return;
}

