/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define MAKE3D

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void
    make3D (SISLSurf *ps, SISLSurf **rsnew, int *jstat)
#else
void 
   make3D (ps, rsnew, jstat)
     SISLSurf *ps;
     SISLSurf **rsnew;
     int *jstat;
#endif
{
  
  int kk1,kk2,kn1,kn2;  
  double *st1,*st2,*scoef; 
  double *s3coef=SISL_NULL;  
  int kkm1,kkm2;        
  int kincre;           
  int ki,kj,kl,kstop;
  double tsum,*sp,*sq;
  //int kstat=0,kpos=0;
   
  if (!ps) goto errnull;
  if (ps->idim != 1) goto errdim;


  kk1   = ps -> ik1;
  kk2   = ps -> ik2;
  kn1   = ps -> in1;
  kn2   = ps -> in2;
  st1   = ps -> et1;
  st2   = ps -> et2;
  scoef = ps -> ecoef;

  
  
  if((s3coef = newarray(kn1*kn2*3,DOUBLE)) == SISL_NULL) goto err101;
  
  
  
  
  
  
  
  
  kkm1    = kk1 - 1;
  kincre  = 3*kn1;
  
  for (ki=0,kl=0,sp=s3coef ; ki<kn1 ; ki++,kl+=3,sp+=3)
    {
      tsum = (double)0.0;
      kstop = ki+kk1;
      for (kj=ki+1;kj<kstop;kj++)
        tsum +=st1[kj];
      
      tsum = tsum/kkm1;
      
      
      
      for (kj=0,sq=sp ; kj<kn2 ; kj++,sq+=kincre) *sq = tsum;
      
    }
  
  
  
  kkm2 = kk2 - 1;
  for (ki=0,sp=s3coef+1 ; ki< kn2 ; ki++)
    {
      tsum = (double)0.0;
      kstop = ki+kk2;
      for (kj=ki+1;kj<kstop;kj++)
        tsum +=st2[kj];
      
      tsum  = tsum/kkm2;
      
      
      
      for (kj=0 ; kj<kn1 ; kj++,sp+=3) *sp = tsum;
      
    }
  
  
  
  for (kj=0,sp=s3coef+2,sq=scoef ; kj < kn2 ; kj++)
     for (ki=0 ; ki<kn1 ; ki++,sp+=3,sq++)
	*sp = *sq;
  
  
  
  if(((*rsnew) = newSurf(kn1,kn2,kk1,kk2,st1,st2,s3coef,1,3,1)) == SISL_NULL) goto err101;
   
  goto out;

  
  
err101:
   *jstat = -101;
  s6err ("make3D", *jstat, 0);
  goto out;

  
errnull:
   *jstat = -200;
  s6err ("make3D", *jstat, 0);
  goto out;

    
errdim:
   *jstat = -201;
  s6err ("make3D", *jstat, 0);
  goto out;

out:
   if (s3coef) freearray(s3coef);
     
}

