/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define PO_CRV_TANG

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
   po_crv_tang(SISLCurve *pcurve, double point[], double ang_tol,
	    double guess_par, double *iter_par, int *jstat)
#else
      void po_crv_tang(pcurve, point, ang_tol, guess_par, iter_par, jstat)
	 SISLCurve   *pcurve;
	 double       point[];
	 double       ang_tol;
	 double       guess_par; 
	 double      *iter_par;
	 int         *jstat;
#endif
                                                               
{                                                                     
  int kstat   = 0;             
  int kpos    = 0;             
  int kdim    = 2;             
  int der0    = 0;               
  int der1    = 1;               
  int kleft   = 0;               
  int narr    = 1;               
  double zero = 0;              
  double iter_ang;             
  double diff[2];              
  double sder[4];              
  double trans_arr[9];         
  SISLCurve *testcurve = SISL_NULL; 
  SISLPoint *p1 = newPoint (&zero, 1, 0);
  
  
  
  if (!p1) goto err101;
  if (pcurve->idim != kdim) goto err105;
  
  
  trans_arr[0] = DZERO;
  trans_arr[1] = -1;
  trans_arr[2] = DZERO;
  trans_arr[3] = 1;
  trans_arr[4] = DZERO;
  trans_arr[5] = DZERO;
  trans_arr[6] = -point[1];
  trans_arr[7] =  point[0];
  trans_arr[8] = 1;

  
  
  s1893(pcurve, trans_arr,kdim+1,narr=1,der0=0,der1=1, &testcurve, &kstat);
  if (kstat < 0) goto error;
  
  
  
  s1771(p1, testcurve, REL_COMP_RES, testcurve->et[testcurve->ik - 1],
	testcurve->et[testcurve->in], guess_par, iter_par, &kstat); 
  if (kstat < 0) goto error;
  
  
  s1221(pcurve, der1=1, *iter_par, &kleft, sder, &kstat);
  if (kstat < 0) goto error;

  s6diff(sder,point,kdim,diff);
  iter_ang = s6ang(sder+2, diff, kdim);
  
  if (iter_ang < ang_tol)   *jstat = 1;
  else   *jstat = 2;

  goto out;
  

  

  
 err101: *jstat = -101;
  s6err("po_crv_tang",*jstat,kpos);
  goto out;
  
  
 err105: *jstat = -105;
  s6err("po_crv_tang",*jstat,kpos);
  goto out;
  
  
  error : *jstat = kstat;
  s6err("po_crv_tang",*jstat,kpos);
  goto out;
  
 out:
  
  if (testcurve) freeCurve(testcurve);
  if (p1)        freePoint(p1);
 
  return;
}                                               
                                           
                       

