/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1174

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void s1174_s9corr(double [],double,double,double,double,double,double);
static void s1174_s9dir(double *,double *,double []);
#else
static void s1174_s9corr();
static void s1174_s9dir();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
s1174(SISLSurf *psurf,double estart[],
     double eend[], double enext[], double gpos[],int *jstat)
#else
void s1174(psurf,estart,eend,enext,gpos,jstat)
     SISLSurf         *psurf;
     double       estart[];
     double       eend[];
     double       enext[];
     double       gpos[];
     int          *jstat;
#endif
{
  int kstat = 0;            
  int kpos = 0;             
  int kleft1=0;             
  int kleft2=0;             
  int kder=2;               
  int knbit;                
  int kdir;                 
  double tdelta[2];         
  double tdist = 0.0;       
  double tprev;             
  double td[2],t1[2],tdn[2];
  double sval[7];           
  double *snorm=sval+7;     
  double snext[2];          
  double tol = (double)10000.0*REL_COMP_RES; 
  

  
  if (psurf->idim != 1) goto err106;

  

  tdelta[0] = psurf->et1[psurf->in1] - psurf->et1[psurf->ik1 - 1];
  tdelta[1] = psurf->et2[psurf->in2] - psurf->et2[psurf->ik2 - 1];


  
  gpos[0] = enext[0];
  gpos[1] = enext[1];

  
  s1421(psurf,kder,gpos,&kleft1,&kleft2,sval,snorm,&kstat);
  if (kstat < 0) goto error;

  
  tprev = sqrt(sval[1]*sval[1] + sval[2]*sval[2]);

  
  s1174_s9dir(td,td+1,sval);

  if ( (fabs(td[0]/tdelta[0]) <= REL_COMP_RES) &&
      (fabs(td[1]/tdelta[1]) <= REL_COMP_RES))
     goto stop_it;

  
  t1[0] = td[0];
  t1[1] = td[1];
  s1174_s9corr(t1,gpos[0],gpos[1],estart[0],eend[0],estart[1],eend[1]);

  

  for (knbit = 0; knbit < 50; knbit++)
    {
      

      snext[0] = gpos[0] + t1[0];
      snext[1] = gpos[1] + t1[1];

      s1421(psurf,kder,snext,&kleft1,&kleft2,sval,snorm,&kstat);
      if (kstat < 0) goto error;

      
      tdist = sqrt(sval[1]*sval[1] + sval[2]*sval[2]);

      
      s1174_s9dir(tdn,tdn+1,sval);

      

      kdir = (s6scpr(td,tdn,2) >= DZERO);     

      if (tdist <= tprev || kdir)
	{
	  

          gpos[0] += t1[0];
          gpos[1] += t1[1];

          td[0] = t1[0] = tdn[0];
          td[1] = t1[1] = tdn[1];

	  
	  s1174_s9corr(t1,gpos[0],gpos[1],estart[0],eend[0],estart[1],eend[1]);


          if ( (fabs(t1[0]/tdelta[0]) <= REL_COMP_RES) &&
	      (fabs(t1[1]/tdelta[1]) <= REL_COMP_RES))
	    {
	      gpos[0] += t1[0];
	      gpos[1] += t1[1];

	      break;
	    }

          tprev = tdist;
	}

      else
	{
	  

          t1[0] /= (double)2;
          t1[1] /= (double)2;
	  
	}
    }

  

  stop_it:

  if (tdist <= tol)
    *jstat = 1;
  else
    *jstat = 0;


  
  if (fabs(gpos[0] - psurf->et1[kleft1])/tdelta[0] < tol)
    gpos[0] = psurf->et1[kleft1];
  else if (fabs(gpos[0] - psurf->et1[kleft1+1])/tdelta[0] < tol)
    gpos[0] = psurf->et1[kleft1+1];

  if (fabs(gpos[1] - psurf->et2[kleft2])/tdelta[1] < tol)
    gpos[1] = psurf->et2[kleft2];
  else if (fabs(gpos[1] - psurf->et2[kleft2+1])/tdelta[1] < tol)
    gpos[1] = psurf->et2[kleft2+1];

  
  goto out;

 
  
 err106: *jstat = -106;
  s6err("s1174",*jstat,kpos);
  goto out;

  
  error : *jstat = kstat;
  s6err("s1174",*jstat,kpos);
  goto out;

 out:;
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1174_s9corr(double gd[], double acoef1,double acoef2,double astart1,
		   double aend1,double astart2, double aend2)
#else
static void s1174_s9corr(gd,acoef1,acoef2,astart1,aend1,astart2,aend2)
     double gd[];
     double acoef1;
     double acoef2;
     double astart1;
     double aend1;
     double astart2;
     double aend2;
#endif
{
  if (acoef1 + gd[0] < astart1)  gd[0] = astart1 - acoef1;
  else if (acoef1 + gd[0] > aend1) gd[0] = aend1 - acoef1;

  if (acoef2 + gd[1] < astart2)  gd[1] = astart2 - acoef2;
  else if (acoef2 + gd[1] > aend2) gd[1] = aend2 - acoef2;
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1174_s9dir(double *cdiff1, double *cdiff2,double evals[])
#else
static void s1174_s9dir(cdiff1,cdiff2,evals)
     double *cdiff1;
     double *cdiff2;
     double evals[];
#endif
{
  double tdiv;		      
  double ta11,ta12,ta21,ta22; 
  double tmax;                
  double tb1,tb2;             
  double tderx,tderxx;        
  double tdery,tderyy;
  double tderxy;
  double tdeltax,tdeltay;   
  

  
  tderx  = evals[1];
  tdery  = evals[2];
  tderxx = evals[3];
  tderxy = evals[4];
  tderyy = evals[5];
  tdeltax = DZERO;
  tdeltay = DZERO;
  *cdiff1  = DZERO;
  *cdiff2  = DZERO;


  

  ta11 = tderxx;
  ta12 = tderxy;
  ta21 = tderxy;
  ta22 = tderyy;
  tb1  = -tderx;
  tb2  = -tdery;

  tmax = max(fabs(ta11),max(fabs(ta12),max(fabs(ta21),fabs(ta22))));

  if (DEQUAL(tb1+tmax,tmax) && DEQUAL(tb2+tmax,tmax))
    {
      
    }
  else
    {
      tdiv    = ta11*ta22 - ta21*ta12;
      if (fabs(tdiv) > MAX(tmax*REL_COMP_RES,REL_COMP_RES))
	{
	  
	  tdeltax = tb1*ta22 - tb2*ta12;
	  tdeltay = ta11*tb2 - ta21*tb1;
	  tdeltax /= tdiv;
	  tdeltay /= tdiv;
	}
      else if (max (fabs(ta11),fabs(ta22)) > REL_COMP_RES)
	{
	   if (fabs(ta11) > fabs(ta22))
	     tdeltax = tb1/ta11;
	   else
	     tdeltay = tb2/ta22;
	}

    }

  *cdiff1  = tdeltax;
  *cdiff2  = tdeltay;

}

