/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1220
#include "sislP.h"
 


#if defined(SISLNEEDPROTOTYPES)
void 
s1220(double *et,int ik,int in,int *ileft,
	   double ax,int ider,double ebder[],int *jstat)
#else
void s1220(et,ik,in,ileft,ax,ider,ebder,jstat)
     double *et;
     int    ik;
     int    in;
     int    *ileft;
     double ax;
     int    ider;
     double ebder[];
     int    *jstat;
#endif
                                     
{
  int kstat=0;        
  int kpos=0;         
  int kdeg;           
  int kleft;          
  int ki,kj,ks;       
  int ki1,ki2,kjh;    
  int kder;           
  double td1,td2;     
  double tw1,tw2;     
  double ts1,ts2;     
  double tt,tth;      
                                         
  
  if (ider < 0) goto err178;
  
  
  
  s1219(et,ik,in,ileft,ax,&kstat);
  
  if (kstat < 0) goto error;
  
  
  
  kleft = *ileft;
  kdeg = ik - 1;
  kder = min(ik-1,ider);
  
  
  
  ki2 = (ik-1)*(ider+1) + kder;
  ebder[ki2] = (double)1.0;
  
  if (ik == 1)
  {
     
     
     *jstat = 0;
     goto out;
  }
  
  
  
  ki1 = ki2 - ider - 2;
  if (kder == kdeg)
    {
      ebder[ki2-1] = (double)1.0;
      ki1 -= 1;
    }
  
  
  
  ki2 = ki1 + ider + 1;
  
  
  
  for(ks=1; ks<ik; ks++)
    {
      
      
      
      kj = kleft - ks + 1;
      kjh = kleft + 1;
      tt = et[kjh++];
      tth = tt - et[kj];
      if (tth <= (double)0.0) goto err112;
      td2 = (double)1.0/tth;
      tw2 = (tt-ax)*td2;
      
      ebder[++ki1] = tw2*ebder[++ki2];
      
      
      
      if (ks >= kdeg-kder && kder > 0)
	{
	  
	  
	  if (ks < kdeg)
	    ebder[ki1-1] = ebder[ki1];
	  
	  
	  
	  ts2 = ks*td2;
	  for (ki=0; ki<ks-kdeg+kder; ki++)
	    ebder[++ki1] = -ts2*ebder[++ki2];
	  
	  
	  
	  ki1 += ider - kder + kdeg - ks;
	  ki2 = ki1 + ider + 1;
	}
      else
	{
	  
	  
	  
	  ki1 += ider;
	  ki2 += ider;
	}
      
      
      
      for (kj=kleft-ks+2; kj<=kleft; kj++)
	{
	  
	  
	  
	  tt = et[kjh++];
	  tth = tt - et[kj];
	  if (tth <= (double)0.0) goto err112;
	  td1 = td2; td2 = (double)1.0/tth;
	  tw1 = (double)1.0 - tw2; tw2 = (tt-ax)*td2;
	  
	  ki1 += 1;
	  ebder[ki1] = tw1*ebder[ki1] + tw2*ebder[++ki2];
	  
	  
	  
	  if (ks >= kdeg-kder && kder > 0)
	    {
	      
	      
	      
	      if (ks < kdeg)
		ebder[ki1-1] = ebder[ki1];
	      
	      
	      
	      ts1 = ts2; ts2 = ks*td2;
	      for (ki=0; ki<ks-kdeg+kder; ki++)
		{
		  ki1 += 1;
		  ebder[ki1] = ts1*ebder[ki1] - ts2*ebder[++ki2];
		}
	      
	      
	      
	      ki1 += ider - kder + kdeg - ks;
	      ki2 = ki1 + ider + 1;
	    }
	  else
	    {
	      
	      
	      
	      ki1 += ider;
	      ki2 += ider;
	    }
	}
      
      
      
      td1 = td2;
      tw1 = (double)1.0 - tw2;
      
      ki1 += 1;
      ebder[ki1] = tw1*ebder[ki1];
      
      
      
      if (ks >= kdeg-kder && kder > 0)
	{
	  
	  
	  
	  if (ks < kdeg)
	    ebder[ki1-1] = ebder[ki1];
	  
	  
	  
	  ts1 = ts2;
	  for (ki=0; ki<ks-kdeg+kder; ki++)
	    {
	      ki1 += 1;
	      ebder[ki1] = ts1*ebder[ki1];
	    }
	  
	  
	  
	  ki1 -= ks - kdeg + kder + 1;
	}
      
      
      
      ki1 -= (ks+1)*(ider+1) + 1;
      ki2 = ki1 + ider + 1;
    }
  
  
  
  for (ki=kder+1; ki<=ider; ki++)
    {
      ki1 = ki;
      for (kj=0; kj<ik; kj++)
	{
	  ebder[ki1] = (double)0.0;
	  ki1 += ider + 1;
	}
    }
  
  
  
  *jstat = 0;
  goto out;
  
  
  
 err112: *jstat = -112;
  s6err("s1220",*jstat,kpos);
  goto out;
  
  
 err178: *jstat = -178;
  s6err("s1220",*jstat,kpos);
  goto out;
  
  
  
 error:  *jstat = kstat;
  s6err("s1220",*jstat,kpos);
  goto out;
  
 out: return;
}

