/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1222

#include "sislP.h"
 


#if defined(SISLNEEDPROTOTYPES)
void 
   s1222(double et[], int ik, int in, int ibase, 
	   double ax, int ider, double ebder[], int *jstat)
#else
void s1222(et, ik, in, ibase, ax, ider, ebder, jstat)
     double et[];
     int    ik;
     int    in;
     int    ibase;
     double ax;
     int    ider;
     double ebder[];
     int    *jstat;
#endif
                                     
{
   int kpos=0;          
   int ki, kj, kl;      
   int degree;          
   int numb_places;     
   int offset;          
   int pre_deriv;       
   int startA;          
   int stopA;           
   int indexA;          
   int indexB;          
   int kder;            
   int kleft;           
   int idx1;            
   int lknot, rknot;    
   double width;         
   double td1, td2;     
   double tw1, tw2;     
   double ts1, ts2;     

   *jstat = 0;
  
   
  
   degree = ik - 1;
  
   
  
   if (in < ik || ik < 1 || ibase < 0 || ibase > (in - 1))
      goto err112;
   
   
   
   if (ider < 0)
      goto err178;
  
   
   
   if ((ax < et[ibase] && et[ibase] > et[ik - 1])
       || (ax > et[ibase + ik] && et[ibase + ik] < et[in]))
   {
      stopA = ik*(ider + 1);
      for (ki = 0; ki < stopA; ki++)
	 ebder[ki] = 0.;
      goto out;
   }
   else
   {
      for (kleft = max(ibase, ik - 1); kleft < (ibase + ik); kleft++)
      {
	 if (ax < et[kleft + 1] || kleft == (in - 1))
	    break;
      }
   }
   offset = kleft - ibase;
   
   kder = min(degree, ider);
   numb_places = kder + 1;
   pre_deriv = degree - kder;
   
   
   
   ebder[0] = 1.;
   
   for (ki = 1; ki < ik; ki++)
   {
      
      startA = min(offset + 1, ki);
      stopA  = max(0, (offset - ik + ki));
      
      
      
      lknot = kleft - startA + 1;
      rknot = lknot + ki;
      width = et[rknot] - et[lknot];
      if (width < REL_PAR_RES)
	 goto err112;
      td2 = (double)1./width;
      tw2 = (et[rknot] - ax)*td2;
      ts2 = ki*td2;
      
      indexB = numb_places*startA;
      indexA = indexB - numb_places;
      
      if (ki < (offset + 1))
      {
	 
	 
	 
	 ebder[indexB] = tw2*ebder[indexA];
	 
	 
	 
	 if (ki > pre_deriv && kder > 0)
	 {
	    for (kl = 1; kl < (ki - pre_deriv + 1); kl++)
	       ebder[indexB + kl] = - ts2*ebder[indexA + kl - 1];
	    
	 }
	 
      }
      
      lknot++;
      rknot++;
      indexB = indexA;
      indexA -= numb_places;
     
      
      
      for (kj = (startA - 1); kj > stopA; kj--)
      {
	 
	 width = et[rknot] - et[lknot];
	 if (width < REL_PAR_RES)
	    goto err112;
	 td1 = td2;
	 td2 = (double)1./width;
	 tw1 = (double)1. - tw2;
	 tw2 = (et[rknot] - ax)*td2;
	 
	 
	 
	 if (ki > pre_deriv && kder > 0)
	 {
	    ts1 = ts2;
	    ts2 = ki*td2;
	    for (kl = (ki - pre_deriv); kl > 0; kl--)
	       ebder[indexB + kl] = ts1*ebder[indexB + kl - 1] 
		  - ts2*ebder[indexA + kl - 1];
	    
	 }
	 
	 
	 
	 ebder[indexB] = tw1*ebder[indexB] + tw2*ebder[indexA];
	 
	 indexB = indexA;
	 indexA -= numb_places;
	 lknot++;
	 rknot++;
      }
      
      
      
      if (ki < (ik - offset))
      {
	 
	 
	 
	 tw1 = (double)1. - tw2;
	 
	 
	 
	 if (ki > pre_deriv && kder > 0)
	 {
	    ts1 = ts2;
	    for (kl = (ki - pre_deriv); kl > 0; kl--)
	       ebder[indexB + kl] = ts1*ebder[indexB + kl - 1]; 
	    
	 }
	 
	 
	 
	 ebder[indexB] = tw1*ebder[indexB];
	 
      }
      
   }
   
   
   
   idx1 = offset*numb_places;
   for (ki = 0; ki < numb_places; ki++)
      ebder[ki] = ebder[idx1 + ki];
   
   stopA = ik*(ider + 1);
   for ( ; ki < stopA; ki++)
      ebder[ki] = 0.;
   
   
   
   goto out;
   
   
   
   err112: 
      *jstat = -112;
   s6err("s1222",*jstat,kpos);
   goto out;
   
   
   
   err178: 
      *jstat = -178;
   s6err("s1222",*jstat,kpos);
   goto out;


   out: 
      return;
}

