/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1236

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1236(double et[],int in,int ik,int inpar,double epar[],int *jstat)
#else
void s1236(et,in,ik,inpar,epar,jstat)
     double et[];
     int    in;
     int    ik;
     int    inpar;
     double epar[];
     int    *jstat;
#endif
{
  int kstat = 0;   
  int kpos = 0;    
  int ki,kj;       
  int kpar;        
  int knumb;       
  int knbreak = 0; 
  double tend;     
  double tlast;    
  double tval;     
  double tdist;    
  double *spar;    
  double *sbreak = SISL_NULL;  
  
  
  
  if (ik < 1) goto err110;
  if (in < ik) goto err111;
  
  
  
  s1235(et,in,ik,&knbreak,&sbreak,&kstat);
  if (kstat < 0) goto error;
  
  
  
  kpar = inpar - knbreak;
  
  

  if (kpar < 0)
    {
      sbreak[1] = sbreak[knbreak-1];
      knbreak = 2;
      kpar = inpar - 2;
    }
  
  
  
  spar = epar;
  tlast = *spar = sbreak[0];
  tend = sbreak[knbreak-1];
  
  for (spar++,ki=1; ki<knbreak; ki++)
    {
      
      
      
      tval = sbreak[ki];
      tdist = tval - tlast;
      knumb = (int)(kpar*tdist/(tend-tlast));
      
      
      
      kpar -= knumb;
      
      
      
      tdist /= (knumb + 1);
      
      
      
      for (kj=0; kj<knumb; kj++) *(spar++) = tlast + (kj+1)*tdist;
      
      
      
      *(spar++) = tval;
      tlast = tval;
    }
  
  
  
  *jstat = 0;
  goto out;
  
  
  
 err110: *jstat = -110;
  s6err("s1236",*jstat,kpos);
  goto out;
  
  
  
 err111: *jstat = -111;
  s6err("s1236",*jstat,kpos);
  goto out;
  
  
  
  error : *jstat = kstat;
  s6err("s1236",*jstat,kpos);
  goto out;
  
 out:
  
  
  
  if (sbreak != SISL_NULL) freearray(sbreak);
  
  return;
}

