/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1239

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void s1239_s9sort(double [],int [],int);
#else
static void s1239_s9sort();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
s1239(SISLCurve *pcpar,int ipar,double apar,SISLCurve *pcurve,double aepsco,
	   double aepsge,SISLCurve **vpartc,int imax,int *jpartc,int *jstat)
#else
void s1239(pcpar,ipar,apar,pcurve,aepsco,aepsge,vpartc,imax,jpartc,jstat)
     SISLCurve  *pcpar;
     int    ipar;
     double apar;
     SISLCurve  *pcurve;
     double aepsco;
     double aepsge;
     SISLCurve  **vpartc;
     int    imax;
     int    *jpartc;
     int    *jstat;
#endif
{
  int kstat = 0;      
  int kpos = 0;       
  int ki,kj;          
  int kpt;            
  int kcrv;           
  int kleft = 0;      
  int kder = 1;       
  int kpar = 0;       
  int *linter = SISL_NULL; 
  int kinter;         
  int kpartc = 0;     
  double tzero = (double)0.0;   
  double tstart,tend; 
  double tclstart,tclend; 
  double t1,t2;       
  double tchange;     
  double spoint[2];   
  double snorm[2];    
  double *spt = SISL_NULL; 
  double sder1[4];    
  double sder2[4];    
  double *spar = SISL_NULL;      
  SISLIntcurve **ucrv;   
  SISLCurve *qc1 = SISL_NULL; 
  SISLCurve *qc2 = SISL_NULL; 

  

  if (pcurve -> idim != 2) goto err108;

  

  tclstart = *(pcurve->et + pcurve->ik - 1);
  tclend   = *(pcurve->et + pcurve->in);

  

  tstart = *(pcpar->et + pcpar->ik - 1);
  tend = *(pcpar->et + pcpar->in);

  if (ipar == 0)
    {
      spoint[0] = apar;  spoint[1] = tstart;
      snorm[0] = tend-tstart;   snorm[1] = tzero;
    }
  else
    {
      spoint[0] = tstart;    spoint[1] = apar;
      snorm[0] = tzero;       snorm[1] = tend-tstart;
    }

  

  s1850(pcurve,spoint,snorm,2,aepsco,aepsge,&kpt,&spt,&kcrv,&ucrv,&kstat);
  if (kstat < 0) goto error;

  

  for (kj=0,ki=0; ki<kpt; ki++)
    {
      if (s6equal(spt[ki],tclstart,(double)1.0) ||
	  s6equal(spt[ki],tclend,(double)1.0))
	{
	  if (kj == 0) kj++;
	  else
	    {
	      for (kj=ki+1; kj<kpt; kj++) spt[kj-1] = spt[kj];
	      kpt--;
	    }
	}
    }

  

  if (kpt+2*kcrv > 0)
    {
      if ((spar = newarray(kpt+2*kcrv,double)) == SISL_NULL) goto err101;
      if ((linter = new0array(kpt+2*kcrv,int)) == SISL_NULL) goto err101;
    }

  

  for (ki=0; ki<kcrv; ki++)
    {

      

      t1 = *((*(ucrv+ki))->epar1);
      t2 = *((*(ucrv+ki))->epar1 + (*(ucrv+ki))->ipoint - 1);
      if (t2 < t1)
	{
	  tchange = t1;  t1 = t2;  t2 = tchange;
	}

      

      s1227(pcurve,kder,t1,&kleft,sder1,&kstat);
      if (kstat < 0) goto error;

      s1221(pcurve,kder,t2,&kleft,sder2,&kstat);
      if (kstat < 0) goto error;

      

      if (sder1[2+ipar]*sder2[2+ipar] > tzero)
	kinter = 2;    
      else kinter = 1;  

      

      t1 = MAX(MIN(sder1[1-ipar],sder2[1-ipar]),tstart);
      t2 = MIN(MAX(sder1[1-ipar],sder2[1-ipar]),tend);

      

      linter[kpar] = kinter;   spar[kpar++] = t1;
      if (kpar == imax) goto err190;
      linter[kpar] = kinter;   spar[kpar++] = t2;
      if (kpar == imax) goto err190;
    }

  

  for (ki=0; ki<kpt; ki++)
    {

      

      s1221(pcurve,kder,spt[ki],&kleft,sder1,&kstat);
      if (kstat < 0) goto error;

      if (!(s6equal(spt[ki],tclstart,(double)1.0) ||
	    s6equal(spt[ki],tclend,(double)1.0)))
	{

	  

	  s1227(pcurve,kder,spt[ki],&kleft,sder2,&kstat);
	  if (kstat < 0) goto error;
	}
      else
	{

	  

	  t1 = (s6equal(spt[ki],tclstart,(double)1.0)) ? tclend : tclstart;
	  s1221(pcurve,kder,t1,&kleft,sder2,&kstat);
	  if (kstat < 0) goto error;
	}

      if (DEQUAL(sder1[2]*sder2[3] - sder1[3]*sder2[2],(double)0.0))
	{

	  

	  if (DEQUAL(sder1[2+ipar],(double)0.0))
	    {

	      

	      s1710(pcurve,spt[ki],&qc1,&qc2,&kstat);
	      if (kstat < 0) goto error;

	      

	      t1 = apar;  kj = 0;
	      while (qc1 && DEQUAL(t1,apar))
		{
		  kj++;
		  t1 = *(qc1->ecoef+2*(qc1->in-kj-1)+ipar);
		}

	      t2 = apar;  kj = 0;
	      while (qc2 && DEQUAL(t2,apar))
		{
		  kj++;
		  t2 = *(qc2->ecoef+2*kj+ipar);
		}

	      

	      if (qc1 != SISL_NULL) freeCurve(qc1);  qc1 = SISL_NULL;
	      if (qc2 != SISL_NULL) freeCurve(qc2);  qc2 = SISL_NULL;
	    }

	  if (DNEQUAL(sder1[2+ipar],(double)0.0) ||
	      (t1-apar)*(t2-apar) < (double)0.0)
	    {

	      

	      t1 = sder1[1-ipar];
	      if (t1 < tstart) t1 = tstart;
	      else if (t1 > tend) t1 = tend;

	      

	      spar[kpar++] = t1;
	      if (kpar == imax) goto err190;
	    }
	}
      else
	{

	  

	  if (sder1[2+ipar]*sder2[2+ipar] > (double)0.0)
	    {

	      

	      t1 = sder1[1-ipar];
	      if (t1 < tstart) t1 = tstart;
	      else if (t1 > tend) t1 = tend;

	      

	      spar[kpar++] = t1;
	      if (kpar == imax) goto err190;
	    }
	}
    }

  

  s1239_s9sort(spar,linter,kpar);

  

  ki = 0;
  while (ki < kpar-1)
    {

      

      kj = ki + (linter[ki] == 0 && linter[ki+1] > 0) + 1;
      while (kj < kpar-1 && ((ki < kj-1 && linter[kj] == 1) ||
			     (ki == kj-1 && linter[kj] == 2))) kj += ((linter[kj+1] > 0) + 1);
      kj = MIN(kj,kpar-1);

      if (DNEQUAL(spar[kj],spar[ki]))
	{

	  

	  s1712(pcpar,spar[ki],spar[kj],vpartc+kpartc,&kstat);
	  if (kstat < 0) goto error;

	  

	  ki = kj + 1;
	  kpartc++;
	}
      else ki = kj + 1;
    }

  

  *jpartc = kpartc;
  *jstat = 0;
  goto out;

  

 err101: *jstat = -101;
  s6err("s1239",*jstat,kpos);
  goto out;

  

 err108: *jstat = -108;
  s6err("s1239",*jstat,kpos);
  goto out;

  

 err190: *jstat = -190;
  s6err("s1239",*jstat,kpos);
  goto out;

  

  error : *jstat = kstat;
  s6err("s1239",*jstat,kpos);
  goto out;

 out:

  

  if (spt != SISL_NULL) free(spt);
  if (spar != SISL_NULL) freearray(spar);
  if (linter != SISL_NULL) freearray(linter);

  return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1239_s9sort(double epar[],int nint[],int ipar)
#else
static void s1239_s9sort(epar,nint,ipar)
     double epar[];
     int    nint[];
     int    ipar;
#endif
{
  int ki,kj;      
  int kchange;    
  double tchange; 

  for (ki=0; ki<ipar; ki++)
    for (kj=ki+1; kj<ipar; kj++)
      if (epar[kj] < epar[ki])
	{
	  tchange = epar[ki];  epar[ki] = epar[kj];  epar[kj] = tchange;
	  kchange = nint[ki];  nint[ki] = nint[kj];  nint[kj] = kchange;
	}

  return;
}

