/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1251

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1251(SISLCurve *pcurve,double aepsco,double *clength,int *jstat)
#else
void s1251(pcurve,aepsco,clength,jstat)
     SISLCurve  *pcurve;
     double aepsco;
     double *clength;
     int    *jstat;
#endif
{
  int kstat = 0;            
  int kpos = 0;             
  int ki;                   
  int kn;                   
  int kk;                   
  int kdim;                 
  int knbez;                
  double tpol = (double)0.0;
  double tdist =(double)0.0;
  double tdum;              
  double tmid;              
  double tlength1,tlength2; 
  double *s1;               
  SISLCurve *qc1 = SISL_NULL;        
  SISLCurve *qc2 = SISL_NULL;        

  

  kn = pcurve -> in;
  kk = pcurve -> ik;
  kdim = pcurve -> idim;

  

  for (ki=1,s1=pcurve->ecoef+kdim; ki<kn; ki++,s1+=kdim)
    tpol += s6dist(s1-kdim,s1,kdim);

  

  tdist = s6dist(pcurve->ecoef,pcurve->ecoef+(kn-1)*kdim,kdim);

  

  if (DEQUAL(tpol + tdist,(double)0.0)) tdum = (double)0.0;
  else tdum = (tpol - tdist)/(tpol + tdist);

  if (tdum < aepsco)

    

    *clength = tdist;
  else if ((tdist <= REL_COMP_RES && tpol <= (double)10*REL_COMP_RES) ||
	   tpol-tdist <= REL_COMP_RES)

     

     *clength = (double)0.5*(tdist+tpol);
  else if (pcurve->ik == pcurve->in)
    {

      

      tmid = ((double)0.5)*(*(pcurve->et+pcurve->ik-1) + *(pcurve->et+kn));

      s1710(pcurve,tmid,&qc1,&qc2,&kstat);
      if (kstat < 0) goto error;

      

      s1251(qc1,aepsco,&tlength1,&kstat);
      if (kstat < 0) goto error;

      

      s1251(qc2,aepsco,&tlength2,&kstat);
      if (kstat < 0) goto error;

      *clength = tlength1 + tlength2;
    }
  else
  {
     

     s1730(pcurve, &qc1, &kstat);
     if (kstat < 0) goto error;

     

     
     

     knbez = qc1->in/kk;

     tlength1 = DZERO;
     for (ki=0; ki<knbez; ki++)
     {
	

	if (qc1->ikind == 1 || qc1->ikind == 3)
	   qc2 = newCurve(kk, kk, qc1->et+ki*kk, qc1->ecoef+ki*kk*kdim,
			  qc1->ikind, kdim, 0);
	else
	   qc2 = newCurve(kk, kk, qc1->et+ki*kk, qc1->rcoef+ki*kk*(kdim+1),
			  qc1->ikind, kdim, 0);

	if (qc2 == SISL_NULL) goto err101;

      

      s1251(qc2,aepsco,&tlength2,&kstat);
      if (kstat < 0) goto error;

      tlength1 += tlength2;

      if (qc2 != SISL_NULL) freeCurve(qc2);
      qc2 = SISL_NULL;
     }

     *clength = tlength1;
  }

  

  *jstat = 0;
  goto out;

  

  err101: *jstat = -101;
  s6err("s1251",*jstat,kpos);
  goto out;

  

  error : *jstat = kstat;
  s6err("s1251",*jstat,kpos);
  goto out;

 out:

  

  if (qc1 != SISL_NULL) freeCurve(qc1);
  if (qc2 != SISL_NULL) freeCurve(qc2);

  return;
}

