/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1252

#include "sislP.h"

#if defined (SISLNEEDPROTOTYPES)
static void s1252_s6corr(double *,double,double [],int,int,int *,int *);
static void s1252_s6dir(double *,double,double [],double,double);
#else
static void s1252_s6corr();
static void s1252_s6dir();
#endif

#if defined (SISLNEEDPROTOTYPES)
void

     s1252(SISLCurve *pcurve,double aepsge,double astart,double *cpos,int *jstat)
#else
void s1252(pcurve,aepsge,astart,cpos,jstat)
     SISLCurve  *pcurve;
     double aepsge;
     double astart;
     double *cpos;
     int    *jstat;
#endif
{
  int kstat = 0;        
  int kpos = 0;         
  int kleft=0;          
  int kder=3;           
  int kdim;             
  int knbit;            
  int kn,kk;            
  int kdir=1;           
  double tstart,tend;   
  double tdelta;        
  double tdist=DZERO;   
  double td;        	
  double tnext;         
  double tprev;         
  double sval[4];       
  double *st;           
  double ref;           

  

  if (pcurve->idim != 1) goto err106;

  kdim = pcurve -> idim;

  

  st = pcurve->et;
  kn = pcurve->in;
  kk = pcurve->ik;

  tstart = *(pcurve->et + pcurve->ik - 1);
  tend   = *(pcurve->et + pcurve->in);
  tdelta = tend - tstart;
  if (tdelta == DZERO) tdelta = fabs(tend);
  if (tdelta == DZERO) tdelta = (double)1.0;

  

  tnext = astart;

  

  s1221(pcurve,kder,tnext,&kleft,sval,&kstat);
  if (kstat < 0) goto error;

  tprev = sval[0];

  

  s1252_s6dir(&td,tnext,sval,tstart,tend);

  

  s1252_s6corr(&td,tnext,st,kn,kk,&kleft,&kdir);

  

  for (knbit = 0; knbit < 20; knbit++)
    {

      

      if (kdir == -2 || kdir == 2)
	{
	  double tder1,tder2;
	  

	  s1221(pcurve,kder,tnext,&kleft,sval,&kstat);
	  if (kstat < 0) goto error;
	  tder2 = sval[1];

	  s1227(pcurve,kder,tnext,&kleft,sval,&kstat);
	  if (kstat < 0) goto error;
	  tder1 = sval[1];

	  

	  if (tder1>=DZERO && tder2<=DZERO) break;

	  
	}


      

      if (kdir>=1)
	{
	  s1221(pcurve,kder,tnext+td,&kleft,sval,&kstat);
	  if (kstat < 0) goto error;
	}
      else
	{
	  s1227(pcurve,kder,tnext+td,&kleft,sval,&kstat);
	  if (kstat < 0) goto error;
	}

        tdist = sval[0];
        if (fabs(tdist) < (double)1.0) ref = (double)2.0;
	else                           ref = DZERO;

        if (tdist >= tprev || DEQUAL(ref+tdist,ref+tprev))
	{
	   tnext += td;

	   
	   s1252_s6dir(&td,tnext,sval,tstart,tend);
	   s1252_s6corr(&td,tnext,st,kn,kk,&kleft,&kdir);

	   if (fabs(td/tdelta) <= REL_COMP_RES) break;

	   tprev = tdist;

	}

      

      else
	{

	  td /= (double)2;
	  if (fabs(td/tdelta) <= REL_COMP_RES) break;
	}


    }


  

  if (tdist <= aepsge)
    *jstat = 1;
  else
    *jstat = 2;

  
  
  if (DEQUAL(tnext,pcurve->et[kleft]))
    *cpos = pcurve->et[kleft];
  else if (DEQUAL(tnext,pcurve->et[kleft+1]))
    *cpos = pcurve->et[kleft+1];
  else
    *cpos = tnext;

  

  goto out;


  

 err106: *jstat = -106;
  s6err("S1252",*jstat,kpos);
  goto out;

  

  error : *jstat = kstat;
  s6err("S1252",*jstat,kpos);
  goto out;

 out:;
}

#if defined (SISLNEEDPROTOTYPES)
static void
   s1252_s6corr(double *gdn,double acoef,double et[],
		int in,int ik,int *ileft,int *jdir)
#else
static void s1252_s6corr(gdn,acoef,et,in,ik,ileft,jdir)
     double *gdn;
     double acoef;
     double et[];
     int    in;
     int    ik;
     int    *ileft;
     int    *jdir;
#endif
{
  int kmult,kstat;

  

  *gdn = MAX(et[ik-1]-acoef,*gdn);
  *gdn = MIN(et[in]  -acoef,*gdn);

  if (acoef+*gdn<et[*ileft] && acoef>et[*ileft])
    {
      *gdn = MAX(et[*ileft]-acoef,*gdn);
    }

  else if(acoef<et[*ileft+1] && acoef+*gdn>et[*ileft+1])
    {
      

      *gdn = MIN(et[*ileft+1]-acoef,*gdn);
    }

  

  if (*gdn>=0)
    {
      *jdir = 1;
    }
  else
    {
      *jdir = -1;
    }

  kmult = s6knotmult(et,ik,in,ileft,acoef,&kstat);

  if (acoef==et[*ileft])
    {

      if(kmult>ik-2)
        {
	  if (*jdir == -1)
            {
	      *jdir = -2;
            }
	  else
            {
	      *jdir =  2;
            }
        }
    }
}

#if defined (SISLNEEDPROTOTYPES)
static void
  s1252_s6dir(double *cdiff,double acoef,double eval[],double astart,
	      double aend)
#else
static void s1252_s6dir(cdiff,acoef,eval,astart,aend)
     double *cdiff;
     double acoef;
     double eval[];
     double astart;
     double aend;
#endif
{
  double t1,t2,t3,t4,t5,t6;   
  double tmax;                
  double ttol=(double)1e-10;  

  
  t1=acoef;
  t2=astart;
  t3=aend;


  t1 =  eval[1];
  t2 =  eval[2];
  t3 =  eval[3]/(double)2.0;

  tmax  = max(fabs(t1),fabs(t2));
  tmax  = max(fabs(t3),tmax);

  if (DEQUAL(tmax,DZERO))                    *cdiff = DZERO;
  else if (fabs(t3)/tmax < ttol) 
	{
          if (fabs(t2) == DZERO )      *cdiff = DZERO;
	  else                        *cdiff = (-t1/t2);
	}
  else
	{
          
	   t4 = t2*t2 - (double)4*t3*t1;
	   if (t4 < DZERO)
	    {
	      
	      if (fabs(t2) == DZERO )      *cdiff = DZERO;
              else                        *cdiff = (-t1/t2);
      	    }

           else
	    {
	       t6 = sqrt(t4);
	       t5 = (-t2 + t6)/((double)2*t3);
	       t6 = (-t2 - t6)/((double)2*t3);
	       t4 = min(fabs(t5),fabs(t6));

               

               if (t4 == DZERO)
                {
	          
	          if (fabs(t2) == DZERO )      *cdiff = DZERO;
                  else                        *cdiff = (-t1/t2);
	        }
               else if (fabs(t5) <= fabs(t6))  *cdiff = t5;
               else                            *cdiff = t6;
             }
	}
}

