/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1301

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1301(double areler,double angle,int idim,SISLCurve **pc,int *jstat)
#else
void s1301(areler,angle,idim,pc,jstat)
     double areler;
     double angle;
     int    idim;
     SISLCurve  **pc;
     int    *jstat;
#endif
{
  int    kant;           
  int    kl;             
  int    kj;             
  int    ki;             
  int    kn;             
  int    kk = 4;         
  int    kpos = 1;       
  int    kstart,kstop;   
  double tangle;         
  double talfa;          
  double tcos,tsin;      
  double *st = SISL_NULL;     
  double *scoef = SISL_NULL;  
  double ta,tb,tc;       
  double tl;             
  double tconst;         
  
  
  if (areler<=(double)0.0) goto err120;
  
  
  if (idim<2) goto err103; 
  
  
  
  tangle = fabs(angle);
  
  
  
  
  if (tangle >= TWOPI)
    {
      tangle = TWOPI;
    }
  
                                                    
  ta = (double)1.0/(double)6.0;
  talfa = PI*pow(areler,ta)/((double)0.4879);
  
  
  
  kant = (int) ((tangle/talfa)+1);
  
  
  
  talfa = tangle/kant;
  tcos = cos(talfa);
  tsin = sin(talfa);
  
  
  
  tconst = (double)1.85530139760811990992528773586425;
  ta     = (double)0.6*tconst - (double)0.9*tcos;
  tb     = ((double)0.4*tconst+(double)1.8)*tsin;
  tc     = ((double)0.4*tconst+(double)1.0)*tcos -
    (double)0.4*tconst - (double)1.0;
  tl     = (-tb+sqrt(tb*tb-(double)4.0*ta*tc))/((double)2.0*ta);
  
  
  
  kn = 2*kant + 2;
  scoef = newarray(kn*idim,DOUBLE);
  st    = newarray(kn+kk,DOUBLE);                       
  if (scoef == SISL_NULL || st == SISL_NULL) goto err101;
  
  
  
  
  kl = 0;
  kj = 2;
  
  
  
  kstop = idim*kn;
  for (ki=0;ki<kstop;ki++)
    {
      scoef[ki] = (double)0.0;
    }
  
  
  
  if (tangle >= TWOPI)
    {
      st[0] = (double)-1;  
      st[1] = (double)-1;
      kstart = 0;
      kstop = kant + 1;
      
      
      
      kj=2;
      
    }
  
  else
    {
      
      
      
      
      tcos = (double)1.0;
      tsin = (double)0.0;
      scoef[0] = (double)1.0;
      scoef[1] = (double)0.0;
      scoef[idim] = (double)1.0;
      scoef[idim+1] = tl;
      st[0] = (double)0.0;  
      st[1] = (double)0.0;
      st[2] = (double)0.0;
      st[3] = (double)0.0;
      kstart = 1;
      kstop = kant;
      
      
      
      kj = 4;
      
    }  
  
  
  for (ki=kstart;ki<kstop;ki++)
    {
      kl = 2*ki*idim;
      talfa = ki*tangle/kant;
      
      
      
      
      tcos = cos(talfa);
      tsin = sin(talfa);
      
      
      scoef[kl]        = tcos + tl*tsin;
      scoef[kl+1]      = tsin - tl*tcos;
      scoef[kl+idim]   = tcos - tl*tsin;
      scoef[kl+idim+1] = tsin + tl*tcos;
      st[kj++] = ki;
      st[kj++] = ki;
    }                     
  
  if (tangle >= TWOPI)
    {
      st[kn+2] = kant+1;
      st[kn+3] = kant+1;
      
      
      
      
      ki = (kn-2)*idim;
      scoef[ki] = scoef[0];
      scoef[ki+1] = scoef[1];
      scoef[ki+2] = scoef[2];
      scoef[ki+3] = scoef[3];
      
      
    }
  else
    {
      
      
      
      tcos = cos(tangle);
      tsin = sin(tangle);
      kl = 2*kant*idim;;
      scoef[kl]        = tcos + tl*tsin;
      scoef[kl+1]      = tsin - tl*tcos;
      scoef[kl+idim]   = tcos;
      scoef[kl+idim+1] = tsin;
      st[kn]   = kant;
      st[kn+1] = kant;
      st[kn+2] = kant;
      st[kn+3] = kant;
    }
  
  
  
  
  if (angle<(double)0.0)
    {
      kl = 1;
      for (ki=0;ki<kn;ki++)
	{
	  scoef[kl] = -scoef[kl];
	  kl+=idim;
	}
    }
  
  
  
  *pc = newCurve(kn,kk,st,scoef,1,idim,1);
  if (*pc == SISL_NULL) goto err101;

  if (tangle >= TWOPI)
    (*pc)->cuopen = SISL_CRV_PERIODIC;
  
  
  
  *jstat = 0;
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("s1301",*jstat,kpos);
  goto out;
  
  
  
 err120: *jstat = -120;
  s6err("s1301",*jstat,kpos);
  goto out;
  
  
  
 err103: *jstat = -103;
  s6err("s1301",*jstat,kpos);
  goto out;
  
  
 out:
  
  if (st != SISL_NULL)    freearray(st);
  if (scoef != SISL_NULL) freearray(scoef);
  
  return;
  
}
    

