/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1333

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1333(int inbcrv,SISLCurve *vpcurv[],int nctyp[],double astpar,
	   int iopen,int iord2,int iflag,
	   SISLSurf **rsurf,double **gpar,int *jstat)
#else
void s1333(inbcrv,vpcurv,nctyp,astpar,iopen,iord2,
           iflag,rsurf,gpar,jstat)
     int    	inbcrv;
     SISLCurve  *vpcurv[];
     int   	nctyp[];
     double	astpar;
     int    	iopen;
     int    	iord2;
     int    	iflag;
     SISLSurf   **rsurf;
     double 	**gpar;
     int    	*jstat;
#endif
{
  int kind,kcopy,kdim;
  int kn1,kord1,knbcrv;
  int kcnsta,kcnend;         
  int ki,kj,kl,km;
  int kleng;                 
  int ktype;                 
  int kopen;                 
  SISLCurve *qc;             
  int *lder = SISL_NULL;	     
  double *spar=SISL_NULL; 	     
  double *spar2=SISL_NULL; 	     
  double *sknot1=SISL_NULL;       
  double *scoef2=SISL_NULL;       
  double tstpar;             
  int kstat = 0;             
  int kpos = 0;              
  int knbpar;                
  int kdimcrv;               
  int kcont;                 
  double *etyp = SISL_NULL;


  if (iopen==SISL_CRV_CLOSED) iopen = SISL_CRV_PERIODIC;




  etyp = newarray (inbcrv, DOUBLE);
  if (etyp == SISL_NULL)
    goto err101;

  

  kdim = vpcurv[0]->idim;

  if (inbcrv < 2) goto err179;

  

  s1931 (inbcrv, vpcurv, &sknot1, &scoef2, &kn1, &kord1, &kstat);
  if (kstat < 0)
    goto error;

  

  s1917 (inbcrv, scoef2, kn1, kdim, nctyp, astpar, iopen,
	 &spar2, &lder, &knbcrv, &kstat);

  if (kstat < 0)
    goto error;

  

  kleng = kn1*kdim;
  for (ki=0 ; ki<knbcrv ; ki++)
    {
       ktype = nctyp[ki];

      if (ktype == 13 && ki+1<knbcrv)
        {
	  

	  for (kj=ki*kleng,kl=kj+kleng,km=0; km <kleng ; kj++,kl++,km++)
	      scoef2[kj] = scoef2[kl] - scoef2[kj];
	  nctyp[ki] = 3;
        }
      else if (ktype == 14 && ki>0)
        {
	  

	  for (kj=ki*kleng,kl=kj-kleng,km=0; km <kleng ; kj++,kl++,km++)
	      scoef2[kj] = scoef2[kj] - scoef2[kl];
	  nctyp[ki] = 4;
        }
    }

  spar = newarray(knbcrv+1,DOUBLE);
  if (spar==SISL_NULL) goto err101;

  

  for (ki=0,kl=0; ki<knbcrv ; ki++)
    {
      if (nctyp[ki] == 1 || nctyp[ki] == 2)
        {
	  spar[kl] = spar2[ki];
	  kl++;
        }
    }

  

  if (iopen != SISL_CRV_OPEN) spar[kl] = spar2[knbcrv];

  

  if (iflag)
    {
      s1918 (knbcrv, sknot1, scoef2, kn1, kord1, kdim, spar2, lder, &kstat);
      if (kstat < 0) goto error;
    }

  

  kcnsta = 0;
  kcnend = 0;
  kdimcrv = kdim*kn1;


  for (ki=0; ki < inbcrv; ki++) etyp[ki] = (double)nctyp[ki];

  *gpar = SISL_NULL;  
  s1358(scoef2,knbcrv,kdimcrv,etyp,spar,kcnsta,kcnend,iopen,iord2,astpar,
	&tstpar,&qc,gpar,&knbpar,&kstat);
  if (kstat<0) goto error;

  if (*gpar) freearray(*gpar);  


  

  

  kind = 1;
  kcopy = 1;
  *rsurf = newSurf(kn1,qc->in,kord1,qc->ik,sknot1,qc->et,qc->ecoef,
		   kind,kdim,kcopy);
  if (*rsurf == SISL_NULL) goto err101;

  
  (*rsurf)->cuopen_2 = qc->cuopen;

  

  freeCurve(qc);

  

  *gpar = spar;

  

  s1333_count(inbcrv,vpcurv,&kcont,&kstat);
  if (kstat<0) goto error;

  if (kcont>=0)
      {
        s1333_cyclic(*rsurf,kcont,&kstat);
	if (kstat<0) goto error;

	
	(*rsurf)->cuopen_1 = SISL_SURF_PERIODIC;
      }
      else
      {
         

         for (kopen=-2, ki=0; ki<inbcrv; ki++)
           kopen = MAX(kopen,vpcurv[ki]->cuopen);
         if (kopen == SISL_CRV_CLOSED) (*rsurf)->cuopen_1 = SISL_SURF_CLOSED;
      }

  

  *jstat = 0;
  goto out;

  

 err101:
  *jstat = -101;
  s6err("s1333",*jstat,kpos);
  goto out;


  

 err179:
  *jstat = -179;
  s6err("s1333",*jstat,kpos);
  goto out;


  

  error :
    *jstat = kstat;
  s6err("s1333",*jstat,kpos);
  goto out;
 out:

  

  if (sknot1 != SISL_NULL) freearray(sknot1);
  if (scoef2 != SISL_NULL) freearray(scoef2);
  if (spar2 != SISL_NULL) freearray(spar2);
  if (lder != SISL_NULL) freearray(lder);

  if (etyp != SISL_NULL)
    freearray (etyp);

  return;
}

