/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1341

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1341(double ep[],int im,int idim,int ipar,double epar[],
	   double eeps[],int ilend,int irend,double afctol,double aepsco,
	   int itmax,int ik,SISLCurve **rc,double emxerr[],int *jstat)
#else
void s1341(ep,im,idim,ipar,epar,eeps,ilend,irend,afctol,aepsco,
           itmax,ik,rc,emxerr,jstat)
     double ep[];
     int    im;
     int    idim;
     int    ipar;
     double epar[];
     double eeps[];
     int    ilend;
     int    irend;
     double afctol;
     double aepsco;
     int    itmax;
     int    ik;
     SISLCurve  **rc;
     double emxerr[];
     int    *jstat;
#endif
{
  double *maxerr = SISL_NULL;    
  double *error1 = SISL_NULL;
  int i;
  int stat = 0;             
  int kpos = 0;
  SISLCurve *ocurve = SISL_NULL; 
  
  
  
  if (im < 2 || idim < 1) goto err103;
  if (ipar < 1 || ipar > 3) ipar = 1;

  

  if(afctol < 0 || afctol > 1) afctol =0;

  

  if (ipar == 3)
    s1350(ep, epar, im, idim, 2, &ocurve, &stat);
  else
    s1351(ep, ipar, im, idim, 2, &ocurve, &stat);
  if (stat<0) goto error;

  

  error1 = newarray(idim, DOUBLE);
  maxerr = newarray(idim, DOUBLE);
  if (error1 == SISL_NULL || maxerr == SISL_NULL) goto err101;
  for (i=0; i<idim; i++)
    maxerr[i] = eeps[i]*afctol;

  

  s1340(ocurve, maxerr, ilend, irend, aepsco, itmax, rc,
	error1, &stat);
  if (stat<0) goto error;
  freeCurve(ocurve);

  

  s1350((*rc)->ecoef, &((*rc)->et)[1], (*rc)->in, 
	idim, ik, &ocurve, &stat);
  if (stat<0) goto error;
  freeCurve(*rc);
  
  

  for (i=0; i<idim; i++)
    maxerr[i] = eeps[i] - error1[i];

  

  s1340(ocurve, maxerr, ilend, irend, aepsco, itmax, rc,
	emxerr, &stat);
  if (stat<0) goto error;

  
  
  (*rc)->cuopen = ocurve->cuopen;
  
  

  for (i=0; i<idim; i++)
    emxerr[i] += error1[i];
  
  *jstat = 0;
  goto out;

  

  err101 :
    *jstat = -101;
  goto out;

  

 err103: 
  *jstat = -103;
  s6err("s1341",*jstat,kpos);
  goto out;
  
  

 error:
  *jstat = stat;
  s6err("s1341",*jstat,kpos);
  goto out;

  

 out:
  if (maxerr != SISL_NULL) freearray(maxerr);
  if (error1 != SISL_NULL) freearray(error1);
  if (ocurve != SISL_NULL) freeCurve(ocurve);
  return;
}

