/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1342

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1342(double ep[],double ev[],int im,int idim,int ipar,double epar[],
	   double eeps[],int ilend,int irend,double aepsco,int itmax,
	   SISLCurve **rc,double emxerr[],int *jstat)
#else
void s1342(ep,ev,im,idim,ipar,epar,eeps,ilend,irend,aepsco,itmax,
           rc,emxerr,jstat)
     double ep[];
     double ev[];
     int    im;
     int    idim;
     int    ipar;
     double epar[];
     double eeps[];
     int    ilend;
     int    irend;
     double aepsco;
     int    itmax;
     SISLCurve  **rc;
     double emxerr[];
     int    *jstat;
#endif
{
  int stat = 0;                
  int kpos = 0;
  SISLCurve *ocurve = SISL_NULL;    

  

  if (im < 2 || idim < 1) goto err103;
  if (ipar < 1 || ipar > 3) ipar = 1;

  

  if (ipar == 3)
    s1379(ep, ev, epar, im, idim, &ocurve, &stat);
  else
    s1380(ep, ev, im, idim, ipar, &ocurve, &stat);
  if (stat < 0) goto error;

  

  s1340(ocurve, eeps, ilend, irend, aepsco, itmax, rc,
	emxerr, &stat);
  if (stat < 0) goto error;

  

  *jstat = 0;
  goto out;
  
  

 err103: 
  *jstat = -103;
  s6err("s1342",*jstat,kpos);
  goto out;
  
  

 error: 
  *jstat = stat;
  s6err("s1342",*jstat,kpos);
  goto out;

  

 out:
  if (ocurve != SISL_NULL) freeCurve(ocurve);
  return;
}

