/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1345

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void s1345(SISLSurf *oldsurf,double eps[],int edgefix[4],double edgeps[],
	   double epsco,int opt,int itmax,SISLSurf **newsurf,
	   double maxerr[],int *stat)
#else
void s1345(oldsurf, eps, edgefix, edgeps, epsco, opt, itmax, newsurf,
	    maxerr, stat)
     SISLSurf 	*oldsurf;
     double	eps[];
     int	edgefix[4];
     double	edgeps[];
     double	epsco;
     int 	opt;
     int	itmax;
     SISLSurf	**newsurf;
     double	maxerr[];
     int	*stat;
#endif

{
  int k1 = oldsurf->ik1;          
  int k2 = oldsurf->ik2;          
  int m1 = oldsurf->in1;
  int m2 = oldsurf->in2;
  int dim = oldsurf->idim;
  int ledgefix[4];                
  double *et1 = SISL_NULL;
  double *et2 = SISL_NULL;

  double *local_eps = SISL_NULL;       
  double *local_edge_eps = SISL_NULL;  
  double *clocal_eps = SISL_NULL;
  double *clocal_err = SISL_NULL;
  double *harray = SISL_NULL;
  double *tempcoef = SISL_NULL;
  SISLSurf *qs_kreg = SISL_NULL;

  double *loct1 = SISL_NULL;
  double *loct2 = SISL_NULL;
  SISLCurve *local_curve = SISL_NULL;
  SISLCurve *newlcurve = SISL_NULL;

  int lopt, i, it, n1, n2, antit, j, jh, lstat;
  double factor;
  int pos = 0;                    


  
  *newsurf = SISL_NULL;

  

  memcopy(ledgefix, edgefix, 4, INT);
  if (oldsurf->cuopen_1 == SISL_CRV_PERIODIC ||
      oldsurf->cuopen_2 == SISL_CRV_PERIODIC)
  {
    make_sf_kreg(oldsurf, &qs_kreg, &lstat);
    if (lstat < 0) goto error;

    

    if (oldsurf->cuopen_1 == SISL_CRV_PERIODIC)
      ledgefix[0] = ledgefix[1] = MAX(MAX(edgefix[0],edgefix[1]), 2);
    if (oldsurf->cuopen_2 == SISL_CRV_PERIODIC)
      ledgefix[2] = ledgefix[3] = MAX(MAX(edgefix[2],edgefix[3]), 2);
  }
  else
    qs_kreg = oldsurf;

  

  et1 = qs_kreg->et1;
  et2 = qs_kreg->et2;


  

  local_eps = newarray(dim, DOUBLE);
  if (local_eps == SISL_NULL) goto err101;

  local_edge_eps = newarray(4*dim, DOUBLE);
  if (local_edge_eps == SISL_NULL) goto err101;

  

  factor = (opt < 3) ? 1.0 : 0.5;

  

  for (i=0; i<dim; i++)
  {
    local_eps[i] = factor*eps[i];
    maxerr[i] = 0.0;
  }

  

  

  for (i=0; i<4*dim; i++) local_edge_eps[i] = edgeps[i];

  

  it = 0; lopt = (opt-1) % 2 + 1;
  n1 = m1; n2 = m2;

  antit = (opt < 3) ? 1 : 2;
  while (it < antit)
  {
    if (lopt == 1)
    {

      

      

      clocal_eps = newarray(dim*n2, DOUBLE);
      if (clocal_eps == SISL_NULL) goto err101;

      clocal_err = newarray(dim*n2, DOUBLE);
      if (clocal_err == SISL_NULL) goto err101;

      

      s1352(et2, n2, k2, local_eps, (local_edge_eps+2*dim),
	    (local_edge_eps+3*dim), dim, ledgefix[2], ledgefix[3],
	    clocal_eps, &lstat);
      if (lstat < 0) goto error;

      

      harray = newarray(n1*n2*dim, DOUBLE);
      if (harray == SISL_NULL) goto err101;

      if (opt == 4)
      {
	

	s6chpar(tempcoef, n1, n2, dim, harray);
	freearray(tempcoef);
	tempcoef = harray;
      }
      else
      {
	s6chpar(qs_kreg->ecoef, n1, n2, dim, harray);
	tempcoef = harray;
      }
      harray = SISL_NULL;

      

      local_curve = newCurve(n1, k1, qs_kreg->et1, tempcoef,
			     1, dim*n2, 0);
      if (local_curve == SISL_NULL) goto err101;

      

      s1340(local_curve, clocal_eps, ledgefix[0], ledgefix[1],
	    epsco, itmax, &newlcurve, clocal_err, &lstat);
      if(lstat<0) goto error;

      

      n1 = newlcurve->in;
      freeCurve(local_curve);
      local_curve = SISL_NULL;

      

      tempcoef = increasearray(tempcoef,n1*n2*dim, DOUBLE);
      if (tempcoef == SISL_NULL) goto err101;

      harray = newlcurve->ecoef;
      s6chpar(harray, n2, n1, dim, tempcoef);
      harray = SISL_NULL;

      

      local_curve = newlcurve;
      freearray(newlcurve->ecoef);
      local_curve->ecoef = tempcoef;
      newlcurve = SISL_NULL;

      

      local_curve->icopy = 0;

      

      loct1 = local_curve->et;
      et1 = loct1;

      

      freeCurve(local_curve);
      local_curve = SISL_NULL;

      

      

      for (i=0; i<dim; i++) clocal_eps[i] = 0.0;
      for (jh=0, i=0; i<n2; i++)
	for (j=0; j<dim; j++, jh++)
	  clocal_eps[j] = MAX(clocal_eps[j], clocal_err[jh]);

      

      for (i=0; i<dim; i++)
      {
	maxerr[i] += clocal_eps[i];
	local_eps[i] = eps[i] - maxerr[i];
      }
      freearray(clocal_eps);
      freearray(clocal_err);
    }
    else
    {


      

      

      clocal_eps = newarray(dim*n1, DOUBLE);
      if (clocal_eps == SISL_NULL) goto err101;

      clocal_err = newarray(dim*n1, DOUBLE);
      if (clocal_err == SISL_NULL) goto err101;

      

      s1352(et1, n1, k1, local_eps, local_edge_eps,
	    local_edge_eps+dim, dim, ledgefix[0], ledgefix[1],
	    clocal_eps, &lstat);
      if (lstat < 0) goto error;

      

      if (opt == 3)
      {
	
	local_curve = newCurve(n2, k2, qs_kreg->et2, tempcoef,
			       1, dim*n1, 0);
      }
      else
	local_curve = newCurve(n2, k2, qs_kreg->et2, qs_kreg->ecoef, 1,
			       dim*n1, 0);
      if (local_curve == SISL_NULL) goto err101;

      

      s1340(local_curve, clocal_eps, ledgefix[2], ledgefix[3],
	    epsco, itmax, &newlcurve, clocal_err, &lstat);
      if(lstat<0) goto error;

      

      n2 = newlcurve->in;

      

      freeCurve(local_curve);
      if (tempcoef != SISL_NULL) freearray(tempcoef);
      local_curve = newlcurve;
      newlcurve = SISL_NULL;

      

      local_curve->icopy = 0;
      tempcoef = local_curve->ecoef;

      

      loct2 = local_curve->et;
      et2 = loct2;

      

      freeCurve(local_curve);
      local_curve = SISL_NULL;

      

      for (i=0; i<dim; i++) clocal_eps[i] = 0.0;
      for (jh=0, i=0; i<n1; i++)
	for (j=0; j<dim; j++, jh++)
	  clocal_eps[j] = MAX(clocal_eps[j], clocal_err[jh]);

      for (i=0; i<dim; i++)
      {
	maxerr[i] += clocal_eps[i];
	local_eps[i] = eps[i] - maxerr[i];
      }
      freearray(clocal_eps);
      freearray(clocal_err);
    }

    

    ++it;
    lopt = 3 - lopt;

  }

  

  if (loct1 == SISL_NULL)
  {
    loct1 = newarray(n1+k1, DOUBLE);
    if (loct1 == SISL_NULL)  goto err101;

    harray = et1;
    for (i=0; i<n1+k1; i++) loct1[i] = harray[i];
  }

  if (loct2 ==SISL_NULL)
  {
    loct2 = newarray(n2+k2, DOUBLE);
    if (loct2 == SISL_NULL)  goto err101;

    harray = et2;
    for (i=0; i<n2+k2; i++) loct2[i] = harray[i];
  }

  

  *newsurf = newSurf(n1, n2, k1, k2, loct1, loct2, tempcoef,
		     1, dim, 2);

  

  loct1 = SISL_NULL;
  loct2 = SISL_NULL;
  tempcoef = SISL_NULL;

  

  if (oldsurf->cuopen_1 == SISL_CRV_CLOSED ||
      oldsurf->cuopen_1 == SISL_CRV_PERIODIC)
    (*newsurf)->cuopen_1 = SISL_CRV_CLOSED;
  if (oldsurf->cuopen_2 == SISL_CRV_CLOSED ||
      oldsurf->cuopen_2 == SISL_CRV_PERIODIC)
    (*newsurf)->cuopen_2 = SISL_CRV_CLOSED;

  *stat = 0;
  goto out;

  

err101:
  *stat = -101;
  s6err("s1345", *stat, pos);
  goto out;

  

error:
  *stat = lstat;
  s6err("s1345", *stat, pos);

  

out:
  if (local_eps != SISL_NULL) freearray(local_eps);
  if (local_edge_eps != SISL_NULL) freearray(local_edge_eps);
  if (clocal_eps != SISL_NULL) freearray(clocal_eps);
  if (qs_kreg != SISL_NULL && qs_kreg != oldsurf) freeSurf(qs_kreg);

  

  if (clocal_eps != SISL_NULL) freearray(clocal_eps);
  if (clocal_err != SISL_NULL) freearray(clocal_err);
  if (tempcoef != SISL_NULL) freearray(tempcoef);
  if (loct1 != SISL_NULL) freearray(loct1);
  if (loct2 != SISL_NULL) freearray(loct2);
  if (local_curve != SISL_NULL) freeCurve(local_curve);  
  if (newlcurve != SISL_NULL) freeCurve(newlcurve);  


  return;

}

