/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1346

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1346(double ep[],int im1,int im2,int idim,int ipar,double epar1[],
	   double epar2[],double eeps[],int nend[],double edgeps[],
	   double afctol,double aepsco,int iopt,int itmax,int ik1,int ik2,
	   SISLSurf **rs,double emxerr[],int *jstat)
#else
void s1346(ep,im1,im2,idim,ipar,epar1,epar2,eeps,nend,
           edgeps,afctol,aepsco,iopt,itmax,ik1,ik2,
           rs,emxerr,jstat)
     double ep[];
     int    im1;
     int    im2;
     int    idim;
     int    ipar;
     double epar1[];
     double epar2[];
     double eeps[];
     int    nend[];
     double edgeps[];
     double afctol;
     double aepsco;
     int    iopt;
     int    itmax;
     int    ik1;
     int    ik2;
     SISLSurf   **rs;
     double emxerr[];
     int    *jstat;
#endif
{
  int in1,in2;                
  int newin1, newin2;
  int fouridim=4*idim;
  int i;                      
  int stat=0, kpos=0;         
  double *par1 = SISL_NULL;
  double *par2 = SISL_NULL;
  double *knot1 = SISL_NULL;       
  double *knot2 = SISL_NULL;
  double *error1 = SISL_NULL;      
  double *error2 = SISL_NULL;
  double *maxerr = SISL_NULL;
  double *newcoeff = SISL_NULL;    
  SISLCurve *ocurve1 = SISL_NULL;  
  SISLCurve *ocurve2 = SISL_NULL;
  SISLSurf *osurf1 = SISL_NULL;    
  SISLSurf *osurf2 = SISL_NULL;

  

  if (im1 < 2 || im2 < 2 || ik1 < 1 || ik2 < 1 || idim < 1)
    goto err103;
  if (ipar < 1 || ipar > 3) ipar = 1;

  if (ipar != 3)
    {
      

      s1528(idim, im1, im2, ep, ipar, SISL_CRV_OPEN, SISL_CRV_OPEN,
	    &par1, &par2, &stat);
      if (stat<0) goto error;
    }
  else
    {
      

      par1 = epar1;
      par2 = epar2;
    }

  

  knot1 = newarray(im1+2, DOUBLE);
  knot2 = newarray(im2+2, DOUBLE);
  if(knot1 == SISL_NULL || knot2 == SISL_NULL) goto err101;
  memcopy(&knot1[1],par1,im1,DOUBLE);
  memcopy(&knot2[1],par2,im2,DOUBLE);
  knot1[0] = knot1[1];
  knot2[0] = knot2[1];
  knot1[im1+1] = knot1[im1];
  knot2[im2+1] = knot2[im2];
  osurf1 = newSurf(im1, im2, 2, 2, knot1, knot2, ep,
		   1,idim, 1);
  if (osurf1 == SISL_NULL) goto err101;
  if (knot1) freearray(knot1);
  if (knot2) freearray(knot2);

  

  maxerr = newarray(idim, DOUBLE);
  error1 = newarray(idim, DOUBLE);
  error2 = newarray(fouridim, DOUBLE);
  if (error1 == SISL_NULL || error2 == SISL_NULL || maxerr == SISL_NULL)
    goto err101;
  for (i=0; i<fouridim; i++)
    {
      edgeps[i] = MIN(edgeps[i], eeps[(i+idim)%idim]);
      error2[i] = afctol * edgeps[i];
    }
  for (i=0; i<idim; i++)
    error1[i] = afctol*eeps[i];

  

  s1345(osurf1, error1, nend, error2, aepsco, iopt, itmax,
	&osurf2, maxerr, &stat);
  if (stat<0) goto error;

  in1 = osurf2->in1;
  in2 = osurf2->in2;

  

  if(osurf1 != SISL_NULL)
    {
      freeSurf(osurf1);
      osurf1 = SISL_NULL;
    }

  

  

  s1350(osurf2->ecoef,&(osurf2->et2)[1], in2,
	in1 * idim, ik2, &ocurve1, &stat);
  if (stat<0) goto error;

  newin2 = ocurve1->in;

  

  if( (newcoeff = newarray(idim * in1 * newin2, DOUBLE)) == SISL_NULL )
    goto err101;
  s6chpar(ocurve1->ecoef, in1, newin2, idim, newcoeff);

  

  s1350(newcoeff, &(osurf2->et1)[1], in1,
	idim*newin2, ik1, &ocurve2, &stat);
  if (stat<0) goto error;
  newin1 = ocurve2->in;

  

  if(osurf2 != SISL_NULL)
    {
      freeSurf(osurf2);
      osurf2 = SISL_NULL;
    }

  

  newcoeff = increasearray(newcoeff,
			   idim * newin1 * newin2, DOUBLE);
  if (newcoeff == SISL_NULL) goto err101;
  s6chpar(ocurve2->ecoef, newin2, newin1, idim, newcoeff);

  

  osurf1 = newSurf(newin1, newin2, ik1, ik2, ocurve2->et,
		   ocurve1->et, newcoeff, 1, idim, 1);

  free(newcoeff); newcoeff = SISL_NULL;

  if (osurf1 == SISL_NULL) goto err101;

  

  osurf1->cuopen_1 = ocurve2->cuopen;
  osurf1->cuopen_2 = ocurve1->cuopen;

  

  for (i=0; i<fouridim; i++)
    error2[i] = edgeps[i]-error2[i];
  for (i=0; i<idim; i++)
    error1[i] = eeps[i]-maxerr[i];

  

  s1345(osurf1, error1, nend, error2, aepsco, iopt, itmax,
	rs, emxerr, &stat);
  if (stat<0) goto error;

  

  for(i=0; i<idim; i++)
    emxerr[i] += maxerr[i];

  

  *jstat = 0;
  goto out;

  

 err101:
  *jstat = -101;
  s6err("s1346",*jstat,kpos);
  goto out;

  

 err103:
  *jstat = -103;
  s6err("s1346",*jstat,kpos);
  goto out;

  

 error:
  *jstat = stat;
  s6err("s1346",*jstat,kpos);
  goto out;

  

 out:
  

  if(ocurve1 != SISL_NULL) freeCurve(ocurve1);
  if(ocurve2 != SISL_NULL) freeCurve(ocurve2);

  

  if(osurf1 != SISL_NULL) freeSurf(osurf1);

  

  if(error1 != SISL_NULL) freearray(error1);
  if(error2 != SISL_NULL) freearray(error2);
  if(maxerr != SISL_NULL) freearray(maxerr);

  if (ipar != 3)
    {
      freearray(par1);
      freearray(par2);
    }

  return;
}

