/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1347

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1347(double ep[],double etang1[],double etang2[],double eder11[],
	   int im1,int im2,int idim,int ipar,double epar1[],double epar2[],
	   double eeps[],int nend[],double edgeps[],double aepsco,int iopt,
	   int itmax,SISLSurf **rs,double emxerr[],int *jstat)
#else
void s1347(ep,etang1,etang2,eder11,im1,im2,idim,ipar,epar1,epar2,
	   eeps,nend,edgeps,aepsco,iopt,itmax,rs,emxerr,jstat)
     double ep[];
     double etang1[];
     double etang2[];
     double eder11[];
     int    im1;
     int    im2;
     int    idim;
     int    ipar;
     double epar1[];
     double epar2[];
     double eeps[];
     int    nend[];
     double edgeps[];
     double aepsco;
     int    iopt;
     int    itmax;
     SISLSurf   **rs;
     double emxerr[];
     int    *jstat;
#endif
{
  int stat=0, kpos=0;         
  double *par1 = SISL_NULL;        
  double *par2 = SISL_NULL;
  SISLSurf *osurf = SISL_NULL;      

  

  if (im1 < 2 || im2 < 2 || idim < 1) 
    goto err103; 
  if (ipar < 1 || ipar > 3) ipar = 1;

  

  if (ipar != 3) 
    { 
      
      
      s1528(idim, im1, im2, ep, ipar, SISL_CRV_OPEN, SISL_CRV_OPEN,
	    &par1, &par2, &stat);
      if (stat<0) goto error;
    }
  else 
    {
      

      par1 = epar1;
      par2 = epar2;
    } 

  

  s1530(ep, etang1, etang2, eder11, par1, par2,
	im1, im2, idim, &osurf, &stat);
  if (stat<0) goto error;

  

  s1345(osurf, eeps, nend, edgeps, aepsco, iopt, itmax,  
        rs, emxerr, &stat);
  if (stat<0) goto error;
  
  

  *jstat = 0;
  goto out;
  
  
  

  err103: 
    *jstat = -103;
    s6err("s1347",*jstat,kpos);
    goto out;
  
  

  error: 
    *jstat = stat;
    s6err("s1347",*jstat,kpos);
    goto out;
  
  

  out:
    

    if(osurf != SISL_NULL) freeSurf(osurf);

    if (ipar != 3)
      {
	if(par1 != SISL_NULL) freearray(par1);
	if(par2 != SISL_NULL) freearray(par2);
      }
    return;
}

