/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1352

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1352(double t[],int n,int k,double inteps[],double lefteps[],
	   double righteps[],int dim,int leftfix,int rightfix,
	   double eps[],int *stat)
#else
void s1352(t, n, k, inteps, lefteps, righteps, dim, leftfix, rightfix,
	   eps, stat)
     double	t[];
     int	n;
     int	k;
     double 	inteps[];
     double	lefteps[];
     double	righteps[];
     int	dim;
     int	leftfix;
     int	rightfix;
     double	eps[];
     int	*stat;
#endif
{
  int ih = 3*MAX(leftfix, rightfix) + 2; 
  double *th = SISL_NULL;			 
  double *hcoef = SISL_NULL;			 
  double *w = SISL_NULL;			 

  SISLCurve *hspline = SISL_NULL;             

                                         

  int left=0, i, lstat, j, jh, pos=0;
  double k1inv, ta, tb, tmid, hh, maxval, sum, cnst, ch, val;

  

  if (k == 1) k1inv = 1.0;
  else k1inv = 1.0 / (float) (k-1);

  

  ta = t[k-1]; tb = t[n];
  tmid = ta + (tb-ta)/2.0;
  hh = 1.0 / (tb-ta);

  

  th = newarray(ih, double);
  if (th == SISL_NULL) goto err101;

  

  hcoef = new0array(2*MAX(leftfix, rightfix)+1, double);
  if (hcoef == SISL_NULL) goto err101;

  hcoef[leftfix] = 1.0;

  

  if (leftfix == 0)
    {
      

      th[0] = MIN(-1.1, (2*t[0]-ta-tb)*hh);
      th[1] = MAX(1.1, (2*t[n+k-1]-ta-tb)*hh);
    }
  else
    {

      

      cnst = PI / (float) (leftfix+1);

      

      for (j=0; j<=leftfix+1; j++)
	th[leftfix+j] = cos((leftfix+1-j)*cnst);

      

      for (j=0; j<leftfix; j++)
	{
	  th[j] = th[leftfix];
	  th[j+2*leftfix+2] = th[j+2*leftfix+1];
	}
    }

  

  w = newarray(MAX((leftfix+1),(rightfix+1)), double);
  if (w == SISL_NULL) goto err101;

  

  hspline = newCurve(2*leftfix+1, leftfix+1, th, hcoef, 1, 1, 0);
  if (hspline == SISL_NULL) goto err101;

  

  s1221(hspline, 0, 0.0, &left, w, &lstat);
  if (lstat < 0) goto err;

  maxval = w[0];

  

  

  if (leftfix > 0)
    {
      for (ih=0; ih<dim; ih++) eps[ih] = lefteps[ih];
    }
  else
    {
      for (ih=0; ih<dim; ih++) eps[ih] = inteps[ih];
    }

  

  if (k == 1) sum = t[1];
  else
    {
      sum = 0.0;

      for (j=2; j<k+1; j++) sum += t[j];

      sum *= k1inv;
    }

  

  i = 1;  ih = dim;

  while (sum < tmid)
    {

      

      ch = (2*sum-ta-tb)*hh;

      

      s1221(hspline, 0, ch, &left, w, &lstat);
      if (lstat < 0) goto err;

      

      val = w[0];
      val /= maxval;

      

      for (jh=0; jh<dim; jh++, ih++)
	eps[ih] = MIN((inteps[jh]-lefteps[jh])*val + lefteps[jh],inteps[jh]);

      ++i;

      

     if (k == 1) sum = t[i];
     else
       {
	 sum = 0.0;

	 for (j=i+1; j<i+k; j++) sum += t[j];

	 sum *= k1inv;
       }
    }

  

  if (leftfix != rightfix)
    {
      if (rightfix == 0)
	{
	  th[0] = MIN(-1.1, (2*t[0]-ta-tb)*hh);
	  th[1] = MAX(1.1, (2*t[n+k-1]-ta-tb)*hh);
	}
      else
	{
	  cnst = PI / (float) (rightfix+1);

	  for (j=0; j<=rightfix+1; j++)
	    th[rightfix+j] = cos((rightfix+1-j)*cnst);
	  for (j=0; j<rightfix; j++)
	    {
	      th[j] = th[rightfix];
	      th[j+2*rightfix+2] = th[j+2*rightfix+1];
	    }
	}

      

      hcoef[leftfix] = 0.0;
      hcoef[rightfix] = 1.0;

      

      freeCurve(hspline);
      hspline = newCurve(2*rightfix+1, rightfix+1, th, hcoef, 1, 1, 0);
      if (hspline == SISL_NULL) goto err101;

      

      s1221(hspline, 0, 0.0, &left, w, &lstat);
      if (lstat < 0) goto err;

      maxval = w[0];
    }

  

  while (i<n-1)
    {
      ch = (2*sum-ta-tb)*hh;

      s1221(hspline, 0, ch, &left, w, &lstat);
      if (lstat < 0) goto err;

      val = w[0];
      val /= maxval;

      for (jh=0; jh<dim; jh++, ih++)
	eps[ih] = MIN((inteps[jh]-righteps[jh])*val + righteps[jh],inteps[jh]);

      ++i;

     if (k == 1) sum = t[i];
     else
       {
	 sum = 0.0;

	 for (j=i+1; j<i+k; j++) sum += t[j];

	 sum *= k1inv;
       }
    }

  

  if (rightfix > 0)
    {
      for (jh=0; jh<dim; jh++, ih++) eps[ih] = righteps[jh];
    }
  else
    {
      for (jh=0; jh<dim; jh++, ih++) eps[ih] = inteps[jh];
    }

  

  *stat = 0;
  goto out;

  

 err101:
  *stat = -101;
  s6err("s1352", *stat, pos);

  goto out;

  

 err:
  *stat = lstat;
  s6err("s1352", *stat, pos);

  

 out:
  if (w != SISL_NULL) freearray(w);
  if (hspline != SISL_NULL) freeCurve(hspline);

  

  if (th != SISL_NULL) freearray(th);
  if (hcoef != SISL_NULL) freearray(hcoef);

  return;

}

