/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define S1355

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1355(SISLCurve *pc,double eeps[],double **epar,
	   int *im, int *jstat)
#else
void s1355(pc,eeps,epar,im,jstat)
     SISLCurve  *pc;
     double eeps[];
     double **epar;
     int *im;
     int    *jstat;
#endif
{
  int i,j,k;               
  int number = 0;
  int idim = pc->idim;     
  int ik, in;              
  int stat = 0;            
  int kpos = 0;
  int multi = 0;           
  int parlen;
  int help;
  double end, start;       
  double maxstep;
  double *coeff;           
  double *et = SISL_NULL;       
  double *par = SISL_NULL;      
  double *maxtab = SISL_NULL;
  double *neweps = SISL_NULL;
  SISLCurve *oc=SISL_NULL;      

  

  if (!pc) goto err150;
  s1707(pc, &stat);
  if (stat<0) goto error;

  

  if (pc->in < pc->ik || pc->ik < 1 || idim < 1) goto err103;

  

  s1720( pc, 4, &oc, &stat);
  if (stat<0) goto error;

  

  ik = oc->ik;
  in = oc->in;
  et = oc->et;
  coeff = oc->ecoef;

  

  parlen = ik*in;
  par    = newarray(parlen, DOUBLE);
  maxtab = newarray(idim, DOUBLE);
  neweps = newarray(idim, DOUBLE);
  if (maxtab == SISL_NULL || par == SISL_NULL || neweps == SISL_NULL) goto err101;

  

  for(i=0; i<idim; i++)
    neweps[i] = eeps[i] * 384 / 5. ;

  

  number = 0;
  multi = 0;
  for(help=0; help<in-ik+1; help++)
    {
      start = et[ik+help-1];
      end = et[ik+help];

      

      

      for(i=0; i<idim; i++)
	maxtab[i] = 0.;
      for(i=help*idim, k=0; k<ik-1; i+=idim, k++)
	for(j=0; j<idim; j++)
	  maxtab[j] = MAX( maxtab[j], fabs(coeff[i+j]) );

      

      maxstep = 1.1*end-start;
      for(i=0; i<idim; i++)
	{
	  if( maxtab[i] > 0. )
	    maxstep = MIN( neweps[i]/maxtab[i], maxstep );
	}
      maxstep = pow(maxstep, 0.25);

      

      if (DEQUAL(maxstep, 0.0))
        goto err103;

      

      while(start<end)
	{
	  par[number] = start;
	  start += maxstep;
	  number++;

	  

	  if (number+1>=parlen)
	    {
	      parlen = 2*parlen;
	      if ((par = increasearray(par, parlen, double)) == SISL_NULL)
		goto err101;
            }
	}

      

      if(end == start)
	{
	  

	  if(multi == pc->ik - 3)
	    {
	      par[number] = start;
	      number++;
	      multi=0;
	    }
	  else
	    multi++;
	}
      else
	multi = 0;
    }

  par[number] = end;
  if ((par = increasearray(par,number+1,DOUBLE)) == SISL_NULL) goto err101;
  *epar = par;
  *im = number+1;

  

  *jstat = 0;
  goto out;

  

 err101:
   *jstat = -101;
   s6err("s1355",*jstat,kpos);
   goto out;

  

  err103:
    *jstat = -103;
    s6err("s1355",*jstat,kpos);
    goto out;

  

  err150:
    *jstat = -150;
    s6err("s1355",*jstat,kpos);
    goto out;

  

  error:
    *jstat = stat;
    s6err("s1355",*jstat,kpos);
    goto out;

  

  out:
    if( maxtab != SISL_NULL) freearray(maxtab);
    if( neweps != SISL_NULL) freearray(neweps);
    if( oc != SISL_NULL) freeCurve(oc);
    return;
}

