/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1377

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1377 (SISLCurve * pcurv, double econic[], int ideg, int idim,
       SISLCurve ** rcurv, int *jstat)
#else
void
s1377 (pcurv, econic, ideg, idim, rcurv, jstat)
     SISLCurve *pcurv;
     double econic[];
     int ideg;
     int idim;
     SISLCurve **rcurv;
     int *jstat;
#endif
{
  int ikind;			
  int kn;			
  int kk;			
  int kjkk;			
  int kjkn;			
  int kdim;			
  int kstat;			
  int kpos = 0;			
  int kzero = 0;		
  int kone = 1;			
  int cuopen;			
  int ki, kj;			
  int kleft = 0;		
  int *der = SISL_NULL;		
  double *st = SISL_NULL;		
  double *scentr = econic;	
  double *saxis = econic + 3;	
  double tbigr = *(econic + 6);	
  double tsmalr = *(econic + 7);
  double tbigr2 = tbigr * tbigr;
  double tdiffr2 = tbigr2 - tsmalr * tsmalr;	
  double *sval1 = SISL_NULL;		
  double *sval2 = SISL_NULL;
  double *sgt = SISL_NULL;		
  double sy[3];			
  double tzn;			
  double tyy;			
  double tzz;			
  double ty;			
  double tz;			
  double sder[4];		
  double ww;			
  double *par = SISL_NULL;
  SISLCurve *tempcurv = SISL_NULL;	

  *jstat = 0;
  if (idim != pcurv->idim) goto err104;
  if (ideg != 1001) goto err200;
  
  

  kn = pcurv->in;
  kk = pcurv->ik;
  kdim = pcurv->idim;
  st = pcurv->et;
  ikind = pcurv->ikind;

  if (ikind == 2 || ikind == 4)
    {
      tempcurv = newCurve (kn, kk, st, pcurv->rcoef, ikind - 1, kdim + 1, 0);
      if (tempcurv == SISL_NULL)
	goto err171;
      tempcurv->cuopen = pcurv->cuopen;
    }
  else
    {
      tempcurv = pcurv;
    }


  

  if (kdim != 3)
    goto err104;


  

  s1376 (st, kn, kk, &sgt, &kjkn, &kjkk, &kstat);
  if (kstat < 0)
    goto error;


  

  s1890 (sgt, kjkk, kjkn, &par, &der, &kstat);
  if (kstat < 0)
    goto error;


  

  sval1 = newarray (kjkn, DOUBLE);
  if (sval1 == SISL_NULL)
    goto err101;


  

  for (ki = 0; ki < kjkn; ki++)
    {
      

      s1221 (tempcurv, 0, par[ki], &kleft, sder, &kstat);
      if (kstat < 0)
	goto error;

      

      if (ikind == 2 || ikind == 4)
	{
	  for (kj = 0; kj < 3; kj++)
	    sy[kj] = sder[kj] - sder[3] * scentr[kj];
	  ww = sder[3] * sder[3];
	}
      else
	{
	  for (kj = 0; kj < 3; kj++)
	    sy[kj] = sder[kj] - scentr[kj];
	  ww = (double) 1.0;
	}

      tzn = s6scpr (sy, saxis, 3);

      tyy = (double) 0.0;
      tzz = (double) 0.0;


      

      for (kj = 0; kj < 3; kj++)
	{
	  ty = sy[kj];
	  tz = ty - tzn * saxis[kj];
	  tyy += ty * ty;
	  tzz += tz * tz;
	}


      sval1[ki] = tyy * tyy + ((double) 2.0 * ww * tyy
			       + ww * ww * tdiffr2) * tdiffr2
	- (double) 4.0 *ww * tbigr2 * tzz;
    }

  cuopen = TRUE;

  s1891 (par, sval1, kone, kjkn, kone, der, cuopen, sgt, &sval2, &kjkn, kjkk,
	 kzero, kzero, &kstat);
  if (kstat < 0)
    goto error;

  *rcurv = SISL_NULL;
  *rcurv = newCurve (kjkn, kjkk, sgt, sval2, 1, 1, 1);
  if (*rcurv == SISL_NULL)
    goto err171;
  (*rcurv)->cuopen = pcurv->cuopen;

 
  

  goto out;


  

error:
  *jstat = kstat;
  s6err ("s1377", *jstat, kpos);
  goto out;

  

err101:
  *jstat = -101;
  s6err ("s1377", *jstat, kpos);
  goto out;

  

err104:
  *jstat = -104;
  s6err ("s1377", *jstat, kpos);
  goto out;

  

err171:
  *jstat = -171;
  s6err ("s1377", *jstat, kpos);
  goto out;

  

err200:
  *jstat = -200;
  s6err ("s1377", *jstat, kpos);
  goto out;

out:

  

  if (sgt != SISL_NULL)
    freearray (sgt);
  if (par != SISL_NULL)
    freearray(par);
  if (der != SISL_NULL)
    freearray(der);
  if (sval1 != SISL_NULL)
    freearray (sval1);
  if (sval2 != SISL_NULL)
    freearray (sval2);
  if ((ikind == 2 || ikind == 4) && (tempcurv != SISL_NULL))
    freeCurve (tempcurv);

  return;
}

