/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1500

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1500(double base[],double norm[],double axisA[],double alpha,
         double ratio,int idim,int inumb,double carray[],int *jstat)
#else
void s1500(base,norm,axisA,alpha,ratio,idim,inumb,carray,jstat)
     double base[];
     double norm[];
     double axisA[];
     double alpha;
     double ratio;
     int    idim;
     int    inumb;
     double carray[];
     int    *jstat;
#endif
{
  int kdimp1;         
  int kstop;          
  int ki,kj,kl;       
  int kpos=0;         
  int kstat;          
  double a1,a2,a3;    
  double b1,b2,b3;    
  double n1,n2,n3;    
  double temp;        
  double cosT;        
  double cosT2;       
  double sinT;        
  double sinT2;       
  double cosTsinT;    
  double ndirec[3];   
  double A[3];        
  double B[3];        
  double asize;       
  double r;           
  double Adotp;       
  double Bdotp;       
  double ndotp;       
  
  
  
  if (inumb <1 ) inumb = 1;
  if (idim != 3 ) goto err104;
  
  kdimp1 = idim + 1;
  kstop  = kdimp1*kdimp1;
  
  for (ki=0;ki<kstop;ki++)
    {
      carray[ki] = DZERO;
    }
  
  
  
  (void)s6norm(norm,idim,ndirec,&kstat);
  
  
  if(kstat == 0) goto err174;

  
          
  
  asize=s6norm(axisA,idim,A,&kstat);
  
  
  if(kstat == 0) goto err174;

          
          
  
  (void)s6crss(ndirec,A,B);
  
  
  
  cosT=cos(alpha);
  sinT=sin(alpha);
  cosT2=cosT*cosT;
  sinT2=sinT*sinT;
  cosTsinT=cosT*sinT;
  
  

  r=ratio*ratio;

  
  

  a1 = A[0];
  a2 = A[1];
  a3 = A[2];
  b1 = B[0];
  b2 = B[1];
  b3 = B[2];
  n1 = ndirec[0];
  n2 = ndirec[1];
  n3 = ndirec[2];
  
  

  Adotp = s6scpr(A,base,idim);
  Bdotp = s6scpr(B,base,idim);
  ndotp = s6scpr(ndirec,base,idim);

  
  
  
  carray[0] = cosT2*(a1*a1+r*b1*b1)-sinT2*n1*n1;
  
  
  
  carray[5] = cosT2*(a2*a2+r*b2*b2)-sinT2*n2*n2;
  
  
  
  carray[10] = cosT2*(a3*a3+r*b3*b3)-sinT2*n3*n3;
  
  
  
  temp = cosT2*(a1*a2+r*b1*b2)-sinT2*n1*n2;
  carray[1] = temp;
  carray[4] = temp;
  
  
  
  
  temp = cosT2*(a1*a3+r*b1*b3)-sinT2*n1*n3;
  carray[2] = temp;
  carray[8] = temp;
  
  
  
  
  temp = cosT2*(a2*a3+r*b2*b3)-sinT2*n2*n3;
  carray[6] = temp;
  carray[9] = temp;
  
  
  
  
  
  temp = -cosT2*(Adotp*a1+r*Bdotp*b1)+(asize*cosTsinT+sinT2*ndotp)*n1;
  carray[3]  = temp;
  carray[12] = temp;
  
  
  
  
  temp = -cosT2*(Adotp*a2+r*Bdotp*b2)+(asize*cosTsinT+sinT2*ndotp)*n2;
  carray[7]  = temp;
  carray[13] = temp;
  
  
  
  
  temp = -cosT2*(Adotp*a3+r*Bdotp*b3)+(asize*cosTsinT+sinT2*ndotp)*n3;
  carray[11] = temp;
  carray[14] = temp;
  
  
  
  temp = cosT*asize+sinT*ndotp;
  carray[15] = cosT2*(Adotp*Adotp+r*Bdotp*Bdotp)-temp*temp;
  
  
  
  
  kj = kstop;
  for (ki=1;ki<inumb;ki++)
    {
      for (kl=0;kl<kstop;kl++,kj++)
        {
	  carray[kj] = carray[kl];
        }
    }
  
  *jstat = 0;
  goto out;
  
  
 err104: *jstat = -104;
  s6err("s1500",*jstat,kpos);
  goto out;
  
  
 err174: *jstat = -174;
  s6err("s1500",*jstat,kpos);
  goto out;
  
 out:
  return;
}

