/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1505

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1505(const SISLSurf *ps1,int ider,int m1, int m2,double *ebder1,double
      *ebder2, int *ileft1,int *ileft2,double eder[],double norm[],int *jstat)
#else
void s1505(ps1,ider,m1,m2,ebder1,ebder2,ileft1,ileft2,eder,norm,jstat)
     SISLSurf   *ps1;
     int    ider;
     int m1;
     int m2;
     double *ebder1;
     double *ebder2;
     int    *ileft1;
     int    *ileft2;
     double eder[];
     double norm[];
     int    *jstat;
#endif
{
  int kstat=0;        
  int kpos=0;         
  int i1,i2;          
  int i1pos,i2pos;    
  int kn1,kn2;        
  int kk1,kk2;        
  int kdim;           
  int kleft1,kleft2;  
  int ki,kx,kjh;      
  int kih2;           
  int ky,kl,kl1,kl2;  
  double *scoef;      
  double tt;          
  double *ew=SISL_NULL;    
  double *sder=SISL_NULL;  
  double *enorm=SISL_NULL; 
  int size;           
  int sizeh;          
  int size1,size2;    
  int ederpos;       
  int normpos;       

  int knumb2;         

  int tot,temp;       

  kn1 = ps1 -> in1;
  kn2 = ps1 -> in2;
  kk1 = ps1 -> ik1;
  kk2 = ps1 -> ik2;
  kdim = ps1 -> idim;
  

  if (kdim < 1) goto err102;
  if (kk1 < 1) goto err115;
  if (kn1 < kk1 || kn2 < kk2) goto err116;
  if (ider < 0) goto err178;

  *jstat = 0;

  if (ps1->ikind == 2 || ps1->ikind == 4)
  {
    scoef = ps1 -> rcoef;
    kdim +=1;
  }
  else
  {
    scoef = ps1 -> ecoef;
  }
  sizeh = kdim*(ider+1)*(ider+2)/2;
  if((sder=newarray(sizeh,DOUBLE)) == SISL_NULL) goto err101;
  if((enorm=newarray(ps1->idim,DOUBLE)) == SISL_NULL) goto err101;

  size = ps1->idim*(ider+1)*(ider+2)/2;
  size1 = (ider+1)*kk1;
  size2 = (ider+1)*kk2;

  

  knumb2 = kn1*(ider+1)*kdim;
  if((ew=newarray(knumb2,DOUBLE)) == SISL_NULL) goto err101;

  ederpos = 0;
  normpos = 0;

  
  for(i2=0, i2pos=0; i2<m2; i2++, i2pos += size2)
  {
    kleft2 = ileft2[i2];

    

    
    for(ki=0; ki<knumb2; ki++) ew[ki] = DZERO;

    

    kih2 = i2pos;
    for (ki=kleft2-kk2+1; ki<=kleft2; ki++)
    {
      

      kjh = 0; kl1 = kdim*ki*kn1;
      for (kx=0; kx<=ider; kx++)
      {
        

        tt = ebder2[kih2++]; kl2 = kl1;
        for (kl=0; kl<kdim*kn1; kl++,kjh++,kl2++)
        {
           ew[kjh] += scoef[kl2]*tt;
        }
      }
    }

    
    for(i1=0, i1pos=0; i1<m1; i1++, i1pos += size1)
    {
      kleft1 = ileft1[i1];

      
      for(ki=0; ki<sizeh; ki++) sder[ki] = DZERO;

      for(ky=0; ky<=ider; ky++)
      {
	kl1 = kdim * (ky * kn1 + kleft1 - kk1 + 1);
	for(kx=0; kx<=ider-ky; kx++)
	{
          tot = kx + ky;
          temp = ((tot * (tot+1)) >> 1) + ky;
	  kjh = temp * kdim;

	  for(ki=0; ki<kk1; ki++)
          {
	    tt = ebder1[i1pos + (ider+1) * ki + kx];
	    for(kl=0; kl<kdim; kl++)
	    {
	      sder[kjh+kl] += ew[kl1 + kdim * ki + kl] * tt;
	    }
          }
        }
      }

      

      if (ps1->ikind == 2 || ps1->ikind == 4)
      {
        s6strider(sder,ps1->idim,ider,eder+ederpos,&kstat);
        if (kstat<0) goto error;
      }
      else
      {
        for(ki=0; ki<sizeh; ki++) eder[ederpos+ki] = sder[ki];
      }

      

      if (ider>0 && ps1->idim ==3)
        {

          s6crss(eder+ederpos+ps1->idim,eder+ederpos+2*ps1->idim,enorm);

          s6norm(enorm,3,norm+normpos,&kstat);
         }

      ederpos += size;
      normpos += 3;
    }
  }

  
  if(sder != SISL_NULL) freearray(sder);
  if(enorm != SISL_NULL) freearray(enorm);
  if (ew != SISL_NULL) freearray(ew);

  goto out;

  
 err101: *jstat = -101;
  s6err("s1505",*jstat,kpos);
  goto out;

  
 err102: *jstat = -102;
  s6err("s1505",*jstat,kpos);
  goto out;

  
 err115: *jstat = -115;
  s6err("s1505",*jstat,kpos);
  goto out;

  
 err116: *jstat = -116;
  s6err("s1505",*jstat,kpos);
  goto out;

  
 err178: *jstat = -178;
  s6err("s1221",*jstat,kpos);
  goto out;

  

 error:  *jstat = kstat;
  s6err("s1505",*jstat,kpos);
  goto out;

 out:
    return;
}

