/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1513

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1513 (SISLSurf * psurf, double qpoint[], double bvec[], int idim, 
	SISLSurf ** rsurf, int *jstat)
#else
void 
s1513 (psurf, qpoint, bvec, idim, rsurf, jstat)
     SISLSurf *psurf;
     double qpoint[];
     double bvec[];
     int idim;
     SISLSurf **rsurf;
     int *jstat;
#endif
{
  double cross[3];		
  double diff[3];		
  int kn1;			
  int kk1;			
  int kn2;			
  int kk2;			
  int kjkk1;			
  int kjkn1;			
  int kjkk2;			
  int kjkn2;			
  int kdim;			
  int kstat;			
  int kpos = 0;			
  int kzero = 0;		
  int kone = 1;			
  int cuopen;			
  int ki, kj;	     	        
  int kp;			
  int klfs = 0;			
  int klft = 0;			

  double *st1 = SISL_NULL;		
  double *st2 = SISL_NULL;		
  double *sval1 = SISL_NULL;		
  double *sval2 = SISL_NULL;
  double *sval3 = SISL_NULL;

  double *sgt1 = SISL_NULL;		
  double *sgt2 = SISL_NULL;		
  double sder[12];		
  double spar[2];		
  double snorm[3];		
  int i;			
  int ikind;			
  double tutv[3];		
  double tvt[3];		
  double ttu[3];		
  SISLSurf *tempsurf = SISL_NULL;	
  double *par1=SISL_NULL;
  double *par2=SISL_NULL;
  int    *der1=SISL_NULL;
  int    *der2=SISL_NULL;


  if (idim != psurf->idim) goto err104;

  

  kn1 = psurf->in1;
  kk1 = psurf->ik1;
  kn2 = psurf->in2;
  kk2 = psurf->ik2;
  kdim = psurf->idim;
  st1 = psurf->et1;
  st2 = psurf->et2;
  ikind = psurf->ikind;

  

  if (kdim != 3)
    goto err104;

  if (ikind == 2 || ikind == 4)
    {
      
     
      tempsurf = newSurf (kn1, kn2, kk1, kk2, st1, st2, psurf->rcoef, ikind - 1, kdim + 1, 0);
      if (tempsurf == SISL_NULL) goto err171;
			    
      tempsurf->cuopen_1 = psurf->cuopen_1;			    
      tempsurf->cuopen_2 = psurf->cuopen_2;			    
      kjkk1 = 4 * kk1 - 4;
      kjkk2 = 4 * kk2 - 4;
    }
  else
    {
      tempsurf = psurf;
      kjkk1 = 3 * kk1 - 3;
      kjkk2 = 3 * kk2 - 3;
    }

  

  s1381 (st1, kn1, kk1, &sgt1, &kjkn1, kjkk1, &kstat);
  if (kstat < 0)
    goto error;

  

  s1890(sgt1,kjkk1,kjkn1,&par1,&der1,&kstat);
  if (kstat < 0)
    goto error;


  

  s1381 (st2, kn2, kk2, &sgt2, &kjkn2, kjkk2, &kstat);
  if (kstat < 0)
    goto error;

  

  s1890(sgt2,kjkk2,kjkn2,&par2,&der2,&kstat);
  if (kstat < 0)
    goto error;

  

  sval1 = newarray (kjkn1 * kjkn2, DOUBLE);
  if (sval1 == SISL_NULL)
    goto err101;

  

  

  kp = 0;

  for (kj = 0; kj < kjkn2; kj++)
    {

      spar[1] = par2[kj];

      for (ki = 0; ki < kjkn1; ki++)
	{
	  

	  spar[0] = par1[ki];

	  s1421 (tempsurf, 1, spar, &klfs, &klft, sder, snorm, &kstat);
	  if (kstat < 0)
	    goto error;

	  if (ikind == 2 || ikind == 4)
	    {
	      

	      s6crss (sder + 4, sder + 8, tutv);
	      s6crss (sder + 8, sder, tvt);
	      s6crss (sder, sder + 4, ttu);
	      for (i = 0; i < 3; i++)
		{
		  snorm[i] = sder[3] * tutv[i] + sder[7] * tvt[i] + sder[11] * ttu[i];
		  diff[i] = sder[i] - sder[3] * qpoint[i];
		}
	    }
	  else
	    {
	      s6diff (sder, qpoint, kdim, diff);
	    }

	  

	  s6crss (snorm, diff, cross);
	  sval1[kp++] = s6scpr (cross, bvec, kdim);
	}
    }

  cuopen = TRUE;

  

  s1891(par2,sval1,kjkn1,kjkn2,kone,der2,cuopen,sgt2,&sval2,&kjkn2,kjkk2,kzero,kzero,&kstat);
  if (kstat < 0)
    goto error;


  

  s1891(par1,sval2,kone,kjkn1,kjkn2,der1,cuopen,sgt1,&sval3,&kjkn1,kjkk1,kzero,kzero,&kstat);
  if (kstat < 0)
    goto error;

  if (kstat < 0)
    goto error;

  *rsurf = SISL_NULL;
  *rsurf = newSurf (kjkn1, kjkn2, kjkk1, kjkk2, sgt1, sgt2, sval3, 1, 1, 1);

  if (*rsurf == SISL_NULL)
    goto err171;

  (*rsurf)->cuopen_1 = psurf->cuopen_1;  
  (*rsurf)->cuopen_2 = psurf->cuopen_2; 
  
  


  *jstat = 0;
  goto out;

  

error:
  *jstat = kstat;
  s6err ("s1513", *jstat, kpos);
  goto out;

  
err101:
  *jstat = -101;
  s6err ("s1513", *jstat, kpos);
  goto out;

  
err104:
  *jstat = -104;
  s6err ("s1513", *jstat, kpos);
  goto out;

  

err171:
  *jstat = -171;
  s6err ("s1513", *jstat, kpos);
  goto out;

out:

  

  if (sgt1 != SISL_NULL)
    freearray (sgt1);
  if (sgt2 != SISL_NULL)
    freearray (sgt2);
  if (sval1 != SISL_NULL)
    freearray (sval1);
  if (sval2 != SISL_NULL)
    freearray (sval2);
  if (sval3 != SISL_NULL)
    freearray (sval3);
  if (par1 != SISL_NULL)
    freearray(par1);
  if (par2 != SISL_NULL)
    freearray(par2);
  if (der1 != SISL_NULL)
    freearray(der1);
  if (der2 != SISL_NULL)
    freearray(der2);
  if ((ikind == 2 || ikind == 4) && (tempsurf != SISL_NULL))
    freeSurf (tempsurf);

  return;
}

