/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1541

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1541   ( SISLCurve  *pc1,
	  int         npol,
	  double      ebder[],
	  int         ileft[],
	  double      eder[],
	  int        *jstat )

#else
void
s1541 ( pc1, npol, ebder, ileft, eder, jstat )
     SISLCurve*  pc1;
     int         npol;
     double      ebder[];
     int         ileft[];
     double      eder[];
     int        *jstat;

#endif
{
   int m = 0, m1 = 0;
   int my, my1;
   int np, i;
   int ik;
   double bas;
   double cx, cy, cz;
   double* ecoef = SISL_NULL;


   

   if ( pc1->idim != 3 ) goto err104;


   

   ik    = pc1 -> ik;
   ecoef = pc1 -> ecoef;


   for ( np = 0; np < npol; np++ )
   {
     my  = ileft[ np ] - ik;

     cx = cy = cz = 0.0;

     for ( i = 0; i < ik; i++ )
     {
       my++;
       my1 = 3*my;
       bas = ebder[ m1++ ];

       cx += ecoef[ my1   ]*bas;
       cy += ecoef[ my1+1 ]*bas;
       cz += ecoef[ my1+2 ]*bas;
     }
     eder[ m++ ] = cx;
     eder[ m++ ] = cy;
     eder[ m++ ] = cz;
   }


  

   *jstat = 0;
   goto out;

 err104: *jstat = -104;
         s6err( "s1541", *jstat, 0 );
         goto out;

out: return;

}

