/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1614

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1614(double epoint[], int inbpnt, int idim, int eptyp[],
	   double spoint[], int *jnbpnt, int sptyp[], int *jstat)

#else
void s1614(epoint, inbpnt, idim, eptyp,spoint, jnbpnt, sptyp, jstat)
     double epoint[];
     int    inbpnt;
     int    idim;
     int    eptyp[];
     double spoint[];
     int    *jnbpnt;
     int    sptyp[];
     int    *jstat;
#endif
{
  int ki, kp, kk, kki;
  int khelp;
  int ktell = 0;
  int kflag = 0;
  int knbpnt = 0;
  int ktyp = 1;
  int ktypm1;
  double tdum;
  int kpos = 0;

  *jstat = 0;


  

  for (ki = 0; ki < inbpnt; ki++)
    {
      ktypm1 = ktyp;
      ktyp = eptyp[ki];

      if (!((ktyp < 1 || ktyp > 4) ||
	    (knbpnt == 0 && ktyp == 4) ||
	    (ktyp == 3 && ktypm1 == 3) ||
	    (ktyp == 4 && ktypm1 == 4) ||
	    (ktyp == 4 && ktypm1 == 3)))
	{
	  

	  sptyp[knbpnt] = ktyp;

	  kki = idim * ki;
	  kk = idim * knbpnt;

	  for (kp = 0; kp < idim; kp++)
	    spoint[kk + kp] = epoint[kki + kp];
	  knbpnt++;
	}
    }

  

  if (ktyp == 3)
    knbpnt--;


  

  if (knbpnt > 5)
    {
      knbpnt = 5;

      

      if (sptyp[knbpnt - 1] == 3)
	{
	  

	  sptyp[knbpnt - 1] = sptyp[knbpnt];

	  kk = idim * (knbpnt - 1);

	  for (kp = 0; kp < idim; kp++)
	    spoint[kk + kp] = spoint[kk + idim + kp];
	}
    }


  

  ktyp = sptyp[0];

  if (ktyp > 2)
    {
      

      kflag = 1;

      sptyp[0] = 1;
      sptyp[1] = 4;

      for (kp = 0; kp < idim; kp++)
	{
	  tdum = spoint[kp];
	  spoint[kp] = spoint[kp + idim];
	  spoint[kp + idim] = tdum;
	}
    }

  

  ktyp = sptyp[knbpnt - 1];

  if (ktyp > 2)
    {

      

      kflag = 1;

      sptyp[knbpnt - 1] = 1;
      sptyp[knbpnt - 2] = 3;

      kk = idim * (knbpnt - 1);

      for (kp = 0; kp < idim; kp++)
	{
	  tdum = spoint[kk + kp];
	  spoint[kk + kp] = spoint[kk - idim + kp];
	  spoint[kk - idim + kp] = tdum;
	}
    }

  

  for (ki = 0; ki < knbpnt; ki++)
    {
      ktyp = sptyp[ki];
      if (ktyp < 3)
	ktell++;
    }

  if (ktell <= 1)
    goto err181;


  

  ktyp = 1;

  if (kflag == 1)
    {
      khelp = knbpnt;

      ktypm1 = ktyp;

      for (ki = 0; ki < khelp; ki++)
	{
	  ktyp = sptyp[ki];

	  if ((ktyp == 3 && ktypm1 == 3) ||
	      (ktyp == 4 && ktypm1 == 4) ||
	      (ktyp == 4 && ktypm1 == 3))
	    {
	      

	      knbpnt--;

	      for (kk = ki; kk < knbpnt; kk++)
		{
		  sptyp[kk] = sptyp[kk + 1];

		  kki = idim * kk;

		  for (kp = 0; kp < idim; kp++)
		    spoint[kki + kp] = spoint[kki + idim + kp];
		}
	    }
	}
    }


  *jnbpnt = knbpnt;

  goto out;


  

err181:
  *jstat = -181;
  s6err ("s1614", *jstat, kpos);
  goto out;

out:

  return;
}

