/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1618

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void s1618(double ematrix[], double eright[], double esol[], int in,
	   double *adiff)
#else
void s1618(ematrix, eright, esol, in, adiff)
     double ematrix[];
     double eright[];
     double esol[];
     int    in;
     double *adiff;
#endif
{
  int ki, kj;
  int kdim;
  double tdiff, tdum;
  double tmax = (double) 0.0;
  double tmxel = (double) 0.0;


  

  kdim = in *in;

  for (ki = 0; ki < kdim; ki++)
    {
      tdum = fabs (ematrix[ki]);
      if (tdum > tmxel)
	tmxel = tdum;
    }
  if (tmxel == (double) 0.0)
    tmxel = (double) 1.0;


  

  for (ki = 0; ki < in; ki++)
    {
      tdum = (double) 0.0;

      for (kj = 0; kj < in; kj++)
	tdum += ematrix[ki * in +kj] *esol[kj];

      tdiff = tdum - eright[ki];

      tdum = fabs (tdiff) / tmxel;
      if (tdum > tmax)
	tmax = tdum;
    }

  *adiff = tmax;

  return;
}

