/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1701

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1701(int ij,int imy,int ik,int in,int *jpl,int *jfi,int *jla,
	   double *et,double *etau,double *ep,double *galfa,int *jstat)
#else
void s1701(ij,imy,ik,in,jpl,jfi,jla,et,etau,ep,galfa,jstat)
     int    ij;
     int    imy;
     int    ik;
     int    in;
     int    *jpl;
     int    *jfi;
     int    *jla;
     double *et;
     double *etau;
     double *ep;
     double *galfa;
     int    *jstat;
#endif
{
  int kp;                  
  int kv,kkv;              
  double *ah;              
  double tbeta,tbeta1;     
  double td1,td2;          
  double *tu;              
  
  
  
  
  kp=ij+1; kkv=ij+ik; in--;
  while ((et[kp] == etau[imy]) && kp<kkv) {kp++; imy--;}
  
  
  
  
  for (kp=imy+1,kv=0,ij++; ij<kkv; ij++)
    if (et[ij] == etau[kp]) kp++;
    else      ep[kv++] = et[ij];
  
  
  
  
  *jpl=ik-imy-1;
  
  
  
  
  galfa += *jpl;
  
  
  
  
  galfa[imy] = 1;
  
  
  
  
  for (kp=0,kkv=ik-kv,ij=in+kv-1,in +=ik; kp<kv; kp++,kkv++,ep++)
    {
      
      
      
      
      
      if (kp>=imy) tbeta1=(*ep - *etau)*(*galfa)/(etau[kkv] - *etau);
      else         tbeta1=0;
      
      *jfi=max(1,imy-kp); *jla=min(imy,ij-kp);
      
      
      
      
      for (et=etau+ *jfi,tu=etau+ *jla,ah=galfa+ *jfi; et<=tu; et++,ah++)
	{
	  td1 = *ep - *et;
	  td2 = et[kkv] - *ep;
	  tbeta = *ah/(td1+td2);
	  *(ah-1) = td2*tbeta + tbeta1;
	  tbeta1 = td1*tbeta;
	}
      
      
      
      
      if (*jla<imy)
	{
	  et = etau + in;
	  *(ah-1) = tbeta1+(*et-*ep)*(*ah)/(*et - *(tu+1));
	} else  *(ah-1) = tbeta1;
    }
  
  
  
  
  if (kv) (*jfi)--;
  else   *jfi = *jla = imy;
  
  if ((*jfi)<0) *jfi = 0;
  if ((*jla)>in-ik) *jla = in-ik;
  
  
  
  
  *jstat = 0;
  goto out;
  
  
 out:
  return;
}

