/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1771

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static
void
   s1771_s9point(SISLCurve *,double [],double [],double [],double,double,
		 int,double *,double *,double,double *,double,int,int *);
static
double
s1771_s9del(double *,double *,double *,int);
#else
static void s1771_s9point();
static double s1771_s9del();
#endif


#if defined(SISLNEEDPROTOTYPES)
void
s1771(SISLPoint *ppoint,SISLCurve *pcurve,double aepsge,
	   double astart,double aend,double anext,double *cpos,int *jstat)
#else
void s1771(ppoint,pcurve,aepsge,astart,aend,anext,cpos,jstat)
     SISLPoint  *ppoint;
     SISLCurve  *pcurve;
     double aepsge;
     double astart;
     double aend;
     double anext;
     double *cpos;
     int    *jstat;
#endif
{
  int kstat = 0;         
  int kpos = 0;          
  int kleft=0;           
  int kdim;              
  double tdelta;         
  double tdist;          
  double td;             
  double tprev;          
  double *sval=SISL_NULL;     
  double *sdiff;         
  int quick = (*jstat);  
  int max_it = 20;       

  if (quick) max_it = 10;

  

  if (ppoint->idim != pcurve->idim) goto err106;

  kdim = pcurve -> idim;

  

  tdelta = pcurve->et[pcurve->in] - pcurve->et[pcurve->ik - 1];

  

  sval = newarray(4*kdim,double);
  if (sval == SISL_NULL) goto err101;

  sdiff = sval + 3*kdim;

  

  tprev = (double)HUGE;

  

  s1221(pcurve,2,anext,&kleft,sval,&kstat);
  if (kstat < 0) goto error;

  s6diff(ppoint->ecoef,sval,kdim,sdiff);

  tdist = s6length(sdiff,kdim,&kstat);

  td = s1771_s9del(sdiff,sval+kdim,sval+kdim+kdim,kdim);

  

  if (anext + td < astart) td = astart - anext;
  else if (anext + td > aend) td = aend - anext;

  s1771_s9point(pcurve,ppoint->ecoef,sval,sdiff,astart,aend,max_it,&anext,
	  &td,tdelta,&tdist,tprev,kleft,&kstat);
  if (kstat < 0) goto error;

  

  if (tdist <= aepsge)
    *jstat = 1;
  else
    *jstat = 2;

  *cpos = anext;

  

  goto out;

  

 err101: *jstat = -101;
  s6err("s1771",*jstat,kpos);
  goto out;

  

 err106: *jstat = -106;
  s6err("s1771",*jstat,kpos);
  goto out;

  

  error : *jstat = kstat;
  s6err("s1771",*jstat,kpos);
  goto out;

 out:    if (sval != SISL_NULL) freearray(sval);
}

#if defined(SISLNEEDPROTOTYPES)
static
void
s1771_s9point(SISLCurve *pcurve,double eval1[],double eval2[],double ediff[],
	      double astart,double aend,int max_it,double *cnext,double *ad,
	      double adel,double *cdist,double aprev,int ileft,int *jstat)
#else
static void s1771_s9point(pcurve,eval1,eval2,ediff,astart,aend,max_it,cnext,ad,adel,
			  cdist,aprev,ileft,jstat)
     SISLCurve  *pcurve;
     double eval1[];
     double eval2[];
     double ediff[];
     double astart;
     double aend;
     int    max_it;
     double *cnext;
     double *ad;
     double adel;
     double *cdist;
     double aprev;
     int    ileft;
     int    *jstat;
#endif
{
  int kstat = 0;            
  int kdim = pcurve->idim;  
  int knbit;                
  int kdiv = 0;             
  int kdiv2 = 0;            

  

  if (*cnext + *ad < astart)    *ad = astart - *cnext;
  else if (*cnext + *ad > aend) *ad = aend - *cnext;

  for (knbit=0;knbit<max_it;knbit++)
    {
      

      s1221(pcurve,2,*cnext + *ad,&ileft,eval2,&kstat);
      if (kstat < 0) goto error;

      s6diff(eval1,eval2,kdim,ediff);

      *cdist = s6length(ediff,kdim,&kstat);

      if (*cdist -aprev  <= REL_COMP_RES)
	{
	   if (kdiv2 > 4) break;
	   if (*cdist -aprev >= DZERO) kdiv2++;

	   kdiv = 0;
	  aprev = *cdist;
	  *cnext += *ad;

	  *ad = s1771_s9del(ediff,eval2+kdim,eval2+kdim+kdim,kdim);

	  

	  if (*cnext + *ad < astart)    *ad = astart - *cnext;
	  else if (*cnext + *ad > aend) *ad = aend - *cnext;
	}
      else
	{
	   kdiv++;
	   if (kdiv > 3) break;
	  (*ad) /= (double)2;
	  
	}
      if (fabs((*ad)/MAX(fabs(*cnext),adel)) <= REL_COMP_RES) break;
    }

  goto out;

  

  error : *jstat = kstat;
  s6err("s1771_s9point",*jstat,0);
  goto out;

 out:   ;
}

#if defined(SISLNEEDPROTOTYPES)
static
double
s1771_s9del(double *eco,double *eco1,double *eco2,int idim)
#else
static double s1771_s9del(eco,eco1,eco2,idim)
     double *eco;
     double *eco1;
     double *eco2;
     int    idim;
#endif
{
  double t1,t2,t3,t4,t5,t6;   
  double tmax,tmax1;          
  double ttol=(double)1e-10;  

  t1 =  s6scpr(eco,eco1,idim);
  t3 =  s6scpr(eco1,eco1,idim);
  t2 =  t3 - s6scpr(eco,eco2,idim);
  t4 =  -(double)2 * s6scpr(eco1,eco2,idim);

  tmax  = max(fabs(t1),fabs(t2));
  tmax1 = max(fabs(t3),fabs(t4));
  tmax  = max(tmax1,tmax);

  if (DEQUAL(tmax,DZERO))                    return DZERO;

  else if (fabs(t4)/tmax < ttol) 
    {
      if (fabs(t2)/tmax < ttol)
	{
          if (fabs(t3)/tmax < ttol)        return DZERO;
          else                             return (t1/t3);
	}
      else                                  return (t1/t2);
    }
  else  
    {
      t5 = t2*t2 - (double)2*t4*t1;
      if (t5 < DZERO)                       return (t1/t3);
      else
	{
          t6 = sqrt(t5);
          t5 = (t2 + t6)/t4;
          t6 = (t2 - t6)/t4;
	  t1 *= t3;


          

          if (t1 < DZERO)
	    {
	      if (t5 <= DZERO && t6 <= DZERO)
		{
		  if (t5 > t6)             return t5;
	          else                     return t6;
		}
	      else if (t5 <= DZERO)        return t5;
	      else if (t6 <= DZERO)        return t6;
              else                         return min(t5,t6);
	    }
	  else if (t1 > DZERO)
	    {
	      if (t5 >= DZERO && t6 >= DZERO)
		{
		  if (t5 < t6)             return t5;
	          else                     return t6;
		}
	      else if (t5 >= DZERO)        return t5;
	      else if (t6 >= DZERO)        return t6;
              else                         return max(t5,t6);
	    }
	  else                             return min(fabs(t5),fabs(t6));
	}
    }
}

