/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1780

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1780(SISLCurve *pc1,SISLCurve *pc2,SISLIntpt *vipt[],int *jstat)
#else
void s1780(pc1,pc2,vipt,jstat)
     SISLCurve *pc1;
     SISLCurve *pc2;
     SISLIntpt *vipt[];
     int   *jstat;
#endif
{ 
  int kstat = 0;     
  int kpos = 0;      
  int ki,kj;         
  int kdim;          
  int kord;          
  int kder;          
  int klefs = 0;     
  int kleft = 0;     
  double *sder1 = SISL_NULL; 
  double *sder2 = SISL_NULL; 
  double tpar1,tpar2;   
  double tang;          


  

  *jstat = 0;

  kdim = pc1 -> idim;
  if (kdim != pc2->idim) goto err106;

  

  tpar1 = min(vipt[0]->epar[0],vipt[1]->epar[0]);
  tpar2 = max(vipt[0]->epar[0],vipt[1]->epar[0]);

  for (ki=0;  pc1->et[ki] <= tpar1; ki++);
  ki--;

  for (kj=0;  pc1->et[kj] < tpar2; kj++);

  if (kj-ki > 1)
    goto out;

  tpar1 = min(vipt[0]->epar[1],vipt[1]->epar[1]);
  tpar2 = max(vipt[0]->epar[1],vipt[1]->epar[1]);

  for (ki=0;  pc2->et[ki] <= tpar1; ki++);
  ki--;

  for (kj=0;  pc2->et[kj] < tpar2; kj++);

  if (kj-ki >1)
    goto out;


  

  kord = MAX(pc1->ik,pc2->ik);
  kder = kord/2;
  kder = MAX(kder,kord-kder);

  

  if ((sder1 = newarray((2*(kder+1)*kdim),double)) == SISL_NULL) goto err101;
  sder2 = sder1 + (kder+1)*kdim;



  

  if (kder > 1)
    {

      

      

      s1221(pc1,kder,vipt[0]->epar[0],&kleft,sder1,&kstat);
      if (kstat < 0) goto error;

      s1221(pc2,kder,vipt[0]->epar[1],&klefs,sder2,&kstat);
      if (kstat < 0) goto error;

      for (ki=1; ki<kder; ki++)
	{

	  

	  tang = s6ang(sder1+(ki*kdim),sder2+(ki*kdim),kdim);

	  tang = min(tang,fabs(PI - tang));

	  if (tang > ANGULAR_TOLERANCE) goto out;
	}
    }

  kder = kord - kder;
  if (kder > 1)
    {

      

      

      s1221(pc1,kder,vipt[1]->epar[0],&kleft,sder1,&kstat);
      if (kstat < 0) goto error;

      s1221(pc2,kder,vipt[1]->epar[1],&klefs,sder2,&kstat);
      if (kstat < 0) goto error;

      for (ki=1; ki<kder; ki++)
	{

	
	  tang = s6ang(sder1+(ki*kdim),sder2+(ki*kdim),kdim);

	  tang = min(tang,fabs(PI - tang));

	  if (tang > ANGULAR_TOLERANCE) goto out;
	}
    }

  

  *jstat = 1;
  goto out;

  

err101: *jstat = -101;
        s6err("s1780",*jstat,kpos);
        goto out;
  
  

err106: *jstat = -106;
        s6err("s1780",*jstat,kpos);
        goto out;

  

error : *jstat = kstat;
        s6err("s1780",*jstat,kpos);
        goto out;

 out:

  

  if (sder1 != SISL_NULL) freearray(sder1);
}

