/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define S1785

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
#else
#endif

#if defined(SISLNEEDPROTOTYPES)
void
        s1785(SISLCurve *pcurve,SISLSurf *psurf,double aepsge,
	   double epar1[],double epar2[],int icur,int *jstat)
#else
   void s1785(pcurve,psurf,aepsge,epar1,epar2,icur,jstat)
      SISLCurve  *pcurve;
      SISLSurf   *psurf;
      double     aepsge;
      double     epar1[];
      double     epar2[];
      int        icur;
      int        *jstat;
#endif
{
	 int kstat;          
      int ki;             
      int kleftc=0;       
      int kleft1=0;       
      int kleft2=0;       
      int kleft1prev,kleft2prev;  
      int khelp;          
      int kn;             
      int kk;             
      int kk1,kk2,kn1,kn2;
      int kdimc;          
      int kdims;          
      int kpos=0;         
      int kderc=2;        
      int kders=1;        
      int kdum;           
      int kpar;           
      double tclose1,tclose2;  
      double snorm[3];    
      double s3dinf1[10]; 
      double *st;         
      double *st1;        
      double *st2;        
      double sfirst[2];   
      double slast[2];    
      double tfirst;      
      double tend;        
      double sderc[9];    
      double sders[18];   
      double tx,tx1,tx2;  
      double tcstep;      
      double tstep;       
      double tmaxinc;     
      double tlengthend;  
      double tincre;      
      double tsmax,tcmax; 
      double tdist;       
      double tref;        
      double sstart[2];   
      double send[2];     
      double snext[3];    
      double spos[3];     
      double snext2[2];   
      SISLPoint *qpoint=SISL_NULL;
      SISLCurve *qc = SISL_NULL;   
      
      *jstat = 0;
      
      
      
      sh1992su(psurf,0,aepsge,&kstat);
      if (kstat < 0) goto error;
      
      tsmax = MAX(psurf->pbox->e2max[0][0] - psurf->pbox->e2min[0][0],
		  psurf->pbox->e2max[0][1] - psurf->pbox->e2min[0][1]);
      tsmax = MAX(tsmax,psurf->pbox->e2max[0][2] - psurf->pbox->e2min[0][2]);
      
      
      
      sh1992cu(pcurve,0,aepsge,&kstat);
      if (kstat < 0) goto error;
      
      tcmax = MAX(pcurve->pbox->e2max[0][0] - pcurve->pbox->e2min[0][0],
		  pcurve->pbox->e2max[0][1] - pcurve->pbox->e2min[0][1]);
      tcmax = MAX(tcmax,pcurve->pbox->e2max[0][2] - pcurve->pbox->e2min[0][2]);
      
      
      
      kdimc = pcurve -> idim;
      kk    = pcurve -> ik;
      kn    = pcurve -> in;
      st    = pcurve -> et;
      
      
      
      kdims = psurf -> idim;
      kk1   = psurf -> ik1;
      kk2   = psurf -> ik2;
      kn1   = psurf -> in1;
      kn2   = psurf -> in2;
      st1   = psurf -> et1;
      st2   = psurf -> et2;
      
      
      
      tref = MAX(st[kn]-st[kk-1],MAX(st1[kn1]-st1[kk1-1],st2[kn2]-st2[kk2-1]));

      
      
      if (kdimc != 3 || kdims != 3) goto err105;
      
      sstart[0] = st1[kk1-1];
      sstart[1] = st2[kk2-1];
      send[0] = st1[kn1];
      send[1] = st2[kn2];
      
      
      
      if (icur ==1)
	 if ( epar1[0]<epar2[0] )
	 {
	    sfirst[0] = epar1[1];
	    sfirst[1] = epar1[2];
	    slast[0]  = epar2[1];
	    slast[1]  = epar2[2];
	    tfirst    = epar1[0];
	    tend      = epar2[0];
	 }
	 else
	 {
	    sfirst[0] = epar2[1];
	    sfirst[1] = epar2[2];
	    slast[0]  = epar1[1];
	    slast[1]  = epar1[2];
	    tfirst    = epar2[0];
	    tend      = epar1[0];
	 }
      else
	 if ( epar1[2]<epar2[2] )
	 {
	    sfirst[0] = epar1[0];
	    sfirst[1] = epar1[1];
	    slast[0]  = epar2[0];
	    slast[1]  = epar2[1];
	    tfirst    = epar1[2];
	    tend      = epar2[2];
	 }
	 else
	 {
	    sfirst[0] = epar2[0];
	    sfirst[1] = epar2[1];
	    slast[0]  = epar1[0];
	    slast[1]  = epar1[1];
	    tfirst    = epar2[2];
	    tend      = epar1[2];
	 }
      
      
      
      tfirst = MAX(tfirst,st[kk-1]);
      tend   = MIN(tend,st[kn]);
      if (DEQUAL(tfirst,tend)) goto out;
      
      
      
      spos[0] = sfirst[0];
      spos[1] = sfirst[1];
      
      
      
      tx2 = tfirst;
      kdum = MAX(kk1,kk2);
      kdum = MAX(kdum,kk);
      tmaxinc = (tend-tfirst)/(kdum*kdum);      
      
      
      
      s1221(pcurve,kderc,tx2,&kleftc,sderc,&kstat);
      if (kstat<0) goto error;
      
      
      
      s1421(psurf,kders,spos,&kleft1,&kleft2,sders,snorm,&kstat);
      if (kstat < 0) goto error;
      
      
      
      while (tx2 < tend)
      {
	 
	 
	 tx1 = tx2;
	 snext[0] = spos[0];
	 snext[1] = spos[1];
	 kleft1prev = kleft1;
	 kleft2prev = kleft2;
	 
	 
	 
	 s1307(sderc,kdimc,s3dinf1,&kstat);
	 if (kstat<0) goto error;
	 
	 
	 
	 tcstep = s1311(s3dinf1[3*kdimc],aepsge,tsmax,&kstat);
	 if (kstat<0) goto error;
	 
	 
	 
	 tlengthend = s6length(sderc+kdimc,kdimc,&kstat);
	 if (kstat<0) goto error;     
	 
	 

	 tstep = tcstep;
	 
	 
	 
	 
	 
	 
	 
	 if (DEQUAL(tlengthend,DZERO))
	    tincre = REL_PAR_RES;
	 else
	    tincre = tstep/tlengthend;
	 
	 
	 
	 tincre = MIN(tincre,tmaxinc);
	 tx2 = MIN(tx1 + tincre,st[kleftc+1]);
	 
	 for (ki=0, tx=(tx1+tx2)/(double)2.0; ki<2; ki++, tx=tx2)
	 {
	    if (tx >= tend) break;
	    
	    
	    
	    s1221(pcurve,kderc,tx,&kleftc,sderc,&kstat);
	    if (kstat<0) goto error;
	    
	    
	    
	    qpoint = newPoint(sderc,kdimc,0);
	    if (qpoint==SISL_NULL) goto err101;
	    
	    snext2[0] = snext[0];
	    snext2[1] = snext[1];
	    s1773(qpoint,psurf,aepsge,sstart,send,snext2,spos,&kstat);
	    if(kstat<0) goto error;
	    
	    freePoint(qpoint);  qpoint = SISL_NULL;
	    
	    
	    
	    s1421(psurf,kders,spos,&kleft1,&kleft2,sders,snorm,&kstat);
	    if (kstat<0) goto error;
	    
	    
	    
	    tdist = s6dist(sderc,sders,kdimc);
	    
	    if (tdist>aepsge)
	    {
	       
	       goto war01;
	    }
	    
	    
	    
	    
	    
	    for (khelp=kleft1prev-1; khelp >= 0 && DEQUAL(st1[khelp],st1[kleft1prev]); khelp--);
	    if (kleft1 != kleft1prev && 
		((DNEQUAL(spos[0]+tref,st1[kleft1]+tref) &&
		 DNEQUAL(snext[0]+tref,st1[kleft1]+tref)) || 
		 kleft1 != kleft1prev+1) &&
		((DNEQUAL(snext[0]+tref,st1[kleft1prev]+tref) &&
		 DNEQUAL(spos[0]+tref,st1[kleft1prev]+tref)) || kleft1 != khelp))
	    {
	       
	       
	       if (kleft1 > kleft1prev) kpar = kleft1prev + 1;
	       else if (snext[0] != st1[kleft1prev]) kpar = kleft1prev;
	       else kpar = khelp;
	       
	       
	       
	       s1437(psurf,st1[kpar],&qc,&kstat);
	       if (kstat < 0) goto error;
	       
	       
	       
	       s1770(pcurve,qc,aepsge,tx1,st2[kk2-1],tx,st2[kn2],(tx1+tx)/(double)2.0,
		     st2[kleft2],&tclose1,&tclose2,&kstat);
	       if (kstat < 0) goto error;
	       
	       
	       
	       spos[0] = st1[kpar];
	       spos[1] = tclose2;
	       tx2 = tclose1;
	       
              
	       
	       tx = (tx1 + tx2)/(double)2.0;
	       s1221(pcurve,kderc,tx,&kleftc,sderc,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       qpoint = newPoint(sderc,kdimc,0);
	       if (qpoint==SISL_NULL) goto err101;
	       
	       snext2[0] = snext[0];
	       snext2[1] = snext[1];
	       s1773(qpoint,psurf,aepsge,sstart,send,snext2,snext2,&kstat);
	       if(kstat<0) goto error;
	       
	       freePoint(qpoint);  qpoint = SISL_NULL;
	       
	       
	       
	       s1421(psurf,kders,snext2,&kleft1,&kleft2,sders,snorm,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       tdist = s6dist(sderc,sders,kdimc);
	       
	       if (tdist>aepsge)
	       {
		  
		  goto war01;
	       }

	       
	       
	       s1221(pcurve,kderc,tx2,&kleftc,sderc,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       s1421(psurf,kders,spos,&kleft1,&kleft2,sders,snorm,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       tdist = s6dist(sderc,sders,kdimc);
	       
	       if (tdist>aepsge)
	       {
		  
		  goto war01;
	       }
	       
	       
	       
	       ki = 2;
	       
	       
	       
	       if (qc != SISL_NULL) freeCurve(qc);  qc = SISL_NULL;
	    }
	    
	    
	    
	    
	    
	    for (khelp=kleft2prev-1; khelp >= 0 && DEQUAL(st2[khelp],st2[kleft2prev]); khelp--);
	    if (kleft2 != kleft2prev && 
		((DNEQUAL(spos[1]+tref,st2[kleft2]+tref) &&
		 DNEQUAL(snext[1]+tref,st2[kleft2]+tref)) || 
		 kleft2 != kleft2prev+1) &&
		((DNEQUAL(snext[1]+tref,st2[kleft2prev]+tref) &&
		 DNEQUAL(spos[1]+tref,st2[kleft2prev]+tref)) ||
		 kleft2 != khelp))
	    {
	       
	       
	       if (kleft2 > kleft2prev) kpar = kleft2prev + 1;
	       else if (snext[1] != st2[kleft2prev]) kpar = kleft2prev;
	       else kpar = khelp;
	       
	       
	       
	       s1436(psurf,st2[kpar],&qc,&kstat);
	       if (kstat < 0) goto error;
	       
	       
	       
	       s1770(pcurve,qc,aepsge,tx1,st1[kk1-1],tx,st1[kn1],(tx1+tx)/(double)2.0,
		     st1[kleft1],&tclose1,&tclose2,&kstat);
	       if (kstat < 0) goto error;
	       
	       
	       
	       spos[0] = tclose2;
	       spos[1] = st2[kpar];
	       tx2 = tclose1;
	       
	       
	       
	       tx = (tx1 + tx2)/(double)2.0;
	       s1221(pcurve,kderc,tx,&kleftc,sderc,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       qpoint = newPoint(sderc,kdimc,0);
	       if (qpoint==SISL_NULL) goto err101;
	       
	       snext2[0] = snext[0];
	       snext2[1] = snext[1];
	       s1773(qpoint,psurf,aepsge,sstart,send,snext2,snext2,&kstat);
	       if(kstat<0) goto error;
	       
	       freePoint(qpoint);  qpoint = SISL_NULL;
	       
	       
	       
	       s1421(psurf,kders,snext2,&kleft1,&kleft2,sders,snorm,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       tdist = s6dist(sderc,sders,kdimc);
	       
	       if (tdist>aepsge)
	       {
		  
		  goto war01;
	       }

	       
	       
	       s1221(pcurve,kderc,tx2,&kleftc,sderc,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       s1421(psurf,kders,spos,&kleft1,&kleft2,sders,snorm,&kstat);
	       if (kstat<0) goto error;
	       
	       
	       
	       tdist = s6dist(sderc,sders,kdimc);
	       
	       if (tdist>aepsge)
	       {
		  
		  goto war01;
	       }
	       
	       
	       
	       ki = 2;
	       
	       
	       
	       if (qc != SISL_NULL) freeCurve(qc);  qc = SISL_NULL;
	    }
	 }
      }
      
      
      
      *jstat = (DEQUAL(tfirst,tend)) ? 0 : 1;
      goto out;
      
      
      war01: *jstat = 0;
      goto out;
      
      
      
      err101: *jstat = -101;
      s6err("S1785",*jstat,kpos);
      goto out;
      
      
      
      err105: *jstat = -105;
      s6err("S1785",*jstat,kpos);
      goto out;
      
      
      
      error:  *jstat = kstat;
      s6err("S1785",*jstat,kpos);
      goto out;
      
      
      out:
	 
	 return;
      }          
	 
#if 0
	 
#if defined(SISLNEEDPROTOTYPES)
   static
      void
            s1785_s9eval(double eders[],double enorms[],double etanc[], 
		   double ederc[],int idim, int *jstat)
#else
static void s1785_s9eval(eders,enorms,etanc,ederc,idim,jstat)
	double eders[];
	double enorms[];
	double etanc[];
	double ederc[];
	int idim;
	int *jstat;
#endif

{
   int kstat = 0;         
   int ki;                
   int ksign = 1;         
   double tfac1,tfac2,tfac3;  
   
   
   
   memcopy(ederc,eders,idim,DOUBLE);
   
   
   
   s6findfac(eders+idim,eders+2*idim,enorms,etanc,idim,ksign,&tfac1,&tfac2,
	     &tfac3,&kstat);
   if (kstat < 0) goto error;
   
   
   
   for (ki=0; ki<idim; ki++)
   {
      ederc[idim+ki] = tfac1*eders[idim+ki] + tfac2*eders[2*idim+ki];
      ederc[2*idim+ki] = tfac1*tfac1*eders[3*idim+ki] 
	 + (double)2.0*tfac1*tfac2*eders[4*idim+ki] + tfac2*tfac2*eders[5*idim+ki];
   }
   
   *jstat = 0;
   goto out;
   
   
   
   error:
      *jstat = kstat;
   goto out;
   
   out:
      return;
}

#endif 

