/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1790

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s1790(SISLObject *po1,SISLObject *po2,double aepsge,int *jstat)
#else
void s1790(po1,po2,aepsge,jstat)
     SISLObject *po1;
     SISLObject *po2;
     double aepsge;
     int    *jstat;
#endif
                                     
{
  int kstat = 0;        
  int kpos = 0;         
  int kant;             
  int kbez = 0;         
  int kdim;	      
  int ki,kj=0;          
  double t1,t2,t3,t4;   
  double *tmin1,*tmax1; 
  double *tmin2,*tmax2; 
  
  
  
  if (po1->iobj == SISLPOINT)
    {
      
      
      kdim = po1->p1->idim;
      
      
      
      
      if (po1->p1->pbox == SISL_NULL)
	{
	  
	  
	  s1992(po1,&kstat);
	  if (kstat<0) goto error;
	}
      
      
      
      tmax1 = po1->p1->pbox->emax;
      tmin1 = po1->p1->pbox->emin;
    }
  else
    if (po1->iobj == SISLCURVE)
      {
	
	
	kdim = po1->c1->idim;
	
	
	
	if (po1->c1->in == po1->c1->ik) kbez = 1;
	
	
	
	if (po1->c1->pbox == SISL_NULL)
	  {
	    
	    
	    s1992(po1,&kstat);
	    if (kstat<0) goto error;
	  }
	
	
	
	tmax1 = po1->c1->pbox->emax;
	tmin1 = po1->c1->pbox->emin;
      }
    else
      if (po1->iobj == SISLSURFACE)
	{
	  
	  
	  kdim = po1->s1->idim;
	  
	  
	  
	  if (po1->s1->in1 == po1->s1->ik1 &&
	      po1->s1->in2 == po1->s1->ik2)    kbez = 1;
	  
	  
	  
	  if (po1->s1->pbox == SISL_NULL)
	    {
	      
	      
	      s1992(po1,&kstat);
	      if (kstat<0) goto error;
	    }
	  
	  
	  
	  tmax1 = po1->s1->pbox->emax;
	  tmin1 = po1->s1->pbox->emin;
	}
      else  goto err121;
  
  
  
  
  if (po2->iobj == SISLPOINT)
    {
      
      
      ki = po2->p1->idim;
      
      
      
      
      if (po2->p1->pbox == SISL_NULL)
	{
	  
	  
	  s1992(po2,&kstat);
	  if (kstat<0) goto error;
	}
      
      
      
      tmax2 = po2->p1->pbox->emax;
      tmin2 = po2->p1->pbox->emin;
    }
  else
    if (po2->iobj == SISLCURVE)
      {
	
	
	ki = po2->c1->idim;
	
	
	
	if (po2->c1->in == po2->c1->ik) kbez = 1;
	
	
	
	if (po2->c1->pbox == SISL_NULL)
	  {
	    
	    
	    s1992(po2,&kstat);
	    if (kstat<0) goto error;
	  }
	
	
	
	tmax2 = po2->c1->pbox->emax;
	tmin2 = po2->c1->pbox->emin;
      }
    else
      if (po2->iobj == SISLSURFACE)
	{
	  
	  
	  ki = po2->s1->idim;
	  
	  
	  
	  if (po2->s1->in1 == po2->s1->ik1 &&
	      po2->s1->in2 == po2->s1->ik2)    kbez = 1;
	  
	  
	  
	  if (po2->s1->pbox == SISL_NULL)
	    {
	      
	      
	      s1992(po2,&kstat);
	      if (kstat<0) goto error;
	    }
	  
	  
	  
	  tmax2 = po2->s1->pbox->emax;
	  tmin2 = po2->s1->pbox->emin;
	}
      else goto err121;
  
  
  
  if (ki != kdim ) goto err106;
  else
    if (kdim < 1 )   goto err105;
  
  
  
  
  if (kdim == 3) kant = 12;
  else
    if (kdim == 2) kant = 4;
    else           kant = kdim;
  
  
  	
  
  for (ki=0; ki<kant; ki++,tmin1++,tmax1++,tmin2++,tmax2++)
    {
      
      
      if (*tmax1 > *tmax2)
	{
	  t1 = *tmax1;
	  t2 = *tmin1;
	  t3 = *tmax2;
	  t4 = *tmin2;
	}
      else
	{
	  t1 = *tmax2;
	  t2 = *tmin2;
	  t3 = *tmax1;
	  t4 = *tmin1;
	}
      
      
      if ((po1->iobj == SISLPOINT || po2->iobj == SISLPOINT) && kdim != 1)
	{
	  t1 += 0.1 *  aepsge;
	  t2 -= 0.1 *  aepsge;
	}
      
      if (t1 - min(t2,t4) <= aepsge)
	kj++;                 
      else if (t3 < t2)
	{
	  *jstat = 0;           
	  goto out;
	}
      else if (kdim != 1 && t3 - t2 <= aepsge && t3 - t4 > aepsge &&
	       t1 - t2 > aepsge && kbez)
	{
	  *jstat = 2;           
	  goto degenerate;
	}
      else if ( kdim == 1 && (t1 - t4 <= aepsge || t3 - t2 <= aepsge) &&
	       kbez)
	{
	  *jstat = 2;           
	  goto out;
	}
      
    }
  
  if (kj == kant)
    *jstat = 3;                   
  else
    *jstat = 1;                   
  
  
  
  
 degenerate:
  
  if (kdim != 1 && po1->iobj > SISLPOINT)
    {
      if (po1->iobj == SISLCURVE)
	{
	  tmin1 = po1->c1->pbox->emin;
	  tmax1 = po1->c1->pbox->emax;
	}
      
      else if (po1->iobj == SISLSURFACE)
	{
	  tmin1 = po1->s1->pbox->emin;
	  tmax1 = po1->s1->pbox->emax;
	}
      
      for(ki=0;ki<kdim;ki++,tmin1++,tmax1++)
	if (DNEQUAL(*tmin1,*tmax1)) break;
      
      if (ki == kdim)
	{
	  *jstat = 4;
	  goto out;
	}
    }
  
  if (kdim != 1 && po2->iobj > SISLPOINT)
    {
      if (po2->iobj == SISLCURVE)
	{
	  tmin1 = po2->c1->pbox->emin;
	  tmax1 = po2->c1->pbox->emax;
	}
      
      else if (po2->iobj == SISLSURFACE)
	{
	  tmin1 = po2->s1->pbox->emin;
	  tmax1 = po2->s1->pbox->emax;
	}
      
      for(ki=0;ki<kdim;ki++,tmin1++,tmax1++)
	if (DNEQUAL(*tmin1,*tmax1)) break;
      
      if (ki == kdim)
	{
	  *jstat = 4;
	  goto out;
	}
    }
  goto out;
  
  
  
 err106: *jstat = -106;
  s6err("s1790",*jstat,kpos);
  goto out;
  
  
  
 err105: *jstat = -105;
  s6err("s1790",*jstat,kpos);
  goto out;
  
  
  
 err121: *jstat = -121;
  s6err("s1790",*jstat,kpos);
  goto out;
  
  
  
 error:  *jstat = kstat;
  s6err("s1790",*jstat,kpos);
  goto out;
  
 out:	return;
}






