/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1834

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
static void
s1834_s9mat2d(double [],double []);
static void
s1834_s9mat3d(double [],double [],double []);
#else
static void s1834_s9mat2d();
static void s1834_s9mat3d();
#endif

#if defined(SISLNEEDPROTOTYPES)
void 
s1834(double ecoef1[],int in1,double ecoef2[],int in2,int idim,
	   double edir1[],double edir2[],int *jstat)
#else
void s1834(ecoef1,in1,ecoef2,in2,idim,edir1,edir2,jstat)
     double ecoef1[];
     int    in1;
     double ecoef2[];
     int    in2;
     int    idim;
     double edir1[];
     double edir2[];
     int    *jstat;
#endif
{                                   
  int kpos = 0;    
  int kopen;       
  int kclose;      
  double tleng;    
  double *smat;    
  double *smin1,*smax1; 
  double *smin2,*smax2; 
  double *s1,*s2,*s3,*s4;  
  
  smat = smin1 = smin2 = smax1 = smax2 = SISL_NULL;
  
  
  
  if (idim != 2 && idim != 3) goto err105;
  
  
  
  smin1 = newarray(idim,double);
  smin2 = newarray(idim,double);
  smax1 = newarray(idim,double);
  smax2 = newarray(idim,double);
  smat = new0array(idim*idim,double);
  if (smin1 == SISL_NULL || smin2 == SISL_NULL || smax1 == SISL_NULL ||
      smax2 == SISL_NULL || smat == SISL_NULL) goto err101;
  
  
  
  for (s2=smin1+idim; smin1<s2; smin1++,smin2++,smax1++,smax2++)
    {
      *smin1 = *smin2 = (double)9999999999.0;
      *smax1 = *smax2 = (double)-9999999999.0;
    }
  smin1 -= idim;
  smin2 -= idim;
  smax1 -= idim;
  smax2 -= idim;
  
  
  
  if (idim == 2)
    
    
    
    s1834_s9mat2d(smat,edir1);
  else
    
    
    
    s1834_s9mat3d(smat,edir1,edir2);
  
  
  
  
  
  for (s1=smat,s3=smat+idim*idim; s1<s3; s1+=idim,smin1++,smax1++)
    for (s2=ecoef1,s4=s2+idim*in1; s2<s4; s2+=idim)
      {
	tleng = s6scpr(s1,s2,idim);
	*smin1 = MIN(*smin1,tleng);
	*smax1 = MAX(*smax1,tleng);
      }
  smin1 -= idim;
  smax1 -= idim;
  
  
  
  for (s1=smat,s3=smat+idim*idim; s1<s3; s1+=idim,smin2++,smax2++)
    for (s2=ecoef2,s4=s2+idim*in2; s2<s4; s2+=idim)
      {
	tleng = s6scpr(s1,s2,idim);
	*smin2 = MIN(*smin2,tleng);
	*smax2 = MAX(*smax2,tleng);
      }
  smin2 -= idim;
  smax2 -= idim;
  
  
  
  kopen = 0;
  kclose = 1;
  for (s2=smin1+idim; smin1<s2; smin1++,smax1++,smin2++,smax2++)
    {
      if (DEQUAL(MIN(*smin1,*smin2),MAX(*smax1,*smax2)))
	continue;
      
      if (*smin1 > *smax2 || *smin2 > *smax1)
	
	
	
	kclose = 0;
      else if (DEQUAL(*smin1,*smax2) || DEQUAL(*smin2,*smax1))
	
	
	
	kopen = 1;
    }
  smin1 -= idim;
  smax1 -= idim;
  smin2 -= idim;
  smax2 -= idim;
  
  if (kopen == 0 && kclose == 1)
    
    
    
    *jstat = 1;
  else if (kopen == 1)
    
    
    
    *jstat = 2;                                            
  else
    
    
    
    *jstat = 0;
  
  
  
  goto out;
  
  
  
 err101: *jstat = -101;
  s6err("s1834",*jstat,kpos);
  goto out;
  
  
  
 err105: *jstat = -105;
  s6err("s1834",*jstat,kpos);
  goto out;
  
 out:
  
  
  
  if (smin1 != SISL_NULL) freearray(smin1);
  if (smin2 != SISL_NULL) freearray(smin2);
  if (smax1 != SISL_NULL) freearray(smax1);
  if (smax2 != SISL_NULL) freearray(smax2);
  if (smat != SISL_NULL) free0array(smat);
  
  return;
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1834_s9mat2d(double emat[],double edir[])
#else
static void s1834_s9mat2d(emat,edir)
     double emat[];
     double edir[];
#endif
{          
  int kstat = 0;   
  double tlength;  
  double sdir[2];  
  
  tlength = s6norm(edir,2,sdir,&kstat);
  if (kstat == 0)
    
    
    
    emat[0] = emat[3] = (double)1.0;
  else
    {          
      
      
      
      emat[0] = sdir[0];
      emat[1] = -sdir[1];
      emat[2] = sdir[1];
      emat[3] = sdir[0];
    }
}

#if defined(SISLNEEDPROTOTYPES)
static void
s1834_s9mat3d(double emat[],double edir1[],double edir2[])
#else
static void s1834_s9mat3d(emat,edir1,edir2)
     double emat[];
     double edir1[];
     double edir2[];
#endif
{          
  int kstat = 0;    
  double snorm[3];  
  double sdir[3];   
  double *s1;       
  double tleng1,tleng2; 
  double ta1,ta2,ta3,tb1,tb2,tb3,td1,td2,tl1,tl2,tl3; 
  
  
  
  s6crss(edir1,edir2,snorm);
  
  
  
  tleng1 = s6norm(snorm,3,snorm,&kstat);
  
  
  
  tleng2 = s6norm(edir1,3,sdir,&kstat);
  
  
  
  ta1 = snorm[0];
  ta2 = snorm[1];
  ta3 = snorm[2];
  tl1 = sqrt(ta2*ta2+ta3*ta3);
  
  
  
  if ((DEQUAL(tleng1,DZERO) || DEQUAL(tl1,DZERO)) && DEQUAL(tleng2,DZERO))
    
    
    
    emat[0] = emat[4] = emat[8] = (double)1.0;
  else if (DEQUAL(tleng1,DZERO) || DEQUAL(tl1,DZERO))
    {
      
      
      
      tb1 = sdir[0];
      tb2 = sdir[1];
      tb3 = sdir[2];
      tl3 = sqrt(tb1*tb1+tb2*tb2);
      
      if (DEQUAL(tl3,DZERO)) emat[0] = emat[4] = emat[8] = (double)1.0;
      else
	{
	  s1      = emat;    
	  *(s1++) = tb1;
	  *(s1++) = tb2;
	  *(s1++) = tb3;
	  *(s1++) = -tb2/tl3;
	  *(s1++) = tb1/tl3;  
	  *(s1++) = DZERO;
	  *(s1++) = -tb1*tb3/tl3;
	  *(s1++) = -tb2*tb3/tl3;  
	  *(s1++) = tl3;
	}
    }
  else
    {
      td1 = edir1[0]/tl1;
      td2 = (ta3*edir1[1] - ta2*edir1[2])/tl1;
      tl2 = sqrt(td1*td1+td2*td2);
      
      if (DEQUAL(tl2,DZERO))
	{
	  
	  
	  
	  s1      = emat;
	  *(s1++) = tl1;
	  *(s1++) = -ta1*ta2/tl1;
	  *(s1++) = -ta1*ta3/tl1;
	  *(s1++) = DZERO;
	  *(s1++) = ta3/tl1;
	  *(s1++) = -ta2/tl1;
	  *(s1++) = ta1;
	  *(s1++) = ta2;
	  *(s1++) = ta3;
	}
      else
	{
	  
	  
	  
	  s1      = emat;
	  *(s1++) = td1*tl1/tl2;
	  *(s1++) = (-ta1*ta2*td1 + ta3*td2)/(tl1*tl2);
	  *(s1++) = (-ta1*ta3*td1 - ta2*td2)/(tl1*tl2);
	  *(s1++) = -td2*tl1/tl2;
	  *(s1++) = (ta1*ta2*td2 + ta3*td1)/(tl1*tl2);
	  *(s1++) = (ta1*ta3*td2 - ta2*td1)/(tl1*tl2);
	  *(s1++) = ta1;
	  *(s1++) = ta2;
	  *(s1++) = ta3;
	}
    }
}

