/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1891

#include "sislP.h"
#define MAX_SIZE  50


#if defined(SISLNEEDPROTOTYPES)
void
s1891 (double etau[], double epoint[], int idim, int inbpnt, int iright,
       int eder[], int iopen, double et[], double *ebcoef[], int *in,
       int ik, int inlr, int inrc, int *jstat)

#else
void
s1891 (etau, epoint, idim, inbpnt, iright, eder, iopen, et, ebcoef,
       in, ik, inlr, inrc, jstat)
     double etau[];
     double epoint[];
     int idim;
     int inbpnt;
     int iright;
     int eder[];
     int iopen;
     double et[];
     double *ebcoef[];
     int *in;
     int ik;
     int inlr;
     int inrc;
     int *jstat;
#endif
{
  int kstat = 0;
  int kpos = 0;			
  int ii;			
  int limit1, limit2;		
  int kj, kl;
  int kdum, stop;
  int nur;			
  int inlx;			
  int inrx;			
  int edarray[MAX_SIZE];        
  int alloc_needed=FALSE;
  int *ed = SISL_NULL;		
  double *ewarray=SISL_NULL;         
  double *ew1 = SISL_NULL;		
  double *ew2 = SISL_NULL;
  double *ew3 = SISL_NULL;

  *jstat = 0;


  

  if (ik < 1 || idim < 1) goto err112;

  

  *in = inbpnt;
  if (iopen != SISL_CRV_OPEN)    *in +=ik - 1;

  *ebcoef = new0array (*in *idim * iright, DOUBLE);
  if (*ebcoef == SISL_NULL) goto err101;

  if ((nur = inbpnt - inlr) > MAX_SIZE)
    alloc_needed = TRUE;

  

  inlx = MAX (1, inlr);
  inrx = MAX (1, inrc);
  limit1 = (ik * nur) + (inrx * nur) + (inlx * inbpnt);
  
  if ((ewarray = new0array(limit1 + 1,DOUBLE)) == SISL_NULL) goto err101;
  
  ew1 = ewarray;
  ew2 = ew1 + (ik * nur);
  ew3 = ew2 + (inrx * nur);

  if (alloc_needed)
    {
       if ((ed = new0array(nur,INT)) == SISL_NULL)
	 goto err101;
    }
  else
    ed = edarray;
  
  s1925 (etau, epoint, inbpnt, eder, et, *ebcoef,*in, ik, iright, 
	 idim, ew1, nur, ed, ew2, inrc, ew3, inlr, &kstat);
  if (kstat < 0) goto error;

  

  if (iopen != SISL_CRV_OPEN)
    {
      stop = ik - 1;
      for (kl = 0; kl < iright; kl++)
	{
	  kdum = *in *kl;
	  for (kj = 0; kj < stop; kj++)
	    {
	      limit2 = (kj + kdum) * idim;
	      limit1 = inbpnt * idim + limit2;
	      for (ii = 0; ii < idim; ii++)
		(*ebcoef)[limit1 + ii] = (*ebcoef)[limit2 + ii];
	    }
	}
    }

  goto out;

  

  error:
    *jstat = kstat;
    s6err ("s1891", *jstat, kpos);
    goto out;

  

  err101:
    *jstat = -101;
    s6err ("s1891", *jstat, kpos);
    goto out;

  

  err112:
    *jstat = -112;
    s6err ("s1891", *jstat, kpos);
    goto out;

  out:
    if (alloc_needed)    freearray (ed);
    if (ewarray)         freearray (ewarray);
    return;
}
#undef MAX_SIZE

