/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1900

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
   s1900 (double param[], double knots[], double econd[], int ntype[],
       int inpt, int ik, int idim, int iopen,
       double *cendpar, SISLCurve ** rcurve,
       double **gpar, int *jnbpar, int *jstat)
#else
void
s1900 (param, knots, econd, ntype, inpt, ik, idim, iopen,
       cendpar, rcurve, gpar, jnbpar, jstat)
     double param[];
     double knots[];
     double econd[];
     int ntype[];
     int inpt;
     int ik;
     int idim;
     int iopen;
     double *cendpar;
     SISLCurve **rcurve;
     double **gpar;
     int *jnbpar;
     int *jstat;
#endif
{
  int kstat = 0;		
  int kpos = 0;
  int ki;			
  int knpt;			
  int kn;			
  int kordr;			
  int kright = 1;		
  int knlr = 0;			
  int knrc = 0;			
  double *scoef = SISL_NULL;		
  int *ltype = SISL_NULL;		
  double *scond = SISL_NULL;		
  double *lpar = SISL_NULL;		
  int *sder = SISL_NULL;		
  SISLCurve *qc = SISL_NULL;		
  SISLCurve *qc2 = SISL_NULL;	

  *jstat = 0;

  

  s1908 (econd, ntype, param, inpt, ik, idim, iopen,
	 &scond, &ltype, &lpar, &knpt, &kstat);
  if (kstat < 0) goto error;

  

  if ((sder = newarray (knpt, INT)) == SISL_NULL)
    goto err101;

  for (ki = 0; ki < knpt; ki++)
    sder[ki] = (int) fabs ((double) ltype[ki]);

  

  kordr = MIN (ik, knpt);

  if (iopen != SISL_CRV_OPEN)
    {
      knlr = kordr / 2;
      knrc = kordr - knlr - 1;
      knpt--;
    }
  

  s1891 (lpar, scond, idim, knpt, kright, sder, iopen, knots,
	 &scoef, &kn, kordr, knlr, knrc, &kstat);
  if (kstat < 0) goto error;

  

  qc = newCurve (kn, kordr, knots, scoef, 1, idim, 1);
  if (qc == SISL_NULL) goto err101;

  if (!(iopen == SISL_CRV_OPEN))
    {
      

      s1713 (qc, knots[kordr - 1], knots[kn], &qc2, &kstat);
      if (kstat < 0) goto error;

      if (qc != SISL_NULL) freeCurve (qc);
      qc = qc2;
    }

  if (kordr < ik)
    {
      

      qc2 = SISL_NULL;
      s1750 (qc, ik, &qc2, &kstat);
      if (kstat < 0) goto error;
      if (qc != SISL_NULL) freeCurve (qc);
      qc = qc2;
    }

  

  qc->cuopen = iopen;

  

  

  *gpar = lpar;

  *jnbpar = 1;
  for (ki = 1; lpar[ki] < *cendpar; ki++)
    {
      if (lpar[ki - 1] < lpar[ki])
	(*gpar)[(*jnbpar)++] = lpar[ki];
    }
  (*gpar)[(*jnbpar)++] = lpar[ki];

  *gpar = increasearray (*gpar, *jnbpar, DOUBLE);

  *rcurve = qc;
  goto out;

  

  err101:
    *jstat = -101;
    s6err ("s1900", *jstat, kpos);
    goto out;

  

  error:
    *jstat = kstat;
    s6err ("s1900", *jstat, kpos);
    goto out;

out:
  

  if (scond != SISL_NULL)    freearray (scond);
  if (scoef != SISL_NULL)    freearray (scoef);
  if (sder != SISL_NULL)     freearray (sder);
  if (ltype != SISL_NULL)    freearray (ltype);

  return;
}

