/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1905

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1905 (double econd1[], int ntype1[], int inpt1, int ik, int idim,
       int iopen, double **gcond2, int **mtype2, int *jnpt2, int *jstat)
#else
void
s1905 (econd1, ntype1, inpt1, ik, idim, iopen, gcond2, mtype2, jnpt2, jstat)
     double econd1[];
     int ntype1[];
     int inpt1;
     int ik;
     int idim;
     int iopen;
     double **gcond2;
     int **mtype2;
     int *jnpt2;
     int *jstat;
#endif
{
   int kstat = 0;
  int kpos = 0;
  int kmaxpt = inpt1 + ik*(iopen != SISL_CRV_OPEN); 
  int knpt = 0;			
  int ki, kj, kl;		
  int ktype;			
  int kneg;			
  int kder;			
  int *lder = SISL_NULL;		
  double *sdum = SISL_NULL;		
  double tdist;                 
  double tref;                  

  *jstat = 0;


  

  if ((*gcond2 = newarray (kmaxpt * idim, DOUBLE)) == SISL_NULL)
    goto err101;
  if ((*mtype2 = newarray (kmaxpt, INT)) == SISL_NULL)
    goto err101;

  

  if ((lder = new0array (ik, INT)) == SISL_NULL)
    goto err101;
  if ((sdum = newarray (idim, DOUBLE)) == SISL_NULL)
    goto err101;


  

  for (ki = 0; ki < inpt1; ki++)
    if (ntype1[ki] == 0)
      break;

  lder[0] = 1;
  (*mtype2)[0] = 0;
  memcopy (*gcond2, econd1 + ki * idim, idim, DOUBLE);
  knpt++;
  ki++;

  

  for (kj = ki - 2; kj >= 0 && ntype1[kj] < 0; kj--)
    {
      ktype = abs (ntype1[kj]);
      if (ktype >= ik)
	continue;		
      if (lder[ktype])
	continue;		
      lder[ktype] = 1;
      (*mtype2)[knpt] = ktype;
      memcopy ((*gcond2) + knpt * idim, econd1 + kj * idim, idim, DOUBLE);
      knpt++;
    }

  

  for (; ki < inpt1 && ntype1[ki] > 0; ki++)
    {
      ktype = ntype1[ki];
      if (ktype >= ik)
	continue;		
      if (lder[ktype])
	continue;		
      lder[ktype] = 1;
      (*mtype2)[knpt] = ktype;
      memcopy ((*gcond2) + knpt * idim, econd1 + ki * idim, idim, DOUBLE);
      knpt++;
    }

  

  for (; ki < inpt1; ki = kj)
    {
      

      for (kj = 0; kj < ik; kj++)
	lder[kj] = 0;

      

      kneg = 1;
      for (kj = ki; kj < inpt1 && (kneg || ntype1[kj] > 0); kj++)
	{
	  ktype = abs (ntype1[kj]);
	  if (ktype == 0)
	    kneg = 0;		
	  if (ktype >= ik)
	    continue;		
	  if (lder[ktype])
	    continue;		
	  lder[ktype] = 1;
	  (*mtype2)[knpt] = ntype1[kj];
	  memcopy ((*gcond2) + knpt * idim, econd1 + kj * idim, idim, DOUBLE);
	  knpt++;
	}
    }

  if (iopen != SISL_CRV_OPEN)
  {
     
     
     
     
     for (kj=ki-1; kj<=0; kj--)
        if (ntype1[kj] == 0) break;
     for (kl=0;  kl<inpt1; kl++)
        if (ntype1[kl] == 0) break;
     tdist = s6dist(econd1+kl*idim,econd1+kj*idim,idim);
     tref = MAX(s6length(econd1+kl*idim,idim,&kstat),
		s6length(econd1+kj*idim,idim,&kstat));
     
     if (DNEQUAL(tdist+tref,tref))
     {
	
	
	for (kj = 0; kj < ik; kj++)
	   lder[kj] = 0;
	
	
	
	for (kj = ki; kj < inpt1 && ntype1[kj] < 0; kj++)
	{
	   ktype = abs (ntype1[kj]);
	   if (ktype >= ik)
	      continue;		
	   if (lder[ktype])
	      continue;		
	   lder[ktype] = 1;
	   (*mtype2)[knpt] = ntype1[kj];
	   memcopy ((*gcond2) + knpt * idim, econd1 + kj * idim, idim, DOUBLE);
	   knpt++;
	}
	
	
	
	for (kj = 0; kj < inpt1 && ntype1[kj] > 0; kj++)
	{
	   ktype = ntype1[kj];
	   if (ktype >= ik)
	      continue;		
	   if (lder[ktype])
	      continue;		
	   lder[ktype] = 1;
	   (*mtype2)[knpt] = -ktype;
	   memcopy ((*gcond2) + knpt * idim, econd1 + kj * idim, idim, DOUBLE);
	   knpt++;
	}
	
	
	
	for (; ntype1[kj] != 0; kj++) ;
	
	(*mtype2)[knpt] = 0;
	memcopy ((*gcond2) + knpt * idim, econd1 + kj * idim, idim, DOUBLE);
	knpt++;
     }
  }

  

  kneg = 1;
  for (kder = 0, ki = knpt - 1; ki >= 0 && (kneg || (*mtype2)[ki] < 0); ki--, kder--)
    {
      if ((*mtype2)[ki] != kder)
	{
	  for (kj = ki - 1; kj >= 0 && (kneg || (*mtype2)[kj] < 0) &&
	       (*mtype2)[kj] != kder; kj--) ;
	  if ((*mtype2)[kj] == kder)
	    {
	      

	      memcopy (sdum, (*gcond2) + kj * idim, idim, DOUBLE);
	      memcopy ((*gcond2) + kj * idim, (*gcond2) + ki * idim, idim, DOUBLE);
	      memcopy ((*gcond2) + ki * idim, sdum, idim, DOUBLE);
	      (*mtype2)[kj] = (kder == 0) ? -(*mtype2)[ki] : (*mtype2)[ki];
	      (*mtype2)[ki] = kder;
	      if (kder == 0)
		{
		  for (kj++; kj<ki; kj++)
		    (*mtype2)[kj] *= -1;
		}
	    }
	}
      kneg = 0;
    }

  

  *gcond2 = increasearray (*gcond2, knpt * idim, DOUBLE);
  if (*gcond2 == SISL_NULL)
    goto err101;
  *mtype2 = increasearray (*mtype2, knpt, INT);
  if (*mtype2 == SISL_NULL)
    goto err101;

  

  *jnpt2 = knpt;
  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1905", *jstat, kpos);
  goto out;

out:
  

  if (lder != SISL_NULL)
    freearray (lder);
  if (sdum != SISL_NULL)
    freearray (sdum);

  return;
}

