/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1911

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1911 (double econd[], int ntype[], int inpt, int idim, int iopen,
       double astpar, double *cendpar, double *epar1[],
       double *epar2[], int *jstat)
#else
void
s1911 (econd, ntype, inpt, idim, iopen, astpar, cendpar, epar1, epar2, jstat)
     double econd[];
     int ntype[];
     int inpt;
     int idim;
     int iopen;
     double astpar;
     double *cendpar;
     double *epar1[];
     double *epar2[];
     int *jstat;
#endif
{
  int count1, count2, count3;	
  int knpt;			

  double kpar, npar;		
  int kpos = 0;

  *jstat = 0;


  if (iopen != SISL_CRV_OPEN)
    knpt = inpt + 1;
  else
    knpt = inpt;


  

  *epar1 = newarray (knpt, DOUBLE);
  if (*epar1 == SISL_NULL)
    goto err101;

  *epar2 = newarray (knpt, DOUBLE);
  if (*epar2 == SISL_NULL)
    goto err101;

  (*epar1)[0] = astpar;
  kpar = astpar;

  for (count1 = 1, kpar = astpar; count1 < inpt; count1++)
    {
      if (ntype[count1] == 0)
	{
	  kpar += sqrt (s6dist (&econd[(count1 - 1) * idim],
				&econd[count1 * idim], idim));
	  (*epar1)[count1] = kpar;
	}
      else
	{
	  

	  for (count2 = count1 + 1; ntype[count2] != 0 && count2 < inpt; count2++) ;

	  if (count2 < inpt)
	    {
	      npar = kpar + sqrt (s6dist (&econd[(count1 - 1) * idim],
					  &econd[count2 * idim], idim));
	      (*epar1)[count2] = npar;
	    }

	  for (count3 = count1; count3 < count2; count3++)
	    {
	      if (ntype[count3] > 0)
		(*epar1)[count3] = kpar;
	      else
		(*epar1)[count3] = npar;
	    }
	  count1 = count2;
	  kpar = npar;
	}
    }

  if (iopen != SISL_CRV_OPEN)
    {
      

      for (count1 = 0; count1 < inpt && ntype[count1] != 0; count1++) ;

      for (count2 = inpt - 1; count2 >= 0 && ntype[count2] != 0; count2--) ;

      if (count1 >= inpt || count2 < 0)
	goto err101;		

      (*epar1)[inpt] = kpar + sqrt (s6dist (&econd[count1 * idim],
					    &econd[count2 * idim], idim));
    }

  *cendpar = (*epar1)[knpt - 1];

  count2 = 1;
  (*epar2)[0] = (*epar1)[0];

  for (count1 = 1; count1 < knpt; count1++)
    {
      if ((*epar1)[count1 - 1] < (*epar1)[count1])
	{
	  (*epar2)[count2] = (*epar1)[count1];
	  count2++;
	}
    }

  *epar2 = increasearray (*epar2, count2, DOUBLE);
  if (*epar2 == SISL_NULL)
    goto err101;


  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1911", *jstat, kpos);
  goto out;

out:

  return;
}

