/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1919

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1919 (double et[], double prev[], double curr[], double deriv[],
       double follow[], int in, int ik, int idim, int iip, int iif,
       double ap, double ac, double af, int *jstat)
#else
void
s1919 (et, prev, curr, deriv, follow, in, ik, idim, iip, iif, ap, ac, af, jstat)
     double et[];
     double prev[];
     double curr[];
     double deriv[];
     double follow[];
     int in;
     int ik;
     int idim;
     int iip;
     int iif;
     double ap;
     double ac;
     double af;
     int *jstat;
#endif
{
  SISLCurve *tcurve = SISL_NULL;	
  int knh;			
  int left;			
  int pos, pos2;		
  int ki, kj;			
  double tdist1;		
  double tdist2;		
  double tlength;		
  double tdiff;			
  double tfak;
  double tval1;			
  double tval2;
  double *kpar = SISL_NULL;		
  int *kder = SISL_NULL;
  double *kpc = SISL_NULL;		
  double *kdc = SISL_NULL;		
  double *kcc = SISL_NULL;		
  double *kfc = SISL_NULL;		
  double *epd = SISL_NULL;		
  int kstat = 0;
  int kpos = 0;

  *jstat = 0;


  

  if (ik <= 1 || in <ik)
    goto err112;


  

  s1890 (et, ik, in, &kpar, &kder, &kstat);
  if (kstat < 0)
    goto error;


  

  kpc = newarray (idim * in, DOUBLE);
  if (kpc == SISL_NULL)
    goto err101;
  kcc = newarray (idim * in, DOUBLE);
  if (kcc == SISL_NULL)
    goto err101;
  kdc = newarray (idim * in, DOUBLE);
  if (kdc == SISL_NULL)
    goto err101;
  kfc = newarray (idim * in, DOUBLE);
  if (kfc == SISL_NULL)
    goto err101;


  if (iip == 1)
    {
      

      tcurve = newCurve (in, ik, et, prev, 1, idim, 1);
      if (tcurve == SISL_NULL)
	goto err101;

      left = 0;
      for (ki = 0; ki < in; ki++)
	{
	  s1221 (tcurve, 0, kpar[ki], &left, &kpc[ki * idim], &kstat);
	  if (kstat < 0)
	    goto error;
	}
      if (tcurve != SISL_NULL)
	freeCurve (tcurve);
    }

  

  tcurve = newCurve (in, ik, et, curr, 1, idim, 1);
  if (tcurve == SISL_NULL)
    goto err101;

  left = 0;
  for (ki = 0; ki < in; ki++)
    {
      s1221 (tcurve, 0, kpar[ki], &left, &kcc[ki * idim], &kstat);
      if (kstat < 0)
	goto error;
    }
  if (tcurve != SISL_NULL)
    freeCurve (tcurve);


  

  tcurve = newCurve (in, ik, et, deriv, 1, idim, 1);
  if (tcurve == SISL_NULL)
    goto err101;

  left = 0;
  for (ki = 0; ki < in; ki++)
    {
      s1221 (tcurve, 0, kpar[ki], &left, &kdc[ki * idim], &kstat);
      if (kstat < 0)
	goto error;
    }
  if (tcurve != SISL_NULL)
    freeCurve (tcurve);


  if (iif == 1)
    {
      

      tcurve = newCurve (in, ik, et, follow, 1, idim, 1);
      if (tcurve == SISL_NULL)
	goto err101;

      left = 0;
      for (ki = 0; ki < in; ki++)
	{
	  s1221 (tcurve, 0, kpar[ki], &left, &kfc[ki * idim], &kstat);
	  if (kstat < 0)
	    goto error;
	}
      if (tcurve != SISL_NULL)
	freeCurve (tcurve);
    }

  

  if (iip == 1)
    tval1 = fabs (ac - ap);
  if (iif == 1)
    tval2 = fabs (af - ac);

  pos = 0;
  for (ki = 0; ki < in; ki++)
    {
      tdist1 = (double) 0.0;
      tdist2 = (double) 0.0;
      tlength = (double) 0.0;

      for (kj = 0; kj < idim; kj++)
	{
	  if (iip == 1)
	    {
	      tdiff = kcc[pos] - kpc[pos];
	      tdist1 += tdiff * tdiff;
	    }
	  if (iif == 1)
	    {
	      tdiff = kfc[pos] - kcc[pos];
	      tdist2 += tdiff * tdiff;
	    }
	  tlength += kdc[pos] * kdc[pos];
	  pos++;
	}

      tdist1 = sqrt (tdist1);
      tdist2 = sqrt (tdist2);
      tlength = sqrt (tlength);
      if (tlength == (double) 0.0)
	tlength = (double) 1.0;


      

      tfak = (double) 1.0;
      if (iip == 1 && iif != 1)
	tfak = tdist1 / (tval1 * tlength);
      else if (iip != 1 && iif == 1)
	tfak = tdist2 / (tval2 * tlength);
      else if (iip == 1 && iif == 1)
	tfak = min (tdist1 / (tval1 * tlength),
		    tdist2 / (tval2 * tlength));


      

      pos2 = pos - idim;
      for (kj = 0; kj < idim; kj++)
	{
	  kdc[pos2 + kj] *= tfak;
	}
    }

  

  s1891 (kpar, kdc, idim, in, 1, kder, 1, et, &epd, &knh, ik, 0, 0, &kstat);
  if (kstat < 0)
    goto error;

  memcopy (deriv, epd, idim * in, DOUBLE);


  

  goto out;


  

err101:
  *jstat = -101;
  s6err ("s1919", *jstat, kpos);
  goto out;

  

err112:
  *jstat = -112;
  s6err ("s1919", *jstat, kpos);
  goto out;

  

error:
  *jstat = kstat;
  s6err ("s1919", *jstat, kpos);
  goto out;

out:
  if (epd != SISL_NULL)
    freearray (epd);
  if (kpc != SISL_NULL)
    freearray (kpc);
  if (kcc != SISL_NULL)
    freearray (kcc);
  if (kdc != SISL_NULL)
    freearray (kdc);
  if (kfc != SISL_NULL)
    freearray (kfc);
  if (kpar != SISL_NULL)
    freearray (kpar);
  if (kder != SISL_NULL)
    freearray (kder);

  return;
}

