/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1935

#include "sislP.h"


#if defined(SISLNEEDPROTOTYPES)
void s1935 (double et1[], int in1, double et2[], int in2,
	    double *knt[], int *in, int ik, int *jstat)
#else
void
s1935 (et1, in1, et2, in2, knt, in, ik, jstat)
     double *et1;
     int in1;
     double *et2;
     int in2;
     double *knt[];
     int *in;
     int ik;
     int *jstat;

#endif
{
  int kpos = 0;			
  int pek1, pek2;		
  int stop1;			
  int stop2;			
  double curr;			

  *jstat = 0;

  

  if (ik < 1) goto err110;
  if ((in1 < ik) || (in2 < ik)) goto err111;


  

  if((*knt = newarray (2 * ik + in1 + in2, DOUBLE))==SISL_NULL) goto err101;

  

  if (et1[ik - 1] >= et1[in1]) goto err112;

  if (et2[ik - 1] >= et2[in2]) goto err112;

  

  *in = 0;
  
  pek1 = 0;
  pek2 = 0;
  stop1 = in1 + ik;
  stop2 = in2 + ik;

  while ((pek1 < stop1) && (pek2 < stop2))
    {
      

      curr = MIN (et1[pek1], et2[pek2]);

      if ((et1[pek1] < curr) || (et2[pek2] < curr)) goto err112;

      if ( DEQUAL(et1[pek1],curr) ) pek1++;
      if ( DEQUAL(et2[pek2],curr) ) pek2++;
      (*knt)[*in] = curr;
      (*in) ++;
    }

  

  if ((pek1 < stop1) || (pek2 < stop2))
    {
      if (pek1 >= stop1)
	{
	  for (; pek2 < stop2; pek2++, (*in) ++)
	    (*knt)[*in] = et2[pek2];
	}
      else
	for (; pek1 < stop1; pek1++, (*in) ++)
	  (*knt)[*in] = et1[pek1];
    }

  

  *in -=ik;
  *knt = increasearray (*knt, *in +ik, DOUBLE);
  if (*knt == SISL_NULL) goto err101;
  goto out;


  

  err101:
    *jstat = -101;
    s6err ("s1935", *jstat, kpos);
    goto out;

  

  err110:
    *jstat = -110;
    s6err ("s1935", *jstat, kpos);
    goto out;

  

  err111:
    *jstat = -111;
    s6err ("s1935", *jstat, kpos);
    goto out;

  

  err112:
    *jstat = -112;
    s6err ("s1935", *jstat, kpos);
    goto out;

  out:
    return;
}

