/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S1956

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s1956(SISLCurve *pc1,SISLCurve *pc2,SISLSurf **rsurf,int *jstat)
#else
void s1956(pc1,pc2,rsurf,jstat)
     SISLCurve *pc1;
     SISLCurve *pc2;
     SISLSurf  **rsurf;
     int   *jstat;
#endif
{
  int kstat = 0;        
  int kpos = 0;         
  int i,ki,kj;          
  int kind1,kind2;      
  int kdim;             
  int kn1;              
  int kk1;              
  int kn2;              
  int kk2;              
  double tdist;         
  double *scoef1;       
  double *scoef2;       
  double *ssurf = SISL_NULL; 
  double *s0;           
  double *s1;           
  double *s2;           
  SISLCurve *ratpc=SISL_NULL; 
  int newdim;          
  double *s1weight;    
  double *s2weight;    
  double *s3;          

  

  if (pc1->idim != pc2->idim) goto err106;

  

  kdim = pc1 -> idim;
  kn1 = pc1 -> in;
  kk1 = pc1 -> ik;
  kind1 = pc1 -> ikind;

  kn2 = pc2 -> in;
  kk2 = pc2 -> ik;
  kind2 = pc2 -> ikind;

  if((kind1 == 2 || kind1 == 4) || (kind2 == 2 || kind2 == 4))
  {
    

    newdim=kdim+1;

    if(kind1 != 2 && kind1 != 4)
    {
	ratpc=s1521(pc1,&kstat);
	if(kstat < 0) goto err102;

	scoef1 = ratpc -> rcoef;
	scoef2 = pc2 -> rcoef;
    }
    else if(kind2 != 2 && kind2 != 4)
    {
	ratpc=s1521(pc2,&kstat);
	if(kstat < 0) goto err102;

	scoef1 = pc1 -> rcoef;
	scoef2 = ratpc -> rcoef;
    }
    else
    {
	scoef1 = pc1 -> rcoef;
	scoef2 = pc2 -> rcoef;
    }


    

    if ((ssurf = newarray(kn1*kn2*newdim,double)) == SISL_NULL) goto err101;

    

    for (s0=ssurf,s2=scoef2,kj=0; kj<kn2; s2+=(kdim+1),kj++)
    {
      for (s1=scoef1,ki=0; ki<kn1; s0++,s1++,ki++)
      {
	  s1weight=s1+kdim;
	  s2weight=s2+kdim;

	  for(i=0,s3=s2; i<kdim; i++,s0++,s1++,s3++)
	  {
	      (*s0) = (*s2weight) * (*s1) - (*s1weight) * (*s3);
	  }

	  (*s0) = (*s1weight) * (*s2weight);
      }
    }

    

    *rsurf = SISL_NULL;
    if ((*rsurf = newSurf(kn1,kn2,kk1,kk2,pc1->et,pc2->et,
			  ssurf,2,kdim,1)) == SISL_NULL) goto err101;
  }
  else
  {
      

    newdim=kdim;

    scoef1 = pc1 -> ecoef;
    scoef2 = pc2 -> ecoef;

    

    if ((ssurf = newarray(kn1*kn2*kdim,double)) == SISL_NULL) goto err101;

    

    for (s0=ssurf,s2=scoef2,kj=0; kj<kn2; s2+=kdim,kj++)
      for (s1=scoef1,ki=0; ki<kn1; s0+=kdim,s1+=kdim,ki++)
        s6diff(s1,s2,kdim,s0);

    

    *rsurf = SISL_NULL;
    if ((*rsurf = newSurf(kn1,kn2,kk1,kk2,pc1->et,pc2->et,
			  ssurf,1,kdim,1)) == SISL_NULL) goto err101;
  }



  

  kstat = 1;
  if (kn1 != kn2 || kk1 != kk2) kstat = 0;
  tdist = s6dist(scoef1,scoef2,newdim);
  if(DNEQUAL(tdist,(double)0.0))
  {
      kstat = 0;
  }
  else
  {
      for (s1=scoef1+newdim,s2=scoef2+newdim,ki=1; ki<kn1 && kstat>0;
           s1+=newdim,s2+=newdim,ki++)
      {
        if (DNEQUAL(s6dist(s1,s2,newdim),tdist))
	{
	    kstat = 0;
	    break;
	}
      }
  }

  if (kstat == 0)
  {

      

      kstat = 2;
      if (kn1 != kn2 || kk1 != kk2) kstat = 0;
      tdist = s6dist(scoef1,scoef2+(kn2-1)*newdim,newdim);
      if(DNEQUAL(tdist,(double)0.0))
      {
          kstat = 0;
      }
      else
      {
          for (s1=scoef1+newdim,s2=scoef2+(kn2-2)*newdim,ki=1;
		 ki<kn1 && kstat>0; s1+=newdim,s2-=newdim,ki++)
          {
            if (DNEQUAL(s6dist(s1,s2,newdim),tdist))
	    {
	        kstat = 0;
	        break;
	    }
	  }
      }
  }


  

  *jstat = kstat;
  goto out;

  

 err101: *jstat = -101;
  s6err("s1956",*jstat,kpos);
  goto out;

  

 err102: *jstat = kstat;
  s6err("s1956",*jstat,kpos);
  goto out;

  

  err106 : *jstat = -106;
  s6err("s1956",*jstat,kpos);
  goto out;

 out:

  

  if (ssurf != SISL_NULL) freearray(ssurf);
  if (ratpc) freeCurve(ratpc);

  return;
}

