/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2511

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
   s2511(SISLSurf *surf, int ider, double derive[], double normal[],
      double *mehlum, int *jstat)
#else
 void s2511(surf, ider, derive, normal, mehlum, jstat)
      SISLSurf *surf;
      int ider;
      double derive[];
      double normal[];
      double *mehlum;
      int *jstat;
#endif
{
   double fundform[10]; 
   double length;       
   double a, b, c, d;   
   double alpha;        
   double beta;         
   double gamma;        
   double delta;        
   double epsilon;      
   double mu;           
   double P, Q, S, T;   
   double numerator;    
   double denominator;  

   if (ider != 0) goto err178;

   if (surf->idim == 1 || surf->idim == 3) 
   {
      s2513(surf, ider, 3, 1, derive, normal, fundform, jstat);
      if (*jstat < 0) goto error;
      
      if (surf->idim == 3)
	 length = normal[0]*normal[0] + normal[1]*normal[1] +
	    normal[2]*normal[2];
      else if (surf->idim == 1)
	 length = 1. + derive[1]*derive[1] + derive[2]*derive[2];
	 
      
      alpha   = s6scpr(&derive[3*(surf->idim)], &derive[surf->idim],
		     surf->idim)/length;
      beta    = s6scpr(&derive[3*(surf->idim)], &derive[2*(surf->idim)],
		     surf->idim)/length;
      gamma   = s6scpr(&derive[4*(surf->idim)], &derive[surf->idim],
		     surf->idim)/length;
      delta   = s6scpr(&derive[4*(surf->idim)], &derive[2*(surf->idim)],
		     surf->idim)/length;
      epsilon = s6scpr(&derive[5*(surf->idim)], &derive[surf->idim],
		     surf->idim)/length;
      mu      = s6scpr(&derive[5*(surf->idim)], &derive[2*(surf->idim)],
		     surf->idim)/length;
      
      a = fundform[1]*fundform[4] - fundform[2]*fundform[3];
      b = fundform[1]*fundform[3] - fundform[0]*fundform[4];
      c = fundform[1]*fundform[5] - fundform[2]*fundform[4];
      d = fundform[1]*fundform[4] - fundform[0]*fundform[5];
      
      P = fundform[6] + 3*(a*alpha + b*beta);
      Q = fundform[7] + c*alpha + d*beta + 2*a*gamma + 2*b*delta;
      S = fundform[8] + 2*c*gamma + 2*d*delta + a*epsilon + b*mu;
      T = fundform[9] + 3*(c*epsilon + d*mu);
      
      numerator = 5*fundform[2]*fundform[2]*fundform[2]*P*P
	 + (fundform[0]*fundform[2] + 4*fundform[1]*fundform[1])
	 *(9*fundform[2]*Q*Q
	 + 9*fundform[0]*S*S
	 + 6*fundform[2]*P*S
	 + 6*fundform[0]*Q*T)
	 + 5*fundform[0]*fundform[0]*fundform[0]*T*T
	 - 2*fundform[1]*(3*fundform[0]*fundform[2] + 2*fundform[1]*fundform[1])
	 *(P*T + 9*Q*S)
	 - 30*fundform[1]*(fundform[2]*fundform[2]*P*Q
	 + fundform[0]*fundform[0]*S*T);
      
      denominator = fundform[0]*fundform[2] - fundform[1]*fundform[1];
      
      *mehlum = numerator/(16*denominator*denominator*denominator);
  }

  else if (surf->idim == 2) 
  {
    

    *mehlum = 0.0;
  }
  else 
  {
    goto err105;
  }

  

  *jstat = 0;
  goto out;


   
err105:
  *jstat = -105;
  s6err("s2511",*jstat,0);
  goto out;

  
err178:
  *jstat = -178;
  s6err("s2511",*jstat,0);
  goto out;
  
error:
  s6err("s2511",*jstat,0);
  goto out;

out:

  return;

}

