/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2534

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
   s2534(SISLSurf *surf, 
	  int u_multinc, 
	  int v_multinc, 
	  int newik1, 
	  int newik2,
	  void evalp(SISLSurf *surf, int ider, int iside1, int iside2, 
		     double parvalue[], int *leftknot1, int *leftknot2, 
		     double *result, int *istat),
	  int eval_dim, 
	  SISLSurf **rsurf, 
	  int *stat)
#else
   void 
      s2534(surf, u_multinc, v_multinc, newik1, newik2, evalp, eval_dim, 
	     rsurf, stat)
      SISLSurf *surf;
      int u_multinc;
      int v_multinc;
      int newik1; 
      int newik2;
      void evalp();
      int eval_dim;
      SISLSurf **rsurf;
      int *stat;
#endif
{
   int ki, kj;            
   int newin1;            
   int newin2;            
   int leftknot1;         
   int leftknot2;         
   int open;              
   int local_in;          
   int nlr = 0;           
   int nrc = 0;           
   int *u_eder = SISL_NULL;    
   int *v_eder = SISL_NULL;    
   double par[2];         
   double *newet1 = SISL_NULL; 
   double *newet2 = SISL_NULL; 
   double *coef1 = SISL_NULL;  
   double *coef2 = SISL_NULL;  
   double *coef3 = SISL_NULL;  
   double *u_par = SISL_NULL;  
   double *v_par = SISL_NULL;  


   
   
   if (surf == SISL_NULL || u_multinc < 0 || v_multinc < 0 ||
       newik1 < (u_multinc + 2) || newik2 < (v_multinc + 2)) goto err150;

   

   s2533 (surf->et1, surf->ik1, surf->in1, u_multinc, newik1, &newin1, 
	  &newet1, stat);
   if (*stat < 0) goto error;

   

   s2533 (surf->et2, surf->ik2, surf->in2, v_multinc, newik2, &newin2, 
	  &newet2, stat);
   if (*stat < 0) goto error;
   
   
   
   if ((coef1 = newarray(newin1*newin2*eval_dim, DOUBLE)) == SISL_NULL) 
      goto err101;
   if ((u_par = newarray(newin1, DOUBLE)) == SISL_NULL) goto err101;
   if ((v_par = newarray(newin2, DOUBLE)) == SISL_NULL) goto err101;
   if ((u_eder = newarray(newin1, INT)) == SISL_NULL) goto err101;
   if ((v_eder = newarray(newin2, INT)) == SISL_NULL) goto err101;
   
   
   
   for (ki = 0; ki < newin1; ki++)
   {
      u_par[ki] = 0.;
      for (kj = 1; kj < newik1; kj++)
	 u_par[ki] += newet1[ki + kj];
      u_par[ki] /= (newik1 - 1);
      
      u_eder[ki] = 0;
   }
   
   for (ki = 0; ki < newin2; ki++)
   {
      v_par[ki] = 0.;
      for (kj = 1; kj < newik2; kj++)
	 v_par[ki] += newet2[ki + kj];
      v_par[ki] /= (newik2 - 1);
      
      v_eder[ki] = 0;
      
      par[1] = v_par[ki];
      for (kj = 0; kj < newin1; kj++)
      {
	 par[0] = u_par[kj];
	 evalp(surf, 0, 1, 1, par, &leftknot1, &leftknot2,
	       &coef1[(ki*newin1 + kj)*eval_dim], stat);
	 if (*stat < 0 || *stat == 2) goto error;
      }
   }
   
   
  
   open = SISL_CRV_OPEN;
   s1891(u_par, coef1, eval_dim, newin1, newin2, u_eder, open, newet1,
	 &coef2, &local_in, newik1, nlr, nrc, stat);
   if (*stat < 0) goto error;
  
   
  
   s1891(v_par, coef2, newin1*eval_dim, newin2, 1, v_eder, open, newet2,
	 &coef3, &local_in, newik2, nlr, nrc, stat);
   if (*stat < 0) goto error;

   
		 
   if ((*rsurf = newSurf(newin1, newin2, newik1, newik2, newet1, newet2,
			 coef3, 1, eval_dim, 2)) == SISL_NULL) goto err101;

  
   goto out;
  
  
  
   

   
   
 err101: 
   *stat = -101;
   s6err("s2534", *stat, 0);
   goto out;

   
   
 err150:
   *stat = -150;
   s6err("s2534", *stat, 0);
   goto out;

   
   
 error:
   s6err("s2534", *stat, 0);
   goto out;

   

 out:
   if (coef1 != SISL_NULL)  freearray(coef1);
   if (coef2 != SISL_NULL)  freearray(coef2);
   if (u_par != SISL_NULL)  freearray(u_par);
   if (v_par != SISL_NULL)  freearray(v_par);
   if (u_eder != SISL_NULL) freearray(u_eder);
   if (v_eder != SISL_NULL) freearray(v_eder);
   
   return;
}

