/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S2561

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s2561( double     derive[],
       int        idim,
       double     p[],
       double     t[],
       double     n[],
       double     b[],
       int       *jstat )
#else
void s2561( derive, idim, p, t, n, b, jstat )
     double      derive[];
     int         idim;
     double      p[];
     double      t[];
     double      n[];
     double      b[];
     int        *jstat;
#endif
{
  int kstat = 0;        
  double a;             
  double crpr[3];       
  double D[9];          


  

  if ( idim == 1 )
  {
      D[0]  = 0.0;   D[1] = derive[0];  D[2]  = 0.0;
      D[3]  = 1.0;   D[4] = derive[1];  D[5]  = 0.0;
      D[6]  = 0.0;   D[7] = derive[2];  D[8]  = 0.0;
  }
  else if ( idim == 2 )
  {
      D[0]  = derive[0];   D[1] = derive[1];   D[2]  = 0.0;
      D[3]  = derive[2];   D[4] = derive[3];   D[5]  = 0.0;
      D[6]  = derive[4];   D[7] = derive[5];   D[8]  = 0.0;
  }
  else
  {
      D[0]  = derive[0];   D[1] = derive[1];   D[2]  = derive[2];
      D[3]  = derive[3];   D[4] = derive[4];   D[5]  = derive[5];
      D[6]  = derive[6];   D[7] = derive[7];   D[8]  = derive[8];
  }

  

  p[0] = D[0];
  p[1] = D[1];
  p[2] = D[2];


  

  a = s6length( D + 3, 3, &kstat );

  t[0] = D[3]/a;
  t[1] = D[4]/a;
  t[2] = D[5]/a;


  s6crss( D + 3, D + 6, crpr );

  a = s6length( crpr, 3, &kstat );

  if ( kstat != 0 )
  {
    b[0] = crpr[0]/a;
    b[1] = crpr[1]/a;
    b[2] = crpr[2]/a;

    s6crss( b, t, n );
  }
  else
  {
    n[0] = 0.0;
    n[1] = 0.0;
    n[2] = 0.0;

    b[0] = 0.0;
    b[1] = 0.0;
    b[2] = 0.0;

    goto war002;
  }



 

  *jstat = 0;
  goto out;


  
war002:
  *jstat = 2;
  goto out;


 out:

 return;

}

