/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6ANGLE

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
double
     s6angle(double evec1[],double evec2[],double enorm[],int idim,int *jstat) 
#else
double s6angle(evec1,evec2,enorm,idim,jstat)
     double evec1[];
     double evec2[];
     double enorm[];
     int    idim;
     int *jstat;
#endif     
                                     
{
  double sa[3];
  double sb[3];
  double sn[3];
  
  double tscpr1,tscpr2,tang,tlength1,tlength2,tcos;
  int    kstat1,kstat2,ki;
    
  if (idim != 3) goto err104;
  
  tscpr1 = s6scpr(evec1,enorm,idim);
  tscpr2 = s6scpr(evec2,enorm,idim);
  
  for (ki=0; ki<idim; ki++)
    {
      sa[ki] = evec1[ki] - tscpr1*enorm[ki];
      sb[ki] = evec2[ki] - tscpr2*enorm[ki];
    }
  
  tscpr1 = s6scpr(sa,sb,idim);

  tlength1 = s6length(sa,idim,&kstat1);
  tlength2 = s6length(sb,idim,&kstat2);
  
  if (!kstat1 || !kstat2)
    tang = DZERO;
  else
    {
      tcos = tscpr1/(tlength1*tlength2);
      tcos = MIN((double)1.0,tcos);
      tcos = MAX(-(double)1.0,tcos);
      tang = acos(tcos);
    }
  
  s6crss(sa,sb,sn);
  if (s6scpr(sn,enorm,idim) < DZERO) tang = TWOPI - tang;
  
  *jstat = 0;
  goto out;
  
 err104:
  *jstat = -104;
  goto out;
  
 out:
  return(tang);
}

