/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6IDGET

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
s6idget(SISLObject *po1,SISLObject *po2,int ipar,double apar,SISLIntdat *pintdat,
	     SISLIntdat **rintdat,int *jstat)
#else
void s6idget(po1,po2,ipar,apar,pintdat,rintdat,jstat)
     SISLObject *po1;
     SISLObject *po2;
     int    ipar;
     double apar;
     SISLIntdat *pintdat;
     SISLIntdat **rintdat;
     int    *jstat;
#endif
                                     
{
  int kstat;                    
  int kpos=0;                   
  int ki,kj,kn;
  double tstart[4];
  double tend[4];
  double spar[4];               
  SISLIntpt *qpt;
  
  
  if (pintdat == SISL_NULL)
    goto out;
  
  if (po1->iobj == SISLCURVE)
    {
      tstart[0] = po1->c1->et[po1->c1->ik - 1];
      tend[0]   = po1->c1->et[po1->c1->in];
    }
  else if (po1->iobj == SISLSURFACE)
    {
      tstart[0] = po1->s1->et1[po1->s1->ik1 - 1];
      tend[0]   = po1->s1->et1[po1->s1->in1];
      tstart[1] = po1->s1->et2[po1->s1->ik2 - 1];
      tend[1]   = po1->s1->et2[po1->s1->in2];
    }
  
  if (po2->iobj == SISLCURVE)
    {
      tstart[po1->iobj]   = po2->c1->et[po2->c1->ik - 1];
      tend[po1->iobj]     = po2->c1->et[po2->c1->in];
    }
  else if (po2->iobj == SISLSURFACE)
    {
      tstart[po1->iobj]   = po2->s1->et1[po2->s1->ik1 - 1];
      tend[po1->iobj]     = po2->s1->et1[po2->s1->in1];
      tstart[po1->iobj+1] = po2->s1->et2[po2->s1->ik2 - 1];
      tend[po1->iobj+1]   = po2->s1->et2[po2->s1->in2];
    }
  
  tstart[ipar] = tend[ipar] = apar;
  
  
  
  
  for (ki=0; ki<pintdat->ipoint; ki++)
    {
      for (kj=0; kj<pintdat->vpoint[ki]->ipar; kj++)
	if((DNEQUAL(pintdat->vpoint[ki]->epar[kj],tstart[kj]) &&
	    pintdat->vpoint[ki]->epar[kj] < tstart[kj]) ||
	   (DNEQUAL(pintdat->vpoint[ki]->epar[kj],tend[kj]) &&
	    pintdat->vpoint[ki]->epar[kj] > tend[kj]))
	  break;
      
      if (kj == pintdat->vpoint[ki]->ipar)
	{
	  for(kn=0; kn<ipar; kn++) spar[kn] = pintdat->vpoint[ki]->epar[kn];
	  for(; kn<pintdat->vpoint[ki]->ipar-1; kn++)
	    spar[kn] = pintdat->vpoint[ki]->epar[kn+1];
	  
	  
	  
	  qpt = newIntpt(pintdat->vpoint[ki]->ipar-1,spar,DZERO);
	  if (qpt == SISL_NULL) goto err101;
	  
	  s6idnpt(rintdat,&qpt,1,&kstat);
	  if (kstat < 0) goto error;
	}
    }
  
  *jstat = 0;
  goto out;
  
  

  err101: 
    *jstat = -101;
    s6err("s6idget",*jstat,kpos);
    goto out;

  

  error: 
    *jstat = kstat;
    s6err("s6idget",*jstat,kpos);
    goto out;

  out: ;
}

