/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"




#define S6INV4

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
s6inv4 (double em[], double einv[], int *jstat)
#else
void
s6inv4 (em, einv, jstat)
     double em[];
     double einv[];
     int *jstat;
#endif
{
  int ki;
  double det;

  *jstat = 0;


  

  det = em[0] * em[5] * (em[10] * em[15] - em[14] * em[11])
    - em[0] * em[6] * (em[9] * em[15] - em[13] * em[11])
    + em[0] * em[7] * (em[9] * em[14] - em[13] * em[10])
    - em[1] * em[4] * (em[10] * em[15] - em[14] * em[11])
    + em[1] * em[6] * (em[8] * em[15] - em[12] * em[11])
    - em[1] * em[7] * (em[8] * em[14] - em[12] * em[10])
    + em[2] * em[4] * (em[9] * em[15] - em[13] * em[11])
    - em[2] * em[5] * (em[8] * em[15] - em[12] * em[11])
    + em[2] * em[7] * (em[8] * em[13] - em[12] * em[9])
    - em[3] * em[4] * (em[9] * em[14] - em[13] * em[10])
    + em[3] * em[5] * (em[8] * em[14] - em[12] * em[10])
    - em[3] * em[6] * (em[8] * em[13] - em[12] * em[9]);


  

  einv[0] = em[5] * (em[10] * em[15] - em[14] * em[11])
    - em[6] * (em[9] * em[15] - em[13] * em[11])
    + em[7] * (em[9] * em[14] - em[13] * em[10]);

  einv[4] = -em[4] * (em[10] * em[15] - em[14] * em[11])
    + em[6] * (em[8] * em[15] - em[12] * em[11])
    - em[7] * (em[8] * em[14] - em[12] * em[10]);

  einv[8] = em[4] * (em[9] * em[15] - em[13] * em[11])
    - em[5] * (em[8] * em[15] - em[12] * em[11])
    + em[7] * (em[8] * em[13] - em[12] * em[9]);

  einv[12] = -em[4] * (em[9] * em[14] - em[13] * em[10])
    + em[5] * (em[8] * em[14] - em[12] * em[10])
    - em[6] * (em[8] * em[13] - em[12] * em[9]);


  einv[1] = -em[1] * (em[10] * em[15] - em[14] * em[11])
    + em[2] * (em[9] * em[15] - em[13] * em[11])
    - em[3] * (em[9] * em[14] - em[13] * em[10]);

  einv[5] = em[0] * (em[10] * em[15] - em[14] * em[11])
    - em[2] * (em[8] * em[15] - em[12] * em[11])
    + em[3] * (em[8] * em[14] - em[12] * em[10]);

  einv[9] = -em[0] * (em[9] * em[15] - em[13] * em[11])
    + em[1] * (em[8] * em[15] - em[12] * em[11])
    - em[3] * (em[8] * em[13] - em[12] * em[9]);

  einv[13] = em[0] * (em[9] * em[14] - em[13] * em[10])
    - em[1] * (em[8] * em[14] - em[12] * em[10])
    + em[2] * (em[8] * em[13] - em[12] * em[9]);


  einv[2] = em[1] * (em[6] * em[15] - em[14] * em[7])
    - em[2] * (em[5] * em[15] - em[13] * em[7])
    + em[3] * (em[5] * em[14] - em[13] * em[6]);

  einv[6] = -em[0] * (em[6] * em[15] - em[14] * em[7])
    + em[2] * (em[4] * em[15] - em[12] * em[7])
    - em[3] * (em[4] * em[14] - em[12] * em[6]);

  einv[10] = em[0] * (em[5] * em[15] - em[13] * em[7])
    - em[1] * (em[4] * em[15] - em[12] * em[7])
    + em[3] * (em[4] * em[13] - em[12] * em[5]);

  einv[14] = -em[0] * (em[5] * em[14] - em[13] * em[6])
    + em[1] * (em[4] * em[14] - em[12] * em[6])
    - em[2] * (em[4] * em[13] - em[12] * em[5]);


  einv[3] = -em[1] * (em[6] * em[11] - em[10] * em[7])
    + em[2] * (em[5] * em[11] - em[9] * em[7])
    - em[3] * (em[5] * em[10] - em[9] * em[6]);

  einv[7] = em[0] * (em[6] * em[11] - em[10] * em[7])
    - em[2] * (em[4] * em[11] - em[8] * em[7])
    + em[3] * (em[4] * em[10] - em[8] * em[6]);

  einv[11] = -em[0] * (em[5] * em[11] - em[9] * em[7])
    + em[1] * (em[4] * em[11] - em[8] * em[7])
    - em[3] * (em[4] * em[9] - em[8] * em[5]);

  einv[15] = em[0] * (em[5] * em[10] - em[9] * em[6])
    - em[1] * (em[4] * em[10] - em[8] * em[6])
    + em[2] * (em[4] * em[9] - em[8] * em[5]);


  for (ki = 0; ki < 16; ++ki)
    einv[ki] /= det;

  return;
}

