/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define S6METRIC

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
     s6metric(double epoint[],int in,int idim,double emat[],int *jstat)
#else
void s6metric(epoint,in,idim,emat,jstat)
   double epoint[];
   int    in;
   int    idim;
   double emat[];
   int    *jstat;
#endif
{
   int kstat = 0;        
   int ki,kk;            
   double tmedium;       
   double tdot;          
   double *smat1 = SISL_NULL; 
   double *smat2 = SISL_NULL; 
   double *s1,*s2,*s3;   
   
   
   
   if ((smat1 = newarray(in*idim,DOUBLE)) == SISL_NULL) goto err101;
   if ((smat2 = newarray(idim*idim,DOUBLE)) == SISL_NULL) goto err101;
   
   
   
   memcopy(smat1,epoint,in*idim,DOUBLE);
   
   for (kk=0; kk<idim; kk++)
   {
      
      
      for (tmedium=DZERO, s1=smat1+kk, s2=s1+in*idim; s1<s2; s1+=idim)
	 tmedium += (*s1);
      tmedium /= in;
      
      
      
      for (s1=smat1+kk; s1<s2; s1+=idim) *s1 -= tmedium;
   }
   
   
   
   for (ki=0; ki<idim; ki++)
      for (kk=0; kk<idim; kk++)
      {
	 for (tdot=DZERO, s1=smat1+ki, s2=smat1+kk, s3=s1+in*idim;
	      s1<s3; s1+=idim, s2+=idim)
	    tdot += (*s1)*(*s2);
	 smat2[ki*idim+kk] = tdot;
      }
   
   
   
   s6invert(smat2,idim,emat,&kstat);
   if (kstat < 0) goto error;
		  
   
		  
   *jstat = kstat;
   goto out;
   
   
   
   err101 : *jstat = -101;
   goto out;
   
   
   
   error : *jstat = kstat;
   goto out;
   
   out :
      
      
      if (smat1 != SISL_NULL) freearray(smat1);
      if (smat2 != SISL_NULL) freearray(smat2);
			 
      return;
}
      

