/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"

#define SH1262

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
static void sh1262_s9blendcoef(double [],double [],double [],int,int,double *,
			       double *,int *);
static void sh1262_s9blendder(double [],double  [],int,int,int,double,double,
			      double,double,double [],double [],double [],
			      double [],int *);
static void sh1262_s9hermit(double [],int,double,double,int,int *);
#else
static void sh1262_s9blendcoef();
static void sh1262_s9blendder();
static void sh1262_s9hermit();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
      sh1262(SISLCurve *vcurve[],int iedge,int inmbx,double ecoef[],int *jstat)
#else	 
void sh1262(vcurve,iedge,inmbx,ecoef,jstat)
     int iedge,inmbx,*jstat;
     double ecoef[];
     SISLCurve *vcurve[];
#endif     
{
  int kstat = 0;     
  int kstat1 = 0;    
  int ki,kj,kk;      
  int ki4,ki12;      
  int kj4,kj12;      
  int kdim = 3;      
  int kleft = 0;     
  int kder = 1;      
  int kncurve = 3*iedge;  
  int kncond = 4*iedge;   
  int ksign;              
  int krang;              
  double ang_tol = (double)0.00001;   
  double tfac;            
  double tref=(double)0.0; 
  double tpar1,tpar2;     
  double thelp1,thelp2,thelp3,thelp4;   
  double tnorm1,tnorm2;   
  double *spar1 = SISL_NULL;   
  double *spar2 = SISL_NULL;   
  double *sder = SISL_NULL;    
  double sa[12];          
  double sb[3];           
  double snorm1[3];       
  double snorm2[3];       
  SISLCurve *qc;              
  
  

  for (ki=0; ki<iedge; ki++)
    if (vcurve[ki]->idim != kdim) goto err104;
  
  
  
  if ((sder = newarray(12*kdim*iedge,DOUBLE)) == SISL_NULL) goto err101;
  if ((spar1 = newarray(kncurve,DOUBLE)) == SISL_NULL) goto err101;
  if ((spar2 = newarray(kncurve,DOUBLE)) == SISL_NULL) goto err101;  
  
  

  for (ki=0; ki<kncurve; ki++)
    {
      qc=vcurve[ki];
      
      

      spar1[ki] = tpar1 = *(qc->et + qc->ik - 1);
      spar2[ki] = tpar2 = *(qc->et + qc->in);
      
      kj = ki/3;
      kk = ki % 3;
      
      

      s1221(qc,kder,tpar1,&kleft,sder+2*(6*kj+kk)*kdim,&kstat);
      if (kstat < 0) goto error;
      
      

      s1221(qc,kder,tpar2,&kleft,sder+2*(6*kj+kk+3)*kdim,&kstat);
      if (kstat < 0) goto error;
    }
  
  

  for (ki=0; ki<iedge; ki++)
    {
      

      kj = (ki > 0) ? ki-1 : iedge-1;
      ki4 = 4*ki, ki12 = 12*ki;
      kj4 = 4*kj, kj12 = 12*kj;
      
      

      ksign = -1;
      sh1262_s9blendcoef(sder+(ki12+2)*kdim,sder+(ki12+4)*kdim,sder+(kj12+7)*kdim,
		  kdim,ksign,ecoef+ki4,ecoef+kncond+ki4,&kstat);
      if (kstat < 0) goto error;
      kstat1 = MAX(kstat,kstat1);
      
      

      ksign = 1;
      sh1262_s9blendcoef(sder+(ki12+8)*kdim,sder+(ki12+10)*kdim,
		  sder+(((ki+1)%iedge)*12+1)*kdim,
		  kdim,ksign,ecoef+ki4+3,ecoef+kncond+ki4+3,&kstat);
      if (kstat < 0) goto error;
      kstat1 = MAX(kstat,kstat1);
    }
  
  for (tref=(double)0.0, ki=0; ki<iedge; ki++)
    {
      

      kj = (ki > 0) ? ki-1 : iedge-1;
      ki4 = 4*ki, ki12 = 12*ki;
      kj4 = 4*kj, kj12 = 12*kj;
      
      for (kk=0; kk < kdim; kk++)
	{
          sa[kk*4] = sder[(ki12+2)*kdim+kk];
	  sa[kk*4+1] = sder[(ki12+4)*kdim+kk];
	  sa[kk*4+2] = sder[(kj12+8)*kdim+kk];
	  sa[kk*4+3] = sder[(kj12+10)*kdim+kk];

	  sb[kk] = - ecoef[ki4]*sder[(ki12+3)*kdim+kk]
	    - ecoef[kncond+ki4]*sder[(ki12+5)*kdim+kk]
	    - ecoef[kj4+3]*sder[(kj12+9)*kdim+kk]
		- ecoef[kncond+kj4+3]*sder[(kj12+11)*kdim+kk];
	  
	  tref = MAX(tref,MAX(sa[kk*4],sa[kk*4+1]));
	  tref = MAX(tref,MAX(sa[kk*4+2],sa[kk*4+3]));
	}

      

      s6crss(sder+(ki12+2)*kdim,sder+(ki12+4)*kdim,snorm1);
      tnorm1 = s6length(snorm1,kdim,&kstat);
      
      s6crss(sder+(kj12+8)*kdim,sder+(kj12+10)*kdim,snorm2);
      tnorm2 = s6length(snorm2,kdim,&kstat);
      
      thelp1 = s6ang(snorm1,sder+(kj12+8)*kdim,kdim);
      thelp2 = s6ang(snorm1,sder+(kj12+10)*kdim,kdim);

      thelp3 = s6ang(snorm2,sder+(ki12+2)*kdim,kdim);
      thelp4 = s6ang(snorm2,sder+(ki12+4)*kdim,kdim);

      if ((DNEQUAL(tnorm1+tref,tref) && 
	   fabs(thelp1-PIHALF) < ang_tol &&
	   fabs(thelp2-PIHALF) < ang_tol) ||
	  (DNEQUAL(tnorm2+tref,tref) &&
	   fabs(thelp3-PIHALF) < ang_tol &&
	   fabs(thelp4-PIHALF) < ang_tol) ||
	  (DEQUAL(tnorm1+tref,tref) &&
	   (fabs(thelp1-PIHALF) < ang_tol ||
 	    fabs(thelp2-PIHALF) < ang_tol)) ||
	  (DEQUAL(tnorm2+tref,tref) &&
	   (fabs(thelp3-PIHALF) < ang_tol ||
 	    fabs(thelp4-PIHALF) < ang_tol)) ||
	  (DEQUAL(tnorm1+tref,tref) && DEQUAL(tnorm2+tref,tref)))
	krang = 2;
      else
	krang = 3;
      
      

	sh1262_s9blendder(sa,sb,4,kdim,krang,spar1[3*ki],spar2[3*ki],spar1[3*kj],
		 spar2[3*kj],ecoef+ki4,ecoef+kncond+ki4,ecoef+kj4,
		 ecoef+kncond+kj4,&kstat);
      if (kstat < 0) goto error;
      kstat1 = MAX(kstat,kstat1);
    }
   
  for (ki=0; ki<iedge; ki++)
    {
      

       sh1262_s9hermit(ecoef+4*ki,4,spar1[3*ki],spar2[3*ki],1,&kstat);
      if (kstat < 0) goto error;

      sh1262_s9hermit(ecoef+kncond+4*ki,4,spar1[3*ki],spar2[3*ki],1,&kstat);
      if (kstat < 0) goto error;
      
      
      
      tfac = MAX(MAX(fabs(ecoef[4*ki+3]),fabs(ecoef[4*ki])),
		  MAX(fabs(ecoef[kncond+4*ki+3]),fabs(ecoef[kncond+4*ki])));
      for (kj=1; kj<3; kj++)
      {
	 if (ecoef[4*ki+kj] < MIN(ecoef[4*ki+3],ecoef[4*ki])-tfac ||
	     ecoef[4*ki+kj] > MAX(ecoef[4*ki+3],ecoef[4*ki])+tfac ||
	     ecoef[4*ki+kj] < DZERO)
	 {
	    ecoef[4*ki+kj] = ((double)(3-kj)*ecoef[4*ki] +
	       (double)kj*ecoef[4*ki+3])/(double)3.0;
	    kstat1 = MAX(kstat1,1);
	 }
	 
	 if (ecoef[kncond+4*ki+kj] < 
	     MIN(ecoef[kncond+4*ki+3],ecoef[kncond+4*ki])-tfac ||
	     ecoef[kncond+4*ki+kj] > 
	     MAX(ecoef[kncond+4*ki+3],ecoef[kncond+4*ki])+tfac)
	 {
	    ecoef[kncond+4*ki+kj] = ((double)(3-kj)*ecoef[kncond+4*ki] +
	       (double)kj*ecoef[kncond+4*ki+3])/(double)3.0;
	    kstat1 = MAX(kstat1,1);
	 }
      }
    }
  
  
  

  *jstat = kstat1;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err104 :
    *jstat = -104;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :
    
    

    if (sder != SISL_NULL) freearray(sder);
  if (spar1 != SISL_NULL) freearray(spar1);
  if (spar2 != SISL_NULL) freearray(spar2);

  return;
}

					
#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1262_s9blendcoef(double evecu[],double evecv[],double etang[],
			       int idim,int isign,double *coef1,
			       double *coef2,int *jstat)
#else
static void sh1262_s9blendcoef(evecu,evecv,etang,idim,isign,coef1,coef2,jstat)
     int idim,isign,*jstat;
     double evecu[],evecv[],etang[],*coef1,*coef2;
#endif     
{
  
  int kstat = 0;           
  int ki;                  
  double tdotuu;           
  double tdotuv;           
  double tdotutang;        
  double tdotvv;           
  double tdotvtang;        
  double tdiv;             
  
  

  *coef1 = (double)0.0;
  *coef2 = (double)0.0;
  
  

  tdotuu = s6scpr(evecu,evecu,idim);
  tdotuv = s6scpr(evecu,evecv,idim);
  tdotutang = (double)isign*s6scpr(evecu,etang,idim);
  tdotvv = s6scpr(evecv,evecv,idim);
  tdotvtang = (double)isign*s6scpr(evecv,etang,idim);

  tdiv = tdotuv*tdotuv - tdotuu*tdotvv;
  if (DEQUAL(tdiv,DZERO))
    {
      if (DEQUAL(tdotuu,DZERO) && DEQUAL(tdotvv,DZERO));
      else if (DEQUAL(tdotuu,DZERO))
	  *coef2 = s6length(etang,idim,&kstat)/sqrt(tdotvv);
      else
	*coef1 = s6length(etang,idim,&kstat)/sqrt(tdotuu);
      goto out;
    }
  
  

  *coef1 = (tdotvtang*tdotuv - tdotutang*tdotvv)/tdiv;
  *coef2 = (tdotutang*tdotuv - tdotvtang*tdotuu)/tdiv;

  

  for (ki=0; ki<idim; ki++)
    if (fabs((*coef1)*evecu[ki]+(*coef2)*evecv[ki]-isign*etang[ki]) > REL_PAR_RES) 
      break;
  
  if (ki < idim) goto warn1;

  *jstat = 0;
  goto out;
  
  

  warn1 :
    *jstat = 2;
  goto out;
  
  out :
    return;
}


#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1262_s9hermit(double econd[],int icond,double astart,
			    double aend,int idim,int *jstat)
#else
static void sh1262_s9hermit(econd,icond,astart,aend,idim,jstat)
     int icond,idim,*jstat;
     double econd[],astart,aend;
#endif     
{
  int ki;    

  

  if (icond != 4) goto err001;
  
  

  for (ki=0; ki<idim; ki++)
    {
      econd[idim+ki] = ONE_THIRD*(aend-astart)*econd[idim+ki] + econd[ki];
      econd[2*idim+ki] = -ONE_THIRD*(aend-astart)*econd[2*idim+ki] + econd[3*idim+ki];
    }
  
  *jstat = 0;
  goto out;
  
  

  err001 :
    *jstat = -1;
  goto out;
  
  out :
    return;
}


#if defined(SISLNEEDPROTOTYPES)
   static
      void
	    sh1262_s9blendder(double ea[],double eb[],int ix,int ieq,
			      int irang,double astarti,double aendi,
			      double astartj,double aendj,double ealfai[],
			      double ebetai[],double ealfaj[],
			      double ebetaj[],int *jstat)
#else
static void sh1262_s9blendder(ea,eb,ix,ieq,irang,astarti,aendi,astartj,aendj,ealfai,
		       ebetai,ealfaj,ebetaj,jstat)
     int ix,ieq,irang,*jstat;
     double ea[],eb[],astarti,aendi,astartj,aendj,ealfai[],ebetai[],
       ealfaj[],ebetaj[];
#endif     
{
  int kstat = 0;          
  int ki,kj,kk,kh;        
  int l[2][2];            
  int lpiv[3];            
  int kcount;             
  double teps = 0.001;    
  double tdiff;           
  double tdum;            
  double tdet;            
  double sdet[2];         
  double sx[4];           
  double sxx[4];          
  double sb[3];           
  double smat[9];         
  
  

  if (ix != 4 || ieq != 3) goto err001;
  
  

  sx[0] = (ealfai[3] - ealfai[0])/(aendi - astarti);
  sx[1] = (ebetai[3] - ebetai[0])/(aendi - astarti);
  sx[2] = (ealfaj[3] - ealfaj[0])/(aendj - astartj);
  sx[3] = (ebetaj[3] - ebetaj[0])/(aendj - astartj);  
  
  if (irang == 2)
    {
      
      

      sdet[0] = sdet[1] = (double)0.0;
      
      for (ki=0; ki<ieq; ki++)
	for (kj=ki+1; kj<ieq; kj++)
	  for (kk=0,kh=0; kh<2; kk+=2,kh++)
	    {
	      tdet = ea[4*ki+kk]*ea[4*kj+kk+1] - ea[4*ki+kk+1]*ea[4*kj+kk];
	      if (fabs(tdet) > fabs(sdet[kh]))
		{
		  sdet[kh] = tdet;
		  l[kh][0] = ki, l[kh][1] = kj;
		}
	    }
      for (kh=0; kh<2; kh++)
	if (DEQUAL(sdet[kh],DZERO)) sdet[kh] = (double)1.0;
      
      kcount = 0;
      
      

      ki = 0, kk = 2;
      sxx[ki] = sx[ki];
      sxx[ki+1] = sx[ki+1];
      
      while (1)
	{
	  

	  

	  for (kj=0; kj<ieq; kj++)
	    sb[kj] = eb[kj] - ea[4*kj+ki]*sxx[ki] - ea[4*kj+ki+1]*sxx[ki+1];
	  
	  

	  kj = kk/2;
	  sxx[kk] = (-ea[4*l[kj][0]+kk+1]*sb[l[kj][1]] 
		       + ea[4*l[kj][1]+kk+1]*sb[l[kj][0]])/sdet[kj];
	  sxx[kk+1] = (ea[4*l[kj][0]+kk]*sb[l[kj][1]] 
		       - ea[4*l[kj][1]+kk]*sb[l[kj][0]])/sdet[kj];
	  
	  

	  tdiff = fabs(sxx[0] - sx[0]);
	  for (kh=1; kh<4; kh++)
	    if (fabs(tdiff-fabs(sxx[kh]-sx[kh])) > teps) break;
	  
	  if (kh == 4 || kcount == 4) break;   
	  
	  

	  sxx[kk] = (sx[kk] + sxx[kk])/(double)2.0;
	  sxx[kk+1] = (sx[kk+1] + sxx[kk+1])/(double)2.0;
	  
	  ki = (ki + 2) % 4;
	  kk = (kk + 2) % 4;

	  kcount++;
	}
    }
  else if (irang == 3)
    {
      
      ki = 0;
      kcount = 0;
      
      while (1)
	{
	  

	  

	  sxx[ki] = sx[ki];
	  
	  

	  for (kj=0; kj<ieq; kj++)
	    {
	      sb[kj] = eb[kj] - ea[kj*4+ki]*sxx[ki];
	      for (kk=0,kh=0; kk<ix; kk++)
		{
		  if (kk == ki) continue;
		  smat[kj*(ix-1)+kh] = ea[kj*ix+kk];
		  kh++;
		}
	    }
	  
	  

	  s6lufacp(smat,lpiv,ieq,&kstat);
	  if (kstat < 0) goto error;
	  
	  s6lusolp(smat,sb,lpiv,ieq,&kstat);
	  if (kstat < 0) goto error;
	  
	  for (kk=0,kh=0; kk<ix; kk++)
	    {
	      if (kk == ki) continue;
	      sxx[kk] = sb[kh++];
	    }
	  
	  

	  tdiff = fabs(sxx[0] - sx[0]);
	  for (kh=1; kh<4; kh++)
	    if (fabs(tdiff-fabs(sxx[kh]-sx[kh])) > teps) break;
	  
	  if (kh == 4 || kcount == 4) break;   
	  
	  

	  kj = (ki + 1) % 4;
	  sxx[kj] = (sx[kj] + sxx[kj])/(double)2.0;
	  
	  ki = kj;
	  kcount++;
	}
      
    }
  
   

  ealfai[1] = sxx[0];
  ebetai[1] = sxx[1];
  ealfaj[2] = sxx[2];
  ebetaj[2] = sxx[3];
  
  

  for (ki=0; ki<ieq; ki++)
    {
      tdum = (double)0.0;
      for (kj=0; kj<ix; kj++)
	tdum += ea[ki*ix+kj]*sxx[kj];
      if (fabs(tdum - eb[ki]) > REL_PAR_RES) break;
    }
  if (ki < ieq) goto warn1;
  
  *jstat = 0;
  goto out;
  
  

  warn1 :
    *jstat = 1;
  goto out;
  
  

  err001 :
    *jstat = -1;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :
    return;
}

