/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define SH1365

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      sh1365(SISLCurve *pcurve,double etau[],int ik,int in,int ileftfix,
	     int irightfix,SISLCurve **rnewcurve,double **gmaxerr,
	     double **gl2err,int *jstat)
#else
void sh1365(pcurve,etau,ik,in,ileftfix,
	    irightfix,rnewcurve,gmaxerr,gl2err,jstat)
     SISLCurve *pcurve;
     double etau[];
     int ik;
     int in;
     int ileftfix;
     int irightfix;
     SISLCurve **rnewcurve;
     double **gmaxerr;
     double **gl2err;
     int *jstat;
#endif     
{
  int kstat = 0;            
  int kdim = pcurve->idim;  
  int kkind = 1;            
  int kcopy = 1;            
  int kn = pcurve->in;      
  double *scoef = SISL_NULL;     
  double *sa = SISL_NULL;        
  double *sb = SISL_NULL;
  int *lfirst = SISL_NULL;
  int *llast = SISL_NULL;
  int *l2sta = SISL_NULL;
  double *sc = SISL_NULL;
  int kk = pcurve->ik;
  int knh = in - ileftfix - irightfix;

  

  if (kk != ik) goto err109;

  

  if ((*gmaxerr = new0array(kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((*gl2err = new0array(kdim,DOUBLE)) == SISL_NULL) goto err101;
  
  

  if ((scoef = newarray(in*kdim,DOUBLE)) == SISL_NULL) goto err101;
  
  
  
  if (kn == in)
  {
     memcopy(scoef,pcurve->ecoef,in*kdim,DOUBLE);
     
     
     
     if ((*rnewcurve = newCurve(in,ik,etau,scoef,kkind,kdim,kcopy)) == SISL_NULL)
	goto err101;
     (*rnewcurve)->cuopen = pcurve->cuopen;
  }     
  else
  {
     
     
     if ((sa = newarray(kn*ik,DOUBLE)) == SISL_NULL) goto err101;
     if ((sb = newarray(in*ik,DOUBLE)) == SISL_NULL) goto err101;
     if ((lfirst = newarray(kn,INT)) == SISL_NULL) goto err101;
     if ((llast = newarray(kn,INT)) == SISL_NULL) goto err101;
     if ((l2sta = newarray(kn,INT)) == SISL_NULL) goto err101;
     
     
     
     sh1922(pcurve->et,kn,ik,etau,in,sa,lfirst,llast,&kstat);
     if (kstat < 0) goto error;
		    
     
		    
     if (ileftfix == 0 && irightfix == 0)
     {
	
	
	sh1926(etau,ik,in,kdim,pcurve->et,pcurve->ecoef,kn,
	       sa,lfirst,llast,sb,l2sta,scoef,&kstat);
	if (kstat < 0) goto error;
      }
     else
     {
	
	
	if ((sc = newarray(kn*kdim,DOUBLE)) == SISL_NULL) goto err101;
	memcopy(sc,pcurve->ecoef,kn*kdim,DOUBLE);
     
	
	
	sh1927(etau,ik,in,kdim,pcurve,ileftfix,irightfix,scoef,&kstat);
	if (kstat < 0) goto error;
		       
        
		       
        if (knh > 0)
	{
	   
	   
	   sh1930(sa,lfirst,llast,sc,scoef,ik,in,kn,kdim,ileftfix,
		  irightfix,&kstat);
	   if (kstat < 0) goto error;
			  
           
			  
	   sh1928(etau,ik,in,kdim,pcurve->et,sc,kn,ileftfix,
		  irightfix,sa,knh,lfirst,llast,sb,scoef,l2sta,&kstat);
	   if (kstat < 0) goto error;
        }
     }
     
     if (knh > 0)
     {
	
	
	sh1923(sb,knh,ik,l2sta,&kstat);
	if (kstat < 0) goto error;
		       
        
		       
        sh1924(sb,scoef+ileftfix*kdim,knh,ik,kdim,l2sta,&kstat);
	if (kstat < 0) goto error;
      }

     
     
     if ((*rnewcurve = newCurve(in,ik,etau,scoef,kkind,kdim,kcopy)) == SISL_NULL)
	goto err101;
     (*rnewcurve)->cuopen = pcurve->cuopen;
     
     
     
     sh1925(pcurve,*rnewcurve,kdim,sa,lfirst,llast,*gmaxerr,*gl2err,
	    ileftfix,irightfix,&kstat);
     if (kstat < 0) goto error;
  }
  
  

  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err109 :
    *jstat = -109;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :

    

    if (scoef != SISL_NULL) freearray(scoef);
    if (sc != SISL_NULL) freearray(sc);
    if (sa != SISL_NULL) freearray(sa);
    if (sb != SISL_NULL) freearray(sb);
    if (lfirst != SISL_NULL) freearray(lfirst);
    if (llast != SISL_NULL) freearray(llast);
    if (l2sta != SISL_NULL) freearray(l2sta);
  
    return;
}

