/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1462

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
typedef void (*fshapeProc)(double [],double [],int,int,int *);
#else
typedef void (*fshapeProc)();
#endif

#if defined(SISLNEEDPROTOTYPES)
void 
  sh1462(fshapeProc fshape,SISLCurve *vboundc[],int icurv,
	 double etwist[],double etang[],double eder[],int *jstat)
#else	 
void sh1462(fshape,vboundc,icurv,etwist,etang,eder,jstat)
     fshapeProc    fshape;
     double        etwist[],etang[],eder[];
     SISLCurve     *vboundc[];
     int           icurv,*jstat;
#endif     
{
  int kstat = 0;         
  int kder = 2;          
  int ki;                
  int kdim = 3;          
  double tonethird = (double)1.0/(double)3.0;  
  double tonesixth = (double)1.0/(double)6.0;  
  double sbar[3];        
  double sder[18];       

  

  sbar[0] = sbar[1] = sbar[2] = tonethird;
  
  

  sh1466(vboundc,etwist,kder,sbar,sder,&kstat);
  if (kstat < 0) goto error;
   
  

  for (ki=0; ki<kdim; ki++)
    {
      etang[ki] = -sder[kdim+ki]*tonethird + sder[2*kdim+ki]*tonesixth;
      etang[kdim+ki] = sder[kdim+ki]*tonesixth - sder[2*kdim+ki]*tonethird;
      etang[2*kdim+ki] = sder[kdim+ki]*tonesixth + sder[2*kdim+ki]*tonesixth;
    }
  
  

  fshape(sder,etang,kdim,icurv,&kstat);
  if (kstat < 0) goto error;
  
  

  memcopy(eder,sder,kdim,DOUBLE);
  memcopy(eder+kdim,etang+2*kdim,kdim,DOUBLE);
  memcopy(eder+2*kdim,etang,kdim,DOUBLE);
  
  

  for (ki=0; ki<kdim; ki++)
    {
      eder[3*kdim+ki] = sder[3*kdim+ki]*tonesixth*tonesixth 
	+ (double)2.0*sder[4*kdim+ki]*tonesixth*tonesixth 
	  + sder[5*kdim+ki]*tonesixth*tonesixth;
      eder[4*kdim+ki] = -sder[3*kdim+ki]*tonesixth*tonethird 
	+ sder[4*kdim+ki]*tonesixth*(tonesixth - tonethird)
	  + sder[5*kdim+ki]*tonesixth*tonesixth;
      eder[5*kdim+ki] = sder[3*kdim+ki]*tonethird*tonethird 
	- (double)2.0*sder[4*kdim+ki]*tonethird*tonesixth 
	  + sder[5*kdim+ki]*tonesixth*tonesixth;
    }
  
  *jstat = 0;
  goto out;
  
  

 error:
  *jstat = kstat;
  goto out;
  
  out :
    return;
}

