/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1465

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
   static void sh1465_s9der2(double [],double [],double [],double [],
			     double [],int,int,double [],int *);
   typedef void (*fshapeProc)(double [],double [],int,int,int *);
#else
   static void sh1465_s9der2();
   typedef void (*fshapeProc)();
#endif  

#if defined(SISLNEEDPROTOTYPES)
void
  sh1465(fshapeProc fshape,SISLCurve *vboundc[],int icurv,
	 double etwist[],double etang[],double eder[],int *jstat)
#else
void sh1465(fshape,vboundc,icurv,etwist,etang,eder,jstat)
     fshapeProc fshape;
     double etwist[],etang[],eder[];
     SISLCurve *vboundc[];
     int icurv,*jstat;
#endif     
{
  int kstat = 0;      
  int kder = 0;       
  int kder1 = 1;      
  int ki,kj;          
  int kdim = 3;       
  int kleft = 0;      
  int kcurv2;         
  double tpar;        
  double t1;          
  double trad1,trad2; 
  double ta1;         
  double tb1,tb2;     
  double tang1,tang2; 
  double tscal1;      
  double tdist;       
  double smid[3];     
  double snorm1[3];   
  double snorm[3];    
  double svec[6];     
  double *sder = SISL_NULL;   
  double *stang = SISL_NULL;  
  SISLCurve *qc;      
  
  kcurv2 = icurv/2;

  

  if ((sder = newarray(2*icurv*kdim,DOUBLE)) == SISL_NULL) goto err101;
  if ((stang = newarray(icurv*kdim,DOUBLE)) == SISL_NULL) goto err101;

  for (ki=0; ki<icurv; ki++)
    {  
      qc = vboundc[2*ki];

      

      if (qc->idim != kdim) goto err102;

      

      tpar = (*(qc->et + qc->ik - 1) + *(qc->et + qc->in))/(double)2.0;
      
      

      s1221(qc,kder1,tpar,&kleft,sder+2*ki*kdim,&kstat);
      if (kstat < 0) goto error;
      
      if (ki < 2) memcopy(svec+ki*kdim,sder+(2*ki+1)*kdim,kdim,DOUBLE);
      if (ki == 1) kder1 = 0;
      
      

      s1221(vboundc[2*ki+1],kder,tpar,&kleft,sder+(2*ki+1)*kdim,&kstat);
      if (kstat < 0) goto error;
    }
  
  for (ki=0; ki<kcurv2; ki++)
    {
       

      s6curvrad(sder+2*ki*kdim,sder+2*(ki+kcurv2)*kdim,sder+(2*ki+1)*kdim,
		kdim,&trad1,&kstat);
      s6curvrad(sder+2*ki*kdim,sder+2*(ki+kcurv2)*kdim,sder+(2*(ki+kcurv2)+1)*kdim,
		kdim,&trad2,&kstat);
  
      

      tb1 = s6norm(sder+(2*ki+1)*kdim,kdim,sder+(2*ki+1)*kdim,&kstat);
      tb2 = s6norm(sder+(2*(ki+kcurv2)+1)*kdim,kdim,sder+(2*(ki+kcurv2)+1)*kdim,&kstat);
      
      

      tdist = s6dist(sder+2*ki*kdim,sder+2*(ki+kcurv2)*kdim,kdim);
      
      

      tscal1  = s6scpr(sder+(2*ki+1)*kdim,sder+(2*(ki+kcurv2)+1)*kdim,kdim);

      if (tscal1 >= DZERO)
	tscal1  = MIN((double)1.0,tscal1);
      else
	tscal1  = MAX((double)-1.0,tscal1);

      ta1 = acos(tscal1);

      if (fabs(ta1) < ANGULAR_TOLERANCE) ta1 = DZERO;

      if (DNEQUAL(ta1,DZERO))
	{
	  

	  tang1 = s1325(trad1,ta1);
	  tang2 = s1325(trad2,ta1);
	}

      

      if (DEQUAL(ta1,DZERO) || trad1 < 0) tang1 = tdist/(double)3.0;
      if (DEQUAL(ta1,DZERO) || trad2 < 0) tang2 = tdist/(double)3.0;
      if (tang1 > (double)0.5*tdist || tang2 > (double)0.5*tdist) 
	{
	  tang1 = tb1;
	  tang2 = tb2;
	}
      
      

      for (kj=0; kj<kdim; kj++)
	{
	  sder[(2*ki+1)*kdim+kj] *= tang1;
	  sder[(2*(ki+kcurv2)+1)*kdim+kj] *= tang2;
	} 
    }  
  

  

  for (kj=0; kj<kdim; kj++) 
    {
      smid[kj] = (double)0.0;
      snorm[kj] = (double)0.0;
      
      for (ki=0; ki<kcurv2; ki++)
	{
	  

	  smid[kj] += (sder[2*ki*kdim+kj] + sder[2*(ki+kcurv2)*kdim+kj])/(double)2.0
	    + (sder[(2*ki+1)*kdim+kj] 
	       + sder[2*(ki+kcurv2)*kdim+kdim+kj])/(double)8.0;
	  
	  

	  stang[(ki+kcurv2)*kdim+kj] = (double)1.5*(sder[2*(ki+kcurv2)*kdim+kj]
					- sder[2*ki*kdim+kj])
	    + (double)0.25*(sder[2*(ki+kcurv2)*kdim+kdim+kj]
			   - sder[(2*ki+1)*kdim+kj]);
	  stang[ki*kdim+kj] = - stang[(ki+kcurv2)*kdim+kj];
	}
      smid[kj] /= (double)kcurv2;
    }

  

  for (ki=0; ki<kcurv2; ki++)
    {
      s6crss(stang+ki*kdim,stang+(ki+1)*kdim,snorm1);
  
      for (kj=0; kj<kdim; kj++) 
	snorm[kj] += snorm1[kj]/(double)kcurv2;
    }
  
  (void)s6norm(snorm,kdim,snorm,&kstat);
  
  

  for (ki=0; ki<icurv; ki++)
    {
      t1 = -s6scpr(stang+ki*kdim,snorm,kdim);
      for (kj=0; kj<kdim; kj++) stang[ki*kdim+kj] += t1*snorm[kj];
    }
  
  

  fshape(smid,stang,kdim,icurv,&kstat);
  if (kstat < 0) goto error;
  
  

  sh1465_s9der2(sder,smid,stang,snorm,svec,icurv,kdim,eder+3*kdim,&kstat);
  if (kstat < 0) goto error;
  
  

  for (ki=0; ki<icurv*kdim; ki++) stang[ki] *= (double)0.5;
  for (ki=3*kdim; ki<6*kdim; ki++) eder[ki] *= (double)0.25;
  
  

  memcopy(eder,smid,kdim,DOUBLE);
  memcopy(eder+kdim,stang,2*kdim,DOUBLE);
  
  

  memcopy(etang,stang+kdim,(icurv-1)*kdim,DOUBLE);
  memcopy(etang+(icurv-1)*kdim,stang,kdim,DOUBLE);
					
  *jstat = 0;
  goto out;
  
  

  err101 :
    *jstat = -101;
  goto out;
  
  

  err102 :
    *jstat = -102;
  goto out;
  
  

 error:
  *jstat = kstat;
  goto out;
  
  out :

    

    if (sder) freearray(sder);
  if (stang) freearray(stang);
  
  return;
}


#if defined(SISLNEEDPROTOTYPES)
static void
  sh1465_s9der2(double ebound[],double epoint[],double etang[],
		double enorm[],double evec[],int icurv,
		int idim,double eder2[],int *jstat)
#else	       
static void sh1465_s9der2(ebound,epoint,etang,enorm,evec,icurv,
			  idim,eder2,jstat)
     int       icurv,idim,*jstat;
     double    ebound[],epoint[],etang[],enorm[],evec[],eder2[];
#endif     
{
  int kstat = 0;     
  int ki,kj;         
  int kleft = 0;     
  int knbpnt = 6;    
  int kcnsta = 0;    
  int kcnend = 0;    
  int kopen = 1;     
  int kcurv2;        
  int kord = 6;      
  int knpar;         
  int kder = 2;      
  int ll[3];         
  double tstpar = (double)0.0;   
  double tendpar;    
  double tpar;       
  double tncurv;     
  double te,tf,tg;   
  double tl,tm,tn;   
  double talfa;      
  double tcos;       
  double tdudt;      
  double tdvdt;      
  double tform1;     
  double spoint[18]; 
  double stype[6];   
  double *spar = SISL_NULL;  
  double sder[18];   
  double scurv[3];   
  double smat[9];    
  SISLCurve *qc = SISL_NULL;  
  
  kcurv2 = icurv/2;
  
  

  if (idim != 3) goto err104;
  
  

  for (ki=0; ki<2; ki++)
    {
      

      for (kj=0; kj<idim; kj++)
	{
	  spoint[kj] = ebound[2*(kcurv2+ki)*idim+kj];
	  spoint[idim+kj] = ebound[(2*(kcurv2+ki)+1)*idim+kj];
	  spoint[2*idim+kj] = epoint[kj];
	  spoint[3*idim+kj] = etang[ki*idim+kj];
	  spoint[4*idim+kj] = ebound[2*ki*idim+kj];
	  spoint[5*idim+kj] = -ebound[(2*ki+1)*idim+kj];
	}
      
      

      stype[0] = (double)1.0;
      stype[1] = (double)4.0;
      stype[2] = (double)1.0;
      stype[3] = (double)4.0;
      stype[4] = (double)1.0;
      stype[5] = (double)4.0;

      

      s1334(spoint,knbpnt,idim,stype,kcnsta,kcnend,kopen,kord,tstpar,&tendpar,
	    &qc,&spar,&knpar,&kstat);
      if (kstat < 0) goto error;
      
      
				
      tpar = spar[1];
      s1221(qc,kder,tpar,&kleft,sder+3*ki*idim,&kstat);
      if (kstat < 0) goto error;
    }
  
  

  memcopy(eder2,sder+2*idim,idim,DOUBLE);
  memcopy(eder2+2*idim,sder+5*idim,idim,DOUBLE);
  
  

  s6curvature(sder,idim,scurv,&kstat);

  
  
  tncurv = s6scpr(scurv,enorm,idim);

  

  talfa = TWOPI/(double)icurv;
  tcos = cos(talfa);
  tdudt = (DEQUAL(tcos+(double)1.0,(double)1.0)) ? (double)0.0 : (double)1.0/tcos;
  tdvdt = (double)1.0;
  
  

  te = s6scpr(sder+idim,sder+idim,idim);
  tf = s6scpr(sder+idim,sder+4*idim,idim);
  tg = s6scpr(sder+4*idim,sder+4*idim,idim);
  
  

  tform1 = te*tdudt*tdudt + (double)2.0*tf*tdudt*tdvdt + tg*tdvdt*tdvdt;
  
  

  tl = s6scpr(sder+2*idim,enorm,idim);
  tn = s6scpr(sder+5*idim,enorm,idim);
  
  

  eder2[idim] = tm = (tncurv*tform1 - tl*tdudt*tdudt - tn*tdvdt*tdvdt)/(double)2.0;
  
  

  eder2[idim+1] = (double)0.0;
  
  

  eder2[idim+2] = (double)0.0;
  
  

  memcopy(smat,enorm,idim,DOUBLE);
  memcopy(smat+idim,sder+idim,idim,DOUBLE);
  memcopy(smat+2*idim,sder+4*idim,idim,DOUBLE);
  
  s6lufacp(smat,ll,3,&kstat);
  if (kstat < 0) goto error;
  
  s6lusolp(smat,eder2+idim,ll,3,&kstat);
  if (kstat < 0) goto error;
  
  *jstat = 0;
  goto out;
  
  

  err104 :
    *jstat = -104;
  goto out;
  
  

  error :
    *jstat = kstat;
  goto out;
  
  out :
    return;
}     



