/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1782

#include "sislP.h"



#if defined(SISLNEEDPROTOTYPES)
static void
sh1782_s9sf_pt (SISLObject *, SISLObject *, double, SISLIntdat **,
		SISLIntpt **, int, int, int *);
static void
sh1782_s9sf_cu (SISLObject *, SISLObject *, double, SISLIntdat **,
		SISLIntpt **, int, int, int *);
static void
sh1782_s9sf_sf (SISLObject *, SISLObject *, double, SISLIntdat **,
		SISLIntpt **, int, int, int *);
#else
static void sh1782_s9sf_pt ();
static void sh1782_s9sf_cu ();
static void sh1782_s9sf_sf ();
#endif

#if defined(SISLNEEDPROTOTYPES)
void
sh1782 (SISLObject * po1, SISLObject * po2, double aepsge,
	SISLIntdat * pintdat, int ipar, double apar,
	SISLIntdat ** rintdat, int *jnewpt, int *jstat)
#else
void
sh1782 (po1, po2, aepsge, pintdat, ipar, apar, rintdat, jnewpt, jstat)
     SISLObject *po1;
     SISLObject *po2;
     double aepsge;
     SISLIntdat *pintdat;
     int ipar;
     double apar;
     SISLIntdat **rintdat;
     int *jnewpt;
     int *jstat;
#endif
{
  int kstat = 0;		
  int kdim;			
  int knpoint;			
  SISLIntpt **uintpt = SISL_NULL;	

  *jnewpt = 0;

  

  if (pintdat == SISL_NULL)
    {
      *jstat = 0;
      goto out;
    }

  

  if (po1->iobj == SISLPOINT)
    kdim = po1->p1->idim;
  else if (po1->iobj == SISLCURVE)
    kdim = po1->c1->idim;
  else
    kdim = po1->s1->idim;

  

  sh6idput (po1, po2, rintdat, pintdat, ipar, apar, &uintpt, &knpoint, &kstat);
  if (kstat < 0)
    goto error;

  if (knpoint == 0)
    {
      *jstat = 0;
      goto out;
    }


  

  if (kdim <= 2 && ((po1->iobj == SISLSURFACE && po2->iobj == SISLPOINT)
		    || (po2->iobj == SISLSURFACE && po1->iobj == SISLPOINT)))
    {
      

      sh1782_s9sf_pt (po1, po2, aepsge, rintdat, uintpt, knpoint,
		      ipar, &kstat);
      if (kstat < 0)
	goto error;

    }

  else if (kdim == 3 && 
	 ((po1->iobj == SISLSURFACE && po2->iobj == SISLCURVE) ||
	  (po1->iobj == SISLCURVE && po2->iobj == SISLSURFACE )))
    {
      

      sh1782_s9sf_cu (po1, po2, aepsge, rintdat, uintpt, knpoint,
		      ipar, &kstat);
      if (kstat < 0)
	goto error;
    }

  else if (kdim == 3 && po1->iobj == SISLSURFACE
	   && po2->iobj == SISLSURFACE)
    {
      

      sh1782_s9sf_sf (po1, po2, aepsge, rintdat, uintpt, knpoint,
		      ipar, &kstat);
      if (kstat < 0)
	goto error;
    }

  

  *jstat = 0;
  goto out;

  

error:*jstat = kstat;
  goto out;

out:if (uintpt != SISL_NULL)
    freearray (uintpt);
}




#if defined(SISLNEEDPROTOTYPES)
static void
sh1782_s9sf_pt (SISLObject * po1, SISLObject * po2, double aepsge,
		SISLIntdat ** rintdat, SISLIntpt ** uintpt, int kpoint,
		int ipar, int *jstat)
#else
static void
sh1782_s9sf_pt (po1, po2, aepsge, rintdat, uintpt, kpoint, ipar, jstat)
     SISLObject *po1;
     SISLObject *po2;
     double aepsge;
     SISLIntdat **rintdat;
     SISLIntpt **uintpt;
     int kpoint;
     int ipar;
     int *jstat;
#endif
{
  int kstat = 0;		
  int kdim;			
  double tdum, tsign;		
  double *sder;			
  double *sdum;			
  int ind;			
  int ki, kj, klow, khigh;	
  int index1, index2;		
  int kant;			
  int *edge_f, *edge_l;		
  SISLSurf *ps;			
  

  
  kdim = (po1->iobj == SISLSURFACE) ? po1->s1->idim : po2->s1->idim;
  if (kdim != 1 && kdim !=2)
    goto err106;

  if (ipar == 0)
    ind = 1;
  else
    ind = 0;

  if (po1->iobj == SISLSURFACE)
    tsign = (double) 1.0;
  else
    tsign = -(double) 1.0;

  
  for (ki = 0; ki < kpoint; ki++)
    for (kj = ki + 1; kj < kpoint; kj++)
      
      {
	sh6getlist (uintpt[ki], uintpt[kj], &index1, &index2, &kstat);
	if (kstat < 0)
	  goto error;
	if (!kstat)
	  {
	    
	    if (uintpt[ki]->epar[ind] <
		uintpt[kj]->epar[ind])
	      {
		klow = ki;
		khigh = kj;
	      }
	    else
	      {
		klow = kj;
		khigh = ki;
	      }

	    
	    sh6getgeom ((po1->iobj == SISLSURFACE) ? po1 : po2,
			(po1->iobj == SISLSURFACE) ? 1 : 2,
			uintpt[klow], &sder, &sdum, aepsge, &kstat);

	    if (kstat < 0)
	      goto error;

	    if (ipar == 0)
	      tdum = tsign * sder[1];
	    else
	      tdum = -tsign * sder[2];

	    if (tdum > DZERO)
	      sh6setdir (uintpt[klow], uintpt[khigh], &kstat);
	    else if (tdum < 0)
	      sh6setdir (uintpt[khigh], uintpt[klow], &kstat);

	      sh6setcnsdir (uintpt[klow], uintpt[khigh], ipar, &kstat);

	    
	    if (sh6ismain (uintpt[ki]) && sh6ismain (uintpt[kj]) &&
		po1->o1 == po1 &&
		po2->o1 == po2)
	      {

		ps = po1->s1;
		edge_f = &(uintpt[ki]->edge_1);
		edge_l = &(uintpt[kj]->edge_1);

		if (po2->iobj == SISLSURFACE)
		  {
		    edge_f = &(uintpt[ki]->edge_2);
		    edge_l = &(uintpt[kj]->edge_2);
		    ps = po2->s1;
		  }

		kant = 0;
		if (ipar == 0)
		  {
		    if (DEQUAL (uintpt[ki]->epar[ipar],
				ps->et1[ps->ik1 - 1]))
		      kant = -1;
		    else if (DEQUAL (uintpt[ki]->epar[ipar],
				     ps->et1[ps->in1]))
		      kant = 1;
		  }
		else
		  {
		    if (DEQUAL (uintpt[ki]->epar[ipar], ps->et2[ps->ik2 - 1]))
		      kant = 1;
		    else if (DEQUAL (uintpt[ki]->epar[ipar],
				     ps->et2[ps->in2]))
		      kant = -1;
		  }

		if (kant)
		  {
		    if ((double) kant * tdum > DZERO)
		      *edge_f = *edge_l = SI_RIGHT;
		    else if ((double) kant * tdum < DZERO)
		      *edge_f = *edge_l = SI_LEFT;

		  }

	      }

	  }
      }

  *jstat = 0;
  goto out;

  

err106:*jstat = -106;
  goto out;

  

error:*jstat = kstat;
  goto out;

out:
  return;
}


#if defined(SISLNEEDPROTOTYPES)
static void
sh1782_s9sf_cu (SISLObject * po1, SISLObject * po2, double aepsge,
		SISLIntdat ** rintdat, SISLIntpt ** uintpt, int kpoint,
		int ipar, int *jstat)
#else
static void
sh1782_s9sf_cu (po1, po2, aepsge, rintdat, uintpt, kpoint, ipar, jstat)
     SISLObject *po1;
     SISLObject *po2;
     double aepsge;
     SISLIntdat **rintdat;
     SISLIntpt **uintpt;
     int kpoint;
     int ipar;
     int *jstat;
#endif
{
  int kstat = 0;		
  int kdim;			
  double tdum;			
  double *val_s1;		
  double *val_s2;		
  double *snorm1;		
  double *snorm2;		
  double *sdir;	        	
  double *stang;		
  int ind;			
  int ki, kj, klow, khigh;	
  int index1, index2;		
  int kcurve;			
  
  
  *jstat = 0;

  
  
  if (po1->iobj == SISLCURVE && po2->iobj == SISLSURFACE)
  {
     kcurve = 1;
     kdim = po1->c1->idim;
     if (kdim != 3)
	goto err104;
     if (kdim != po2->s1->idim)
	goto err106;
  }
  else if (po1->iobj == SISLSURFACE && po2->iobj == SISLCURVE)
  {
     kcurve = 2;
     kdim = po1->s1->idim;
     if (kdim != 3)
	goto err104;
     if (kdim != po2->c1->idim)
	goto err106;
  }
  else
    goto err104;
  
  
  if (kcurve == 1 && ipar == 1)
    ind = 2;
  else  if ((kcurve == 1 && ipar == 2) ||
	    (kcurve == 2 && ipar == 0))
    ind = 1;
  else  if (kcurve == 2 && ipar == 1)
    ind = 0;
  else
     goto out;
  
  
  for (ki = 0; ki < kpoint; ki++)
    for (kj = ki + 1; kj < kpoint; kj++)
      
      {
	sh6getlist (uintpt[ki], uintpt[kj], &index1, &index2, &kstat);
	if (kstat < 0)
	  goto error;
	if (!kstat)
	  {
	    
	    if (uintpt[ki]->epar[ind] <
		uintpt[kj]->epar[ind])
	      {
		klow = ki;
		khigh = kj;
	      }
	    else
	      {
		klow = kj;
		khigh = ki;
	      }

	    
	    sh6getgeom (po1, 1, uintpt[klow], &val_s1, &snorm1, aepsge, &kstat);
	    if (kstat < 0)
	      goto error;

	    
	    sh6getgeom (po2, 2, uintpt[klow], &val_s2, &snorm2, aepsge, &kstat);
	    if (kstat < 0)
	      goto error;

	    
	    if (ipar == 0)
	      stang = val_s1 + 6;
	    else if (ipar == 1 && kcurve == 2)
	      stang = val_s1 + 3;
	    else if (ipar == 1 && kcurve == 1)
	      stang = val_s2 + 6;
	    else
	      stang = val_s2 + 3;

	    
	    if (kcurve == 1)
	       sdir = val_s1 + 3;
	    else
	       sdir = val_s2 + 3;

	    tdum = s6scpr (sdir, stang, kdim);

	    if (tdum > 0)
	      sh6setdir (uintpt[klow], uintpt[khigh], &kstat);
	    else if (tdum < 0)
	      sh6setdir (uintpt[khigh], uintpt[klow], &kstat);

	      sh6setcnsdir (uintpt[klow], uintpt[khigh], ipar, &kstat);
	  }
      }
  *jstat = 0;
  goto out;


  

err104:*jstat = -104;
  goto out;

  

err106:*jstat = -106;
  goto out;

  

error:*jstat = kstat;
  goto out;

out:
  return;
}



#if defined(SISLNEEDPROTOTYPES)
static void
sh1782_s9sf_sf (SISLObject * po1, SISLObject * po2, double aepsge,
		SISLIntdat ** rintdat, SISLIntpt ** uintpt, int kpoint,
		int ipar, int *jstat)
#else
static void
sh1782_s9sf_sf (po1, po2, aepsge, rintdat, uintpt, kpoint, ipar, jstat)
     SISLObject *po1;
     SISLObject *po2;
     double aepsge;
     SISLIntdat **rintdat;
     SISLIntpt **uintpt;
     int kpoint;
     int ipar;
     int *jstat;
#endif
{
  int kstat = 0;		
  int kdim;			
  double tdum;			
  double *val_s1;		
  double *val_s2;		
  double *snorm1;		
  double *snorm2;		
  double sdir[3];		
  double *stang;		
  int ind;			
  int ki, kj, klow, khigh;	
  int index1, index2;		
  int *edge_f, *edge_l;		
  int kant;			
  SISLSurf *ps;			
  

  
  kdim = po1->s1->idim;
  if (kdim != 3)
    goto err104;
  if (kdim != po2->s1->idim)
    goto err106;

  
  if (ipar == 0)
    ind = 1;
  else if (ipar == 1)
    ind = 0;
  else if (ipar == 2)
    ind = 3;
  else
    ind = 2;

  
  for (ki = 0; ki < kpoint; ki++)
    for (kj = ki + 1; kj < kpoint; kj++)
      
      {
	sh6getlist (uintpt[ki], uintpt[kj], &index1, &index2, &kstat);
	if (kstat < 0)
	  goto error;
	if (!kstat)
	  {
	    
	    if (uintpt[ki]->epar[ind] <
		uintpt[kj]->epar[ind])
	      {
		klow = ki;
		khigh = kj;
	      }
	    else
	      {
		klow = kj;
		khigh = ki;
	      }




	    
	    sh6getgeom (po1, 1, uintpt[klow], &val_s1, &snorm1, aepsge, &kstat);
	    if (kstat < 0)
	      goto error;

	    
	    sh6getgeom (po2, 2, uintpt[klow], &val_s2, &snorm2, aepsge, &kstat);
	    if (kstat < 0)
	      goto error;

	    
	    
	    if (ipar == 0)
	      stang = val_s1 + 6;
	    else if (ipar == 1)
	      stang = val_s1 + 3;
	    else if (ipar == 2)
	      stang = val_s2 + 6;
	    else
	      stang = val_s2 + 3;

	    
	    s6crss (snorm1, snorm2, sdir);

	    tdum = s6scpr (sdir, stang, kdim);

	    if (tdum > 0)
	      sh6setdir (uintpt[klow], uintpt[khigh], &kstat);
	    else if (tdum < 0)
	      sh6setdir (uintpt[khigh], uintpt[klow], &kstat);

	      sh6setcnsdir (uintpt[klow], uintpt[khigh], ipar, &kstat);
	    
	    if (sh6ismain (uintpt[ki]) && sh6ismain (uintpt[kj]) &&
		po1->o1 == po1 &&
		po2->o1 == po2)
	      {

		if (ipar < 2)
		  
		  {
		    edge_f = &(uintpt[ki]->edge_1);
		    edge_l = &(uintpt[kj]->edge_1);
		    ps = po1->s1;
		  }
		else
		  {
		    edge_f = &(uintpt[ki]->edge_2);
		    edge_l = &(uintpt[kj]->edge_2);
		    ps = po2->s1;
		  }

		kant = 0;
		if (ipar == 0 || ipar == 2)
		  {
		    if (DEQUAL (uintpt[ki]->epar[ipar],
				ps->et1[ps->ik1 - 1]))
		      kant = -1;
		    else if (DEQUAL (uintpt[ki]->epar[ipar],
				     ps->et1[ps->in1]))
		      kant = 1;
		  }
		else
		  {
		    if (DEQUAL (uintpt[ki]->epar[ipar], ps->et2[ps->ik2 - 1]))
		      kant = 1;
		    else if (DEQUAL (uintpt[ki]->epar[ipar],
				     ps->et2[ps->in2]))
		      kant = -1;
		  }

		if (kant)
		  {
		    if ((double) kant * tdum > DZERO)
		      *edge_f = *edge_l = SI_RIGHT;
		    else if ((double) kant * tdum < DZERO)
		      *edge_f = *edge_l = SI_LEFT;

		  }





	      }
	  }
      }
  *jstat = 0;
  goto out;


  

err104:*jstat = -104;
  goto out;

  

err106:*jstat = -106;
  goto out;

  

error:*jstat = kstat;
  goto out;

out:
  return;
}

