/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1850

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void 
sh1850(SISLCurve *pc1,double epoint[],double enorm[],int idim,double aepsco,
       double aepsge,int trackflag, int *jtrack, SISLTrack *** wtrack,
       int *jpt,double **gpar,int **pretop,int *jcrv,
       SISLIntcurve ***wcurve,int *jstat)
#else
void sh1850(pc1,epoint,enorm,idim,aepsco,aepsge,
	    trackflag,jtrack,wtrack,jpt,gpar,pretop,jcrv,wcurve,jstat)
     SISLCurve    *pc1;
     double   epoint[];
     double   enorm[];
     int      idim;
     double   aepsco;
     double   aepsge;
     int       trackflag;
     int       *jtrack;
     SISLTrack ***wtrack;
     int      *jpt;
     double   **gpar;
     int      **pretop;
     int      *jcrv;
     SISLIntcurve ***wcurve;
     int      *jstat;
#endif
                                                               
{                                                                     
  double *nullp = SISL_NULL;
  int ikind;               
  int kstat = 0;           
  int kpos = 0;            
  int kn;                  
  int kk;                  
  int ki;                  
  int kdim = 1;            
  double *st;              
  double *scoef;           
  double *sc = SISL_NULL;       
  double spoint[1];        
  double *spar = SISL_NULL;     
  double *sdiff = SISL_NULL;    
  double *s1,*s2;          
  SISLCurve *qc = SISL_NULL;        
  SISLPoint *qp = SISL_NULL;        
  SISLObject *qo1 = SISL_NULL;      
  SISLObject *qo2 = SISL_NULL;      
  SISLIntdat *qintdat = SISL_NULL;  
  double *rscoef;          
  double *s3;              
  double wmin,wmax;        
  double scale;            
  int i;                   
  int idimp1;              
  int      ksurf=0;         
  SISLIntsurf **wsurf=SISL_NULL;    
  int      kdeg=2000;       
  SISLObject *track_obj=SISL_NULL;
  
  
  
  if (!(track_obj = newObject (SISLCURVE)))
    goto err101;
  track_obj->c1 = pc1;
  

  

  *jpt  = 0;
  *jcrv = 0;
  *jtrack = 0;

  if ( !(idim == 2 || idim == 3)) goto err104;
  if (pc1 -> idim != idim) goto err103;

  

  kn    = pc1 -> in;
  kk    = pc1 -> ik;
  st    = pc1 -> et;
  ikind = pc1 -> ikind;
  
  
  if(ikind == 2 || ikind == 4)
  {
      
      idimp1=idim+1;
      rscoef = pc1 -> rcoef;
      wmin=rscoef[idim];
      wmax=rscoef[idim];
      for(i=idim; i< kn*idimp1; i+=idimp1)
      {
          if(rscoef[i] < wmin) wmin=rscoef[i];
          if(rscoef[i] > wmax) wmax=rscoef[i];
      } 
      scale=1.0/sqrt(wmin*wmax);
      scoef=newarray(kn*idimp1,DOUBLE);
      for(i=0; i< kn*idimp1; i++)
      {
          scoef[i]=rscoef[i]*scale;
      } 
  }
  else
  {
      scoef = pc1 -> ecoef;
  }


  
                         
  if ((sdiff = newarray(idim,double)) == SISL_NULL) goto err101;
  if ((sc = newarray(kn,double)) == SISL_NULL) goto err101;

  

  for (s1=scoef,s2=sc,ki=0; ki<kn; ki++,s1+=idim,s2++)
    {
      if(ikind == 2 || ikind == 4)
      {
      
          s3=s1+idim;
          for(i=0; i<idim; i++)
          {
              sdiff[i]=(*(s1+i)) - (*s3) * epoint[i];
          }
          *s2 = s6scpr(sdiff,enorm,idim);
	  s1++;
      }
      else
      {
      
          s6diff(s1,epoint,idim,sdiff);
          *s2 = s6scpr(sdiff,enorm,idim);
      }
    }                                 

  if(ikind == 2 || ikind == 4) freearray(scoef);

  

  if(ikind == 2 || ikind == 4) ikind--;
  qc = newCurve(kn,kk,st,sc,ikind,kdim,0);
  if (qc == SISL_NULL) goto err101;
  qc->cuopen = pc1->cuopen;		  
      
  

  if (!(qo1 = newObject(SISLCURVE))) goto err101;
  qo1 -> c1 = qc;
  qo1 -> o1 = qo1;
  
  

  if(!(qo2 = newObject(SISLPOINT))) goto err101;
  spoint[0] = DZERO;
  if(!(qp = newPoint(spoint,kdim,1))) goto err101;
  qo2 -> p1 = qp;

  

  sh1761(qo1,qo2,aepsge,&qintdat,&kstat);
  if (kstat < 0) goto error;
  
  
  int_join_per( &qintdat,track_obj,track_obj,nullp,kdeg,aepsge,&kstat);
  if (kstat < 0)
    goto error;

  
  if (trackflag && qintdat)
    {
      make_tracks (qo1, qo2, 0, nullp,
		   qintdat->ilist, qintdat->vlist, 
		   jtrack, wtrack, aepsge, &kstat);
      if (kstat < 0)
	goto error;
    }

  

  if (qintdat)
    {
      hp_s1880(track_obj, track_obj, kdeg,
	       1,0,qintdat,jpt,gpar,&spar,pretop,jcrv,wcurve,&ksurf,&wsurf,&kstat);
      if (kstat < 0) goto error;
    }
  
  

  *jstat = 0;
  goto out;

  

 err101: *jstat = -101;
        s6err("sh1850",*jstat,kpos);
        goto out;

  

 err103: *jstat = -103;
        s6err("sh1850",*jstat,kpos);
        goto out;

  

 err104: *jstat = -104;                          
        s6err("sh1850",*jstat,kpos);
        goto out;

  

  error : *jstat = kstat;
        s6err("sh1850",*jstat,kpos);
        goto out;

 out:

  

  if (sc)      freearray(sc);
  if (sdiff)   freearray(sdiff);       
  if (spar)    freearray(spar);
  if (qo1)     freeObject(qo1);
  if (qo2)     freeObject(qo2);
  if (qintdat) freeIntdat(qintdat);
  if (track_obj)
    {
       track_obj->c1 = SISL_NULL;
       freeObject(track_obj);
    }

return;
}                                               
                                           
                       


