/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"



#define SH1859

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
sh1859 (SISLSurf * ps1, SISLSurf * ps2, double aepsco, double aepsge,
	int trackflag, int *jtrack, SISLTrack *** wtrack,
	int *jpt, double **gpar1, double **gpar2, int **pretop, int *jcrv, SISLIntcurve *** wcurve, int *jsurf, SISLIntsurf *** wsurf, int *jstat)
#else
void 
sh1859 (ps1, ps2, aepsco, aepsge,
	trackflag, jtrack, wtrack, jpt, gpar1, gpar2, pretop, jcrv, wcurve,jsurf,wsurf, jstat)
     SISLSurf *ps1;
     SISLSurf *ps2;
     double aepsco;
     double aepsge;
     int trackflag;
     int *jtrack;
     SISLTrack ***wtrack;
     int *jpt;
     double **gpar1;
     double **gpar2;
     int **pretop;
     int *jcrv;
     SISLIntcurve ***wcurve;
     int      *jsurf;
     SISLIntsurf ***wsurf;
     int *jstat;
#endif
{
  double *nullp = SISL_NULL;
  int kstat = 0;		
  int kpos = 0;			
  SISLObject *qo1 = SISL_NULL;	
  SISLObject *qo2 = SISL_NULL;	
  SISLIntdat *qintdat = SISL_NULL;	
  int      kdeg=0;              

  

  *jpt = 0;
  *jcrv = 0;
  *jtrack = 0;
  *jsurf = 0;

  if (ps1->idim != ps2->idim)
    goto err106;

  

  if ((qo1 = newObject (SISLSURFACE)) == SISL_NULL)
    goto err101;
  qo1->s1 = ps1;
  qo1->o1 = qo1;

  if ((qo2 = newObject (SISLSURFACE)) == SISL_NULL)
    goto err101;
  qo2->s1 = ps2;
  qo2->o1 = qo2;

  

  sh1761 (qo1, qo2, aepsge, &qintdat, &kstat);
  if (kstat < 0)
    goto error;

  

  sh6degen(qo1,qo2,&qintdat,aepsge,&kstat);
  if (kstat < 0) goto error;

  
  if (trackflag && qintdat)
    {
      refine_all( &qintdat,qo1,qo2,nullp,kdeg=0,aepsge,&kstat);
      if (kstat < 0)
	goto error; 
    }

  
  int_join_per( &qintdat,qo1,qo2,nullp,kdeg=0,aepsge,&kstat);
  if (kstat < 0)
    goto error;
  
  
  if (trackflag && qintdat)
    {
      make_tracks (qo1, qo2, 0, nullp,
		   qintdat->ilist, qintdat->vlist, 
		   jtrack, wtrack, aepsge, &kstat);
      if (kstat < 0)
	goto error;
    }


  

  if (qintdat)			
    {
      hp_s1880(qo1, qo2, 0,
	       2,2,qintdat,jpt,gpar1,gpar2,pretop,jcrv,wcurve,jsurf,wsurf,&kstat);
      if (kstat < 0)
	goto error;
    }


  

  *jstat = 0;
  goto out;

  

err101:*jstat = -101;
  s6err ("sh1859", *jstat, kpos);
  goto out;

  

err106:*jstat = -106;
  s6err ("sh1859", *jstat, kpos);
  goto out;

  

error:*jstat = kstat;
  s6err ("sh1859", *jstat, kpos);
  goto out;

out:

  

  if (qo1)
    {
      qo1->s1 = SISL_NULL;
      freeObject (qo1);
    }
  if (qo2)
    {
      qo2->s1 = SISL_NULL;
      freeObject (qo2);
    }
  if (qintdat)
    freeIntdat (qintdat);

  

  return;
}


