/* SISL - SINTEF Spline Library version 4.4.                              */
/* Definition and interrogation of NURBS curves and surface.              */
/* Copyright (C) 1978-2005, SINTEF ICT, Applied Mathematics, Norway.      */

/* This program is free software; you can redistribute it and/or          */
/* modify it under the terms of the GNU General Public License            */
/* as published by the Free Software Foundation version 2 of the License. */

/* This program is distributed in the hope that it will be useful,        */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/* GNU General Public License for more details.                           */

/* You should have received a copy of the GNU General Public License      */
/* along with this program; if not, write to the Free Software            */
/* Foundation, Inc.,                                                      */
/* 59 Temple Place - Suite 330,                                           */
/* Boston, MA  02111-1307, USA.                                           */

/* Contact information: e-mail: tor.dokken@sintef.no                      */
/* SINTEF ICT, Department of Applied Mathematics,                         */
/* P.O. Box 124 Blindern,                                                 */
/* 0314 Oslo, Norway.                                                     */

/* SISL commercial licenses are available for:                            */
/* - Building commercial software.                                        */
/* - Building software whose source code you wish to keep private.        */


#include "copyright.h"


#define SH1922

#include "sislP.h"

#if defined(SISLNEEDPROTOTYPES)
void
      sh1922(double et[],int im,int ik,double etau[],int in,double ea[],
	     int nfirst[],int nlast[],int *jstat)
#else
void sh1922(et,im,ik,etau,in,ea,nfirst,nlast,jstat)
   double et[];
   int im;
   int ik;
   double etau[];
   int in;
   double ea[];
   int nfirst[];
   int nlast[];
   int *jstat;
#endif     
{ 
   int kstat = 0;
   int kj1,kj2;
   int kih1,kih2;
   int ki,kj,kjh,kkj;
   int kmu;
   int kmuprm;
   int knu;
   double *sah = SISL_NULL;
   
   if ((sah = new0array(ik,DOUBLE)) == SISL_NULL) goto err101;
   
   

   for (kmu=in+ik-1; DEQUAL(etau[kmu-1],etau[kmu]); kmu--);
   
   for (kjh=im-1; et[kjh+ik]>etau[kmu]; kjh--);
   
   ki = kjh;
   if (DEQUAL(et[kjh+ik],etau[kmu]))
      {
	 for (; DEQUAL(et[ki-1+ik],et[kjh+ik]); ki--);
      }
      
   kj2 = MIN(im-1,ki+in+ik-kmu-1);
   
   
       
   for (kmu=0; DEQUAL(etau[kmu+1],etau[kmu]); kmu++);
   
   for (kjh=0; et[kjh]<etau[kmu]; kjh++);
   
   ki = kjh;
   if (DEQUAL(et[kjh],etau[kmu]))
      {
	 for (; DEQUAL(et[ki+1],et[kjh]); ki++);
      }
   kj1 = MAX(0,ki-kmu);      
      
   
      
   memzero(ea,im*ik,DOUBLE);
   
   
   
   
   for (kj=0; kj<kj1; kj++)
     {
	nfirst[kj] = ik;
	nlast[kj] = 0;
     }
   
   
   
   for (kj=kj2+1; kj<im; kj++)
     {
	nfirst[kj] = ik;
	nlast[kj] = 0;
     }
   
   
   
   for (kj=kj1; kj<=kj2; kj++)
     {
	for (; etau[kmu+1] <= et[kj]; kmu++);
	kkj = kj;
	sh1929(etau,in,ik,kmu,et,im,kkj,sah,&kmuprm,&knu,&kstat);
	if (kstat < 0) goto error;
		       
       
		       
       nfirst[kj] = MAX(kmuprm-knu,0);
       nlast[kj] = MIN(kmuprm,in-1);
       kih1 = nfirst[kj] + ik - kmuprm - 1;
       kih2 = nlast[kj] + ik - kmuprm - 1;
       for (ki=kih1; ki<=kih2; ki++)
	 ea[kj*ik+ki] = sah[ki];
     }

   

   *jstat = 0;
   goto out;
   
   
   
   err101: *jstat = -101;
   goto out;
   
   
   
   error: *jstat = kstat;
   goto out;
   
   out:
      
      
      if (sah != SISL_NULL) freearray(sah);
	  
      return;
}
   

