/*********************************************************************
*      This file is part of the source code of AXEL library          *
*   Author(s): G. Gatellier                                          *
*              B. Mourrain, GALAAD, INRIA                            *
**********************************************************************/
/** \file                                                           **/
#ifndef axel_mesh_H__
#define axel_mesh_H__

/*********************************************************************/
#include <synaps/init.h>
#include <synaps/linalg/MethodName.h>
#include <synaps/linalg/MATRIX3D.h>
#include <synaps/mpol/MPOLDST.m>
#include <synaps/topology/mesh/cell3d.h>
#include <synaps/topology/box3d.h>
#include <synaps/topology/mesh/cube3d.h>
#include <synaps/topology/Octree.h>
#include <synaps/topology/mesh/bezier3d.h>
#include <synaps/topology/mesh/scene.h>
#include <synaps/topology/mesh/subdivisor.h>
#include <synaps/topology/mesh/geomview.h>

__BEGIN_NAMESPACE_SYNAPS

/*********************************************************************/

template<class MPOL, class REP3D>
void mesh(scene<MPOL, REP3D> & scn,
	  const MPOL & polynome, 
	  const typename MPOL::coeff_t& x0, const typename MPOL::coeff_t& x1, 
	  const typename MPOL::coeff_t& y0, const typename MPOL::coeff_t& y1, 
	  const typename MPOL::coeff_t& z0, const typename MPOL::coeff_t& z1,
	  const typename MPOL::coeff_t& min,const typename MPOL::coeff_t& max )
{
  scn.x=x0; scn.y=y0; scn.z=z0;
  scn.u=x1-x0; scn.v=y1-y0; scn.w=z1-z0;
  cell3d<MPOL,REP3D> bz3d(polynome, x0, x1, y0, y1, z0, z1);

  // Construction of the grid.
  subdivisor<cell3d<MPOL,REP3D> > subdiv(min, max);

  // Subdivision of space and construction of the octree.
  //  axel::Clock T;
  //  T.start();
  adaptiveSplit(scn,bz3d,subdiv);
  //  T.stop();
  //  double t=T.time();
  //  std::cout<<"\n-> time split computing: "<<t<<"\n";

  // Construction of the mesh.
  //  axel::Clock T2;
  //  T2.start();
  buildMesh(scn,bz3d,subdiv);
  //  T2.stop();
  //  t=T2.time();
  //  std::cout<<"-> time build mesh: "<<t<<"\n\n";
}

/*********************************************************************/

__END_NAMESPACE_SYNAPS

#endif // axel_mesh_H__
