/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 ********************************************************************/
#ifndef synaps_mpol_mpoldse_regularity_h
#define synaps_mpol_mpoldse_regularity_h
//===================================================================
#include <synaps/init.h>
#include <synaps/mpol/MPolDse.h>
//===================================================================
__BEGIN_NAMESPACE_SYNAPS
//===================================================================
#define _is_small(x) ((x) < 1e-6 && (x) > -1e-6)


/// Check if the coefficients of the polynomial are of the same sign.
template<class C>
bool has_no_sign_change(const MPolDse<C, mpoldse::bernstein<C> >& pol)
{
  bool hasroot;
  const C* coeff = pol.begin();
  while( _is_small(*coeff) ) ++coeff;
  C lastC = *coeff;
  while( ++coeff < pol.end() )
    if ( ! _is_small(*coeff) )
      {
	if ( lastC * (*coeff) < 0 ) return false;
	lastC = *coeff;
      };
  return true;
}

/** Check if the coefficients of the derivative of p with respects to the variable xi 
    are of the same sign.
    **/
template<class C>
bool is_regular(const MPolDse<C, mpoldse::bernstein<C> >& p, int i)
{
  MPolDse<C, mpoldse::bernstein<C> > dp;
  diff(dp,p,i);
  return has_no_sign_change(dp);
}
//===================================================================
__END_NAMESPACE_SYNAPS
#endif //
