/********************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: topology2d.h,v 1.2 2005/09/28 06:40:47 mourrain Exp $
 ********************************************************************/
#ifndef synaps_subdivision_sbd2dbdg_H
#define synaps_subdivision_sbd2dbdg_H
//--------------------------------------------------------------------
#include <synaps/topology/sbd2d.h>
#include <synaps/arithm/Interval.h>
//#include <synaps/arithm/interval/interval.h>
//--------------------------------------------------------------------
__BEGIN_NAMESPACE_SYNAPS
//--------------------------------------------------------------------
template<class X, int N>
void print_mx(const MatrDse<Interval<X,N> >& M)
{

  double dmx=0, imx=0,imn=1;
  for(unsigned i=0; i< M.nbrow();i++)
    for(unsigned j=0; j< M.nbcol();j++)
      {
	imx= std::max(imx,M(i,j).sup()-M(i,j).inf());
	imn= std::min(imn,M(i,j).sup()-M(i,j).inf());
	if( M(i,j).sup()<0)
	  dmx = std::max(dmx,-M(i,j).sup());
	else
	      dmx = std::max(dmx,M(i,j).sup());
	  }
      std::cout<<"DMax "<<dmx<<std::endl;
      std::cout<<"IMax "<<imx<<std::endl;
      std::cout<<"IMin "<<imn<<std::endl;
}

template<class X, int N>
void print_sign(const MatrDse<Interval<X,N> >& M)
{
  for(unsigned i=0;i<M.nbrow();i++)
    {
      std::cout<<"[";
      for(unsigned j=0; j<M.nbcol();j++)
      {
	if(M(i,j).sup()<0)
	  std::cout<<"-";
	else if(M(i,j).inf()>=0)
	  std::cout<<"+";
	else if(M(i,j).inf()<0 && M(i,j).sup()>0)
	  std::cout<<"?";
	else
	  std::cout<<" ";
	//std::cout<<M(i,j).sup()-M(i,j).inf()<<std::endl;
      }
      std::cout<<"]"<<std::endl;
    }
}
template<class Int>
bool IsSmall (const Int& I)
{
  return (I.inf()<= 0 && I.sup() >= 0 && (I.inf()!=I.sup()));
}

template<class X, int N>
int sign(const Interval<X,N>& I)
{
  if(I.sup()< (0))
    return -1;
  else if(I.inf()>(0))
    return 1;
  else if(I.sup()==0 && I.inf()==0)
    return 0;
  else
    return 2;
}
// template<class X, int N>
// int sign(const std2::interval<double>& I)
// {
//   if(I.sup()< (0))
//     return -1;
//   else if(I.inf()>(0))
//     return 1;
//   else if(I.sup()==0 && I.inf()==0)
//     return 0;
//   else
//     return 2;
// }

namespace bezier 
{
  template<class X>
  struct bzislv: public Isole<X> 
  {
    X eps;
    bzislv():eps(1e-4){}

    template<class UPOL>
    Seq<X> solve(const UPOL& p, X x0, X x1)
    {
      typedef typename UPOL::value_type coeff_t;
      bezier::rep1d<X> ubp(p.size()), dbp(p.size());
      for(unsigned i=0;i<p.size();i++)
	{
	  ubp[i]=p[i].sup();
	  dbp[i]=p[i].inf();
	  assert(dbp[i]<= ubp[i]);
	}
      this->Run(dbp,ubp,eps,false);
      Seq<X> sol; sol.resize(0);
      this->GetMidPoints(sol);
      for (unsigned i = 0; i < sol.size(); i ++ )
	{
	  if ( this->m_res[i].t )
	    std::cout<<" xxx"<<std::endl;
	}
      RConvert(sol,x0,x1);	       
      return sol;
    }
  };
  
 
}//namespace bezier
//--------------------------------------------------------------------
__END_NAMESPACE_SYNAPS
//====================================================================
#endif //synaps_subdivision_topology_H
