#!/bin/sh
N=20;
SLV="IslSturmQQ  SturmSeq IslBzIntegerZZ IslBzBdgSturmQQ SlvAberthQQ SolveCORE IslContFrac"
#IslBzStdDb IslBzBdgQQ  SolveCORE  SolveNiX SturmAberth

NAME="Rnd";

while test $# -gt 0
do
case $1 in
  -N)
      N=${2:?$usage}
      shift ;;
  -T)
      NAME=${2:?$usage}
      shift ;;
  -tex) 
      OUTPUT="latex color";EXT=".tex";;
  -ps) 
      OUTPUT="postscript color solid lw 3";EXT=".ps";;
  -pswb) 
      OUTPUT="postscript lw 3";EXT=".ps";;
  -pdf) 
      OUTPUT="pdf";EXT=".pdf";;
  -png) 
      OUTPUT="png";EXT=".png";;
  -pbm) 
      OUTPUT="pbm color";EXT=".pbm";;
  -clean)
    clean="yes";;
  -rm)
    clean="yes";
    remove="yes";;
esac
shift
done;

case $NAME in
  Mign)
    DATA="../../upol/data/mignotte";;
  Rnd)
    DATA="../../upol/data/random";;
  Int)
    DATA="../../upol/data/introots";;
  Rat)
    DATA="../../upol/data/ratroots";;
  BiMign)
    DATA="../../upol/data/mignotte2";;
  MuMign)
    DATA="../../upol/data/mignotte3";;
  *)
    DATA="unknown";;
esac;

if ! test -d ${DATA}; then 
    echo "Data ${DATA} not found"; 
    exit;
fi;
PREF=$NAME$N
if [ "$clean" ]; then 
    for slv in ${SLV}
    do
      rm ${slv}${PREF}.xml
    done;
fi;
if [ "$remove" ]; then exit; fi;

if [ "$OUTPUT" ]; then 
HEADER="set terminal $OUTPUT
set output \"$PREF$EXT\"
"; 
fi;
echo "Nb of bits: $N Tests: ${DATA} Pref: $PREF";

echo "$HEADER
set style line 3 lt 3 lw 4 pt 4 ps 1
set title \"${NAME}$N\" 
set xlabel \"Degree\" 
set ylabel \"Average time (s)\" 
set autoscale x
set autoscale y
set pointsize 0.6
plot 'zero'\\"> ${PREF}.gp;
for slv in ${SLV}
do
if test -f "./src/${slv}"; then 
   ext=""; 
else 
   ext=".ex"; 
fi;
echo "${slv}${PREF}.bch: ../src/${slv}${ext}" >bch/${slv}-${PREF}.d
if ! test -f ${slv}${PREF}.xml; then 
echo "<bench>
<define var=\"DATADIR\" val=\"../../upol/data/${DATA}\"/>
<define var=\"TOOL\" val=\"./src/${slv}${ext}\"/>
<define var=\"METHOD\" val=\"${slv}\"/>
for j in 03 04 05 06 07 08 09 10 11 12 13 14 15 16 17 18 19 20
do
 for i in $N
   do 
    <run tool=\"$METHOD\">\$TOOL \$DATADIR/d\${j}_n\${i}.dat</run>
   done
done
</bench>" > ${slv}${PREF}.xml;
fi;
echo ", 'gp/${slv}${PREF}.gp' using (\$1):(\$2/100) title \"${slv}\"  with linesp  lw 2 ps 3 pt 13 \\">>${PREF}.gp;
done
echo "

pause -1 \"OK?\"">>${PREF}.gp
make run
gnuplot ${PREF}.gp;
