#include <iostream>
#define USING_NAMESPACE_SYNAPS 1

#include "synaps/util/time.H"
#include "synaps/upol.H"
#include "synaps/solve/bezier/Std.H"
#include "synaps/solve/bezier/Isolation.H"
#include "synaps/solve/Aberth.H"
#include "synaps/solve/Exclusion.H"
#include "synaps/solve/Algebraic.H"


using std::cin;
using std::cout;
using std::endl;

typedef double NT;

int main()
{ 
  using namespace std;
  UPolDse<NT> P("2*x0^7+3*x0^6+2*x0^5+1*x0^4-1*x0^3-2*x0^2-3*x0+1");
  
  unsigned N=1000, nr=0;
  double t = time();
  for ( unsigned i = 0; i < N; i++ )
    { 
      RootOf(P,Bezier<NT,Rkw>(1e-6)); 
    }
  t = time()-t;
  cout << "<method>Rockwood</method> ";
  cout << "<nb_root>"<<N<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(N/t)<<"</speed>"<<endl;

  t = time();
  for ( unsigned i = 0; i < N; i++ ) Isole(P,Bezier<NT,Isolation>(1.e-6));  
  t = time()-t;
  cout << "<method>1st Bezier</method> ";
  cout << "<nb_root>"<<N<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(N/t)<<"</speed>"<<endl;


//   t = time();
//   for ( unsigned i = 0; i < N; i++ ) RootOf(P,2,Algebraic<NT>(1.e-6));  
//   t = time()-t;
//   cout << "<method>2nd Bezier</method> ";
//   cout << "<nb_root>"<<N<<"</nb_root> ";
//   cout << "<speed>"<<(N/t)<<"</speed>"<<endl;


 //  t = time();
//   for (unsigned i = 0; i < N; i++ ) Solve(P,Bezier<NT,Opt>(1.e-6));
//   t = time()-t;
//   cout << "Opt debit :" << (N/t) << endl;

  nr=0; t = time();
  for (unsigned i = 0; i < N; i++ ) 
    nr+=Solve(P, Algebraic<NT>(1.e-6)).size();
  t = time()-t;
  cout << "<method>Isolation</method> ";
  cout << "<nb_root>"<<nr<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(nr/t)<<"</speed>"<<endl;

  nr=0; t = time(); 
  for ( unsigned i = 0; i < N; i++ ) 
    nr+=Solve(P,Bezier<NT>()).size();  
  t = time()-t;
  cout << "<method>Bezier at eps</method> ";
  cout << "<nb_root>"<<nr<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(nr/t)<<"</speed>"<<endl;

  nr=0;  t = time();
  for (unsigned i = 0; i < N/4; i++ ) 
    nr+=Solve(P,-2.,2.,Exclusion(1e-6)).size();
  t = time()-t;
  cout << "<method>Exclusion</method> ";
  cout << "<nb_root>"<<nr<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(nr/t)<<"</speed>"<<endl;

#ifdef SYNAPS_WITH_MPSOLVE
  nr=0; t = time();
  for ( unsigned i = 0; i < N; i++ ) 
    nr+=Solve(P,Aberth<RR>()).size();  
  t = time()-t;
  cout << "<method>Aberth</method> ";
  cout << "<nb_root>"<<nr<<"</nb_root> ";
  cout << "<time>"<<t<<"</time>";
  cout << "<speed>"<<(nr/t)<<"</speed>"<<endl;
#endif

};

