#ifndef SYNAPS_UPOL_MULT_FFT_H
#define SYNAPS_UPOL_MULT_FFT_H
//======================================================================
#include <synaps/init.h>
#include <synaps/arithm/RealOf.h>
#include <synaps/linalg/FFT.h>
//======================================================================

__BEGIN_NAMESPACE_SYNAPS


namespace UPOL
{
/**
   Multiplication of a specialized polynomial by an other specialized
   polynomial. Result,  given by the first parameter r,  is a polynomial
   of the same type as the two polynomials.
   Method of multiplication is fft.
   We assume that R is a valid class of container of polynomial.
**/
template<class R, class TA, class TB>
void mul_fft(R & r, const TA & a, const TB & b)
{
  #ifdef VERBATIM
	std::cout <<"Using fft for the multiplication of polynomials"<<std::endl;
  #endif

  typedef typename TA::value_type C;
  typedef std::complex<typename RealOf<C>::TYPE> Cpl;
  unsigned long degree = degree(a) + degree(b);
  unsigned long N = (int) pow(2, 1 + (int) (log(degree)/log(2)));
  Cpl  w ;

  Root_Unity(w, N);

  Cpl * XA = new Cpl[N];
  Cpl * XB = new Cpl[N];

  for (long int i = 0; i <= degree(a) ; i++)
    XA[i] = a[i];

  // Bit reversal permutation, to allow Butterfly algorithm to calculate FFT
  // on the two polynomials coefficients.

  Permut(XA, N);
  FFT(N, w, XA,0);

  for (long int i = 0; i <= degree(b) ; i++)
    XB[i] = b[i];

  Permut(XB, N);
  FFT(N, w, XB, 0);

  // multiplication of FFT(polynomial a) by FFT(polynomial b)
  for (unsigned long k = 0; k < N; k++)
    XA[k] *= XB[k] ;

  // result of this multiplication is also permuted for the same reason
  Permut(XA, N);

  // Backward FFT , to retrieve the result of multiplication
  // of the two polynomials
  FFT(N, pow(w,-1), XA, 1);

  for (unsigned long k = 0; k <= degree; k++)
    RealOf<C>::assign(r[k], (XA[k]/N));

  delete [] XA;
  delete [] XB;
}
//----------------------------------------------------------------------
}//UPOL

__END_NAMESPACE_SYNAPS

#endif // SYNAPS_UPOL_MULT_FTT_H

