<TeXmacs|1.0.6>

<style|article>

<\body>
  <section|Real algebraic numbers >

  <\with|par-mode|right>
    <name|E. Tsigaridas>
  </with>

  \;

  A real algebraic number is a real root of a polynomial, whose coefficients
  are integers. We represent such a real algebraic number, by a square-free
  polynomial and an isolating interval, that is an interval with rational
  endpoints, which contains only one root of the polynomial:

  <\eqnarray*>
    <tformat|<table|<row|<cell|\<alpha\> \<equiv\><left|(>f(X),
    [a,b])>|<cell|>|<cell|>>|<row|<cell|f \<in\>
    \<bbb-Z\>[X]>|<cell|>|<cell|>>|<row|<cell|a, b
    \<in\>\<bbb-Q\>>|<cell|>|<cell|>>>>
  </eqnarray*>

  <subsection|The class for computing algebraic numbers >

  The class which specifies the construction of algebraic number is
  parameterised by a number type <verbatim|NT>, corrresponding to the output
  type used to represent the algebraic numbers. It containts the following
  definitions:

  <\code>
    template \<less\> typename NT \<gtr\>

    struct Algebraic\ 

    {

    \ \ typedef NumberTraits\<less\>NT\<gtr\> \ \ NTR; \ \ \ \ \ \ \ \ //
    Number type traits\ 

    \;

    \ \ typedef typename NTR::RT \ \ RT; \ \ \ \ \ \ \ \ \ // Ring number
    type (typically <with|mode|math|\<bbb-Z\>)>

    \ \ typedef typename NTR::FT \ \ FT; \ \ \ \ \ \ \ \ \ // Field number
    type (typically <with|mode|math|\<bbb-Q\>)>

    \ \ typedef typename NTR::XT \ \ XT; \ \ \ \ \ \ \ \ \ // Approximate
    number type\ 

    \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ //
    (typically double)

    \ \ typedef typename NTR::XIT RIT; \ \ \ \ \ \ \ \ \ // Ring interval
    type

    \ \ typedef typename NTR::FIT FIT; \ \ \ \ \ \ \ \ \ // Field interval
    type

    \ \ typedef typename NTR::XIT XIT; \ \ \ \ \ \ \ \ \ // Approximate
    interval type

    \;

    \ \ typedef UPolDse\<less\>RT\<gtr\> \ \ \ \ \ \ Poly; \ \ \ \ \ \ \ \ //
    Univariate polynomial type

    \ \ typedef UPolDse\<less\>RT\<gtr\> \ \ \ \ \ \ RT_Poly; \ \ \ \ \ //
    Ring polynomial type

    \ \ typedef UPolDse\<less\>FT\<gtr\> \ \ \ \ \ \ FT_Poly; \ \ \ \ \ //
    Field polynomial type

    \ \ typedef UPolDse\<less\>XT\<gtr\> \ \ \ \ \ \ XT_Poly; \ \ \ \ \ //
    Approximate polynomial type

    \ \ typedef UPolDse\<less\>RL\<gtr\> \ \ \ \ \ \ RL_Poly; \ \ \ \ \ //
    Real polynomial type

    \ \ typedef ALGEBRAIC::root_of\<less\>RT, Poly\<gtr\> root_t; \ // The
    root_of type

    };
  </code>

  See: <verbatim|synaps/usolve/Algebraic.h>.

  <subsection|Construction of algebraic numbers>

  The first way to define an algebraic number is to solve a polynomial, using
  the algebraic solver:

  <\code>
    \ \ \ Seq\<less\> Algebraic\<less\>ZZ\<gtr\>::root_t \<gtr\>\ 

    \ \ \ \ \ \ \ \ \ solve(const UPolDse\<less\>ZZ\<gtr\> & P,
    Algebraic\<less\>ZZ\<gtr\> mth);
  </code>

  This returns a sequence of real algebraic numbers that are the roots of the
  polynomial <verbatim|P>. \ If one is interested by a specific root, one can
  also use the following instructions:\ 

  <\code>
    \ \ \ Algebraic\<less\>ZZ\<gtr\>::root_t\ 

    \ \ \ \ \ \ \ \ \ solve(const UPolDse\<less\>ZZ\<gtr\> & P,
    Algebraic\<less\>ZZ\<gtr\> mth, int i);
  </code>

  which computes the <with|mode|math|i<rsup|th>> root of <verbatim|P>, if it
  exists. Otherwise, it produces an error. For polynomials of degree up to 4,
  we use precomputed discrimination systems in order to determine the
  square-free polynomial and we compute the isolating interval as function in
  the coefficients of the polynomial.\ 

  For polynomials of higher degree, we use a Sturm-like solver in order to
  isolate the roots of the polynomial.

  <subsection|Operations on real algebraic numbers>

  Currently, the available operations for the algebraic numbers are

  <\itemize-dot>
    <item>comparison: operators <verbatim|\<less\>,\<gtr\>, \<less\>=,
    \<gtr\>=, ==, !=>

    <item>sign of a polynomial <with|mode|math|P \<in\>\<bbb-Z\>[x]> at a
    real algebraic number: <verbatim|sign_at(P,alpha)>.
  </itemize-dot>

  Example: <verbatim|Algebraic.cc>\ 

  \;
</body>

<\initial>
  <\collection>
    <associate|language|english>
    <associate|page-bot|1in>
    <associate|page-even|1in>
    <associate|page-odd|1in>
    <associate|page-right|1in>
    <associate|page-top|1in>
    <associate|page-type|letter>
    <associate|par-width|6.5in>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|1.1|?>>
    <associate|auto-3|<tuple|1.2|?>>
    <associate|auto-4|<tuple|1.3|?>>
    <associate|auto-5|<tuple|1.4|?>>
    <associate|toc-1|<tuple|1|?>>
    <associate|toc-2|<tuple|1.1|?>>
    <associate|toc-3|<tuple|1.2|?>>
    <associate|toc-4|<tuple|2.2|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|1<space|2spc>Real
      algebraic numbers > <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|1.1<space|2spc>The class for computing
      algebraic numbers \ <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2>>

      <with|par-left|<quote|1.5fn>|1.2<space|2spc>Construction of algebraic
      numbers <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|1.5fn>|1.3<space|2spc>Operations on real
      algebraic numbers <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>
    </associate>
  </collection>
</auxiliary>