#ifndef SYNAPS_ARITHM_NUMBER_TRAITS_H
#define SYNAPS_ARITHM_NUMBER_TRAITS_H

#include <synaps/init.h>
#include <synaps/base/type.h>
#include <synaps/arithm/Abs.h>
#include <synaps/arithm/pow.h>
#include <synaps/arithm/inv.h>
#include <synaps/arithm/sign.h>
#include <synaps/arithm/traits.h>
#include <synaps/arithm/functions.h>
#include <synaps/arithm/Interval.h>

#include <synaps/arithm/gmp.h>

//#include <synaps/numerics/Various_defs.h>

__BEGIN_NAMESPACE_SYNAPS


/** This a prototype class. 
    Every number type must provide a specialization of this struct,
    and specializations for the functions.
    This class provides the following definitions for the type @RT_@
	\param RT is the type @RT_@ itself. (eg. @ZZ@).
	\param FT is the field type associated to @RT@ (eg. @QQ@).
	\param XT is the numeric type associated to @RT@ (eg. @double@).
	\param FIT is the type of exact intervals (eg. @Interval<QQ>@).
	\param XIT is the type of approximate intervals (eg. @Interval<double>@).
	
	It also has various tags, in order to provide information abou the presense or not of 
	@gcd@, @isqrt@, @sqrt@ etc.
	\sa Rational, Interval, check_type, Tag_false, Tag_true
**/

template < typename RT_ >
struct NumberTraits {
  typedef typename kernelof< RT_ >::T             K;
  typedef typename K::integer 	                  RT; 
  typedef typename K::rational                    FT;
  typedef typename K::ieee                        XT;
  typedef Interval< FT >                          FIT;
  typedef Interval< XT >                          XIT;
  typedef NumberTraits<RT>                        self_t;
  
  typedef typename hasfunction<RT>::gcd Has_gcd;
  typedef typename hasfunction<RT>::isqrt Has_isqrt;
  typedef typename hasfunction<RT>::sqrt Has_sqrt;
  
  typedef typename isexact<RT>::T          Is_exact;
  typedef typename isring<RT>::T          Is_Ring_type;
  typedef typename isfield<RT>::T         Is_Field_type;
};

template<> 
struct NumberTraits<double> : NumberTraits<GMP::integer> 
{ 
};



/** 
 * \brief Computes the integer square root of a number
 * 
 * \param a 
 * 
 * \return the @isqrt@ 
 */
template < typename T >  inline T
isqrt(const T& a)
{
  assert (a > 0);
  return T(std::floor(std::sqrt(to_double(a))));
};



__END_NAMESPACE_SYNAPS

#endif // SYNAPS_NUMERICS_NUMBER_TRAITS_H

