#ifndef _synaps_solve_Sturm_h_
#define _synaps_solve_Sturm_h_

#include <synaps/init.h>
#include <synaps/arithm/Interval.h>
#include <synaps/arithm/traits.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/usolve/Small_degree.h>
#include <synaps/usolve/sturm/common.h>
#include <synaps/usolve/sturm/solve_sturm.h>

__BEGIN_NAMESPACE_SYNAPS


template < class NT >
struct Sturm : public Small_degree<NT>
{
    typedef Sturm          self_t;
};


/**  Solve the univariate polynomial using Sturm's sequence.
 **/
template < class NT > inline
Seq< typename Sturm<NT>::RO_t > 
solve( const typename Sturm<NT>::Poly& f,
       Sturm<NT>)
{
    typedef Sturm<NT>          K;
    typedef typename K::RO_t          RO_t;
    
    Seq<RO_t> sol;
   
    ALGEBRAIC::solve_sturm(f, std::back_inserter(sol.rep()), K());

    std::stable_sort( sol.begin(), sol.end(), ALGEBRAIC::Refine_compare());
    return sol;
}


template < class NT,
	   class OutputIterator> inline
OutputIterator
solve( const typename Sturm<NT>::Poly& f,
       OutputIterator it,
       Sturm<NT>)
{
    typedef Sturm<NT>          K;

    return ALGEBRAIC::solve_sturm(f, it, K());
}



template < class NT >  inline
typename Sturm<NT>::RO_t 
RootOf( const typename Sturm<NT>::Poly& f, unsigned i, Sturm<NT>)
{
   typedef Sturm<NT>          K;
   return solve( f, K())[i];
}



template < class NT > inline
int
Count_real_roots( const typename Sturm<NT>::Poly& f, Sturm<NT>)
{
    typedef Sturm<NT>          K;
    typedef typename K::RT            RT;
    typedef typename K::FT            FT;
    typedef typename K::FIT           FIT;

    SturmSeq<RT> sth( f, UPOLDAR::diff(f), SUBRES());
    FT b = Bound( f, Cauchy());
    FT a = -b;

    return ( Var( sth, a) - Var( sth, b) );
}


__END_NAMESPACE_SYNAPS

#endif // _SYNAPS_SOLVE_STURM_H_

