
#ifndef SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMMON_H
#define SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMMON_H

#include <synaps/usolve/algebraic/Algebraic_config.h>


__ALGEBRAIC_BEGIN_NAMESPACE

// Forward declaration
template < typename RT, typename P > struct root_of;
//-------------------------------------------------------------------------------

template < typename RT,
           typename POLY >   inline
int
decide_common_interval(
		       const root_of<RT, POLY>&  a,
		       const root_of<RT, POLY>&  b,
		       typename NumberTraits<RT>::FIT& I,
		       int& where
		       )
{
  typedef typename NumberTraits<RT>::FT     FT;


  if (a.interval() > b.interval()) return LARGER;
  if (a.interval() < b.interval()) return SMALLER;
  I = intersection(a.interval(), b.interval());

  if (a.sign_up()  == POSITIVE) where ^= 1;
  if (a.sign_low() == POSITIVE) where ^= 2;
  if (b.sign_up()  == POSITIVE) where ^= 4;
  if (b.sign_low() == POSITIVE) where ^= 8;

  FT new_right = b.right();
  FT Dp2 = a.right() - b.right();
  if (Dp2 > FT(0)) {
    new_right = b.right();
    int sgn_a_right = sign( UPOLDAR::eval_horner(a.poly(), new_right) );
    if ((sgn_a_right == a.sign_low()) || (sgn_a_right == ZERO)) {
      return LARGER;
      // if ZERO change the representation of a
    }
  } else if (Dp2 < FT(0)){
    new_right = a.right();
    int sgn_b_right = sign( UPOLDAR::eval_horner(b.poly(), new_right) );
    if ((sgn_b_right == b.sign_low()) || (sgn_b_right == ZERO)) {
      return SMALLER;
      // if ZERO change the representation of b
    }
  }

  // The right isolating point is new_right
  // for both a and b NOW!!!
  a.set_interval(a.left(), new_right);
  b.set_interval(b.left(), new_right);

  FT new_left = b.left();
  FT Dp1 = a.left() - b.left();
  if (Dp1 > FT(0)) {
    new_left = a.left();
    int sgn_b_left = sign( UPOLDAR::eval_horner(b.poly(), new_left) );
    if ((sgn_b_left == b.sign_up()) || (sgn_b_left == ZERO)) {
      return LARGER;
    }
  } else if (Dp1 < FT(0)) {
    new_left = b.left();
    int sgn_a_left = sign( UPOLDAR::eval_horner(a.poly(), new_left) );
    if ((sgn_a_left == a.sign_up()) || (sgn_a_left == ZERO)) {
      return SMALLER;
    }
  }
  // The left isolating point is new_right
  // for both a and b NOW!!!
  a.set_interval(new_left, a.right());
  b.set_interval(new_left, b.right());

  I = a.interval();
  if (singleton(I)) return EQUAL;
  
  //if (r_I1 == r_I2) { }
  return UNKNOWN;
}




__ALGEBRAIC_END_NAMESPACE


#endif // SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMMON_H

