// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)



#ifndef SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARE_43_HH
#define SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARE_43_HH



#include <synaps/usolve/algebraic/Algebraic_config.h>
#include <synaps/usolve/algebraic/comparison/Common.h>


__ALGEBRAIC_BEGIN_NAMESPACE

// Forward declaration
template < typename RT, typename P > struct root_of;
//-------------------------------------------------------------------------------

// Evaluation of the  full Sturm sequence
template< typename RO_t>   inline
int
eval_full_Sturm_43_pq(
		const RO_t& a,
		const RO_t& b,
		const typename RO_t::FT& isol_1,
		const typename RO_t::FT& isol_2,
		int& where
		)
{
	typedef typename RO_t::NTR   NTR;
	typedef typename RO_t::RT    RT;
	typedef typename RO_t::FT    FT;

	RT a1 = a[4];
	RT b1 = a[3];
	RT c1 = a[2];
	RT d1 = a[1];
	RT e1 = a[0];
	RT a2 = b[3];
	RT b2 = b[2];
	RT c2 = b[1];
	RT d2 = b[0];
//	RT a1 = a.C4(); RT b1 = a.C3();  RT c1 = a.C2(); RT d1 = a.C1();  RT e1 = a.C0();
//	RT a2 = b.C3(); RT b2 = b.C2();  RT c2 = b.C1(); RT d2 = b.C0();

	RT p1 = numerator(isol_1);
	RT q1 = denominator(isol_1);
	RT p2 = numerator(isol_2);
	RT q2 = denominator(isol_2);

	int result;
	RT     J  = calc_brackets(a1, b2, a2, b1);
	RT     G  = calc_brackets(a1, c2, a2, c1);
	RT    S22 = calc_brackets(a2, G, b2, J);
	RT S2_inf = S22;
	if (S2_inf == RT(0)) {
		return eval_deg_Sturm_43_pq(
				a1, b1, c1, d1, e1,
				a2, b2, c2, d2,
				J, G,
				p1, q1, p2, q2,
				where
				);

	}
	RT     M = calc_brackets(a1, d2, a2, d1);
	RT   S21 = calc_brackets(a2, M, c2, J);
	RT    K1 = calc_43_K1(b1, d2, e1, a2, d1, b2);
	RT   S20 = calc_brackets(a2, K1, b2, M);
	RT S2_p2q2 = calc_quadratic(S22, S21, S20, p2, q2);
	if (S2_p2q2 > RT(0)) where ^= 16;
	RT S2_p1q1 = calc_quadratic(S22, S21, S20, p1, q1);
	if (S2_p1q1 >= RT(0)) where ^= 32;

	RT     K2 = calc_43_K2(c1, c2, b2, d1, b1, d2, a2, e1);
	RT    S31 = calc_43_S31(a1, b1, a2, c2, S22, S21, S20, J, G, M, K2);
	RT S3_inf = S31;
	if (S3_inf == RT(0)) {
		RT  Jp = calc_brackets(b1, c2, b2, c1);
		RT S30 = calc_43_S30(e1, a2, b2, d2, S22, S20, G, Jp, M);
		S3_inf = S30;
		if (S3_inf == RT(0))
			return Result_deg_3::on_end(where);
		if (S3_inf > RT(0)) { where ^= 64; where ^= 128;}
		return Result_deg_4::on_end(where);
	}

	RT    Jp = calc_brackets(b1, c2, b2, c1);
	RT   S30 = calc_43_S30(e1, a2, b2, d2, S22, S20, G, Jp, M);

	RT S3_p2q2 = calc_one(S31, S30, p2, q2);
	if (S3_p2q2 >= RT(0)) where ^= 64;
	result = Static_Result::Result_5.on_S3_p2q2(where);
	if (result != UNKNOWN) {
		return result;
	}

	RT S3_p1q1 = calc_one(S31, S30, p1, q1);
	if (S3_p1q1 >= RT(0)) where ^= 128;
	result = Static_Result::Result_5.on_S3_p1q1(where);
	if (result != UNKNOWN) {
		return result;
	}

	RT S4 = calc_43_S4(S22, S21, S20, S31, S30);
	if (S4 == RT(0)) return EQUAL;
	if (S4 > RT(0)) where ^= 256;
	result = Static_Result::Result_5.on_S4(where);
	if (result != UNKNOWN) {
		return result;
	}
	return EQUAL;
}
//-------------------------------------------------------------------------------



// Evaluation of DEGENERATE Sturm sequence
// (degenerate = the first remainder is of degree 1)
// when the second endpoint of the common interval is infinity
template < class RT >
int
eval_deg_Sturm_43_pq(
		const RT& a1,  const RT& b1,  const RT& c1,  const RT& d1,  const RT& e1,
		const RT& a2,  const RT& b2,  const RT& c2,  const RT& d2,
		const RT& J,   const RT& G,
		const RT& p1,  const RT& q1, const RT& p2,  const RT& q2,
		int& where
		)
{
	int result;
	RT      M = calc_brackets(a1, d2, a2, d1);
	RT    S21 = calc_brackets(a2, M, c2, J);
	RT S2_inf = S21;
	if (S2_inf == RT(0)) {
		RT  K1 = calc_43_K1(b1, d2, e1, a2, d1, b2);
		S2_inf = calc_brackets(a2, K1, b2, M);
		if (S2_inf == RT(0)) return EQUAL;
		if (S2_inf > RT(0)) { where ^= 16; where ^= 32; }
		return Result_deg_3::on_end(where);
	}
	RT    K1 = calc_43_K1(b1, d2, e1, a2, d1, b2);
	RT   S20 = calc_brackets(a2, K1, b2, M);

	RT S2_p2q2 = calc_one(S21, S20, p2, q2);
	if (S2_p2q2 >= RT(0)) where^= 16;

	RT S2_p1q1 = calc_one(S21, S20, p1, q1);
	if (S2_p1q1 >= RT(0)) where^= 32;
	result = Static_Result::Result_4.on_S2_p1q1(where);
	if (result != UNKNOWN) {
		return result;
	}

	RT S3 = calc_43_deg_S3(a2, b2, c2, d2, S21, S20);
	if (S3 == RT(0)) return EQUAL;
	if (S2_inf > RT(0)) S3 = -S3;
	if (S3 > RT(0)) where ^= 64;
	result = Static_Result::Result_4.on_S3(where);
	if (result != UNKNOWN) {
		return result;
	}
	return EQUAL;
}
//-------------------------------------------------------------------------------

template < typename RO_t >  inline
int
compare_43(const RO_t& a, const RO_t& b)
{
	typedef typename RO_t::NTR    NTR;
	typedef typename RO_t::RT     RT;
	typedef typename RO_t::FT     FT;
	typedef typename RO_t::FIT    FIT;

	int where = 0;
	FIT I;
	int result = decide_common_interval(a, b, I, where);
	if (result != UNKNOWN)
		return result;

	return eval_full_Sturm_43_pq(a, b, I.lower(), I.upper(), where) * b.sign_up();
}
//-------------------------------------------------------------------------------


template < typename RO_t >  inline
int
compare_34(const RO_t& a, const RO_t& b)
{
	return opposite(compare_43(b, a));
}
//-------------------------------------------------------------------------------




template< class RT >
	inline
RT calc_43_K1(const RT& b1, const RT& d2, const RT& e1, const RT& a2, const RT& d1, const RT& b2)
{ return d2 * b1 - e1 * a2 - d1 * b2; }


template< class RT >
inline
RT calc_43_K2(const RT& c1, const RT& c2, const RT& b2, const RT& d1, const RT& b1, const RT& d2,
		const RT& a2, const RT& e1)
{ return c1 * c2 - b2 * d1 + b1 * d2 - a2 * e1; }


template< class RT >
inline
RT calc_43_S31(
		const RT& a1,  const RT& b1,  const RT& a2,  const RT& c2,
		const RT& S22, const RT& S21, const RT& S20,
		const RT& J,   const RT& G,   const RT& M,   const RT& K2)
{ return S22 * K2 - M*(RT(2) * S21 - a2 * M) + SQR(c2)*(b1 * J -a1 * G); }


template< class RT >
inline
RT calc_43_S30(
		const RT& e1,  const RT& a2,  const RT& b2, const RT& d2,
		const RT& S22, const RT& S20,
		const RT& G,   const RT& Jp,  const RT& M)
{ return -S20 * (Jp + M) - b2 * e1 * (S22 + a2 * G) - d2 * SQR(G); }


template< class RT >
inline
RT calc_43_S4( const RT& S22, const RT& S21, const RT& S20,
		const RT& S31, const RT& S30)
{ return -SQR(S31) * S20 + S30 * (S21 * S31 - S30 * S22); }




template< class RT >
inline
RT calc_43_deg_S3(const RT& a2,  const RT& b2,  const RT& c2, const RT& d2,
		const RT& S21, const RT& S20)
{ return SQR(S21)*(d2 * S21 - c2 * S20) + SQR(S20)*(S21 * b2 - a2 * S20); }



__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_ARITHM_ALGEBRAIC_COMPARISON_COMPARE_43_HH

