// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     University of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)


#ifndef SYNAPS_ARITHM_ALGEBRAIC_SIGN_SIGN_33_H
#define SYNAPS_ARITHM_ALGEBRAIC_SIGN_SIGN_33_H

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>

__ALGEBRAIC_BEGIN_NAMESPACE


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_degJ1_sturm_33(
		const RT& a1, const RT& b1, const RT& c1, const RT& d1,
		const RT& a2, const RT& b2, const RT& c2, const RT& d2,
		const FT& a,  const FT& b,
		int& where)
{
  typedef NumberTraits<RT>    NTR;

	print("where", where);
	print(__FUNCTION__);
	Comparison_result result;

	RT G = calc_brackets(a1, c2, a2, c1);
	RT M = calc_brackets(a1, d2, a2, d1);
	if (G == RT(0)) {
		if (M == RT(0)) return Result_deg_2::on_end(where);
		if (M > RT(0)) { where ^= 16; where ^= 32; }
		return Result_deg_3::on_end(where);
	}
	POLY S2(2, AsSize());
	S2[1] = G;
	S2[0] = M;

	if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
	result = Static_Result::Result_4.on_S2_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
	result = Static_Result::Result_4.on_S2_p1q1(where);
	if (result != UNKNOWN) return result;

	RT P = calc_brackets(c1, b2, c2, b1);
	RT K = P - M;
	RT L = calc_brackets(d1, b2, d2, b1);
	RT Q = calc_brackets(d1, c2, d2, c1);

	RT S3 =  M * (G * L - K * M) + G * (L * M - G * Q);
	if (S3 ==  RT(0)) return EQUAL;
	if (sign(G)*S3 > RT(0)) where ^= 64;
	result = Static_Result::Result_4.on_S3(where);
	if (result != UNKNOWN) return result;
	return EQUAL;
}


template < typename POLY,
		   typename RT,
		   typename FT >  inline
Comparison_result
static_full_sturm_33(
		const RT& a1, const RT& b1, const RT& c1, const RT& d1,
		const RT& a2, const RT& b2, const RT& c2, const RT& d2,
		const FT& a,  const FT& b,
		int& where)
{
  typedef NumberTraits<RT>    NTR;

	print(__FUNCTION__);
	Comparison_result result;

	RT J1 = b2 * a1 - a2 * b1;
	if (J1 == RT(0)) {
		return static_degJ1_sturm_33<POLY>(
			a1, b1, c1, d1,
			a2, b2, c2, d2,
			a, b,
			where);
	}
	RT t2 = d2 * a1 - a2 * d1;
	RT J2 = c2 * a1 - a2 * c1;
	RT J3 = t2;
	POLY S2(3, AsSize());
	S2[2] = J1;
	S2[1] = J2;
	S2[0] = J3;

//	cout << "poly S2 = "; UPOLDAR::print(cout, S2); 
//	cout << "degree S2 = " << UPOLDAR::degree(S2) << endl;
//	if (UPOLDAR::degree(S2) <= 0) return EQUAL;

	if (sign(UPOLDAR::eval_horner(S2, b)) >= 0) where ^= 16;
	print("S2 on b",  sign(UPOLDAR::eval_horner(S2, b)));
	result = Static_Result::Result_5.on_S2_p2q2(where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S2, a)) >= 0) where ^= 32;
	print("S2 on a", sign(UPOLDAR::eval_horner(S2, a)));
	result = Static_Result::Result_5.on_S2_p1q1(where);
	if (result != UNKNOWN) return result;


	RT t4 = J3 * J2;
	RT J5 = d2 * b1 - b2 * d1;
	RT t3 = J1 * J5;
	RT t1 = J2 * J2;
	RT J4 = -b2 * c1 + c2 * b1 + t2;
	POLY S3(2, AsSize());
	S3[1] = J1 * J4 - t1;
	S3[0] = t3 - t4;

//	cout << "poly S3 = "; UPOLDAR::print(cout, S3);
//	cout << "degree S3 = " << UPOLDAR::degree(S3) << endl;
//	if ((S3[1] == 0) && (S3[0] == 0))
	if ((UPOLDAR::degree(S3) <= 0) && (S3[0] == RT(0)))
		return Result_deg_3::on_end(where);

	if (sign(UPOLDAR::eval_horner(S3, b)) >= 0) where ^= 64;
	result = Static_Result::Result_5.on_S3_p2q2(where);
	print("where", where);
	if (result != UNKNOWN) return result;
	if (sign(UPOLDAR::eval_horner(S3, a)) >= 0) where ^= 128;
	print("S3 on a", sign(UPOLDAR::eval_horner(S3, a)));
	result = Static_Result::Result_5.on_S3_p1q1(where);
	if (result != UNKNOWN) return result;


	RT J6 = d2 * c1 - c2 * d1;
	RT S4 = -t1 * J6 + (2 * t4 - t3) * J5 + (J1 * J6 - J3 * J3) * J4;

	if (S4 ==  RT(0)) return EQUAL;
	if (S4 > RT(0)) where ^= 256;
	result = Static_Result::Result_5.on_S4(where);
	if (result != UNKNOWN) return result;
	return EQUAL;
}


template < typename POLY,
           typename RO_t >  inline
int
sign_at_33(const POLY& f, const RO_t& x)
{
	typedef typename RO_t::NTR    NTR;
	typedef typename NTR::RT      RT;
	typedef typename NTR::FT      FT;

	print(__FUNCTION__);
	int where = 0;
	in_isolating_interval(f, x, where);
	return static_full_sturm_33<POLY>(
			x[3], x[2], x[1], x[0],
			f[3], f[2], f[1], f[0],
			x.left(),
			x.right(),
			where);
}



template < typename POLY,
           typename RO_t >  inline
int
sign_at_32(const POLY& f, const RO_t& x)
{
	typedef typename RO_t::NTR    NTR;
	typedef typename NTR::RT      RT;
	typedef typename NTR::FT      FT;

	print(__FUNCTION__);
	int where = 0;
	in_isolating_interval(f, x, where);
	return static_full_sturm_33<POLY>(
			RT(0), x[2], x[1], x[0],
			f[3], f[2], f[1], f[0],
			x.left(),
			x.right(),
			where);
}


template < typename POLY,
           typename RO_t >  inline
int
sign_at_23(const POLY& f, const RO_t& x)
{
	typedef typename RO_t::NTR    NTR;
	typedef typename NTR::RT      RT;
	typedef typename NTR::FT      FT;

	print(__FUNCTION__);
	int where = 0;
	in_isolating_interval(f, x, where);
	print("where", where);
	return static_full_sturm_33<POLY>(
			x[3], x[2], x[1], x[0],
			RT(0), f[2], f[1], f[0],
			x.left(),
			x.right(),
			where);
}

__ALGEBRAIC_END_NAMESPACE

#endif // SYNAPS_ARITHM_ALGEBRAIC_SIGN_SIGN_33_H

