/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: Integer.h,v 1.2 2005/10/04 12:30:23 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_IsoleInteger_H
#define synaps_solve_IsoleInteger_H
/********************************************************************/
#include <synaps/init.h>
#include <synaps/usolve/bezier/integer.h>
//====================================================================
__BEGIN_NAMESPACE_SYNAPS
//====================================================================
template<class O, class X=double>
struct SlvBzInteger
{
  typedef O solution_t;
  X eps;
  SlvBzInteger():eps(1.e-6) {}
  SlvBzInteger(const X& e):eps(e) {}
  SlvBzInteger(const SlvBzInteger<O,X> & mth): eps(mth.eps) {}

};
//====================================================================
/* UPOL: view on an univariate polynom                 */
/*    C: number type used in computation               */
/*    O: type of result                                */
template< class UPOL, class O, class X >
Seq<O> solve( const UPOL& p, const SlvBzInteger<O,X>& mth )
{
  Seq<O> sol;
  bezier::integer_appendsol( sol, p.rep(), mth, false );
  return  sol;
};

//====================================================================
/* UPOL: view on an univariate polynom                 */
/*    C: number type used in computation               */
/*    O: type of result                                */
template< class UPOL, class O, class X, class U >
Seq<O> solve( const UPOL& r, const SlvBzInteger<O,X>& mth, const U& a, const U& b)
{
  typedef typename UPOL::coeff_t coeff_t;
  typedef typename arithm::rationalof<coeff_t>::T rational_t;

  bezier::rep1d<rational_t> bpr(r.size()); // TC->C
  using let::assign;

  assign(bpr,r,a,b);
  
  coeff_t denom=1;
  for ( unsigned i = 0; i < bpr.size(); i++ ) 
    assign(denom,lcm(denom,denominator(bpr[i]))) ;
  
  bezier::rep1d<coeff_t> bp(r.size()); 
  for ( unsigned i = 0; i < bpr.size(); i++ ) 
    assign(bp[i],numerator(rational_t(bpr[i]*denom)));
  //  VECTOR::print(std::cout,bp)<<std::endl;
    
  bezier::IsoleInteger<coeff_t> isole;
  isole.Run( bp,mth.eps,false);

  Seq<O> sol;
  isole.GetMidPoints( sol );
  RConvert(sol, a, b);
  return  sol;
};
//====================================================================
__END_NAMESPACE_SYNAPS 
/********************************************************************/
#endif // synaps_solve_IsoleInteger_H

