/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): B. Mourrain, GALAAD, INRIA
 *   $Id: SymNum.h,v 1.1 2005/09/28 06:30:42 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_BzIsoleSymNum_H
#define synaps_solve_BzIsoleSymNum_H
/********************************************************************/
#include <synaps/usolve/BzIsole.h>
__BEGIN_NAMESPACE_SYNAPS 
//====================================================================
struct SyNum {};
//====================================================================
template< class O, class UPOL>
void AppendSol( Seq<O>& sols, 
	        const UPOL& r, const Bezier<O,SyNum> & mth, 
		const O& a, const O& b, bool isole_= true)
{
  using let::assign;  
  typedef typename NumberTraits<typename UPOL::value_type>::FT rational_t;

  Seq<O> sol;
  bezier::rep1d<double> bp(r.size());
  double u,v;
  assign(u,a); assign(v,b);
  assign(bp,r,u,v);
  bezier::Isole<double>  isole;
  double eps2=1e-3;
  isole.Run(bp,eps2,true);
  isole.GetIntervals(sol);
  RConvert(sol,a,b);
  bezier::rep1d<rational_t> bz(r.size());
  for (  unsigned i = 0; i < sol.size(); i += 2 )
    {
      assign(bz,r, rational_t(sol[i]),rational_t(sol[i+1]));
      int v0=Variation(bz);
      if ( v0==1)
	{
	  sols.push_back( sol[i]   );
	  sols.push_back( sol[i+1] );
	}
      else if(v0>1)
	{
	  //	  std::cout << "IMPROVEMENT"<<std::endl;
	  AppendSol(sols, r, mth, rational_t(sol[i]), rational_t(sol[i+1]));
	}
    }
}
//====================================================================
__END_NAMESPACE_SYNAPS 
#endif synaps_solve_BzIsoleSymNum_H
