/*******************************************************************
 *   This file is part of the source code of the SYNAPS kernel.
 *   Author(s): J.P. Pavone, GALAAD, INRIA
 *   $Id: sbd1d.h,v 1.1 2005/09/28 06:30:42 mourrain Exp $
 ********************************************************************/
#ifndef synaps_solve_subdivision1d_H
#define synaps_solve_subdivision1d_H
/********************************************************************/
#include <synaps/base/Seq.h>
#include <synaps/arithm/rounding.h>
#include <synaps/arithm/rdw.h>
#include <synaps/base/TypeTests.h>
#include <synaps/arithm/hdwi.h>
#include <synaps/arithm/BigUnsigned.h>

#include <assert.h>
#include <vector>
#include <algorithm>

//====================================================================
__BEGIN_NAMESPACE_SYNAPS

#ifdef DEBUG
#define DEBUG_STATEMENT(stmt) { std::cout << #stmt; stmt; }
#else 
#define DEBUG_STATEMENT(stmt) stmt
#endif

//--------------------------------------------------------------------

namespace bezier
{
  template< class C >
  struct IsoleSimple
  {
    typedef unsigned sz_t;
    //  typedef unsigned unsigned_t;
    static const sz_t bitquo = numerics::bit_resolution<C>::nbit / numerics::hdwi<sz_t>::nbit;
    static const sz_t bitrem = numerics::bit_resolution<C>::nbit % numerics::hdwi<sz_t>::nbit;
    typedef BigUnsigned<bitquo+(sz_t)(bitrem!=0)> unsigned_t;
    typedef C creal_t;
    inline
    static void split( creal_t * r, creal_t * l, sz_t sz )
    {
      creal_t * er, * p;
      er = r + (sz-1);
      for ( er = r + (sz-1); er != r; er--, l++ )
	{
	  *l = *r;
	  for ( p = r; p != er; p ++ )
	    *p = (*p+*(p+1))/2;
	};
      *l = *r;
    };
    
    sz_t sgncnt ( creal_t const *  b , sz_t sz )
    {
      creal_t const * last = b + sz;
      sz_t c;
      int prv = (((*b>0)<<1)|(*b<0));
      int crr;
      for ( c = 0; b != last; crr = (((*b>0)<<1)|(*b<0)), c += prv != crr, prv = crr, b ++ );
      return c;
    };
    
    //    struct dmn_t
    //    {
    //      unsigned   h;
    //      unsigned_t v;
    //    };
    
    struct bound
    {
      unsigned  e; 
      unsigned_t m;
      bool operator==( const bound& b )
      {
	if ( e < b.e )
	  {
	    return (m << (b.e-e)) == b.m;
	  };
	return (b.m << (e-b.e)) == m;
      };
    };
    
    inline void split( bound& r, bound& l )
    {
      r.m <<= 1;
      l.m = r.m;
      r.m |=  1;
      r.e ++;
      l.e = r.e;
    };
    
    struct res_t { 
      bound a;
      bound b;
      bool t;  
    };
    
    creal_t  *m_data;
    bound    *m_dmn ;
    sz_t  m_limit;
    sz_t  m_s;
    
    std::vector<res_t> m_res;
    
    IsoleSimple()  { 
      m_data = 0; 
    };
    
    ~IsoleSimple() { 
      Free(); 
    };
    
    inline 
    void Free() 
    { 
      if ( m_data ) 
	{ 
	  delete[] m_data; 
	  delete[] m_dmn; 
	  m_res.resize(0); 
	}; 
    };
        
    inline 
    void alloc( sz_t s, sz_t cs, sz_t deep  )
    { 
      Free();
      m_s = s;
      m_limit = std::min((sz_t)numerics::hdwi<unsigned_t>::nbit,deep);
      m_data  = new creal_t [ cs*m_limit ];
      m_dmn   = new bound   [ m_limit ];
      m_res.resize(0);
    };
  
    inline 
    void alloc( sz_t s, sz_t deep ) 
    { 
      alloc(s,s,deep); 
    };
  
    inline const 
    bound& bcka() const 
    { 
      return m_res.back().a; 
    };

    inline
    bound& bcka()
    {
      return m_res.back().a;
    };
    
    inline const 
    bound& bckb() const 
    {
      return m_res.back().b;
    };
    
    inline
    bound& bckb()
    {
      return m_res.back().b;
    };
    
    inline 
    bool& bckt() 
    {
      return m_res.back().t;
    };
    
    inline 
    bool bckt() const
    {
      return m_res.back().t;
    };
    
    inline 
    void setbck( const bound& a, const bound& b, bool type )
    {
      bcka() = a;
      bckb() = b;
      bckt() = type;
    };
    
    inline 
    void pshbck()
    {
      m_res.push_back(res_t());
    };
    
    inline 
    void sstore( int d, int t = 1 ) 
    {
      // m_data + d*m_s;
      pshbck();
      bcka() = m_dmn[d];
      bckb() = bcka();
      bckb().m += t;
      bckt() = true;
    };
  
    inline 
    void bstore( int d, int t = 1 ) 
    {
      // m_data + d*m_s;
      pshbck();
      bcka() = m_dmn[d];
      bcka().m += t;
      bckb() = bcka();
      bckt() = true;
    };
   inline 
    void estore( int d, int t = 1 ) 
    {
      // m_data + d*m_s;
      pshbck();
      bckb() = m_dmn[d];
      bckb().m += t;
      bcka() = bckb();
      bckt() = true;
    };
  
    inline 
    void mstore( int d ) 
    {
      sstore(d);
      bckt() = false;
    };
    
    inline
    void print( creal_t * p, sz_t  n )
    {
      std::cout << "[";
      for ( sz_t i = 0; i < n-1; i ++ )
	std::cout << p[i] << ",";
      std::cout << p[n-1] << " ]";
    };

    
    void Loop( bool isole = true )
    {
      //      std::cout <<"Starting simple solver loop"<<std::endl;
      creal_t * pup, * pdw;
      int d;
      sz_t h,a,s,cup,cdw;
      unsigned_t v;
      s = m_s;
      pup = m_data;
      m_dmn[0].m = 0;
      m_dmn[0].e = 0;
      for( a = 0, d = 0; d >= 0; d--, a -= s   ) 
	{
	  cup = sgncnt(pup+a,s);
	  if ( cup ) 
	    {
	      if ( isole && cup == 1 ) { sstore(d); continue; };
	      if ( this->m_dmn[d].e == m_limit-1 )
		{
		  if ( cup == 1 ) 
		    {
		      if ( *(pup+a) != 0 )
			{
			  if ( *(pup+a+s-1) == 0 )
			    sstore(d,0);
			  else 
			    sstore(d);
			};
		    }
		  else  
		    {
		      mstore(d);
		      //std::cout<<"s "<<cup<<" "<<d<<" "<<m_limit<<std::endl;
		    }
		  continue;
		};
	      split(pup+a,pup+a+s,s);
	      split(this->m_dmn[d],this->m_dmn[d+1]);
	      d += 2;
	      a += 2*s;
	    };
	};
      //std::cout <<"End of simple solver loop"<<std::endl;
    };
  };
  
  template < class C >
  struct Isole : IsoleSimple<C>
  {
    typedef  C creal_t;
    typedef  unsigned sz_t; 
    typedef  IsoleSimple<C>  Base_t;
    typedef typename Base_t::unsigned_t unsigned_t;
    typedef typename Base_t::res_t      Domain_t;
    typedef typename Base_t::bound    bound;
    sz_t  m_cup;
    sz_t  m_cdw;
    
    inline 
    void alloc( sz_t s, sz_t deep  ) 
    { 
      Base_t::alloc(s,2*s,deep); 
    };
    
    
    void barre( char c, unsigned n )
    {
      char  _bar[ n+1 ];
      std::fill(_bar,_bar+n, c);
      _bar[n] = 0;
      std::cout << _bar << std::endl;
    };
    
    void writebounds( creal_t * pup, creal_t * pdw, unsigned s )
    {
      print(pup,s); std::cout << std::endl;
      print(pdw,s); std::cout << std::endl;
    };
    
    void writedomain( int d )
    {
      std::cout << "Domaine-" << d << std::endl;
      std::cout << "\thauteur = " << this->m_dmn[d].e << std::endl;
      std::cout << "\tvaleur  = " << this->m_dmn[d].m << std::endl;
    };

    void writebck()
    {
      std::cout << "Back " << std::endl;
      std::cout << "\tA = " << this->bcka().e << " , " << this->bcka().m  << std::endl;
      std::cout << "\tB = " << this->bckb().e << " , " << this->bckb().m  << std::endl;
    };
    

    inline 
    bool glue( sz_t cup, sz_t cdw, int d )
    {
      if ( this->bckb() == this->m_dmn[d] )
	{
	  this->bckb() = this->m_dmn[d];
	  this->bckb().m += 1;
	  if ( cup ) m_cup = cup;
	  if ( cdw ) m_cdw = cdw;
	  return true;
	};
      return false;
    };
    
    inline 
    void mstore( sz_t cup, sz_t cdw, int d  ) 
    {
      if ( this->m_res.size() == 0 || !glue(cup,cdw,d)  )
	{
	  Base_t::mstore(d);
	  m_cup = cup;
	  m_cdw = cdw;
	  //std::cout << "Push ";
	};
      //      writebck();
    };
  
    
    inline
    void dwsplit( creal_t * r, creal_t * l, sz_t s )
    {
      numerics::rounding<creal_t> rnd( numerics::rnd_dw() );
      this->split(r,l,s);
    };

    inline 
    void upsplit( creal_t * r, creal_t * l, sz_t s )
    {
      numerics::rounding<creal_t> rnd( numerics::rnd_up() );
      this->split(r,l,s);
    };
    
    void Loop( bool isole = true )
    {
      //  std::cout << "LOOP\n";
      numerics::rounding<creal_t> rnd(numerics::rnd_dw());
      creal_t * pup, * pdw;
      int d;
      sz_t a,s,cup,cdw;
      unsigned_t v;
      s = this->m_s;
      pup = this->m_data;
      pdw = this->m_data + s;
      this->m_dmn[0].m = 0;
      this->m_dmn[0].e = 0;
      for( a = 0, d = 0; d >= 0; d--, a -= 2*s   ) 
	{
	  cup = sgncnt(pup+a,s);
	  cdw = sgncnt(pdw+a,s);
	  if ( cup || cdw )
	    {
	      if ( isole && cup == cdw && cup == 1 ){ 
// 		std::cout << " (b) \n";
		this->sstore(d);
		continue; 
	      };
	      if ( this->m_dmn[d].e == this->m_limit-1 ) 
		{ 
// 		  std::cout << " (c) "<<cup<<" "<<cdw<<"\n";
// 		  if ( cup == cdw && cup == 1 )
// 		    this->sstore(d);
// 		   else
		  this->mstore(cup,cdw,d); 
		  continue; 
		};
	      dwsplit(pdw+a,pdw+a+2*s,s);    
	      upsplit(pup+a,pup+a+2*s,s);
	      this->split(this->m_dmn[d],this->m_dmn[d+1]);
	      d += 2;
	      a += 4*s;
	      continue;
	    }
	  else
	    {
	      //std::cout<<"  (a) ";
	      sz_t k =0, sv;
	      for (  k = 0; k < this->m_s; k ++ )
		{
		  if ( pup[a+k] > 0 != pdw[a+k] > 0 )
		    {
		      sv++;
		      if(sv>1)
			{ 
			  mstore(cup,cdw,d); 
			  break; 
			};
		    }
		}
	      //std::cout<<std::endl;
	    };
	};
      //      std::cout << "END LOOP\n";
    };
    
        
    template<class input>
    void Run( const input& in, const creal_t& eps, bool isole, const Type::true_t&   )
    {
      sz_t prec = numerics::bitprec(eps);
      alloc(in.size(),prec);
      std::copy(in.begin(),in.end(),this->m_data);
      std::copy(in.begin(),in.end(),this->m_data+this->m_s);
      Loop(isole);
    };
    
    template<class input>
    void Run( const input& in, const creal_t& eps, bool isole, const Type::false_t& )
    {
      sz_t prec = numerics::bitprec(eps);
      Base_t::alloc(in.size(),prec);
      std::copy(in.begin(),in.end(),this->m_data);
      Base_t::Loop(isole);
    };
    
    template<class input> inline
    void Run( const input& in, const creal_t& eps, bool isole )
    { 
      typedef typename numerics::is_rounded<creal_t>::result_t round_t;
      Run(in,eps,isole,round_t()); 
    }
    
    template<class input>
    void Run( const input& up, const input& dw, const creal_t& eps, bool isole   )
    {
      assert(up.size()==dw.size());
      sz_t prec = numerics::bitprec(eps);
      //      std::cout << eps << std::endl;
      //std::cout << numerics::bitprec(eps) << std::endl;
      alloc(up.size(),prec);
      std::copy(up.begin(),up.end(),this->m_data);
      std::copy(dw.begin(),dw.end(),this->m_data+this->m_s);
      Loop(isole);
    }
    
    template<class input> inline
    void RootsRun( const input& in, const creal_t& eps )
    {
      Run(in,eps,false);    
    }
    
    template<class input> inline
    void RootsRun( const input& up, const input& dw, const creal_t& eps )
    { 
      Run(up,dw,eps,false); 
    }
    
    template<class input> inline
    void IsoleRun( const input& in, const creal_t& eps )
    { 
      Run(in,eps,true); 
    };

    template<class input>
    void IsoleRun( const input& up, const input& dw, const creal_t& eps )
    { 
      Run(up,dw,eps,true); 
    };
    
    const Domain_t& operator[]( int i ) const 
    { 
      return this->m_res[i]; 
    };
    
    Domain_t& operator[]( int i ) 
    { 
      return this->m_res[i]; 
    };
    
    sz_t size() const 
    { 
      return this->m_res.size(); 
    };
    
    template<class real>
    void Convert( real& a, const bound& b )
    {
      unsigned_t m(b.m);
      numerics::hdwi<unsigned_t>::reverse(b.e+1,m);
      a = 0;
      real x = 1;
      //      std::cout<<"idx "<<m<<std::endl;
      for ( unsigned i = 0; i < b.e + 1; x /= 2, i ++ )
	{
	  if ( m & 1 ) a += x;
	  m >>= 1;
	};
      //      std::cout<<"sol "<<a<<std::endl;
    };
    
    template<class output>
    void GetIntervals( output& o )
    {
      unsigned s = o.size();
      //      typename output::value_type a,b;
      o.resize(s+2*size());
      for ( sz_t i = 0; i < this->m_res.size(); i ++ )
	{
	  Convert(o[s+2*i],this->m_res[i].a);
	  Convert(o[s+2*i+1],this->m_res[i].b);
	};
    };
  
    template<class output>
    void GetFlags( output& o )
    {
      unsigned s = o.size();
      o.resize( s + o.size() );
      for ( sz_t i = 0; i < this->m_res.size(); o[s+i] = this->m_res[i++].t );
    };
    
    template<class output>
    void GetMidPoints( output& o )
    {
      typename output::value_type a,b;
      unsigned s = o.size();
      //      std::cout << " osize  = ";
      //      std::cout << o.size() << std::endl;
      //      std::cout << m_res.size() << std::endl;
      o.resize(this->m_res.size()+o.size());
      for ( sz_t i = 0; i < this->m_res.size(); i ++ )
	{
	  Convert(a,this->m_res[i].a);
	  Convert(b,this->m_res[i].b);
	  o[s+i] = (a+b)/2.0;
	};

    };

    sz_t nb_sol() {return this->m_res.size();}
  };
}

//====================================================================
template< class O, class X >
void RConvert( Seq<O>& sol, unsigned s, const X& mx )
{
  using let::assign;
  for ( unsigned i = 0; i < s;  i ++  )
    if(sol[i] != 1) 
      sol[i]/=(sol[i]-1);
    else
      assign(sol[i],X(-mx));
  for ( unsigned i = 0; i < s/2;  std::swap(sol[s-i-1],sol[i]), i ++ );
  for ( unsigned i = s; i < sol.size();  i ++  )
    if(sol[i] != 1) 
      sol[i] /= (1-sol[i]);
    else
      assign(sol[i],mx);
};

template< class RT, class C >
void RConvert( Seq<RT>& sol, const C &a , const C& b)
{
  using let::assign;
  RT ra,rb;
  assign(ra,a); assign(rb,b);
  for ( unsigned i = 0; i < sol.size();  i ++  )
    {
      sol[i]= ra*(1-sol[i]) + rb*sol[i];
    }
};
//====================================================================
__END_NAMESPACE_SYNAPS
#endif  //synaps_solve_subdivision1d_H
