#ifndef _SYNAPS_SOLVE_STURMABERTH_H_
#define _SYNAPS_SOLVE_STURMABERTH_H_

#include <synaps/arithm/gmp.h>
#include <synaps/base/Seq.h>
#include <synaps/usolve/Algebraic.h>
#include <synaps/usolve/sturm/solve_sturm.h>


__BEGIN_NAMESPACE_SYNAPS

template <class O, class C=QQ, class X=double>
struct IslSturm 
{
  typedef C                RT;
  typedef C                FT;
  typedef Interval<FT>     FIT;
  typedef double           XT;
  typedef Interval<double> XIT;
  typedef O                solution_t;
  typedef O                RO_t;
  typedef UPolDse<FT>      Poly;

  X eps;
  IslSturm():eps(1.e-6) {}
  IslSturm(const X& e):eps(e) {}
  IslSturm(const IslSturm & mth): eps(mth.eps) {}
};


template <class UPOL, class O, class C>
Seq<Interval<O> > 
solve( const UPOL& f,  IslSturm<O,C> mth )
{

  typedef IslSturm<O,C>     K;
  typedef typename UPOL::coeff_t    RT;
  typedef typename K::FT    FT;
  typedef typename K::FIT   FIT;
  typedef typename K::RO_t  RO_t;
  typedef typename K::Poly  Poly;
  Seq<FIT> sol;
  SturmSeq<RT> sth(f,diff(f),SUBRES());
  FT b = max_bound<FT>(sth[0], Cauchy());
  FT a = -b;
  FIT I( a, b);
    
  ALGEBRAIC::sturm_isolation( sth, I, std::back_inserter(sol.rep()), K());
  std::sort(sol.begin(), sol.end());

  return sol;
}


template <class UPOL, class O, class C, class U>
Seq<Interval<O> > 
solve( const UPOL& f,  IslSturm<O,C> mth, const U& u, const U& v)
{

  typedef IslSturm<O,C>     K;
  typedef typename UPOL::coeff_t    RT;
  typedef typename K::FT    FT;
  typedef typename K::FIT   FIT;
  typedef typename K::RO_t  RO_t;
  typedef typename K::Poly  Poly;
  Seq<FIT> sol;
  SturmSeq<RT> sth(f,diff(f),SUBRES());
  FIT I;
  let::assign(I.lower(),u);
  let::assign(I.upper(),v);
    
  ALGEBRAIC::sturm_isolation( sth, I, std::back_inserter(sol.rep()), mth);
  //  std::sort(sol.begin(), sol.end());

  return sol;
}

__END_NAMESPACE_SYNAPS
#endif // _SYNAPS_SOLVE_STURMABERTH_H_

    
