
#ifndef __SYNAPS_SOLVE_STURM_COMMON_H__
#define __SYNAPS_SOLVE_STURM_COMMON_H__


#include <synaps/init.h>
//#include <synaps/usolve.h>
#include <synaps/arithm/Interval.h>
#include <synaps/arithm/double.h>
#include <synaps/upol/UPOLDAR.m>
#include <synaps/arithm/Zp.h>
#include <synaps/linalg/MatrDse.h>
#include <synaps/upol/resultant.h>
#include <synaps/upol/SturmSeq.h>
#include <synaps/usolve/algebraic/Algebraic_config.h>
__ALGEBRAIC_BEGIN_NAMESPACE


template < typename Poly >  inline
void
convert_to_pos_lcoeff(Poly& p)
{
  typedef typename Poly::value_type    NT;
  if (UPOLDAR::lcoeff(p) < NT(0))
    std::transform(p.begin(), p.end(), p.begin(), std::negate<NT>());
  return;
}




template < typename K > inline
typename K::RT
homogeneous_eval_horner(const typename K::Poly& f, 
			const typename K::RT&   b, 
			const typename K::RT&   c)
{
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::XT    XT;
  typedef typename K::XIT   XIT;
  
  RT d(1);

  int p = UPOLDAR::degree(f);
  RT H0 = UPOLDAR::lcoeff(f);
  RT H1;

  for (int i = 1; i <= p; ++i) {
    d = d * c;
    H1 = b * H0 + d * f[p-i];
    std::swap(H0, H1);
  }
  return H0;
}


template < typename K > inline
typename K::RT
homogeneous_eval_horner(const typename K::Poly& f, 
			const typename K::FT&   num)
{
  return homogeneous_eval_horner<K>( f, numerator( num), denominator( num));
}




template < typename K > inline
int
filtered_horner(const typename K::Poly& f, 
		const typename K::FT&   x)
{
  typedef typename K::RT    RT;
  typedef typename K::FT    FT;
  typedef typename K::XT    XT;
  typedef typename K::XIT   XIT;

  return sign( homogeneous_eval_horner<K>( f, x));
  
  //filtering
  XIT dx;
  let::assign( dx, x);

  std::cout << "dx: " << dx << std::endl; 

  UPolDse< XIT > F( degree( f), AsSize());
  for (int i = 0; i < degree( F); ++i) {
    let::assign( F[i], FT( f[i], 1));
  }

  


  dx = UPOLDAR::eval_horner( F, dx);
  std::cout << "1: " << dx <<  std::endl; 
  
  if ( contain_zero( dx) ) {
    return sign( homogeneous_eval_horner<K>( f, x));
  } else {
    std::cout << "2: " << dx << std::endl; 
    return sign( lower( dx));
  }

}








//--------------------------------------------------------------------
/** Return the number of sign changes at a value x.
 **/
template< class K >
unsigned 
VAR( const SturmSeq< typename K::RT >& tab, 
     const typename K::FT& x)
{

//   typedef typename K::XIT     XIT;
//   XIT dx;
//   let::assign( dx, x);
//   for (int i = 0; i < tab.size(); ++i) {
//     UPOLDAR::eval_horner( tab[i], dx);
//   }
  
  
  
  // You must put some filering here

  int i = 0, c = 0;
  int s0 = 0, s = 0;
  while (  (i < tab.size()) && 
	   ((s0 = filtered_horner<K>( tab[i], x)) == 0) )
    i++;
  while ( i < tab.size() ) {
    while( (i < tab.size()) && 
	   ((s = filtered_horner<K>( tab[i], x)) ==0) )
      ++i;
    if(s !=0 && s!=s0) {
      c++;
      s0 = s;
    }
    ++i;
  }
  return c;
}


template < typename K > inline
typename K::FT
simplest_rational_in_interval( typename K::RL x, 
			       typename K::RL y)
{
  typedef typename K::RT     RT;
  typedef typename K::FT     FT;
  typedef typename K::RL     RL;

  if (x == y) {
    return FT(x);
  }
    
  if ( x > y) {
    std::swap( x, y);
  }
    
    
  FT res;
  RT res_numer;
  RT res_denom;
    
  if (x < 0 && y < 0) {
    // Both arguments are negative: solve positive case and negate
    return  - simplest_rational_in_interval<K>(abs(x), abs(y));
  } else if (x <= 0 || y <= 0) {
    // One argument is 0, or arguments are on opposite sides of 0:
    // simplest rational in interval is 0 exactly. 
    res_numer = 0;
    res_denom = 1;
  } else if ( y - x >= 1) {
    res_numer = ceil( x);
    res_denom = 1;
  } else { // x > 0 && y > 0
    RL tx = 1/x;
    RL ty = 1/y;
	
    RL xc = floor(tx); // First coefficient of cf for x.
    RL xr = fracpart(tx); // Remaining fractional part of x.
    RL yc = floor(ty); // First coefficient of cf for y.
    RL yr = fracpart(ty); // Remaining fractional part of y.

    if (xc < yc) {
      // Return 1/(xc+1).
      res_numer = 1;
      res_denom = xc + 1;
    } else if (yc < xc) {
      // Return 1/(yc+1).
      res_numer = 1;
      res_denom = yc + 1;
    } else  {  // xc == yc
      // Recurse to find s, the rational with the lowest denominator
      //  between xr and yr.
      FT  s(simplest_rational_in_interval<K>( xr, yr));

      // Return 1/(xc + s).
      res_numer = denominator(s);
      RT  xc_rt(xc);
      res_denom = numerator(s) + xc_rt * denominator(s);
    }
  }

  return FT( res_numer, res_denom);
}



struct Refine_compare {
  template < typename RO_t>
  bool operator()( const RO_t& a, const RO_t& b)
  {
    typedef typename RO_t::FT    FT;
    typedef typename RO_t::FIT   FIT;
    
    while ( true ) {
      FT ileft  = std::max( a.left(), b.left());
      FT iright = std::min( a.right(), b.right());
      
      if (ileft >= iright) { break; }
      
      a.refine_interval();
      b.refine_interval();
    }
    return (a.right() < b.left());
  }
};


struct CompareFIT {
  template < typename FIT>
  bool operator()( const FIT& a, const FIT& b)
  {
    return ( upper( a) <= lower( b));
  }
};


// CProbabilistic check
struct Is_square_free {
  template < typename Poly >
    bool operator() ( const Poly& p)
    {
      typedef typename Poly::value_type   RT;
      typedef RT  Zp;
      typedef UPolDse< Zp >  Polyp;
      
      Zp   prime = 65003;
      
      Poly dp = diff( p);
      
      Polyp f( degree( p) + 1, AsSize());
      for (int i = 0; i <= degree( p); ++i) {
	RT tmp = p[i];
	f[i] =  tmp; // static_cast<int>( to_double( tmp));
      }
      
      Polyp df( degree( dp) + 1, AsSize());
      for (int i = 0; i <= degree( dp); ++i) {
	RT tmp = dp[i];
	
	df[i] =  tmp; // static_cast<int>( to_double( tmp));
      }
      
      typedef MatrDse< Zp> MAT;
      Zp d = Det( bezout<MAT>( f, df)) % prime;

      return ( d != 0 ? true : false );
    }
};





__ALGEBRAIC_END_NAMESPACE


#endif // __SYNAPS_SOLVE_STURM_COMMON_H__
