#ifndef SYNAPS_ARITHM_ALGEBRAIC_METHOD_NAME_H
#define SYNAPS_ARITHM_ALGEBRAIC_METHOD_NAME_H

#include <synaps/init.h>
#include <synaps/usolve/Algebraic_config.h>
#include <synaps/usolve/root_of.h>
#include <synaps/mpol/MPoly_2_2.h>
//#include <synaps/arithm/algebraic/Apx_root_of.h>
//#include <synaps/arithm/algebraic/Filt_root_of.h>

__BEGIN_NAMESPACE_SYNAPS


/**
 * This class contains all the types needed in order to use
 * the algebraic solvers in the ALGEBRAIC namespace.
 *
 * \sa NumberTraits, root_of
 */
template< typename RT_ >
struct ALG_STURM
{
  typedef ALG_STURM<RT_>                            self_t;

  typedef NumberTraits<RT_>                        NTR;
  typedef typename NTR::RT                          RT;
  typedef typename NTR::FT                          FT;
  typedef typename NTR::XT                          XT;

  typedef typename NTR::FIT                         FIT;

  typedef UPolDse<RT>                               Poly;
  typedef UPolDse< UPolDse<RT> >                    Poly_2;

  typedef UPolDse<RT>                               RT_Poly;
  typedef UPolDse< UPolDse<RT> >                    RT_Poly_2;

  typedef UPolDse<FT>                               FT_Poly;
  typedef UPolDse< UPolDse<FT> >                    FT_Poly_2;

  typedef ALGEBRAIC::MPoly_2_2<RT>                  MPoly_2_2;
  
  typedef UPolDse<XT>                               APX_POLY;
  
  typedef ALGEBRAIC::root_of<RT, Poly>              RO_t;
  // typedef ALGEBRAIC::apx_root_of<self_t>            apx_RO_t;

};


#if 0

/**
 * This the Filtered version of ALG_STURM.
 * This class contains all the types needed in order to use
 * the algebraic solvers in the ALGEBRAIC namespace.
 *
 * \sa NumberTraits, root_of, appx_root_of
 */
template< typename RT_ >
struct ALG_FILT_STURM 
{
  typedef ALG_FILT_STURM<RT_>                    self_t;

  typedef NumberTraits<RT_>                      NTR;
  typedef typename NTR::RT                        RT;
  typedef typename NTR::FT                        FT;
  typedef typename NTR::XT                        XT;

  typedef UPolDse<RT>                             POLY;
  typedef UPolDse<XT>                             APX_POLY;

  typedef ALGEBRAIC::root_of<RT, POLY>            RO_t;
  typedef ALGEBRAIC::apx_root_of<self_t>          apx_RO_t;


  typedef ALG_STURM<RT_>                          Parent_t;
  typedef ALGEBRAIC::filt_root_of<Parent_t>       FRO_t;

};

#endif 

__END_NAMESPACE_SYNAPS


#endif // SYNAPS_ARITHM_ALGEBRAIC_METHOD_NAME_H


