
#ifndef _SYNAPS_SOLVE_STURM_H_
#define _SYNAPS_SOLVE_STURM_H_

#include <synaps/init.h>
#include <synaps/arithm/Interval.h>
#include <synaps/arithm/NumberTraits.h>
#include <synaps/arithm/RealOf.h>
#include <synaps/upol/UPolDse.h>
#include <synaps/usolve.h>

#include <synaps/usolve/solver_base.h>
#include <synaps/usolve/Small_degree.h>
#include <synaps/usolve/sturm/common.h>
#include <synaps/usolve/sturm/solve_sturm.h>
#include <synaps/usolve/Bsolver.h>

__BEGIN_NAMESPACE_SYNAPS



template < class NT >
struct Sturm : public Small_degree<NT>
{
    typedef Sturm          self_t;
};



template < class NT > inline
Seq< typename Sturm<NT>::RO_t > 
solve( const typename Sturm<NT>::Poly& f,
       Sturm<NT>)
{
    typedef Sturm<NT>          K;
    typedef typename K::RO_t          RO_t;
    
    Seq<RO_t> sol;
    ALGEBRAIC::solve_sturm(f, std::back_inserter(sol.rep()), K());

    std::stable_sort( sol.begin(), sol.end(), ALGEBRAIC::Refine_compare());
    return sol;
}


template < class NT,
	   class OutputIterator> inline
OutputIterator
solve( const typename Sturm<NT>::Poly& f,
       OutputIterator it,
       Sturm<NT>)
{
    typedef Sturm<NT>          K;

    return ALGEBRAIC::solve_sturm(f, it, K());
}



template < class NT >  inline
typename Sturm<NT>::RO_t 
RootOf( const typename Sturm<NT>::Poly& f, unsigned i, Sturm<NT>)
{
   typedef Sturm<NT>          K;
   return solve( f, K())[i];
}



template < class NT > inline
int
Count_real_roots( const typename Sturm<NT>::Poly& f, Sturm<NT>)
{
    typedef Sturm<NT>          K;
    typedef typename K::RT            RT;
    typedef typename K::FT            FT;
    typedef typename K::FIT           FIT;

    SturmSeq<RT> sth( f, UPOLDAR::diff(f), SUBRES());
    FT b = Bound( f, Cauchy());
    FT a = -b;

    return ( Var( sth, a) - Var( sth, b) );
}


//-------------- Bivariate solvers ----------------------//


template < class NT > inline
Seq< VectDse< typename Sturm<NT>::RO_t > > 
solve( const typename Sturm<NT>::MPoly_2_2& f1,
       const typename Sturm<NT>::MPoly_2_2& f2,
       Sturm<NT>)
{
  typedef Sturm<NT>             K;
  typedef typename K::RO_t      RO_t;
  

  Seq< VectDse< RO_t > > sol;
  
  ALGEBRAIC::solve( f1, f2, std::back_inserter( sol.rep()), K());
  
  return sol;
  
}

template < typename NT, typename O, typename R >
Seq< VectDse< typename Sturm<NT>::RO_t > >
solve( const MPol<typename Sturm<NT>::RT, O, R>& f1, 
       const MPol<typename Sturm<NT>::RT, O, R>& f2,
       Sturm<NT>)
{
  typedef Sturm<NT>             K;

  return ALGEBRAIC::solve_with_sign( f1, f2, K());
  
}



__END_NAMESPACE_SYNAPS

#endif // _SYNAPS_SOLVE_STURM_H_

