
#ifndef __SYNAPS_SOLVE_STURM_SOLVE_D1_H__
#define __SYNAPS_SOLVE_STURM_SOLVE_D1_H__


#include <synaps/init.h>
#include <synaps/usolve/Algebraic_config.h>
#include <synaps/usolve/Util.h>
#include <synaps/upol/bound.h>



__ALGEBRAIC_BEGIN_NAMESPACE

template < typename K >
typename K::RO_t
make_root_of_1(const typename K::FT& a, K)
{
    typedef typename K::RT     RT;
    typedef typename K::FT     FT;
    typedef typename K::Poly   Poly;
    typedef typename K::RO_t   RO_t;

    Poly f(2, AsSize());
    f[0] = -numerator(a);
    f[1] = denominator(a);
    return RO_t(f, 0, 1, a, a);
}

/** Construction of an algebraic number  from a number @a@.
 * The number belongs to a ring.
 **/
template < typename K >
typename K::RO_t
make_root_of_1(const typename K::RT& a, K)
{
    typedef typename K::RT     RT;
    typedef typename K::FT     FT;
    typedef typename K::Poly   Poly;
    typedef typename K::RO_t   RO_t;

    Poly f(2, AsSize());
    f[0] = -a;
    f[1] = 1;
    return RO_t(f, 0, 1, a, a);
}




/** Construction of an algebraic number from two numbers.
 * The numbers belong to a ring. The algebraic  number is actually
 * a number that belongs to the fraction field of the ring.
 * Optionally, we can set the multiplicity and the root position
 * (in the original equation). This function is useful for the @adjust_poly()@ functions.
 **/
template < typename K >
typename K::RO_t
make_root_of_1(const typename K::RT& c0, const typename K::RT& c1, K)
{
    assert(c1 != 0);
      
    typedef typename K::RT     RT;
    typedef typename K::FT     FT;
    typedef typename K::Poly   Poly;
    typedef typename K::RO_t   RO_t;

    Poly f(2, AsSize());
    f[0] = c0;
    f[1] = c1;
    FT b = to_FT(RT(-c0), c1);
    return  RO_t(f, 0, 1, b, b);
}


__ALGEBRAIC_END_NAMESPACE


#endif // __SYNAPS_SOLVE_STURM_SOLVE_D1_H__
