/*********************************************************************
*      This file is part of the source code of SYNAPS.               *
*   Author(s): O. Ruatta,  GALAAD, INRIA                             *
*              LACO - Universt de Limoges                           *
**********************************************************************
History:
$Id: WeierTools.H,v 1.1 2005/07/11 10:04:00 jppavone Exp $
**********************************************************************/

#ifndef __SYNAPS__WEIERTOOLS__H
#define __SYNAPS__WEIERTOOLS__H

#include "synaps/linalg.H"
#include "synaps/upol.H"
#include "synaps/linalg/norm.H"
#include "synaps/arithm/RealOf.H"

namespace weierstrass {

template<class UPOL>
typename RealOf<typename UPOL::value_type>::TYPE pnorme2(const UPOL& P) {
  typedef typename UPOL::value_type value_type;
  typedef typename RealOf<value_type>::TYPE real_type;
  typedef typename UPOL::iterator IT;

  real_type res(0);

  for (IT it=P.begin(); it != P.end(); it++) res+=abs(*it)*abs(*it);
  return sqrt(res);
}



template<class VECT>
VECT WDenoms (const VECT& V) {
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type;
  typedef UPolDse<value_type> UPOL;
  UPOL P;
  PolByRoots(V,P);
  size_type d( V.size() );
  P = UPOLDAR::diff(P);
  VECT R(d);
  for(size_type i = 0; i < d; i++) R[i] = P(V[i]);
  return R;
}

template<class VECT, class UPOL>
VECT WeierField (const UPOL& P, VECT V) {
  assert ( P.degree() == V.size() );
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type;
  
  size_type d = V.size();
  VECT R(d), TMP(d);
  TMP = WDenoms(V);
  for(size_type i=0; i < d; i++) {
    R[i] = -P(V[i])/TMP[i];
  }
  return R;
}


template<class VECT, class UPOL>
void PolByRoots(const VECT& V, UPOL& P) {
  typedef typename VECT::size_type size_type;
  typedef typename VECT::value_type value_type;
  UPOL R(1);
  R[0] = 1; 
  value_type TMP[2];
  TMP[1]=1;
  for( size_type i = 0; i < V.size(); i++) {
    TMP[0] = -V[i];
    R *= UPOL(2,TMP);
  }
  P = R;
}


} //namespace weierstrass

#endif //__SYNAPS__WEIERTOOLS__H
