// Copyright (c) 2003  INRIA Sophia-Antipolis (France) and
//                     Department of Informatics and Telecommunications
//                     Univeristy of Athens (Greece).
// All rights reserved.
//
// Authors : Elias P. TSIGARIDAS <et@di.uoa.gr>

// Partially supported by INRIA's project "CALAMATA", a
// bilateral collaboration with National Kapodistrian University of
// Athens.
// Partially supported by the IST Programme of the EU as a Shared-cost
// RTD (FET Open) Project under Contract No  IST-2000-26473
// (ECG - Effective Computational Geometry for Curves and Surfaces)

#include <synaps/usolve/algebraic/comparison/auto/Auto_Results.h>
#include <synaps/usolve/algebraic/comparison/auto/Result_length_5.h>

__ALGEBRAIC_BEGIN_NAMESPACE

bool Result_length_5::initialize = false;

Result_length_5::MapResult Result_length_5::map_S0_p2q2;
Result_length_5::MapResult Result_length_5::map_S0_p1q1;
Result_length_5::MapResult Result_length_5::map_S1_p2q2;
Result_length_5::MapResult Result_length_5::map_S1_p1q1;
Result_length_5::MapResult Result_length_5::map_S2_p2q2;
Result_length_5::MapResult Result_length_5::map_S2_p1q1;
Result_length_5::MapResult Result_length_5::map_S3_p2q2;
Result_length_5::MapResult Result_length_5::map_S3_p1q1;
Result_length_5::MapResult Result_length_5::map_S4;



Comparison_result
Result_length_5::on_S0_p2q2(int where)
{
	MapResult_iter it;
	it = map_S0_p2q2.find(where);
	if (it != map_S0_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S0_p1q1(int where)
{
	MapResult_iter it;
	it = map_S0_p1q1.find(where);
	if (it != map_S0_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S1_p2q2(int where)
{
	MapResult_iter it;
	it = map_S1_p2q2.find(where);
	if (it != map_S1_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S1_p1q1(int where)
{
	MapResult_iter it;
	it = map_S1_p1q1.find(where);
	if (it != map_S1_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S2_p2q2(int where)
{
	MapResult_iter it;
	it = map_S2_p2q2.find(where);
	if (it != map_S2_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S2_p1q1(int where)
{
	MapResult_iter it;
	it = map_S2_p1q1.find(where);
	if (it != map_S2_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S3_p2q2(int where)
{
	MapResult_iter it;
	it = map_S3_p2q2.find(where);
	if (it != map_S3_p2q2.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S3_p1q1(int where)
{
	MapResult_iter it;
	it = map_S3_p1q1.find(where);
	if (it != map_S3_p1q1.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}

Comparison_result
Result_length_5::on_S4(int where)
{
	MapResult_iter it;
	it = map_S4.find(where);
	if (it != map_S4.end())
		return to_CR(it->second);
	else
		return UNKNOWN;
}



 void Result_length_5::set_default_values()
{
	map_S0_p1q1[   0] = EQUAL;
	map_S0_p1q1[   3] = EQUAL;

	map_S2_p1q1[  17] = SMALLER;
	map_S2_p1q1[  34] = SMALLER;

	map_S2_p1q1[  29] = LARGER;
	map_S2_p1q1[  46] = LARGER;

	map_S3_p2q2[   2] = SMALLER;
	map_S3_p2q2[  10] = SMALLER;
	map_S3_p2q2[  49] = SMALLER;
	map_S3_p2q2[  57] = SMALLER;
	map_S3_p2q2[  65] = SMALLER;
	map_S3_p2q2[  69] = SMALLER;
	map_S3_p2q2[ 114] = SMALLER;
	map_S3_p2q2[ 118] = SMALLER;

	map_S3_p2q2[   9] = LARGER;
	map_S3_p2q2[  13] = LARGER;
	map_S3_p2q2[  58] = LARGER;
	map_S3_p2q2[  62] = LARGER;
	map_S3_p2q2[  70] = LARGER;
	map_S3_p2q2[  78] = LARGER;
	map_S3_p2q2[ 117] = LARGER;
	map_S3_p2q2[ 125] = LARGER;

	map_S3_p1q1[   1] = SMALLER;
	map_S3_p1q1[   5] = SMALLER;
	map_S3_p1q1[  21] = SMALLER;
	map_S3_p1q1[  25] = SMALLER;
	map_S3_p1q1[  38] = SMALLER;
	map_S3_p1q1[  42] = SMALLER;
	map_S3_p1q1[  50] = SMALLER;
	map_S3_p1q1[  54] = SMALLER;
	map_S3_p1q1[ 194] = SMALLER;
	map_S3_p1q1[ 202] = SMALLER;
	map_S3_p1q1[ 210] = SMALLER;
	map_S3_p1q1[ 214] = SMALLER;
	map_S3_p1q1[ 218] = SMALLER;
	map_S3_p1q1[ 222] = SMALLER;
	map_S3_p1q1[ 225] = SMALLER;
	map_S3_p1q1[ 229] = SMALLER;
	map_S3_p1q1[ 233] = SMALLER;
	map_S3_p1q1[ 237] = SMALLER;
	map_S3_p1q1[ 241] = SMALLER;
	map_S3_p1q1[ 249] = SMALLER;

	map_S3_p1q1[   6] = LARGER;
	map_S3_p1q1[  14] = LARGER;
	map_S3_p1q1[  18] = LARGER;
	map_S3_p1q1[  22] = LARGER;
	map_S3_p1q1[  26] = LARGER;
	map_S3_p1q1[  30] = LARGER;
	map_S3_p1q1[  33] = LARGER;
	map_S3_p1q1[  37] = LARGER;
	map_S3_p1q1[  41] = LARGER;
	map_S3_p1q1[  45] = LARGER;
	map_S3_p1q1[  53] = LARGER;
	map_S3_p1q1[  61] = LARGER;
	map_S3_p1q1[ 201] = LARGER;
	map_S3_p1q1[ 205] = LARGER;
	map_S3_p1q1[ 213] = LARGER;
	map_S3_p1q1[ 217] = LARGER;
	map_S3_p1q1[ 230] = LARGER;
	map_S3_p1q1[ 234] = LARGER;
	map_S3_p1q1[ 250] = LARGER;
	map_S3_p1q1[ 254] = LARGER;

	map_S4[  73] = SMALLER;
	map_S4[  77] = SMALLER;
	map_S4[  85] = SMALLER;
	map_S4[  89] = SMALLER;
	map_S4[  97] = SMALLER;
	map_S4[ 101] = SMALLER;
	map_S4[ 105] = SMALLER;
	map_S4[ 109] = SMALLER;
	map_S4[ 113] = SMALLER;
	map_S4[ 121] = SMALLER;
	map_S4[ 134] = SMALLER;
	map_S4[ 142] = SMALLER;
	map_S4[ 146] = SMALLER;
	map_S4[ 150] = SMALLER;
	map_S4[ 154] = SMALLER;
	map_S4[ 158] = SMALLER;
	map_S4[ 166] = SMALLER;
	map_S4[ 170] = SMALLER;
	map_S4[ 178] = SMALLER;
	map_S4[ 182] = SMALLER;
	map_S4[ 322] = SMALLER;
	map_S4[ 330] = SMALLER;
	map_S4[ 338] = SMALLER;
	map_S4[ 342] = SMALLER;
	map_S4[ 346] = SMALLER;
	map_S4[ 350] = SMALLER;
	map_S4[ 358] = SMALLER;
	map_S4[ 362] = SMALLER;
	map_S4[ 378] = SMALLER;
	map_S4[ 382] = SMALLER;
	map_S4[ 385] = SMALLER;
	map_S4[ 389] = SMALLER;
	map_S4[ 405] = SMALLER;
	map_S4[ 409] = SMALLER;
	map_S4[ 417] = SMALLER;
	map_S4[ 421] = SMALLER;
	map_S4[ 425] = SMALLER;
	map_S4[ 429] = SMALLER;
	map_S4[ 437] = SMALLER;
	map_S4[ 445] = SMALLER;

	map_S4[  66] = LARGER;
	map_S4[  74] = LARGER;
	map_S4[  82] = LARGER;
	map_S4[  86] = LARGER;
	map_S4[  90] = LARGER;
	map_S4[  94] = LARGER;
	map_S4[ 102] = LARGER;
	map_S4[ 106] = LARGER;
	map_S4[ 122] = LARGER;
	map_S4[ 126] = LARGER;
	map_S4[ 129] = LARGER;
	map_S4[ 133] = LARGER;
	map_S4[ 149] = LARGER;
	map_S4[ 153] = LARGER;
	map_S4[ 161] = LARGER;
	map_S4[ 165] = LARGER;
	map_S4[ 169] = LARGER;
	map_S4[ 173] = LARGER;
	map_S4[ 181] = LARGER;
	map_S4[ 189] = LARGER;
	map_S4[ 329] = LARGER;
	map_S4[ 333] = LARGER;
	map_S4[ 341] = LARGER;
	map_S4[ 345] = LARGER;
	map_S4[ 353] = LARGER;
	map_S4[ 357] = LARGER;
	map_S4[ 361] = LARGER;
	map_S4[ 365] = LARGER;
	map_S4[ 369] = LARGER;
	map_S4[ 377] = LARGER;
	map_S4[ 390] = LARGER;
	map_S4[ 398] = LARGER;
	map_S4[ 402] = LARGER;
	map_S4[ 406] = LARGER;
	map_S4[ 410] = LARGER;
	map_S4[ 414] = LARGER;
	map_S4[ 422] = LARGER;
	map_S4[ 426] = LARGER;
	map_S4[ 434] = LARGER;
	map_S4[ 438] = LARGER;
}

__ALGEBRAIC_END_NAMESPACE
